@@ -65,7 +65,7 @@ void PDB_cloneUser(char *name, int32_t cloneid, int32_t *newId);
 void PDB_deleteUser(int32_t id);
 void PDB_createGroup(char *name, int32_t owner, int32_t *newGroupId);
 void PDB_deleteGroup(int32_t groupId);
-void PDB_lookupByName(char *name, int32_t *id);
+void PDB_lookupByName(const char *name, int32_t *id);
 void PDB_lookupById(int32_t id, char **name);
 int PDB_nameInUse(char *name);
 void PDB_changeId(int32_t oldid, int32_t newid);
@@ -81,7 +81,7 @@ void pdb_unpack(PDB_profile *r, void *data, size_t size);
 void PDB_freeProfile(PDB_profile *r);
 void PDB_writeProfile(PDB_HANDLE h, PDB_profile *r);
 void PDB_readProfile(PDB_HANDLE h, int32_t id, PDB_profile *r);
-void PDB_readProfile_byname(PDB_HANDLE h, char *name, PDB_profile *r);
+void PDB_readProfile_byname(PDB_HANDLE h, const char *name, PDB_profile *r);
 void PDB_deleteProfile(PDB_HANDLE h, PDB_profile *r);
 void PDB_printProfile(FILE *out, PDB_profile *r);
 void PDB_updateCps(PDB_HANDLE h, PDB_profile *r);
@@ -94,7 +94,8 @@ void PDB_db_release(void);
 void PDB_db_maxids(PDB_HANDLE h, int32_t *uid, int32_t *gid);
 void PDB_db_update_maxids(PDB_HANDLE h, int32_t uid, int32_t gid, int mode);
 void PDB_db_write(PDB_HANDLE h, int32_t id, char *name, void *buf, size_t size);
-void PDB_db_read(PDB_HANDLE h, int32_t id, char *name, void **buf,size_t *size);
+void PDB_db_read(PDB_HANDLE h, int32_t id, const char *name, void **buf,
+		 size_t *size);
 void PDB_db_delete(PDB_HANDLE h, int32_t id, char *name);
 void PDB_db_delete_xfer(PDB_HANDLE h, char *name);
 int PDB_db_exists(void);
