@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -83,10 +83,6 @@ extern "C" {
 #define O_BINARY 0
 #endif
 
-static void GetPathName();
-
-static char *pName;		/* name to use on PIOCTLS */
-
 typedef struct {
     int			    sTokenSize;
     EncryptedSecretToken    stoken;
@@ -99,7 +95,8 @@ typedef struct {
     auth server.  If setPag is true, a setpag system call is made.
     Returns 0 on success, -1 on failure.  Who knows what setpag did? */
 
-int U_SetLocalTokens(IN int setPag, IN ClearToken *cToken, IN EncryptedSecretToken sToken, IN char *realm)
+int U_SetLocalTokens(IN int setPag, IN ClearToken *cToken,
+		     IN EncryptedSecretToken sToken, IN const char *realm)
 {
     int    rc;
     struct ViceIoctl buffer;
@@ -118,12 +115,11 @@ int U_SetLocalTokens(IN int setPag, IN ClearToken *cToken, IN EncryptedSecretTok
     strncpy(inbuff.realm, realm, MAXHOSTNAMELEN);
 
     buffer.in = (char *)&inbuff;
-    buffer.out = 0;
+    buffer.out = NULL;
     buffer.in_size = sizeof(inbuff);
     buffer.out_size = 0;
 
-    GetPathName(); 
-    rc = pioctl(pName, _VICEIOCTL(3), &buffer, 0);
+    rc = pioctl(NULL, _VICEIOCTL(3), &buffer, 0);
     if(rc) {
 	return(-1);
     }
@@ -145,8 +141,7 @@ int U_GetLocalTokens(OUT ClearToken *cToken, OUT EncryptedSecretToken sToken, IN
     buffer.out = (char *)&outbuff;
     buffer.in_size = strlen(realm) + 1;
     buffer.out_size = sizeof(venusbuff);
-    GetPathName();
-    rc = pioctl(pName, _VICEIOCTL(8), &buffer, 0);
+    rc = pioctl(NULL, _VICEIOCTL(8), &buffer, 0);
     if(rc) {
 #ifdef __CYGWIN__
 	return rc;
@@ -165,29 +160,16 @@ int U_GetLocalTokens(OUT ClearToken *cToken, OUT EncryptedSecretToken sToken, IN
 }
 
 
-int U_DeleteLocalTokens(char *realm)
+int U_DeleteLocalTokens(const char *realm)
  /* Deletes internal state for viceId.  Returns 0.    */
 {
     struct ViceIoctl buffer;
 
-    buffer.in = realm;
-    buffer.out = 0;
+    buffer.in = (char *)realm;
+    buffer.out = NULL;
     buffer.in_size = strlen(realm) + 1;
     buffer.out_size = 0;
-    GetPathName();
-    pioctl(pName, _VICEIOCTL(9), &buffer, 0);
-    return(0);    
-}
-
-#if defined(DJGPP) || defined(__CYGWIN32__)
-#define DFLT_MNT "N:"
-#else
-#define DFLT_MNT "/coda"
-#endif
-
-static void GetPathName()
-{
-    codaconf_init("venus.conf");
-    CODACONF_STR(pName, "mountpoint", DFLT_MNT);
+    pioctl(NULL, _VICEIOCTL(9), &buffer, 0);
+    return(0);
 }
 
