@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -18,9 +18,11 @@ listed in the file CREDITS.
 #ifndef _AVENUS_H_
 #define _AVENUS_H_
 
-int U_DeleteLocalTokens(char *realm);
-int U_GetLocalTokens(ClearToken *cToken, EncryptedSecretToken sToken, const char *realm);
-int U_SetLocalTokens(int setPag, ClearToken *cToken, EncryptedSecretToken sToken, char *realm);
+int U_DeleteLocalTokens(const char *realm);
+int U_GetLocalTokens(ClearToken *cToken, EncryptedSecretToken sToken,
+		     const char *realm);
+int U_SetLocalTokens(int setPag, ClearToken *cToken,
+		     EncryptedSecretToken sToken, const char *realm);
 
 #endif /* _AVENUS_H_ */
 
