@@ -66,8 +66,6 @@ static int key1valid, key2valid;
 static uint8_t auth2key1[AUTH2KEYSIZE];
 static uint8_t auth2key2[AUTH2KEYSIZE];
 
-extern void ntoh_SecretToken(SecretToken *);
-
 /*  Wrapper function when multiple authentication type support was added;
     fits new calling parameters.  We only allow by-token authentication to
     vice
@@ -166,7 +164,7 @@ long GetKeysFromToken(IN RPC2_Integer *AuthenticationType,
 	return(-1);	/* no valid key did the job */
     }
 
-    now = time(0);
+    now = time(NULL);
     if (now > endtimestamp) {
 	LogMsg(10, SrvDebugLevel, stdout,
 	       "End time stamp %d > time %d for user %d",
