@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -98,7 +98,7 @@ int main(int argc, char **argv)
     ClearToken		    testCTok;
     char *hostname=NULL;
     char *username=NULL;
-    char *realm = NULL;
+    const char *realm = NULL;
     long rc;
     int i;
     int testing = 0;
@@ -115,10 +115,6 @@ int main(int argc, char **argv)
     authmethod = AUTH_METHOD_KERBEROS5;
 #endif 
 
-#ifdef DJGPP
-	__djgpp_set_quiet_socket(1);	
-#endif
-
     i = 1;
     while (i < argc) {
 	    if (U_GetAuthMethod(argv[i], &authmethod)) {
@@ -241,26 +237,26 @@ int main(int argc, char **argv)
 	rc = U_Authenticate(srvs, authmethod, username, strlen(username)+1,
 			    &cToken, sToken, verbose, interactive);
 	RPC2_freeaddrinfo(srvs);
-        if (rc != 0) {
-            fprintf (stderr, "Invalid login (%s).\n", RPC2_ErrorMsg(rc));
-            exit (1);
-        }
+	if (rc != 0) {
+	    fprintf (stderr, "Invalid login (%s).\n", RPC2_ErrorMsg(rc));
+	    exit (1);
+	}
     }
 
     if (testing)
 	    printf ("Sending token to venus\n");
 
     if (tofile)
-        WriteTokenToFile(tofile, &cToken, sToken);
+	WriteTokenToFile(tofile, &cToken, sToken);
 
-    if(U_SetLocalTokens(0, &cToken, sToken, realm))
+    if (U_SetLocalTokens(0, &cToken, sToken, realm))
 	printf("Local login only, could not contact venus\n");
-    
+
     if (testing) {
 	    printf("Getting tokens back from venus\n");
 	    if (U_GetLocalTokens(&testCTok, testSTok, realm) < 0)
 		    perror("U_GetLocalTokens");
-	    
+
 	    printf("Comparing clear token\n");
 	    if(memcmp(&cToken, &testCTok, sizeof(ClearToken)) != 0) {
 		    printf("Bad ClearToken\n");
