@@ -19,6 +19,7 @@ listed in the file CREDITS.
 #include <config.h>
 #endif
 
+#include <sys/stat.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "coda_string.h"
@@ -67,7 +68,8 @@ void ReadTokenFromFile(char *filename, ClearToken *cToken,
                        EncryptedSecretToken sToken)
 {
     FILE *f;
-    char *buf = NULL, c;
+    char *buf = NULL;
+    signed char c;
     int len;
 
     f = fopen(filename, "r");
