@@ -34,12 +34,8 @@ extern "C" {
 #include <errno.h>
 #include "coda_string.h"
 #include <stdlib.h>
-#ifdef __BSD44__
-#include <ufs/ufs/dir.h>
-#undef	DIRSIZ
-#else
-#define DIRBLKSIZ	0x200
-#endif
+#include <stdint.h>
+#include <dirent.h>
 #include <lwp/lwp.h>
 #include <lwp/lock.h>
 #include <rvmlib.h>
@@ -51,10 +47,14 @@ extern "C" {
 #endif
 
 #undef MMAP_DIR_CONTENTS
-#if defined(HAVE_MMAP) && !defined(DJGPP)
+#ifdef HAVE_MMAP
 #define MMAP_DIR_CONTENTS 1
 #endif
 
+#ifndef DIRBLKSIZ
+#define DIRBLKSIZ	0x1000
+#endif
+
 /* are we dealing with RVM memory? (yes for Venus, no for Vice)*/
 int dir_data_in_rvm;
 
@@ -64,7 +64,7 @@ void DIR_Print(PDirHeader, FILE *f);
 static int dir_FindBlobs (struct DirHeader **dh, int nblobs);
 static int dir_AddPage (struct DirHeader **dir);
 static int dir_NameBlobs(const char *);
-static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename, 
+static struct DirEntry *dir_FindItem (struct DirHeader *dir, const char *ename,
 				       struct DirEntry **preventry, int *index,
 				       int flags);
 struct DirEntry *dir_GetBlob (struct DirHeader *dir, long blobno);
@@ -72,16 +72,16 @@ static void dir_FreeBlobs(struct DirHeader *dir, int firstblob, int nblobs);
 static struct DirHeader *dir_Extend(struct DirHeader *olddirh, int in_rvm);
 
 
-static int fid_FidEqNFid(struct DirFid *fid, struct DirNFid *nfid);
-static void fid_Fid2NFid(struct DirFid *fid, struct DirNFid *nfid);
-static void fid_NFid2Fid(struct DirNFid *nfid, struct DirFid *fid); 
+static int fid_FidEqNFid(DirFid *fid, struct DirNFid *nfid);
+static void fid_Fid2NFid(DirFid *fid, struct DirNFid *nfid);
+static void fid_NFid2Fid(struct DirNFid *nfid, DirFid *fid);
 static void fid_NFidV2Fid(struct DirNFid *, VolumeId, struct ViceFid *);
 
 struct DirFind {
-	char		*df_ename;
+	const char	*df_ename;
 	struct DirEntry *df_tp;
 	struct DirEntry *df_lp;
-	int             df_index;          	
+	int		 df_index;
 };
 
 /* locking policy: DH_ routines lock.  DIR routines may
@@ -288,7 +288,7 @@ static struct DirHeader *dir_Extend(struct DirHeader *olddir, int in_rvm)
 struct PageHeader *DIR_Page(struct DirHeader *dirh, int page)
 {
 	CODA_ASSERT(page >= 0);
-	return (struct PageHeader *)((void *)dirh + page * DIR_PAGESIZE);
+	return (struct PageHeader *)((char *)dirh + page * DIR_PAGESIZE);
 
 }
 
@@ -374,36 +374,36 @@ static int dir_DirEntry2VDirent(PDirEntry ep, struct venus_dirent *vd, VolumeId
  */
 
 
-static int fid_FidEqNFid(struct DirFid *fid, struct DirNFid *nfid) 
+static int fid_FidEqNFid(DirFid *fid, struct DirNFid *nfid)
 {
 	if ( !fid || !nfid ) 
 		CODA_ASSERT(0);
 	
-	if ( (ntohl(nfid->dnf_vnode) == fid->df_vnode) &&
-	     (ntohl(nfid->dnf_unique) == fid->df_unique) )
+	if ( (ntohl(nfid->dnf_vnode) == fid->Vnode) &&
+	     (ntohl(nfid->dnf_unique) == fid->Unique) )
 		return 1;
 	else 
 		return 0;
 
 }
 
-static void fid_Fid2NFid(struct DirFid *fid, struct DirNFid *nfid) 
+static void fid_Fid2NFid(DirFid *fid, struct DirNFid *nfid)
 {
 	if ( !fid || !nfid ) 
 		CODA_ASSERT(0);
 	
-	nfid->dnf_vnode = htonl(fid->df_vnode);
-	nfid->dnf_unique = htonl(fid->df_unique);
+	nfid->dnf_vnode = htonl(fid->Vnode);
+	nfid->dnf_unique = htonl(fid->Unique);
 
 }
 
-static void fid_NFid2Fid(struct DirNFid *nfid, struct DirFid *fid) 
+static void fid_NFid2Fid(struct DirNFid *nfid, DirFid *fid)
 {
 	if ( !fid || !nfid ) 
 		CODA_ASSERT(0);
 	
-	fid->df_vnode = ntohl(nfid->dnf_vnode);
-	fid->df_unique = ntohl(nfid->dnf_unique);
+	fid->Vnode = ntohl(nfid->dnf_vnode);
+	fid->Unique = ntohl(nfid->dnf_unique);
 
 }
 
@@ -439,7 +439,7 @@ void DH_PrintStats(FILE *fp)
    return 0 if found
    return EONOENT upon failure
 */
-int DIR_LookupByFid(PDirHeader dhp, char *name, struct DirFid *fid) 
+int DIR_LookupByFid(PDirHeader dhp, char *name, DirFid *fid)
 {
 	int i;
 	int code = 0;
@@ -499,7 +499,7 @@ int DIR_Length (struct DirHeader *dir)
 
 
 /* Create an entry in a directory file.  */
-int DIR_Create (struct DirHeader **dh, char *entry, struct DirFid *fid)
+int DIR_Create (struct DirHeader **dh, const char *entry, DirFid *fid)
 {
 	int blobs, firstblob;
 	int i;
@@ -520,9 +520,9 @@ int DIR_Create (struct DirHeader **dh, char *entry, struct DirFid *fid)
 		return ENAMETOOLONG;
 	if ( strlen(entry) == 0 )
 		return EINVAL;
-    
+
 	/* First check if file already exists. */
-	ep = dir_FindItem(dir, entry, 0, 0, CLU_CASE_SENSITIVE);
+	ep = dir_FindItem(dir, entry, NULL, NULL, CLU_CASE_SENSITIVE);
 	if (ep) {
 		return EEXIST;
 	}
@@ -562,7 +562,7 @@ int DIR_Create (struct DirHeader **dh, char *entry, struct DirFid *fid)
    entry descriptors. 
    Return 0 upon success
 */
-int DIR_Delete(struct DirHeader *dir, char *entry)
+int DIR_Delete(struct DirHeader *dir, const char *entry)
 {
 	int nitems, index;
 	struct DirEntry *firstitem;
@@ -604,8 +604,7 @@ int DIR_Delete(struct DirHeader *dir, char *entry)
    We don't do this in conjunction with the parent, to flexibly create
    root directories too. Maybe MakeSubDir is a good idea? 
 */
-int DIR_MakeDir (struct DirHeader **dir,struct DirFid *me, 
-		 struct DirFid *parent)
+int DIR_MakeDir (struct DirHeader **dir, DirFid *me, DirFid *parent)
 {
 	int i;
 	struct DirHeader *dhp;
@@ -681,13 +680,13 @@ void DIR_Free(struct DirHeader *dir, int in_rvm)
 /* print one entry */
 int DIR_PrintEntry(PDirEntry entry, FILE *f)
 {
-	struct DirFid fid;
+	DirFid fid;
 
 	fid_NFid2Fid(&(entry->fid), &fid);
 
-	fprintf(f, "next: %hu, flag %d fid: (%lx.%lx) %s\n",
+	fprintf(f, "next: %hu, flag %d fid: (%x.%x) %s\n",
 		ntohs(entry->next), entry->flag,
-		fid.df_vnode, fid.df_unique, entry->name);
+		fid.Vnode, fid.Unique, entry->name);
 
 
 	return 0;
@@ -778,7 +777,7 @@ static void fid_NFidV2Fid(struct DirNFid *dnfid, VolumeId vol, struct ViceFid *f
    return 0 upon success
    return ENOENT upon failure
 */
-int DIR_Lookup(struct DirHeader *dir, char *entry, struct DirFid *fid,
+int DIR_Lookup(struct DirHeader *dir, const char *entry, DirFid *fid,
 	       int flags)
 {
 	struct DirEntry *de;
@@ -786,7 +785,7 @@ int DIR_Lookup(struct DirHeader *dir, char *entry, struct DirFid *fid,
 	if ( !dir ) 
 		return ENOENT;
 
-	de = dir_FindItem(dir, entry, 0, 0, flags);
+	de = dir_FindItem(dir, entry, NULL, NULL, flags);
 	if (de == 0) {
 		return ENOENT;
 	}
@@ -799,7 +798,7 @@ int DIR_Lookup(struct DirHeader *dir, char *entry, struct DirFid *fid,
 /* an EnumerateDir hook for comparing two directories. */
 int dir_HkCompare(PDirEntry de, void *hook)
 {
-	struct DirFid dfid;
+	DirFid dfid;
 	int rc;
 	PDirHeader dh = (PDirHeader)hook;
 	rc = DIR_Lookup(dh, de->name, &dfid, CLU_CASE_SENSITIVE);
@@ -844,13 +843,13 @@ int DIR_Convert (PDirHeader dir, char *file, VolumeId vol, RealmId realm)
 	   maximum-sized directory entry for each chunk of
 	   directory we write.
 	 */
-#endif /* __BSD44__ */
+#endif
 	len += (len / DIRBLKSIZ + 1) * ((sizeof(struct venus_dirent) + 3) & ~3);
 	len = ((len + (DIRBLKSIZ - 1)) & ~(DIRBLKSIZ - 1));
 
 #ifdef MMAP_DIR_CONTENTS
 	CODA_ASSERT( ftruncate(fd, len) == 0 );
-	buf = mmap(0, len, PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0);
+	buf = mmap(NULL, len, PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0);
 	CODA_ASSERT(buf != MAP_FAILED);
 #else
 	buf = malloc(len);
@@ -1006,9 +1005,9 @@ int DIR_IsEmpty (struct DirHeader *dhp)
 /* Return a pointer to an entry, given its number. */
 struct DirEntry *dir_GetBlob (struct DirHeader *dir, long blobno)
 {
-	if (!dir || blobno <= 0 || blobno > (EPP * DIR_MAXPAGES)) 
-		return 0;
-	
+	if (!dir || blobno <= 0 || blobno > (EPP * DIR_MAXPAGES))
+		return NULL;
+
 	return (struct DirEntry *)
 		((char *)dir + blobno * sizeof(struct DirBlob));
 }
@@ -1032,39 +1031,19 @@ int DIR_Hash (const char *string)
 
 /* an EnumerateDir hook to find a DirEntry with case-insensitive match.*/
 /* BUG: index is always set to 0. smarc */
-int dir_FindCaseInsensitive(PDirEntry de, void *hook){
+int dir_FindCaseInsensitive(PDirEntry de, void *hook)
+{
 	struct DirFind *find = (struct DirFind *) hook;
-	char           *p = NULL;
-	char           *name = strdup(find->df_ename);
-	int            length = strlen(name);
-	char           *name2;
-	int            length2;
-	int            rc = 0;
+	int rc = 0;
 
-      	/* lowercase name to look for */
-	for (p = name; p < name + length; p++)
-		if (isupper((int)*p))
-	       		*p += 'a' - 'A';
-
-	name2 = strdup(de->name);
-       	length2 = strlen(name2);
-
-      	/* lowercase name2 */       
-	for (p = name2; p < name2 + length2; p++)
-      	       	if (isupper((int)*p))
-			*p += 'a' - 'A';
-
-	if (!strcmp(name, name2)){
+	if (strcasecmp(find->df_ename, de->name) == 0)
+	{
 		find->df_tp = de;
 		find->df_index = 0; /* BUG no way to find out about index here. smarc */
-		strcpy(find->df_ename, de->name);
 		rc = 1; /* indicate EnumerateDir to stop */
 	}
-	else 
+	else
 		find->df_lp = de;
-		
-	if (name) free(name);
-	if (name2) free(name2);
 
 	return rc;
 }
@@ -1072,16 +1051,16 @@ int dir_FindCaseInsensitive(PDirEntry de, void *hook){
 /* Find a directory entry, given its name.  This entry returns a
    pointer to DirEntry, and a pointer to the previous DirEntry (to aid
    the delete code) in the hash chain.  If no entry is found a null
-   pointer is returned instead. 
+   pointer is returned instead.
 
 BUG: index doesn't work with CLU_CASE_INSENSITIVE so far. smarc
 */
-static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename,
+static struct DirEntry *dir_FindItem (struct DirHeader *dir, const char *ename,
 				      struct DirEntry **preventry, int *index,
 				      int flags)
 {
 	int rc = 0;
-	int i;	
+	int i;
 	short blobno;
 	struct DirEntry *tp;
 	struct DirEntry *lp = NULL;	/* page of previous entry in chain */
@@ -1089,8 +1068,8 @@ static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename,
 
 	switch (flags) {
 	case CLU_CASE_INSENSITIVE:
-		if (!dir) 
-			return 0;
+		if (!dir)
+			return NULL;
 
 		find.df_ename = ename;
 		find.df_lp = NULL;
@@ -1098,26 +1077,29 @@ static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename,
 		rc = DIR_EnumerateDir(dir, dir_FindCaseInsensitive, (void *) &find);
 		if (rc){
 			if (preventry)
-		          	*preventry = find.df_lp;
-		       	if (index)
-		       		*index = find.df_index;	
-			strcpy(ename, find.df_tp->name);	       							
-			return find.df_tp;			
-		}		       			
+			    *preventry = find.df_lp;
+			if (index)
+			    *index = find.df_index;
+			/* copying the found name back won't work reliably
+			 * especially because the names we're looking for are
+			 * in several cases const strings */
+			//strcpy(ename, find.df_tp->name);
+			return find.df_tp;
+		}
 
-		return 0;
+		return NULL;
 		break;
 
 	case CLU_CASE_SENSITIVE:
-	
-		if (!dir) 
-			return 0;
-	
+
+		if (!dir)
+			return NULL;
+
 		i = DIR_Hash(ename);
 		blobno = ntohs(dir->dirh_hashTable[i]);
 
 		if (blobno == 0)
-			return 0;
+			return NULL;
 
 		tp = dir_GetBlob(dir,blobno);
 		lp = NULL;
@@ -1135,11 +1117,11 @@ static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename,
 
 			/* The end of the line */
 			blobno = ntohs(tp->next);
-			if (blobno == 0) 
-				return 0;
+			if (blobno == 0)
+				return NULL;
 			tp = dir_GetBlob(dir,blobno);
-			if (!tp) 
-				return 0;
+			if (!tp)
+				return NULL;
 		}
 		break;
 	default:
@@ -1147,7 +1129,7 @@ static struct DirEntry *dir_FindItem (struct DirHeader *dir, char *ename,
 		CODA_ASSERT(0);
 	}
 
-	return 0;
+	return NULL;
 }
 
 
