@@ -1,3 +1,4 @@
+
 /* BLURB gpl
 
                            Coda File System
@@ -88,14 +89,17 @@ extern "C" {
 #include "coda_assert.h"
 #include "repio.h"
 
+
+#define DEF_BUF 2048
+
 static char *eatwhite(char *), *eatnonwhite(char *);
-static int acldecode(char *, unsigned int *);
+static int acldecode(const char *, unsigned int *);
 static int growarray(char **arrayaddr, int *arraysize, int elemsize);
 
-
 #define ewrite(f, b, l) do { if (write(f, b, l) != l) goto err; } while(0);
+
 int repair_putdfile(char *fname, int replicaCount, struct listhdr *replicaList)
-    /*	fname		name of file to write list to
+    /*	fname		filename to write list to
      *	replicaCount 	no of replicas
      *	replicaList	array of headers, one per replica
      *
@@ -106,25 +110,22 @@ int repair_putdfile(char *fname, int replicaCount, struct listhdr *replicaList)
     int i, k, n;
     unsigned int x, j;
     struct repair *r;
-    int fd;
+    FILE *file = NULL;
 
-    fd = open(fname, O_WRONLY|O_CREAT|O_EXCL|O_BINARY);
-    if (fd < 0) {
-	perror(fname);
-	return -1;
-    }
+    file = fopen(fname, "w+");
+    CODA_ASSERT(file);
     
     /* Write out number of replicas */
     x = htonl(replicaCount);
-    ewrite(fd, &x, sizeof(int));
+    fwrite(&x, sizeof(int), 1, file);
     
     /* Write out header for each replica */
     for (i = 0; i < replicaCount; i++) {
-	x = htonl(replicaList[i].replicaId);
-	ewrite(fd, &x, sizeof(int));
+        x = htonl(replicaList[i].replicaFid.Volume);
+	fwrite(&x, sizeof(int), 1, file);
 
 	x = htonl(replicaList[i].repairCount);
-	ewrite(fd, &x, sizeof(int));
+	fwrite(&x, sizeof(int), 1, file);
     }
 
     /* Write out list of repairs for each replica */
@@ -132,30 +133,27 @@ int repair_putdfile(char *fname, int replicaCount, struct listhdr *replicaList)
 	r = replicaList[i].repairList;
 	for (j = 0; j < replicaList[i].repairCount; j++) {
 	    x = htonl(r[j].opcode);
-	    ewrite(fd, &x, sizeof(int));
+	    fwrite(&x, sizeof(int), 1, file);
 
 	    n = strlen(r[j].name);
-	    ewrite(fd, r[j].name, n);
+	    fwrite(r[j].name, sizeof(char), n, file);
 
-	    ewrite(fd, "\n", 1);
+	    fwrite("\n", sizeof(char), 1, file);
 
 	    n = strlen(r[j].newname);
-	    ewrite(fd, r[j].newname, n);
+	    fwrite(r[j].newname, sizeof(char), n, file);
+
+	    fwrite("\n", sizeof(char), 1, file);
 
-	    ewrite(fd, "\n", 1);
 	    for (k = 0; k < REPAIR_MAX; k++) {
-		x = htonl(r[j].parms[k]);
-		ewrite(fd, &x, sizeof(int));
+	        x = htonl(r[j].parms[k]);
+		fwrite(&x, sizeof(int), 1, file);
 	    }
 	}
     }
-    close(fd);
-    return 0;
 
-err:
-    perror("repair_putdfile");
-    close(fd);
-    return -1;
+    fclose(file);
+    return 0;
 }
 
 int repair_getdfile(char *fname, int infd, int *replicaCount, struct listhdr **replicaList)
@@ -173,34 +171,69 @@ int repair_getdfile(char *fname, int infd, int *replicaCount, struct listhdr **r
     int i, k;
     unsigned int x, j;
     struct repair *r;
-    FILE *ff;
+    FILE *ff = NULL;
     char *s;
+    char errmsg[DEF_BUF];
 
-#define CHKERR() if (ferror(ff) || feof(ff)) goto ERR
+    perror("repair_getdfile: starting\n");
+
+    if (replicaList == NULL) {
+      sprintf(errmsg, "repair_getdfile: bad replicaList ptr!");
+      goto ERR;
+    }
 
     if (fname == NULL) ff = fdopen(infd, "r");
     else               ff = fopen(fname, "r");
-    if (!ff) { perror((fname ? fname : "input fd")); return(-1); }
+    if (!ff) {
+      sprintf(errmsg, "repair_getdfile: couldn't open file!");
+      perror(errmsg);
+      return -1;
+    }
+
+    perror("repair_getdfile: file opened");
     
-    fread(&x, sizeof(int), 1, ff); CHKERR();
+    fread(&x, sizeof(int), 1, ff);
+    if (ferror(ff) || feof(ff)) {
+      sprintf(errmsg, "repair_getdfile: replicaCount parsing failed!");
+      goto ERR;
+    }
     *replicaCount = ntohl(x);
 
     *replicaList = (struct listhdr *) calloc(*replicaCount, sizeof(struct listhdr));
+    if (*replicaList == NULL) {
+      sprintf(errmsg, "repair_getdfile: replicaList allocation failed!\n\telements= %x\tx= %d\tsizeof(struct listhdr)= %d\n", *replicaCount, x, sizeof(struct listhdr));
+      goto ERR;
+    }
 
-    if (*replicaList == NULL) goto ERR;
-
+    perror("repair_getdfile: list created");
     for (i = 0; i < *replicaCount; i++) {
-	fread(&x, sizeof(int), 1, ff); CHKERR();
-	(*replicaList)[i].replicaId  = ntohl(x);
-	fread(&x, sizeof(int), 1, ff); CHKERR();
+
+	fread(&x, sizeof(int), 1, ff);
+	if (ferror(ff) || feof(ff)) {
+	  sprintf(errmsg, "repair_getdfile: fread failed!");
+	  goto ERR;
+	}
+	(*replicaList)[i].replicaFid.Volume  = ntohl(x);
+
+	fread(&x, sizeof(int), 1, ff);
+	if (ferror(ff) || feof(ff)) {
+	  sprintf(errmsg, "repair_getdfile: fread failed!");
+	  goto ERR;
+	}
+
 	(*replicaList)[i].repairCount  = ntohl(x);	
     }
 
+    perror("repair_getdfile: replicas parsed");
+
     for (i = 0; i < *replicaCount; i++) {
 	if ((*replicaList)[i].repairCount > 0){
 	    r = (struct repair *) calloc((*replicaList)[i].repairCount,
 					 sizeof(struct repair));
-	    if (!r)  return(-1);
+	    if (!r) {
+	      sprintf(errmsg, "repair_getdfile: repair calloc failed!\n\tcount = %d\tsize = %d\n\n", (*replicaList)[i].repairCount, sizeof(struct repair));
+	      goto ERR;
+	    }
 	    (*replicaList)[i].repairList = r;
 	}
 	else {
@@ -209,32 +242,54 @@ int repair_getdfile(char *fname, int infd, int *replicaCount, struct listhdr **r
 	}
 	
 	for (j = 0; j < (*replicaList)[i].repairCount; j++) {
-	    fread(&x, sizeof(int), 1, ff); CHKERR();
+	    fread(&x, sizeof(int), 1, ff);
+	    if (ferror(ff) || feof(ff)) {
+	      sprintf(errmsg, "repair_getdfile: fread failed!\n");
+	      goto ERR;
+	    }
 	    r[j].opcode = ntohl(x);
 
 	    s = r[j].name;
-	    fgets(s, MAXNAMELEN, ff); CHKERR();
+	    fgets(s, MAXNAMELEN, ff);
+	    if (ferror(ff) || feof(ff)) {
+	      sprintf(errmsg, "repair_getdfile: fread failed!\n");
+	      goto ERR;
+	    }
 	    *(s + strlen(s) - 1) = 0;  /* nuke the '\n' left behind by fgets() */
 
 	    s = r[j].newname;
-	    fgets(s, MAXNAMELEN, ff); CHKERR();
+	    fgets(s, MAXNAMELEN, ff);
+	    if (ferror(ff) || feof(ff)) {
+	      sprintf(errmsg, "repair_getdfile: fread failed!\n");
+	      goto ERR;
+	    }
 	    *(s + strlen(s) - 1) = 0;  /* nuke the '\n' left behind by fgets() */
 
 	    for (k = 0; k < REPAIR_MAX; k++) {
-		fread(&x, sizeof(int), 1, ff); CHKERR();
+		fread(&x, sizeof(int), 1, ff);
+		if (ferror(ff) || feof(ff)) {
+		  sprintf(errmsg, "repair_getdfile: fread failed!\n");
+		  goto ERR;
+		}
 		r[j].parms[k] = ntohl(x);
 	    }
 	}
+	perror("repair_getdfile: replica processed");
     }
-    if (fname != NULL) fclose(ff);
+    if (fname != NULL)
+      fclose(ff);
+
+    perror("repair_getdfile: completed!");
+
     return(0);
 
  ERR: /* Error exit */
-    if (fname == NULL) perror("input fd");
-    else { perror(fname); fclose(ff); }
+    perror(errmsg);
+    if(fname != NULL)
+      fclose(ff);
+
     return(-1);
 }
-#undef CHKERR
 
 int repair_getdfile(char *fname, int *replicaCount, struct listhdr **replicaList)
    { return(repair_getdfile(fname, 0, replicaCount, replicaList)); }
@@ -257,7 +312,7 @@ int repair_parseline(char *line, struct repair *rs)
      */
 {
     char *c, *d, *eos;
-    int i;
+    int i, localhost = 0;
 
 #define NEXTFIELD()								\
     /* Set c to start of next field, d to the null at the end of this field */	\
@@ -382,7 +437,7 @@ int repair_parseline(char *line, struct repair *rs)
 	c = eatwhite(c);
 	if (*c) return(-1);
     }
-    return(0);
+    return localhost;
 }
 #undef NEXTFIELD
 #undef ADVANCE
@@ -399,7 +454,7 @@ int repair_parsefile(char *fname, int *hdcount, struct listhdr **hdarray)
      *  Returns -1 with err msg if errors (including syntax errors) were found.
      */
 {
-    int rc, lineno;
+  int rc, lineno;
     char line[MAXPATHLEN];
     FILE *rf;
     struct repair rentry;
@@ -435,19 +490,19 @@ int repair_parsefile(char *fname, int *hdcount, struct listhdr **hdarray)
 	}
 
 	if (rentry.opcode == REPAIR_REPLICA) { /* new replica */
-	    growarray((char **)hdarray, hdcount, sizeof(struct listhdr));
-	    (*hdarray)[*hdcount - 1].replicaId = rentry.parms[0];
-	    (*hdarray)[*hdcount - 1].repairCount = 0;
-	    (*hdarray)[*hdcount - 1].repairList = 0;
+	  growarray((char **)hdarray, hdcount, sizeof(struct listhdr));
+	  (*hdarray)[*hdcount - 1].replicaFid.Volume = rentry.parms[0];
+	  (*hdarray)[*hdcount - 1].repairCount = 0;
+	  (*hdarray)[*hdcount - 1].repairList = 0;
 	}
 	else { /* another entry for the current replica */
-	    struct repair **rearray;
-	    int *recount;
-	    
-	    rearray = &((*hdarray)[(*hdcount - 1)].repairList);
-	    recount  = (int *)&((*hdarray)[(*hdcount - 1)].repairCount);
-	    growarray((char **)rearray, recount, sizeof(struct repair));
-	    (*rearray)[*recount - 1] = rentry; /* struct assignment */
+	  struct repair **rearray;
+	  int *recount;
+
+	  rearray = &((*hdarray)[(*hdcount - 1)].repairList);
+	  recount  = (int *)&((*hdarray)[(*hdcount - 1)].repairCount);
+	  growarray((char **)rearray, recount, sizeof(struct repair));
+	  (*rearray)[*recount - 1] = rentry; /* struct assignment */
 	}
     }
 
@@ -483,7 +538,7 @@ static char *eatnonwhite(char *s) {
 }
 
 
-static int acldecode(char *s, unsigned int *r)
+static int acldecode(const char *s, unsigned int *r)
     /*
 	s	input string
 	r	output rights mask
@@ -492,7 +547,7 @@ static int acldecode(char *s, unsigned int *r)
 	Returns 0 on success, -1 if s is bogus in some way
     */
 {
-    register int i, max;
+    int i, max;
 
     if (!strcmp(s, "none")) s = "";
     if (!strcmp(s, "all")) s = "rlidwka";
@@ -521,8 +576,9 @@ static int acldecode(char *s, unsigned int *r)
 }
 
 
-void repair_printline(struct repair *rs, FILE *ff) {
-    char *c;
+void repair_printline(struct repair *rs, FILE *ff)
+{
+    const char *c;
     char quoted_name[3*MAXNAMELEN];
     int i;
     
@@ -564,10 +620,12 @@ void repair_printfile(char *fname) {
     int repcount, i;
     struct listhdr *list;
     unsigned int j;
+    char buf[200];
     repair_getdfile(fname, 0, &repcount, &list);
     for (i = 0; i < repcount; i++) {
-	printf("New replica: volume id %08x has %d repair entries\n",
-	       list[i].replicaId, list[i].repairCount);
+        sprintf(buf, "New replica: volume id %08x has %d repair entries\n",
+	       list[i].replicaFid.Volume, list[i].repairCount);
+	perror(buf);
 	for (j = 0; j < list[i].repairCount; j++) 
 	    repair_printline(&list[i].repairList[j], stdout);
     }
