@@ -69,6 +69,7 @@ int nextavailindex = -1;
 resdir_entry	**sortedArrByFidName;	/* for sorting the direntries in fid order*/
 resdir_entry	**sortedArrByName;	/* for sorting the direntries in name order */
 int totaldirentries = 0;
+
 /* VolumeId RepVolume; */
 int nConflicts;
 static char AclBuf[2048];
@@ -78,7 +79,7 @@ static int getfid (char *path, ViceFid *Fid, ViceVersionVector *VV, struct ViceI
     char buf[2048];
     vi->out = buf;
     vi->out_size = sizeof(buf);
-                      
+
     if (pioctl(path, _VICEIOCTL(_VIOC_GETFID), vi, 0)) {
 	char symval[MAXPATHLEN];
 	symval[0] = 0;
@@ -201,10 +202,11 @@ int getunixdirreps (int nreplicas, char *names[], resreplica **reps)
   /* allocate space for dir entries  - allocate some extra nodes */
   direntriesarrsize = (nreplicas + 1) * (buf.st_size/AVGDIRENTRYSIZE);
   CODA_ASSERT(direntriesarr = (resdir_entry *)malloc(direntriesarrsize * sizeof(resdir_entry)));
-  nextavailindex = -1;
-  totaldirentries = 0;
 
-  for(j = 0; j < nreplicas; j++){
+  totaldirentries = 0;
+  nextavailindex = -1;
+
+  for(j = 0; j < nreplicas; j++) {
       int count;
       ViceFid Fid;
       ViceVersionVector VV;
@@ -234,19 +236,21 @@ int getunixdirreps (int nreplicas, char *names[], resreplica **reps)
 	  /* get index of direntry */
 	  i = nextindex();
 	  if (Fid.Vnode != 1 || Fid.Unique != 1) {
-	      direntriesarr[i].vno = Fid.Vnode;
-	      direntriesarr[i].uniqfier = Fid.Unique;
 	      direntriesarr[i].MtPt = 0;
 	  }
 	  else {
-	      if (res_getmtptfid(path, &Fid, &VV)) return -1;
-	      direntriesarr[i].vno = Fid.Vnode;
-	      direntriesarr[i].uniqfier = Fid.Unique;
-	      direntriesarr[i].MtPt = 1;
+	    if (res_getmtptfid(path, &Fid, &VV))
+	      return -1;
+	    direntriesarr[i].MtPt = 1;
 	  }
 	  strcpy(direntriesarr[i].name, dp->d_name);
 	  direntriesarr[i].VV = VV;
-	  direntriesarr[i].replicaid = j;
+
+	  direntriesarr[i].fid.Vnode = Fid.Vnode;
+	  direntriesarr[i].fid.Unique = Fid.Unique;
+	  direntriesarr[i].fid.Volume = Fid.Volume;
+
+	  direntriesarr[i].index = j;
 	  direntriesarr[i].lookedAt = 0;
 	  count++;
 	  free(path);
@@ -256,9 +260,12 @@ int getunixdirreps (int nreplicas, char *names[], resreplica **reps)
       /* fill in the resreplica */
       if (res_getfid(names[j], &Fid, &VV)) return -1;
       dirs[j].nentries = count;
-      dirs[j].replicaid = Fid.Volume;
-      dirs[j].vnode = Fid.Vnode;
-      dirs[j].uniqfier = Fid.Unique;
+
+      CODA_ASSERT(Fid.Volume != 0xffffffff); /* will trigger if we have an expanded dirent */
+
+      dirs[j].fid.Volume = Fid.Volume;
+      dirs[j].fid.Vnode = Fid.Vnode;
+      dirs[j].fid.Unique = Fid.Unique;
       dirs[j].path = (char *)malloc(strlen(names[j]) + 1);
       strcpy(dirs[j].path, names[j]);
       /* fill in access list and modebits */
@@ -267,8 +274,10 @@ int getunixdirreps (int nreplicas, char *names[], resreplica **reps)
 	  return(-1);
       }
       dirs[j].modebits = buf.st_mode;
-      if (!(dirs[j].al = res_getacl(names[j]))) /* return -1; */
+      if (!(dirs[j].al = res_getacl(names[j]))) { /* return -1; */
 	  fprintf(stderr, "\t--> getacl on \"%s\" FAILED!!!\n", names[j]);
+	  return -1;
+      }
       dirs[j].owner = buf.st_uid;
   }
   return(0);
@@ -290,12 +299,13 @@ static int nextindex()
     return(nextavailindex);
 }
 
-void MarkEntriesByFid (VnodeId Vnode, Unique_t unique)
+void MarkEntriesByFid (VnodeId Vnode, Unique_t Unique)
 {
     resdir_entry *direntryptr = direntriesarr;
     for(int i = 0; i < totaldirentries; i++)
-	if ((direntryptr[i].vno == Vnode) && (direntryptr[i].uniqfier == unique))
-	    direntryptr[i].lookedAt = 1;
+	if ((direntryptr[i].fid.Vnode == Vnode) &&
+	    (direntryptr[i].fid.Unique == Unique))
+	  direntryptr[i].lookedAt = 1;
     
 }
 
@@ -306,7 +316,7 @@ void MarkEntriesByGroup (resdir_entry **mark_arr, int nentries)
 	mark_arr[i]->lookedAt = 1;
 }
 
-int GetConflictType (int nreplicas, resreplica *dirs, resdir_entry **deGroup, int nentries, int *conflictType, char *volmtpt, char *realm)
+int GetConflictType (int nreplicas, resreplica *dirs, resdir_entry **deGroup, int nentries, int *conflictType, char *realm)
 {
     int i;
     *conflictType = STRONGLY_EQUAL;
@@ -328,20 +338,27 @@ int GetConflictType (int nreplicas, resreplica *dirs, resdir_entry **deGroup, in
 
 void InitListHdr (int nreplicas, resreplica *dirs, struct listhdr **opList)
 {   struct listhdr *lh;
-    *opList = lh = (struct listhdr *) malloc(nreplicas * sizeof(struct listhdr));
+  *opList = lh = (struct listhdr *) malloc(nreplicas * sizeof(struct listhdr));
     for (int i = 0; i < nreplicas; i++){
-	lh[i].replicaId = dirs[i].replicaid;
+	lh[i].replicaFid = dirs[i].fid;
 	lh[i].repairCount = 0;
 	lh[i].repairList = NULL;
     }
 }
 
 /* inserts a copy of a repair struct into the repair ops list */
-int InsertListHdr (struct repair *rep, struct listhdr **ops, int index)
+int InsertListHdr (int nreplicas, struct repair *rep, struct listhdr **ops, int index)
 {
-    int size = (*ops)[index].repairCount;
+    int size;
     struct repair *repList;
-    
+
+    if(!ops)
+      return -1;
+
+    CODA_ASSERT(index < nreplicas);
+
+    size = (*ops)[index].repairCount;
+
     repList = (struct repair *)malloc(sizeof(struct repair) * (size + 1));
     if (repList == 0) return -1;
     if (size > 0) 
@@ -379,7 +396,7 @@ int IsCreatedEarlier (struct listhdr **opList, int index, VnodeId vnode, Unique_
 
 void ResolveConflict (int nreplicas, resreplica *dirs, resdir_entry **deGroup,
 		      int nentries, int conflictType, listhdr **opList,
-		      char *volmtpt, VolumeId RepVolume, char *realm)
+		      VolumeId RepVolume, char *realm)
 {
     /* call the appropriate repair function */
     switch (conflictType){
@@ -388,7 +405,7 @@ void ResolveConflict (int nreplicas, resreplica *dirs, resdir_entry **deGroup,
       case ALL_PRESENT:
 	break;
       case SUBSET_RENAME:
-	RepairRename(nreplicas, dirs, deGroup, nentries, opList, volmtpt,
+	RepairRename(nreplicas, dirs, deGroup, nentries, opList,
 		     RepVolume, realm);
 	break;
       case SUBSET_CREATE:
@@ -410,7 +427,7 @@ void ResolveConflict (int nreplicas, resreplica *dirs, resdir_entry **deGroup,
     /* mark the group - if conflict is resolvable */
     MarkEntriesByGroup(deGroup, nentries);
     if (conflictType == SUBSET_RENAME) 
-	MarkEntriesByFid(deGroup[0]->vno, deGroup[0]->uniqfier);
+      MarkEntriesByFid(deGroup[0]->fid.Vnode, deGroup[0]->fid.Unique);
 }
 
 int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct listhdr **opList, struct repinfo *inf) {
@@ -421,13 +438,15 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
     int uselsoutput = 0;
     int i;
     char replicatedname[MAXPATHLEN];
-    strcpy(replicatedname, dirs[sortedArrByName[first]->replicaid].path);
-    char *lastslash = rindex(replicatedname, '/'); // the trailing /
+    char *lastslash;
+
+    strcpy(replicatedname, dirs[sortedArrByName[first]->index].path);
+    lastslash = strrchr(replicatedname, '/'); // the trailing /
     if (!lastslash) 
 	printf("Couldn't find the parent directory of %s\n", sortedArrByName[first]->name);
     else {
 	*lastslash = '\0';
-	lastslash = rindex(replicatedname, '/'); // the / before the replica name
+	lastslash = strrchr(replicatedname, '/'); // the / before the replica name
 	if (!lastslash)
 	    printf("Couldn't find the parent directory of %s\n", sortedArrByName[first]->name);
 	else {
@@ -437,30 +456,16 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
     }
     if (uselsoutput) {
 	char cmd[2 * MAXPATHLEN];
-	resdir_entry *rde;
-	char *path, *replicaname;
-	char cwdpath[MAXPATHLEN];
-	char *cwd = getcwd(cwdpath, MAXPATHLEN);
-	chdir(replicatedname);
-	for (i = first; i < last; i++) {
-	    rde = sortedArrByName[i];
-	    path = dirs[rde->replicaid].path;  	// this has a trailing /
-	    path[strlen(path)-1] = '\0'; 		// erase the last /
-	    replicaname = rindex(path, '/') + 1;
-	    sprintf(cmd, "/bin/ls -lF %s/%s",
-		    replicaname, sortedArrByName[first]->name);
-	    system(cmd);
-	    path[strlen(path)]='/';	// pretty gross!
-	}
+	sprintf(cmd, "/bin/ls -lF %s/", replicatedname);
+	system(cmd);
 	printf("\n\n");
-	if (cwd) chdir(cwd);
     }
     
     for (i = first; i < last; i++) {
 	resdir_entry *rde = sortedArrByName[i];
 	printf("%s%s\n\tFid: (%08x.%08x) VV:(%d %d %d %d %d %d %d %d)(%x.%x)\n",
-	       dirs[rde->replicaid].path, sortedArrByName[first]->name,
-	       rde->vno, rde->uniqfier, rde->VV.Versions.Site0,
+	       dirs[rde->index].path, sortedArrByName[i]->name,
+	       rde->fid.Vnode, rde->fid.Unique, rde->VV.Versions.Site0,
 	       rde->VV.Versions.Site1, rde->VV.Versions.Site2, rde->VV.Versions.Site3,
 	       rde->VV.Versions.Site4, rde->VV.Versions.Site5, rde->VV.Versions.Site6,
 	       rde->VV.Versions.Site7, rde->VV.StoreId.Host, rde->VV.StoreId.Uniquifier);
@@ -468,11 +473,11 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
     int answers[MAXHOSTS];
     char nnpath[MAXPATHLEN], fixedpath[MAXPATHLEN], lnpath[MAXPATHLEN];
     struct stat sbuf;
-    for (i= 0; i < MAXHOSTS; i++) answers[i] = -1;
+    for (i = 0; i < MAXHOSTS; i++) answers[i] = -1;
     CODA_ASSERT((last-first) <= MAXHOSTS);
     for (i = first; i < last; i++) {
 	resdir_entry *rde = sortedArrByName[i];
-	sprintf(nnpath, "%s%s", dirs[rde->replicaid].path, rde->name);
+	sprintf(nnpath, "%s%s", dirs[rde->index].path, rde->name);
 	if (inf->interactive) {
 	    printf("Should %s be removed? ", nnpath);
 	    answers[i-first] = Parser_getbool("", 0);
@@ -509,20 +514,21 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
 
     if (nno) {
 	/* check that only single unique object will exist */
-	VnodeId goodvnode = (VnodeId)-1;
+	VnodeId  goodvnode = (VnodeId)-1;
 	Unique_t goodunique = (Unique_t)-1;
 	for (i = 0; i < nobjects; i++) {
 	    if (!answers[i]) {
 		if (goodvnode == (VnodeId)-1) {
-		    goodvnode = sortedArrByName[i+first]->vno;
-		    goodunique = sortedArrByName[i+first]->uniqfier;
+		    goodvnode = sortedArrByName[i+first]->fid.Vnode;
+		    goodunique = sortedArrByName[i+first]->fid.Unique;
 		}
-		else if (goodvnode != sortedArrByName[i+first]->vno ||
-			 goodunique != sortedArrByName[i+first]->uniqfier) {
+		else if (goodvnode != sortedArrByName[i+first]->fid.Vnode ||
+			 goodunique != sortedArrByName[i+first]->fid.Unique) {
 		    printf("Please try to rename or remove one of the two objects:\n");
 		    printf("(%08x.%08x) and (%08x.%08x) with name %s\n",
-			   goodvnode, goodunique, sortedArrByName[i+first]->vno, 
-			   sortedArrByName[i+first]->uniqfier,
+			   goodvnode, goodunique,
+			   sortedArrByName[i+first]->fid.Vnode,
+			   sortedArrByName[i+first]->fid.Unique,
 			   sortedArrByName[first]->name);
 		    return(-1);
 		}
@@ -533,14 +539,21 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
     if (nyes) {
 	for (i = 0; i < nobjects; i++) {
 	    struct repair rep;
+	    int index;
 	    if (answers[i]) {
-		if (ISDIRVNODE(sortedArrByName[first+i]->vno))
+		if (ISDIRVNODE(sortedArrByName[first+i]->fid.Vnode))
 		    rep.opcode = REPAIR_REMOVED;
 		else
 		    rep.opcode = REPAIR_REMOVEFSL;
 		strcpy(rep.name, sortedArrByName[first+i]->name);
 		rep.parms[0] = rep.parms[1] = rep.parms[2] = 0;
-		InsertListHdr(&rep, opList, sortedArrByName[first+i]->replicaid);
+
+		for (index = 0; index < nreplicas; index++)
+		    if (((*opList)[index]).replicaFid.Volume ==
+			sortedArrByName[first+i]->fid.Volume)
+			break;
+
+		InsertListHdr(nreplicas, &rep, opList, index);
 	    }
 	}
 	return(1);
@@ -550,7 +563,7 @@ int NameNameResolve(int first, int last, int nreplicas, resreplica *dirs, struct
 
 /* dirresolve : returns NNCONFLICTS(-1) if this resolve is definitely not the last needed compare/repair 
    return 0 if the compare implied that the resulting repair will make the directories equal */
-int dirresolve (int nreplicas, resreplica *dirs, int (*cbfn)(char *), struct listhdr **opList, char *volmtpt, VolumeId RepVolume, struct repinfo *inf, char *realm)
+int dirresolve (int nreplicas, resreplica *dirs, int (*cbfn)(char *), struct listhdr **opList, VolumeId RepVolume, struct repinfo *inf, char *realm)
 {
     int i;
 
@@ -574,8 +587,10 @@ int dirresolve (int nreplicas, resreplica *dirs, int (*cbfn)(char *), struct lis
 	    next++;
 	
 	for (i = first+1; i < next; i++) {
-	    if ((sortedArrByName[i]->vno != sortedArrByName[first]->vno) || 
-		(sortedArrByName[i]->uniqfier != sortedArrByName[first]->uniqfier)) {
+	    if ((sortedArrByName[i]->fid.Vnode !=
+		 sortedArrByName[first]->fid.Vnode) ||
+		(sortedArrByName[i]->fid.Unique !=
+		 sortedArrByName[first]->fid.Unique)) {
 		/* name/name conflict exists - process it */
 		if (NameNameResolve(first, next, nreplicas, dirs, opList, inf)) {
 		    nConflicts++;
@@ -598,19 +613,22 @@ int dirresolve (int nreplicas, resreplica *dirs, int (*cbfn)(char *), struct lis
 	  (int (*)(const void *, const void *))resdirCompareByFidName);
     int j;
     for (i = 0, j = 1; i < totaldirentries; i += j){
-	int conflict, rc;
+	int conflict = 0, rc;
 	for (j = 1; (i + j) < totaldirentries; j ++)
 	    if (resdirCompareByFidName(&(sortedArrByFidName[i]), &sortedArrByFidName[i+j]))
 	 	break;
 	if (sortedArrByFidName[i]->lookedAt) continue;
-	rc = GetConflictType(nreplicas, dirs, &(sortedArrByFidName[i]), j, &conflict, volmtpt, realm);
+
+	rc = GetConflictType(nreplicas, dirs, &(sortedArrByFidName[i]),
+			     j, &conflict, realm);
 	if (rc){
 	    if (inf->interactive)
-		printf("**** Couldnt get conflict type for %s ****\n", sortedArrByFidName[i]->name);
+		printf("**** Couldnt get conflict type for %s ****\n",
+		       sortedArrByFidName[i]->name);
 	    nConflicts++;
 	}
 	else 
-	    ResolveConflict(nreplicas, dirs, &(sortedArrByFidName[i]), j, conflict, opList, volmtpt, RepVolume, realm);
+	    ResolveConflict(nreplicas, dirs, &(sortedArrByFidName[i]), j, conflict, opList, RepVolume, realm);
     }
     free(sortedArrByFidName);
     return 0;
@@ -624,11 +642,17 @@ int resdirCompareByName (resdir_entry **a, resdir_entry **b)
 /* this sorts by Fid as the primary index and the name as the secondary index */
 int resdirCompareByFidName (resdir_entry **a, resdir_entry **b)
 {
-    if ((u_long)((*a)->vno) < (u_long)((*b)->vno)) return -1;
-    else if ((u_long)((*a)->vno) > (u_long)((*b)->vno)) return 1; 
-    if ((u_long)((*a)->uniqfier) < (u_long)((*b)->uniqfier)) return -1;
-    else if ((u_long)((*a)->uniqfier) > (u_long)((*b)->uniqfier)) return 1;
-    return (strcmp((*a)->name, (*b)->name));
+  if ((*a)->fid.Vnode < (*b)->fid.Vnode)
+    return -1;
+  else if ((*a)->fid.Vnode > (*b)->fid.Vnode)
+    return 1;
+
+  if ((*a)->fid.Unique < (*b)->fid.Unique)
+    return -1;
+  else if ((*a)->fid.Unique > (*b)->fid.Unique)
+    return 1;
+
+  return (strcmp((*a)->name, (*b)->name));
 }
 
 /* clean up routine */
@@ -655,8 +679,7 @@ void resClean (int nreplicas, resreplica *dirs, struct listhdr *lh)
     }
 }
 
-int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *volmtpt,
-	      char *dpath, char *childname)
+int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *dpath, char *childname)
 {
     /* returns fid and absolute path of parent */
     int rc;
@@ -678,31 +701,19 @@ int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *volmtpt,
     vi.out_size = sizeof(tmp);
     memset(tmp, 0, sizeof(tmp));
 
-    strcpy(path, "/coda");
-    rc = pioctl(path, _VICEIOCTL(_VIOC_GETPATH), &vi, 0);
+    rc = pioctl(NULL, _VICEIOCTL(_VIOC_GETPATH), &vi, 0);
     if (rc) {
 	//printf("GetParent: Getpath returns error %d, errno = %d\n",
 	//rc, errno);
 	return(rc);
     }
 
-#if 0 /* this seems to be wrong, replaced it by conditionalizing the
-	 strcpy a few lines down. --JH */
-    if (!volmtpt) {
-	strcpy(dpath, path);
-	return(0);
-    }
-#endif
-
     /* form the absolute path name of the parent */
-    char *lastcomp = rindex(tmp, '/');
-    if (lastcomp) 
-	*lastcomp = '\0';
-    char *firstcomp = index(tmp, '/');
+    char *lastcomp = strrchr(tmp, '/');
+    char *firstcomp = strchr(tmp, '/');
+    if (lastcomp) *lastcomp = '\0';
 
-    /* if a volmtpt has not been passed along use "/coda", which already
-     * happens to be what path contains. --JH */
-    if (volmtpt) strcpy(path, volmtpt);
+    strcpy(path, "/coda");
 
     if (firstcomp && (firstcomp != lastcomp)) {
 	strcat(path, "/");
@@ -711,7 +722,7 @@ int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *volmtpt,
     strcpy(dpath, path);
     strcpy(childname, lastcomp + 1);
 
-    // get fid of parent 
+    // get fid of parent
     //ViceVersionVector VV;
     //res_getfid(path, dfid, &VV);
     gp.fid = *cfid;
@@ -720,8 +731,7 @@ int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *volmtpt,
     vi.in_size = sizeof(gp);
     vi.out = tmp;
     vi.out_size = sizeof(tmp);
-    strcpy(path, "/coda");
-    rc = pioctl(path, _VICEIOCTL(_VIOC_GETPFID), &vi, 0);
+    rc = pioctl(NULL, _VICEIOCTL(_VIOC_GETPFID), &vi, 0);
     if (rc) {
 	printf("Error %d occured while trying to get fid of %s's parent\n", rc, childname);
 	return(rc);
