@@ -27,6 +27,7 @@ listed in the file CREDITS.
 /* Header file for definitions used by the 
  * resolution subsystem  
  */
+#include <vcrcommon.h>
 
 #ifdef MAXNAMELEN
 #undef MAXNAMELEN
@@ -58,25 +59,24 @@ struct repinfo {
     char interactive;         /* flag indicating whether repair is interactive */
 };
 
-/* definition of each directory entry in memory as used 
- * by the resolution subsystem */
+/* definition of directory entries within a directory conflict replica
+ * as used by the resolution subsystem */
 typedef struct {
     char    name[MAXNAMELEN + 1];   /* name of the entry */
-    long    vno;		    /* vnode number */
-    long    uniqfier;		    /* for vice dirs; 0 for unix */
+  /* char    realm  // not needed because cross-realm mtpts aren't supported
+   * (Jan has a good comment on authentication issues somewhere) */
+    ViceFid fid;
     ViceVersionVector	VV;
     int	    MtPt;		    /* Is this child a mount point? */
-    int	    replicaid;
     int	    lookedAt;
+    int     index;                  /* Functions as a index for resreplica arrays */
 } resdir_entry;
 
 /* definition of the parent directory replica */
 typedef struct {
     int	    entry1;		    /* index of first child in table */
     int	    nentries;		    /* number of children */
-    long    replicaid;		    /* id of this replica */
-    VnodeId  vnode;		    /* fid of the parent directory */
-    Unique_t uniqfier;
+    ViceFid fid;
     char    *path;		    /* path name of the RO mounted copy */
     u_short modebits;
     struct  Acl *al;
@@ -89,18 +89,18 @@ extern int direntriesarrsize;
 extern int nextavailindex;
 extern resdir_entry	**sortedArrByFidName;	/* for sorting the direntries in fid order*/
 extern resdir_entry	**sortedArrByName;	/* for sorting the direntries in name order */
-extern int totaldirentries;
-/* extern VolumeId RepVolume; */
+extern int  totaldirentries;
 extern int  nConflicts;
 
 
 extern void InitListHdr (int , resreplica *, struct listhdr **);
-extern int InsertListHdr (struct repair *, struct listhdr **, int );
+extern int InsertListHdr (int , struct repair *, struct listhdr **, int);
 extern int InRepairList (struct listhdr *, unsigned, VnodeId, Unique_t);
 extern int IsCreatedEarlier(struct listhdr **, int, VnodeId, Unique_t);
 extern int getunixdirreps (int , char **, resreplica **);
-extern int dirresolve (int , resreplica *, int (*)(char *), struct listhdr **, char *, VolumeId, struct repinfo *, char *realm);
+extern int dirresolve (int, resreplica *, int (*)(char *), struct listhdr **,
+		       VolumeId, struct repinfo *, char *realm);
 extern void resClean (int, resreplica *, struct listhdr *);
-extern int GetParent (char *realm, ViceFid *, ViceFid *, char *, char *, char *);
+extern int GetParent (char *realm, ViceFid *, ViceFid *, char *, char *);
 
 #endif
