@@ -75,12 +75,12 @@ static int ReadGlobalState(int fd) {
 	return 0;
     }
 
-    if (norton_debug) printf("Read maximum vol ID: 0x%lx\n", maxvolid);
+    if (norton_debug) printf("Read maximum vol ID: %08x\n", maxvolid);
 
     rvmlib_begin_transaction(restore);
     VSetMaxVolumeId(maxvolid);
     rvmlib_end_transaction(flush, &(status));
-    if (norton_debug) printf("Set max vol id to 0x%08lx.  RVM Status = %d\n",
+    if (norton_debug) printf("Set max vol id to %08x.  RVM Status = %d\n",
 		      maxvolid, status);
     return 1;
 }
@@ -337,7 +337,7 @@ static int DumpVnodeList(int fd, struct VolumeData *vol, int vol_index,
     int    nvnodes,
 	   vnode_index,
 	   vnode_num;
-    DirInode *inode;
+    PDirInode inode;
     int	   npages;
     
     /* Send in a 0 for device, since noone uses it anyway. */
@@ -347,7 +347,7 @@ static int DumpVnodeList(int fd, struct VolumeData *vol, int vol_index,
 
     nvnodes = v_index.vnodes();
     if (norton_debug) {
-	printf("Volume 0x%lx, has %d %s vnodes\n", vol->volumeInfo->id,
+	printf("Volume %08x, has %d %s vnodes\n", vol->volumeInfo->id,
 	       nvnodes, VNODECLASS(vclass));
     }
     
@@ -374,7 +374,7 @@ static int DumpVnodeList(int fd, struct VolumeData *vol, int vol_index,
 
 	if (vclass == vLarge) {
 	    /* Now write the inode information and directory pages. */
-	    inode = (DirInode *)vnode->inodeNumber;
+	    inode = vnode->node.dirNode;
 	    for (npages = 0; inode->di_pages[npages]; npages++);
 
 	    if (norton_debug) {
@@ -415,16 +415,16 @@ static int DumpVolVnodes(int fd, struct VolumeData *vol, int vol_index) {
     return 1;
 }
 
-int CopyDirInode(DirInode *oldinode, DirInode **newinode)
+int CopyDirInode(PDirInode oldinode, PDirInode *newinode)
 {
-    DirInode    shadowInode;
+    struct DirInode shadowInode;
 
     if (!oldinode){
        LogMsg(29, DirDebugLevel, stdout, "CopyDirInode: Null oldinode");
        return -1;
     }
-    *newinode = (DirInode *)rvmlib_rec_malloc(sizeof(DirInode));
-    memset((void *)&shadowInode, 0, sizeof(DirInode));
+    *newinode = (PDirInode)rvmlib_rec_malloc(sizeof(struct DirInode));
+    memset((void *)&shadowInode, 0, sizeof(struct DirInode));
     for(int i = 0; i < DIR_MAXPAGES; i++)
         if (oldinode->di_pages[i]){
             shadowInode.di_pages[i] = (long *)rvmlib_rec_malloc(DIR_PAGESIZE);
@@ -432,7 +432,7 @@ int CopyDirInode(DirInode *oldinode, DirInode **newinode)
 				oldinode->di_pages[i], DIR_PAGESIZE);
         }
     shadowInode.di_refcount = oldinode->di_refcount;
-    rvmlib_modify_bytes(*newinode, &shadowInode, sizeof(DirInode));
+    rvmlib_modify_bytes(*newinode, &shadowInode, sizeof(struct DirInode));
     return 0;
 
 }
@@ -449,15 +449,15 @@ static int ReadVnodeList(int fd, Volume *vp, VnodeClass vclass, int ResOn) {
     Error	err;
 
     /* We need to allocate this in RVM! */
-    DirInode *inode = (DirInode *)malloc(sizeof(DirInode));
-    DirInode *newinode;
+    PDirInode inode = (PDirInode)malloc(sizeof(struct DirInode));
+    PDirInode newinode;
 
     if (!inode) {
 	fprintf(stderr, "Cannot allocate VM inode.\n");
 	return 0;
     }
     
-    if (read(fd, (void *)&nvnodes, (int)sizeof(nvnodes)) == -1) {
+    if (read(fd, (void *)&nvnodes, sizeof(nvnodes)) == -1) {
 	sprintf(buf, "Reading number of %s vnodes\n", VNODECLASS(vclass));
 	perror(buf);
 	return 0;
@@ -465,13 +465,13 @@ static int ReadVnodeList(int fd, Volume *vp, VnodeClass vclass, int ResOn) {
     if (norton_debug) printf("Reading %d vnodes...\n", nvnodes);
 
     while (nvnodes--) {
-	if (read(fd, (void *)&vnode_num, (int)sizeof(vnode_num)) == -1) {
+	if (read(fd, (void *)&vnode_num, sizeof(vnode_num)) == -1) {
 	    perror("Reading vnode number\n");
 	    return 0;
 	}
-	if (norton_debug) printf("Reading vnode number 0x%lx\n", vnode_num);
+	if (norton_debug) printf("Reading vnode number %08x\n", vnode_num);
 
-	memset((void *)(void *)vnode, 0, VNODESIZE(vclass));
+	memset(vnode, 0, VNODESIZE(vclass));
 	
 	if (read(fd, (void *) vnode, VNODESIZE(vclass)) == -1) {
 	    perror("Reading vnode\n");
@@ -503,7 +503,7 @@ static int ReadVnodeList(int fd, Volume *vp, VnodeClass vclass, int ResOn) {
 	
 	rvmlib_begin_transaction(restore);
 	if (vclass == vLarge) {
-	    memset(inode, 0, sizeof(DirInode));
+	    memset(inode, 0, sizeof(struct DirInode));
 		
 	    /* Read in the directory pages */
 	    if (read(fd, (void *)&npages, (int)sizeof(npages)) == -1) {
@@ -539,9 +539,9 @@ static int ReadVnodeList(int fd, Volume *vp, VnodeClass vclass, int ResOn) {
 		rvmlib_abort(VFAIL);
 		return 0;
 	    }
-		
-	    vnp->disk.inodeNumber = (Inode)newinode;
-	    
+
+	    vnp->disk.node.dirNode = newinode;
+
 	    /* Read in the resolution log */
 	    if (ResOn) {
 		if (!ReadResLog(fd, vp, vnp)) {
@@ -606,8 +606,8 @@ static void NortonSetupVolume(VolHead *vh, Volume *vp, int volindex)
 // Can't re-use code in vol-salvage since this comes from command line.
 static void GetSkipVols(int num, VolumeId *ids, char *vol_nums[]) {
     while(num > 0) {
-	sscanf(*vol_nums, "%lx", ids);
-	printf("Skipping volume: 0x%lx\n", *ids);
+	sscanf(*vol_nums, "%x", ids);
+	printf("Skipping volume: %08x\n", *ids);
 	vol_nums++;
 	ids++;
 	num--;
@@ -647,7 +647,7 @@ static int HasBackVols(VolumeId *skipvols, int nskipvols) {
 		continue;
 	    } else {
 		ret = TRUE;
-		printf("backup volume: %s(0x%lx)\n",
+		printf("backup volume: %s (%08x)\n",
 		       vol->data.volumeInfo->name,
 		       vol->data.volumeInfo->id);
 	    }
@@ -697,12 +697,12 @@ static int load_server_state(char *dump_file, VolumeId *skipvols, int nskipvols)
 	}
 
 	if (InSkipVolumeList(vol_head.header.id, skipvols, nskipvols)) {
-	    printf("Skipping volume 0x%lx\n", vol_head.header.id);
+	    printf("Skipping volume %08x\n", vol_head.header.id);
 	    SkipToNextVolHead(dump_fd);
 	    continue;
 	}
 	else
-	    printf("Reading volume 0x%lx\n", vol_head.header.id);
+	    printf("Reading volume %08x\n", vol_head.header.id);
 
 	rvmlib_begin_transaction(restore);
 
@@ -714,11 +714,11 @@ static int load_server_state(char *dump_file, VolumeId *skipvols, int nskipvols)
 
 	if ((volindex = NewVolHeader(&vol_head.header, &err)) == -1) {
 	    if (err == VVOLEXISTS) {
-		fprintf(stderr, "Volume 0X%x already exists!  Aborting\n",
+		fprintf(stderr, "Volume %08x already exists!  Aborting\n",
 			vol_head.header.id);
 	    } else {
 		fprintf(stderr,
-			"Error code %d while creating volume %x, Aborting\n",
+			"Error code %d while creating volume %08x, Aborting\n",
 			err, vol_head.header.id);
 	    }
 	    rvmlib_abort(VFAIL);
@@ -871,7 +871,7 @@ static int dump_server_state(char *dump_file, VolumeId *skipvols, int nskipvols)
 	  continue;
 	}
 	    
-	printf("Writing volume 0x%lx\n", vol->data.volumeInfo->id);
+	printf("Writing volume %08x\n", vol->data.volumeInfo->id);
 
 	if (!DumpVolHead(dump_fd, vol) ||
 	    !DumpVolDiskData(dump_fd, vol->data.volumeInfo) ||
