@@ -93,7 +93,7 @@ struct inodeops inodeops_ftree = {
 static mode_t   mode=S_IREAD | S_IWRITE;
 
 
-static int f_magic()
+static int f_magic(void)
 {
     return VICEMAGIC;
 }
@@ -188,9 +188,9 @@ static int f_init (union PartitionData **data,
     options->width = val;
 
     i=0;
-    do 
+    do {
 	i++;
-    while ( (1<<i) != options->width &&  i<20 );
+    } while ( (1<<i) != options->width &&  i<20 );
 
     if ( i != 10) {
 	options->logwidth = i;
@@ -207,7 +207,7 @@ static int f_init (union PartitionData **data,
 
     rc = stat(resfilename, &buf);
     if ( rc != 0 || !(buf.st_mode & S_IFREG) ) {
-	dev = 0;
+	dev = NULL;
 	eprint("Error in init of partition %s:%s: no resource database.", 
 	       Partent_host(partent), Partent_dir(partent));
 	perror("");
@@ -217,14 +217,16 @@ static int f_init (union PartitionData **data,
     /* open the resource data file */
     options->resource = open(resfilename, O_RDWR);
     if ( options->resource == -1 ) {
-	dev = 0;
+	dev = NULL;
 	eprint("Error opening resource file!\n");
 	perror("");
 	CODA_ASSERT(0);
     }
 
     /* set up a bitmap */
-    filecount = (long)pow(options->width, options->depth);
+    for (filecount = 1, i = 0; i < options->depth; i++)
+	filecount *= options->width;
+
     freemap = Bitv_new(filecount);
     /*    Bitv_print(freemap, stdout); */
     options->freebm = freemap;
@@ -291,21 +293,19 @@ f_icreate(struct DiskPartition *dp, u_long volume, u_long vnode, u_long unique,
 	  u_long dataversion)
 {
     struct part_ftree_opts *opts=&(dp->d->ftree);
-    int	fd, rc, pos;
-    Inode i, ino;
-    char	filename[FNAMESIZE];
-    struct
-	i_header header={1, volume, vnode, unique, dataversion, VICEMAGIC};
+    int	fd, rc, pos, i;
+    Inode ino;
+    char filename[FNAMESIZE];
+    struct i_header header={1, volume, vnode, unique, dataversion, VICEMAGIC};
 
     /*  Find an available inode. */
-    i = (Inode) Bitv_getfree(opts->freebm);
+    i = Bitv_getfree(opts->freebm);
     if ( i == -1 ) {
-	eprint("No more free entries in freebitmap of %s\n",
-	       dp->name);
+	eprint("No more free entries in freebitmap of %s\n", dp->name);
 	CODA_ASSERT(0);
     }
 	
-    ino = i+1; /*inode numbers start at 1 */
+    ino = (Inode)i+1; /*inode numbers start at 1 */
     
     f_inotostr(dp, ino, filename);
     if (mkpath(filename, 0700) < 0) {
@@ -561,9 +561,7 @@ istat(struct DiskPartition *dp, Inode  inode_number, struct stat *statbuf)
     statbuf->st_nlink = header.lnk;
     statbuf->st_size=header.vnode;
     statbuf->st_blksize=header.unique;
-#ifndef DJGPP
     statbuf->st_blocks=header.dataversion;
-#endif
     return 0;
 }
 
