@@ -60,7 +60,7 @@ extern "C" {
 // ********** Private Routines *************
 static int ComparePhase3Status(res_mgrpent *, int *, ViceStatus *);
 static char *CoordPhase2(res_mgrpent *, ViceFid *, int *, int *, int *, unsigned long *,dirresstats *);
-static int CoordPhase3(res_mgrpent*, ViceFid*, char*, int, int, ViceVersionVector**, dlist*, ResStatus**, unsigned long*, int*);
+static int CoordPhase3(res_mgrpent*, ViceFid*, char*, int, int, ViceVersionVector**, dlist*, ResStatus**, unsigned long*, int*, DirFid *);
 static int CoordPhase4(res_mgrpent *, ViceFid *, unsigned long *, int *);
 static int CoordPhase34(res_mgrpent *, ViceFid *, dlist *, int *);
 static void AllocateBufs(res_mgrpent *, char **, int *);
@@ -100,8 +100,8 @@ static void UpdateStats(ViceFid *, dirresstats *);
 // *		In case of problems it marks the replicas inconsistent
 //
 
-long RecovDirResolve(res_mgrpent *mgrp, ViceFid *Fid, ViceVersionVector **VV, 
-		     ResStatus **rstatusp, int *sizes)
+long RecovDirResolve(res_mgrpent *mgrp, ViceFid *Fid, ViceVersionVector **VV,
+		     ResStatus **rstatusp, int *sizes, DirFid *HintFid)
 {
     int reserror = EINCONS;
     char *AllLogs = NULL;
@@ -157,13 +157,15 @@ long RecovDirResolve(res_mgrpent *mgrp, ViceFid *Fid, ViceVersionVector **VV,
 	PollAndYield();
     }
     // Phase3
-    {	
+    {
 	PROBE(tpinfo, RecovCoorP3Begin);
 	inclist = new dlist((CFN)CompareIlinkEntry);
-	if (CoordPhase3(mgrp, Fid, AllLogs, totalsize, totalentries, VV, 
-			inclist, rstatusp, succFlags, dirlengths)) {
+	if (CoordPhase3(mgrp, Fid, AllLogs, totalsize, totalentries, VV,
+			inclist, rstatusp, succFlags, dirlengths, HintFid))
+	{
 	    LogMsg(0, SrvDebugLevel, stdout,
 		   "RecovDirResolve: Error during phase 3\n");
+	    retval = EINCONS;
 	    goto Exit;
 	}
 	PROBE(tpinfo, RecovCoorP3End);
@@ -190,11 +192,12 @@ long RecovDirResolve(res_mgrpent *mgrp, ViceFid *Fid, ViceVersionVector **VV,
 	PROBE(tpinfo, RecovCoorP4End);
     }
   Exit:
-    // mark object inconsistent in case of error 
+    // mark object inconsistent in case of error
     // Phase5
-    if (reserror) {
+    if (reserror && (!HintFid || !(HintFid->Vnode || HintFid->Unique)))
+    {
 	MRPC_MakeMulti(MarkInc_OP, MarkInc_PTR, VSG_MEMBERS,
-		       mgrp->rrcc.handles, mgrp->rrcc.retcodes, 
+		       mgrp->rrcc.handles, mgrp->rrcc.retcodes,
 		       mgrp->rrcc.MIp, 0, 0, Fid);
 	drstats.dir_conf++;
     }
@@ -352,12 +355,15 @@ static char *ConcatLogs(res_mgrpent *mgrp, char **bufs,
 static int CoordPhase3(res_mgrpent *mgrp, ViceFid *Fid, char *AllLogs,
 		       int logsize, int totalentries, ViceVersionVector **VV,
 		       dlist *inclist, ResStatus **rstatusp,
-		       unsigned long *successFlags, int *dirlengths)
+		       unsigned long *successFlags, int *dirlengths,
+		       DirFid *HintFid)
 {
     RPC2_BoundedBS PBinc;
     char buf[RESCOMM_MAXBSLEN];
     SE_Descriptor sid;
     ViceStatus status;
+    DirFid hint;
+
     // init parms PB, sid, status block
     {
 	PBinc.SeqBody = (RPC2_ByteSeq)buf;
@@ -382,20 +388,48 @@ static int CoordPhase3(res_mgrpent *mgrp, ViceFid *Fid, char *AllLogs,
 	}
     }
     ARG_MARSHALL(IN_OUT_MODE, SE_Descriptor, sidvar, sid, VSG_MEMBERS);
-    ARG_MARSHALL_BS(IN_OUT_MODE, RPC2_BoundedBS, PBincvar, PBinc, VSG_MEMBERS, RESCOMM_MAXBSLEN);
+    ARG_MARSHALL_BS(IN_OUT_MODE, RPC2_BoundedBS, PBincvar, PBinc, VSG_MEMBERS,
+		    RESCOMM_MAXBSLEN);
     ARG_MARSHALL(IN_OUT_MODE, ViceStatus, statusvar, status, VSG_MEMBERS);
+    ARG_MARSHALL(OUT_MODE, DirFid, hintvar, hint, VSG_MEMBERS);
 
     // Ship log to Subordinates & Parse results
     {
-	MRPC_MakeMulti(ShipLogs_OP, ShipLogs_PTR, VSG_MEMBERS,
-		       mgrp->rrcc.handles, mgrp->rrcc.retcodes, 
-		       mgrp->rrcc.MIp, 0, 0, Fid, logsize, 
-		       totalentries, statusvar_ptrs, PBincvar_ptrs, 
-		       sidvar_bufs);
+	if (!HintFid)
+	    MRPC_MakeMulti(ShipLogs_OP, ShipLogs_PTR, VSG_MEMBERS,
+			   mgrp->rrcc.handles, mgrp->rrcc.retcodes,
+			   mgrp->rrcc.MIp, 0, 0, Fid, logsize,
+			   totalentries, statusvar_ptrs, PBincvar_ptrs,
+			   sidvar_bufs);
+	else
+	    MRPC_MakeMulti(NewShipLogs_OP, NewShipLogs_PTR, VSG_MEMBERS,
+			   mgrp->rrcc.handles, mgrp->rrcc.retcodes,
+			   mgrp->rrcc.MIp, 0, 0, Fid, logsize,
+			   totalentries, statusvar_ptrs, PBincvar_ptrs,
+			   hintvar_ptrs, sidvar_bufs);
 	mgrp->CheckResult();
 	int errorCode = 0;
 	if ((errorCode = CheckRetCodes(mgrp->rrcc.retcodes, mgrp->rrcc.hosts, successFlags))) {
-	    LogMsg(0, SrvDebugLevel, stdout,  
+
+	    if (HintFid) {
+		/* we got an error, let's see if any of the servers came up
+		 * with a suggested fid that may resolve better than the
+		 * current one */
+		for (int i = 0; i < VSG_MEMBERS; i++) {
+		    if (mgrp->rrcc.retcodes[i] == EINCONS)
+		    {
+			ARG_UNMARSHALL(hintvar, *HintFid, i);
+			if (HintFid->Vnode || HintFid->Unique)
+			    break;
+		    }
+		}
+
+		LogMsg(0, SrvDebugLevel, stdout,
+		       "CoordPhase3: ERESHINT (Hint = %08x.%08x)",
+		       HintFid->Vnode, HintFid->Unique);
+	    }
+
+	    LogMsg(0, SrvDebugLevel, stdout,
 		   "CoordPhase3: Error %d in ShipLogs", errorCode);
 	    return(errorCode);
 	}
