@@ -82,8 +82,8 @@ static int GatherFids(dlist *, Vnode *, Volume *, arrlist *);
 static int AddRenameChildrenToList(dlist *, Volume *, Vnode *, rsle *);
 static int SetPhase3DirStatus(ViceStatus *, ViceFid *, Volume *, dlist *);
 static int GetResObjs(arrlist *, ViceFid *, Volume **, dlist *);
-static int CheckSemPerformRes(arrlist *, Volume *, ViceFid *, dlist *, 
-			       olist *, dlist *, int *);
+static int CheckSemPerformRes(arrlist *, Volume *, ViceFid *, dlist *,
+			      olist *, dlist *, int *, DirFid *);
 static int CheckRegularCompOp(rsle *, dlist *, vle *, ViceFid *, Volume *, olist *);
 static int PerformRegularCompOp(int, rsle *, dlist *, dlist *, 
 				 olist *, ViceFid *, vle *, Volume *, VolumeId, int *);
@@ -106,14 +106,13 @@ const int Yield_rp3CheckSemPerformRes_Mask = Yield_rp3CheckSemPerformRes_Period
 
 
 
-long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size, 
-		   RPC2_Integer nentries, ViceStatus *status, 
-		   RPC2_BoundedBS *piggyinc, SE_Descriptor *sed) 
+long RS_NewShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
+		    RPC2_Integer nentries, ViceStatus *status,
+		    RPC2_BoundedBS *piggyinc, DirFid *HintFid,
+		    SE_Descriptor *sed)
 {
 
-    SLog(1, 
-	   "RS_ShipLogs: Entering for Fid (0x%x.%x.%x)\n",
-	   Fid->Volume, Fid->Vnode, Fid->Unique);
+    SLog(1, "RS_ShipLogs: Entering for Fid (%s)\n", FID_(Fid));
 
     PROBE(tpinfo, RecovSubP3Begin);
     int errorCode = 0;
@@ -125,6 +124,11 @@ long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
     arrlist *CompOps = NULL;
     int nblocks = 0;
 
+    if (HintFid) {
+	HintFid->Vnode  = 0;
+	HintFid->Unique = 0;
+    }
+
     // fetch log from coordinator 
     char *buf = NULL;
     {
@@ -175,12 +179,12 @@ long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
     // Check Semantics and Perform compensating operations
     {
 	PROBE(tpinfo, RecovPerformResOpBegin);
-	if ((errorCode = CheckSemPerformRes(CompOps, volptr, Fid, 
-					   vlist, AllLogs, inclist, 
-					   &nblocks))) {
-	    SLog(0,  
-		   "RS_ShipLogs: Error %d during CheckSemPerformRes",
-		   errorCode);
+	errorCode = CheckSemPerformRes(CompOps, volptr, Fid, vlist, AllLogs,
+				       inclist, &nblocks, HintFid);
+	if (errorCode)
+	{
+	    SLog(0, "RS_ShipLogs: Error %d during CheckSemPerformRes",
+		 errorCode);
 	    goto Exit;
 	}
 	PROBE(tpinfo, RecovPerformResOpEnd);
@@ -197,12 +201,10 @@ long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
     }
   Exit:
     // Put Objects back to RVM
-    {
-	//PROBE(tpinfo, P3PUTOBJBEGIN);
-	PutObjects(errorCode, volptr, NO_LOCK, vlist, nblocks, 1);
-	//PROBE(tpinfo, P3PUTOBJEND);
-	
-    }
+    /* Do not pass errorCode, this way we end up committing all updates we
+     * managed to apply even when a later operation in the log failed. */
+    PutObjects(0, volptr, NO_LOCK, vlist, nblocks, 1);
+
     // Return list of inconsistencies to coordinator
     {
 	if (!errorCode)
@@ -223,6 +225,13 @@ long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
     return(errorCode);
 }
 
+long RS_ShipLogs(RPC2_Handle RPCid, ViceFid *Fid, RPC2_Integer size,
+		 RPC2_Integer nentries, ViceStatus *status,
+		 RPC2_BoundedBS *piggyinc, SE_Descriptor *sed)
+{
+  return RS_NewShipLogs(RPCid, Fid, size, nentries, status,
+			piggyinc, NULL, sed);
+}
 
 static int FetchLog(RPC2_Handle RPCid, char **buf, int size) {
     int errorCode = 0;
@@ -515,17 +524,18 @@ static int GatherFids(dlist *vlist, Vnode *pvptr,
  *		Look up table to see if operation not allowed
  *	Then perform the operation accordingly
  *
- * 	Return values:
+ *	Return values:
  *		functions return value indicates abort/commit
  *		result[] - array of codes for outcome of each operation
  *			PERFORMOP, NULLOP, MARKPARENTINC, MARKOBJINC
  *			CREATEINCOBJ
- *			
+ *
  */
-static int CheckSemPerformRes(arrlist *ops, Volume *volptr, 
-			       ViceFid *dFid, dlist *vlist, 
-			       olist *AllLogs, dlist *inclist, 
-			       int *nblocks) {
+static int CheckSemPerformRes(arrlist *ops, Volume *volptr,
+			       ViceFid *dFid, dlist *vlist,
+			       olist *AllLogs, dlist *inclist,
+			      int *nblocks, DirFid *HintFid)
+{
     int errorCode = 0;
     *nblocks = 0;
 
@@ -556,24 +566,26 @@ static int CheckSemPerformRes(arrlist *ops, Volume *volptr,
 	    // yield after every few operations
 	    if ((count & Yield_rp3CheckSemPerformRes_Mask) == 0)
 		PollAndYield();
-	    
+
 	    if (r->opcode == ResolveNULL_OP) continue;
-	    
+
 	    // handle renames separately
-	    if (r->opcode == RES_Rename_OP || 
+	    if (r->opcode == RES_Rename_OP ||
 		r->opcode == ResolveViceRename_OP) {
-		if (!(errorCode = CheckAndPerformRename(r, volptr, VSGVolnum, dFid,
-							vlist, AllLogs, inclist, &tblocks)))
+		if (!(errorCode = CheckAndPerformRename(r, volptr, VSGVolnum,
+							dFid, vlist, AllLogs,
+							inclist, &tblocks,
+							HintFid)))
 		    *nblocks += tblocks;
 		continue;
 	    }
-	    
-	    // regular operation check and perform 
+
+	    // regular operation check and perform
 	    {
 		int result = CheckRegularCompOp(r, vlist, pv, dFid, volptr, AllLogs);
 		SLog(9, "CheckRegularCompOp returns %d\n", result);
-		if (!(errorCode = PerformRegularCompOp(result, r, vlist, inclist, 
-						       AllLogs, dFid, pv, volptr, 
+		if (!(errorCode = PerformRegularCompOp(result, r, vlist, inclist,
+						       AllLogs, dFid, pv, volptr,
 						       VSGVolnum, &tblocks)))
 		    *nblocks += tblocks;
 	    }
@@ -979,32 +991,30 @@ static int CheckValidityResOp(rsle *r, int NE,
     return 0;
 }
 
-static int PerformRegularCompOp(int result, rsle *rp, dlist *vlist, dlist *inclist, 
-				 olist *AllLogs, ViceFid *dFid, vle *pv, 
-				 Volume *volptr, VolumeId VSGVolnum, int *nblocks) {
+static int PerformRegularCompOp(int result, rsle *rp, dlist *vlist,
+				dlist *inclist, olist *AllLogs, ViceFid *dFid,
+				vle *pv, Volume *volptr, VolumeId VSGVolnum,
+				int *nblocks)
+{
     int tblocks = 0;
     int errorCode = 0;
-    ViceFid cFid; 
+    ViceFid cFid;
     vle *cv;
     char *name;
     int vntype;
 
     switch (result) {
       case PERFORMOP:
-	SLog(9,  
-	       "PerformRegularCompOp: Going to Perform Op");
-	errorCode = PerformResOp(rp, vlist, AllLogs, pv, 
-				 volptr, VSGVolnum, &tblocks);
+	SLog(9, "PerformRegularCompOp: Going to Perform Op");
+	errorCode = PerformResOp(rp, vlist, AllLogs, pv, volptr, VSGVolnum, &tblocks);
 	break;
       case NULLOP:
-	SLog(9,  
-	       "PerformRegularCompOp: NULL Operation - ignore");
+	SLog(9, "PerformRegularCompOp: NULL Operation - ignore");
 	break;
       case MARKPARENTINC:
-	SLog(9,  
-	       "PerformRegularCompOp: Marking Parent Inc");
+	SLog(9, "PerformRegularCompOp: Marking Parent Inc");
 	MarkObjInc(dFid, pv->vptr);
-	AddILE(*inclist, ".", dFid->Vnode, dFid->Unique, 
+	AddILE(*inclist, ".", dFid->Vnode, dFid->Unique,
 	       dFid->Vnode, dFid->Unique, (long)vDirectory);
 	break;
       case MARKOBJINC:
@@ -1013,42 +1023,39 @@ static int PerformRegularCompOp(int result, rsle *rp, dlist *vlist, dlist *incli
 	cFid.Volume = V_id(volptr);
 	cv = FindVLE(*vlist, &cFid);
 	if (!cv || !cv->vptr) {
-	    SLog(0,  
-		   "MARKOBJINC: couldnt get obj(0x%x.%x)vnode pointer",
+	    SLog(0, "MARKOBJINC: couldnt get obj(0x%x.%x)vnode pointer",
 		   cFid.Vnode, cFid.Unique);
 	    errorCode = EINVAL;
 	}
 	else {
 	    MarkObjInc(&cFid, cv->vptr);
 	    name = ExtractNameFromrsle(rp);
-	    AddILE(*inclist, name, cFid.Vnode, cFid.Unique, 
-		   dFid->Vnode, dFid->Unique, 
+	    AddILE(*inclist, name, cFid.Vnode, cFid.Unique,
+		   dFid->Vnode, dFid->Unique,
 		   (long)(cv->vptr->disk.type));
 	}
 	break;
       case CREATEINCOBJ:
-	// xxx BE CAREFUL WITH CHILD FID AND RENAMES 
-	SLog(9,  "CheckSemPerformRes: Creating Inc Object");
+	// xxx BE CAREFUL WITH CHILD FID AND RENAMES
+	SLog(9, "CheckSemPerformRes: Creating Inc Object");
 	ExtractChildFidFromrsle(rp, &cFid);
 	cFid.Volume = V_id(volptr);
 	name = ExtractNameFromrsle(rp);
 	vntype = ExtractVNTypeFromrsle(rp);
-	errorCode = CreateObjToMarkInc(volptr, dFid, &cFid, name, 
+	errorCode = CreateObjToMarkInc(volptr, dFid, &cFid, name,
 				       vntype, vlist, &tblocks);
 	if (errorCode == 0) {
 	    cv = FindVLE(*vlist, &cFid);
 	    CODA_ASSERT(cv);
 	    CODA_ASSERT(cv->vptr);
 	    MarkObjInc(&cFid, cv->vptr);
-	    AddILE(*inclist, name, cFid.Vnode, cFid.Unique, 
-		   cv->vptr->disk.vparent, 
-		   cv->vptr->disk.uparent,
+	    AddILE(*inclist, name, cFid.Vnode, cFid.Unique,
+		   cv->vptr->disk.vparent, cv->vptr->disk.uparent,
 		   (long)(cv->vptr->disk.type));
 	}
 	break;
       default:
-	SLog(0,  
-	       "Illegal opcode for PerformCompOp\n");
+	SLog(0, "Illegal opcode for PerformCompOp\n");
 	CODA_ASSERT(0);
     }
 
@@ -1056,14 +1063,10 @@ static int PerformRegularCompOp(int result, rsle *rp, dlist *vlist, dlist *incli
     return(errorCode);
 }
 
-
-
-static int PerformResOp(rsle *r, dlist *vlist, olist *AllLogs,
-			 vle *pv, Volume *volptr, VolumeId VSGVolnum, 
-			 int *blocks) {
-    
-    SLog(9,  
-	   "Entering PerformResOp: %s ", PRINTOPCODE(r->opcode));
+static int PerformResOp(rsle *r, dlist *vlist, olist *AllLogs, vle *pv,
+			Volume *volptr, VolumeId VSGVolnum, int *blocks)
+{
+    SLog(9, "Entering PerformResOp: %s ", PRINTOPCODE(r->opcode));
     int errorCode = 0;
     *blocks = 0;
     char *name = ExtractNameFromrsle(r);
@@ -1122,8 +1125,8 @@ static int PerformResOp(rsle *r, dlist *vlist, olist *AllLogs,
 		    int tblocks = (int) (-nBlocks(cv->vptr->disk.length));
 		    CODA_ASSERT(AdjustDiskUsage(volptr, tblocks) == 0);
 		    *blocks += tblocks;
-		    cv->f_sinode = cv->vptr->disk.inodeNumber;
-		    cv->vptr->disk.inodeNumber = 0;
+		    cv->f_sinode = cv->vptr->disk.node.inodeNumber;
+		    cv->vptr->disk.node.inodeNumber = 0;
 		}
 	    }
 	    /* spool log record */
@@ -1203,12 +1206,12 @@ static int PerformResOp(rsle *r, dlist *vlist, olist *AllLogs,
 	    cv->vptr->disk.owner = r->u.slink.owner;
 	    cv->vptr->disk.author = r->u.slink.owner;
 	    /* create the inode */
-	    cv->f_finode = icreate((int)V_device(volptr), (int)V_id(volptr),
-				   (int) cv->vptr->vnodeNumber,
-				   (int) cv->vptr->disk.uniquifier, 1);
+	    cv->f_finode = icreate(V_device(volptr), V_id(volptr),
+				   cv->vptr->vnodeNumber,
+				   cv->vptr->disk.uniquifier, 1);
 	    CODA_ASSERT(cv->f_finode > 0);
-	    cv->vptr->disk.inodeNumber = cv->f_finode;
-	    
+	    cv->vptr->disk.node.inodeNumber = cv->f_finode;
+
 	    /* append log record */
 	    SLog(9, "PerformResOp: Spooling log record SymLink(%s)", name);
 	    if ((errorCode = SpoolVMLogRecord(vlist, pv, volptr, &r->storeid,
@@ -1431,7 +1434,7 @@ static int SetPhase3DirStatus(ViceStatus *status, ViceFid *Fid,
     CODA_ASSERT(ov && ov->vptr);
     
     // check if new vv is legal 
-    int res = VV_Cmp_IgnoreInc(&Vnode_vv(ov->vptr), &DiffVV);
+    VV_Cmp_Result res = VV_Cmp_IgnoreInc(&Vnode_vv(ov->vptr), &DiffVV);
     if (res != VV_EQ && res != VV_SUB) {
 	SLog(0, "SetPhase3DirStatus: 0x%x.%x VV's are in conflict",
 	       ov->vptr->vnodeNumber, ov->vptr->disk.uniquifier);
