@@ -29,8 +29,8 @@ listed in the file CREDITS.
 #include <stdlib.h>
 #include <lwp/lwp.h>
 #include <rpc2/rpc2.h>
-#include <ports.h>
 #include <coda_string.h>
+#include <coda_getservbyname.h>
 
 void Initialize(void)
 {
@@ -52,7 +52,7 @@ void Initialize(void)
     memset(&options, 0, sizeof(options));
     options.Flags = RPC2_OPTION_IPV6;
 
-    rc = RPC2_Init(RPC2_VERSION, &options, 0, -1, &tv);
+    rc = RPC2_Init(RPC2_VERSION, &options, NULL, -1, &tv);
     if (rc != LWP_SUCCESS) {
 	printf("RPC_Init() failed\n");
 	exit(-1);
@@ -78,7 +78,7 @@ long Bind(char *host, short port, long subsys, RPC2_Handle *cid)
 
     bindparms.SideEffectType = 0;
     bindparms.SecurityLevel = RPC2_OPENKIMONO;
-    bindparms.ClientIdent = 0;
+    bindparms.ClientIdent = NULL;
 
     return RPC2_NewBinding(&hostid, &portid, &subsysid, &bindparms, cid);
 }
@@ -101,9 +101,10 @@ int main(int argc, char *argv[])
         host = argv[3];
         port = atoi(argv[2]);
     } else {
+	struct servent *s = coda_getservbyname("codasrv", "udp");
         host = argv[1];
-        port = PORT_codasrv; /* codasrv portnumber */
-	subsys = 1001;       /* SUBSYS_SRV */
+        port = ntohs(s->s_port);
+	subsys = 1001; /* SUBSYS_SRV */
     }
 
     Initialize();
