@@ -39,7 +39,6 @@ listed in the file CREDITS.
 #include <sys/socket.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <netdb.h>
 #include <netinet/in.h>
 #include <time.h>
 #include <sys/time.h>
@@ -50,10 +49,10 @@ listed in the file CREDITS.
 #include <lwp/lwp.h>
 #include <lwp/timer.h>
 #include <rpc2/rpc2.h>
-#include <rpc2/rpc2_addrinfo.h>
 #include <rpc2/se.h>
 #include <rpc2/sftp.h>
-#include <ports.h>
+#include <coda_getservbyname.h>
+#include <coda_getaddrinfo.h>
 
 #include "vice.h"
 
@@ -199,15 +198,15 @@ static int ValidServer(char *s)
 {
     struct RPC2_addrinfo hints, *res = NULL;
     int ret;
-    
+
     memset(&hints, 0, sizeof(hints));
     hints.ai_family   = PF_INET;
     hints.ai_socktype = SOCK_DGRAM;
     hints.ai_protocol = IPPROTO_UDP;
 
-    ret = RPC2_getaddrinfo(s, "codasrv", &hints, &res);
+    ret = coda_getaddrinfo(s, "codasrv", &hints, &res);
     RPC2_freeaddrinfo(res);
-    
+
     return (ret == 0);
 }
 
@@ -285,7 +284,7 @@ static void InitRPC(void)
     memset(&options, 0, sizeof(options));
     options.Flags = RPC2_OPTION_IPV6;
 
-    RPC2_Init(RPC2_VERSION, &options, 0, -1, &tv);
+    RPC2_Init(RPC2_VERSION, &options, NULL, -1, &tv);
 }
 
 
@@ -293,24 +292,27 @@ static void DoProbe(struct server *srv)
 {
     int rc;
 
-    srv->probe = time(0);
+    srv->probe = time(NULL);
     if (!srv->cid)
     {
 	RPC2_HostIdent hi;
 	RPC2_PortIdent pi;
 	RPC2_SubsysIdent si;
 	RPC2_BindParms bparms;
+	struct servent *s = coda_getservbyname("codasrv", "udp");
 
 	hi.Tag = RPC2_HOSTBYNAME;
 	strcpy(hi.Value.Name, srv->srvname);
-	pi.Tag = RPC2_PORTBYNAME;
-	strcpy(pi.Value.Name, "codasrv");
+
+	pi.Tag = RPC2_PORTBYINETNUMBER;
+	pi.Value.InetPortNumber = s->s_port;
+
 	si.Tag = RPC2_SUBSYSBYID;
 	si.Value.SubsysId= SUBSYS_SRV;
 
 	bparms.SideEffectType = 0;
 	bparms.SecurityLevel = RPC2_OPENKIMONO;
-	bparms.ClientIdent = 0;
+	bparms.ClientIdent = NULL;
 
 	rc = (int) RPC2_NewBinding(&hi, &pi, &si, &bparms, &srv->cid);
 	if (rc == RPC2_SUCCESS)
@@ -346,15 +348,15 @@ static void DoSleep(struct server *srv)
 
     /* Compensate the waiting time for the time it took to do the RPC */
     if (srv->succ == srv->probe)
-	NextProbe.tv_sec = (srv->probe + probeinterval) - time(0);
+	NextProbe.tv_sec = (srv->probe + probeinterval) - time(NULL);
 
     if (NextProbe.tv_sec > 0)
-	IOMGR_Select(0, 0, 0, 0, &NextProbe);  /* sleep */
+	IOMGR_Select(0, NULL, NULL, NULL, &NextProbe);  /* sleep */
 }
 
 static void srvlwp(void *arg)
 {
-    int slot = (int)arg;
+    int slot = *(int *)arg;
     srv[slot].cid = 0;
     srv[slot].old = 0;
 
@@ -379,7 +381,7 @@ int main(int argc, char *argv[])
     int i;
 
     GetArgs(argc, argv);
-    
+
     /* create missing rrd databases */
     for (i = 0; i < SrvCount; i++) {
 	sprintf(buf, "%s.rrd", srv[i].srvname);
@@ -393,14 +395,14 @@ int main(int argc, char *argv[])
     /* start monitoring */
     for (i = 0; i < SrvCount; i++) {
         LWP_CreateProcess(srvlwp, 0x8000, LWP_NORMAL_PRIORITY,
-                          (void *)i, srv[i].srvname, &srv[i].pid);
+                          (void *)&i, srv[i].srvname, &srv[i].pid);
     }
 
     /* QSignal/QWait doesn't actually queue anything so we still have to rely
      * on a counter to catch possibly missed events */
     while (SrvCount)
 	LWP_QWait();
-    
+
     exit(0);
 }
 
