@@ -79,8 +79,6 @@ extern "C" {
 #include <rpc2/rpc2.h>
 #include <rpc2/se.h>
 #include <rpc2/sftp.h>
-#include <map.h>
-#include <portmapper.h>
 #include <vice.h>
 #include <util.h>
 
@@ -95,6 +93,7 @@ int utimes(const char *, const struct timeval *);
 #include <volutil.h>
 #include <codaconf.h>
 #include <vice_file.h>
+#include <coda_getservbyname.h>
 #include "update.h"
 #include "getsecret.h"
 
@@ -109,8 +108,8 @@ extern char *ViceErrorMsg(int errorCode);   /* should be in libutil */
 #define LISTLEN 4096
 
 static void CheckLibStructure();
-static int CheckDir(char *prefix, int mode);
-static int CheckFile(char *fileName, int mode);
+static int CheckDir(const char *prefix, int mode);
+static int CheckFile(const char *fileName, int mode);
 static void ProcessArgs(int argc, char ** argv);
 static void ReadConfigFile();
 static void ReConnect();
@@ -130,16 +129,16 @@ static RPC2_Integer operatorUsecs = 0;
 
 static RPC2_Handle con = 0;
 static char host[256];
+static RPC2_Integer port;
 static int waitinterval = 30;	/* 5 min */
 static int reps = 6;
 
 static struct timeval  tp;
 static struct timezone tsp;
 
-static char **hostlist;		/* List of hosts to notify of changes. */
-static RPC2_EncryptionKey vkey;	/* Encryption key for bind authentication */
+static char **hostlist;	/* List of hosts to notify of changes. */
 
-static char *vicedir = NULL;
+static const char *vicedir = NULL;
 static int   nservers = 0;
 
 static RPC2_Unsigned timestamp = 0;  /* last transfered time. */
@@ -243,29 +242,29 @@ int main(int argc, char **argv)
 	if (CheckDir("db", 0644)) {
 	    operatorSecs = 0;  /* if something changed time has elapsed */
 	    for (int i=0; i<nservers; i++) {
-	        if (nservers != 1)
+		if (nservers != 1)
 		    vice_dir_init (vicedir, i+1);
 		/* signal file server to check data bases */
 		file = fopen(vice_file("srv/pid"), "r");
 		if (file == NULL) {
-		    LogMsg(0, SrvDebugLevel, stdout, 
+		    LogMsg(0, SrvDebugLevel, stdout,
 			   "Fopen failed for file %s with %s\n",
 			   vice_file("srv/pid"),
 			   ViceErrorMsg(errno));
 		} else {
 		    RPC2_Handle rpcid;
 		    if (U_BindToServer(hostlist[i], &rpcid) == RPC2_SUCCESS) {
-		        if (VolUpdateDB(rpcid) == RPC2_SUCCESS) {
-			    LogMsg(0, SrvDebugLevel, stdout, 
-			         "Notifying server %s of database updates\n",
-				 hostlist[i]);
+			if (VolUpdateDB(rpcid) == RPC2_SUCCESS) {
+			    LogMsg(0, SrvDebugLevel, stdout,
+				   "Notifying server %s of database updates\n",
+				   hostlist[i]);
 			} else {
-			    LogMsg(0, SrvDebugLevel, stdout, 
+			    LogMsg(0, SrvDebugLevel, stdout,
 				   "VolUpdateDB failed for host %s\n",
 				   hostlist[i]);
 			}
 		    } else {
-		        LogMsg(0, SrvDebugLevel, stdout, 
+			LogMsg(0, SrvDebugLevel, stdout,
 			     "Bind to server %s for database update failed\n",
 			     hostlist[i]);
 		    }
@@ -275,7 +274,7 @@ int main(int argc, char **argv)
 	}
 	if (operatorSecs > 0) {
 	    gettimeofday(&tp, &tsp);
-	    if ((tp.tv_sec < operatorSecs) || 
+	    if ((tp.tv_sec < operatorSecs) ||
 		(tp.tv_sec > (operatorSecs + 2))) {
 		tp.tv_sec = operatorSecs + 1;
 		tp.tv_usec = operatorUsecs;
@@ -303,13 +302,16 @@ static void ProcessArgs(int argc, char **argv)
     for (i = 1; i < argc; i++) {
 	if (!strcmp(argv[i], "-d"))
 	    SrvDebugLevel = atoi(argv[++i]);
+        else
+	    if (!strcmp(argv[i], "-port"))
+		port = atoi(argv[++i]);
 	else
 	    if (!strcmp(argv[i], "-h"))
 		strcpy(host, argv[++i]);
 	else
 	  if (!strcmp(argv[i], "-q")) {
-	        fprintf (stderr, "Old argument -q to update clnt.\n");
-		++i;
+	      fprintf (stderr, "Old argument -q to update clnt.\n");
+	      ++i;
 	  }
 	else
 	    if (!strcmp(argv[i], "-w"))
@@ -318,11 +320,13 @@ static void ProcessArgs(int argc, char **argv)
 	    if (!strcmp(argv[i], "-r"))
 		reps = atoi(argv[++i]);
 	else {
-	    LogMsg(0, SrvDebugLevel, stdout, 
+	    LogMsg(0, SrvDebugLevel, stdout,
 		   "usage: update [-d (debug level)] ");
-	    LogMsg(0, SrvDebugLevel, stdout, 
-		   "[-h (operator console hostname)] ");
-	    LogMsg(0, SrvDebugLevel, stdout, 
+	    LogMsg(0, SrvDebugLevel, stdout,
+		   "[-h (update server hostname)] ");
+	    LogMsg(0, SrvDebugLevel, stdout,
+		   "[-port (port of the update server)] ");
+	    LogMsg(0, SrvDebugLevel, stdout,
 		   "[-r (reps of w for long wait time)] [-w (short wait time)]\n");
 	    exit(-1);
 	}
@@ -336,30 +340,29 @@ ReadConfigFile()
     codaconf_init("server.conf");
 
     CODACONF_STR(vicedir, "vicedir", "/vice");
-    CODACONF_INT(nservers, "numservers", 1); 
+    CODACONF_INT(nservers, "numservers", 1);
 
     vice_dir_init(vicedir, 0);
 
     /* Host name list from multiple configs. */
     hostlist = new char*[nservers];
     if (nservers == 1) {
-        hostlist[0] = new char[256];
+	hostlist[0] = new char[256];
 	hostname(hostlist[0]);
     }
     else {
+	const char *host;
 	char confname[80];
-        for (int i = 0; i<nservers; i++) {
-	    hostlist[i] = new char[256];
-	    hostlist[i][0] = '\0';
-	    sprintf (confname, "server_%d.conf", i+1);
+	for (int i = 0; i < nservers; i++) {
+	    sprintf(confname, "server_%d.conf", i+1);
 	    codaconf_init(confname);
-	    CODACONF_STR(hostlist[i],  "hostname",  "");
-	    if (hostlist[i][0] == '\0') {
-	        LogMsg(0, SrvDebugLevel, stdout,
-		       "No host name for server %d.\n",
+	    host = codaconf_lookup("hostname", NULL);
+	    if (host == NULL) {
+		LogMsg(0, SrvDebugLevel, stdout,"No host name for server %d.\n",
 		       i+1);
 		exit(1);
 	    }
+	    hostlist[i] = strdup(host);
 	}
     }
 
@@ -375,10 +378,10 @@ static void CheckLibStructure()
 	mkdir(vice_sharedfile(NULL),0755);
 	mkdir(vice_sharedfile("db"),0755);
 	mkdir(vice_sharedfile("misc"),0755);
-	
+
 	for (int i=1; i<=nservers; i++) {
 	    if (nservers != 1)
-	        vice_dir_init (vicedir, i);
+		vice_dir_init (vicedir, i);
 	    mkdir(vice_file("srv"),0755);
 	    mkdir(vice_file("vol"),0755);
 	    mkdir(vice_file("spool"),0755);
@@ -391,9 +394,9 @@ static void CheckLibStructure()
 	}
 	for (int i=1; i<=nservers; i++) {
 	    if (nservers != 1)
-	        vice_dir_init (vicedir, i);
+		vice_dir_init (vicedir, i);
 	    if ((stat(vice_file("srv"),&lbuf)) && (errno == ENOENT)) {
-	        printf("Creating %s\n",vice_file("srv"));
+		printf("Creating %s\n",vice_file("srv"));
 		mkdir(vice_file("srv"),0755);
 	    }
 	    if ((stat(vice_file("vol"),&lbuf)) && (errno == ENOENT)) {
@@ -409,7 +412,7 @@ static void CheckLibStructure()
 }
 
 
-static int CheckDir(char *prefix, int mode)
+static int CheckDir(const char *prefix, int mode)
 {
     static char list[LISTLEN],
 	name[256],
@@ -425,7 +428,7 @@ static int CheckDir(char *prefix, int mode)
     if ((!(CheckFile(prefix, 0755))) && !CheckAll)
 	return(0);
 
-    LogMsg(1, SrvDebugLevel, stdout, 
+    LogMsg(1, SrvDebugLevel, stdout,
 	   "Directory %s changed, check files\n", prefix);
 
     strcpy(newname, prefix);
@@ -483,7 +486,7 @@ static int CheckDir(char *prefix, int mode)
 
 
 /* fileName is the full pathname */
-static int CheckFile(char *fileName, int mode)
+static int CheckFile(const char *fileName, int mode)
 {
     static char oldname[1024];
     static char tmpname[1024];
@@ -494,7 +497,7 @@ static int CheckFile(char *fileName, int mode)
     long     rc;
     SE_Descriptor sed;
 
-    LogMsg(1, SrvDebugLevel, stdout, "Checking file %s", fileName); 
+    LogMsg(1, SrvDebugLevel, stdout, "Checking file %s", fileName);
 
     if (stat(fileName, &buff)) {
 	time = 0;
@@ -540,11 +543,11 @@ static int CheckFile(char *fileName, int mode)
     }
 
     if (currentsecs > newtime) {
-        operatorSecs = currentsecs;
+	operatorSecs = currentsecs;
 	operatorUsecs = currentusecs;
     }
     else {
-        operatorSecs = 0;
+	operatorSecs = 0;
     }
 
     if ((newtime != 0) && (time != newtime)) {
@@ -584,6 +587,7 @@ static int CheckFile(char *fileName, int mode)
 
 static void ReConnect()
 {
+    static struct secret_state state = { 0, };
     long     rc;
     RPC2_SubsysIdent ssid;
     RPC2_HostIdent hident;
@@ -591,8 +595,6 @@ static void ReConnect()
     RPC2_CountedBS cident;
     RPC2_EncryptionKey secret;
     char hostname[64];
-    long portmapid;
-    long port;
 
     if (con) {
 	LogMsg(0, SrvDebugLevel, stdout, "Unbinding\n");
@@ -600,19 +602,10 @@ static void ReConnect()
 	con = 0;
     }
 
-    portmapid = portmap_bind(host);
-    if ( !portmapid ) {
-	fprintf(stderr, "Cannot bind to rpc2portmap; exiting\n");
-	return;
-    }
-
-    rc = portmapper_client_lookup_pbynvp(portmapid,
-					 (unsigned char *)"codaupdate", 0, 17,
-					 &port);
-    RPC2_Unbind(portmapid);
-    if (rc) {
-	fprintf(stderr, "Cannot get port from rpc2portmap; exiting\n");
-	return;
+    if (!port) {
+	/* we use the unused codasrv-se port */
+	struct servent *s = coda_getservbyname("codasrv-se", "udp");
+	port = ntohs(s->s_port);
     }
 
     hident.Tag = RPC2_HOSTBYNAME;
@@ -633,12 +626,12 @@ static void ReConnect()
     cident.SeqLen = strlen(hostname) + 1;
     bparms.ClientIdent = &cident;
 
-    GetSecret(vice_sharedfile("db/update.tk"), secret);
+    GetSecret(vice_sharedfile("db/update.tk"), secret, &state);
     bparms.SharedSecret = &secret;
 
     rc = RPC2_NewBinding(&hident, &pident, &ssid, &bparms, &con);
     if (rc) {
-	LogMsg(0, SrvDebugLevel, stdout, "Bind failed with %s\n", 
+	LogMsg(0, SrvDebugLevel, stdout, "Bind failed with %s\n",
 	       (char *)ViceErrorMsg((int)rc));
 	return;
     }
@@ -700,17 +693,10 @@ static void SetCheck()
 static void U_InitRPC()
 {
     PROCESS mylpid;
-    FILE *tokfile;
     SFTP_Initializer sftpi;
     long rcode;
     RPC2_Options options;
 
-    /* store authentication key */
-    tokfile = fopen(vice_sharedfile(VolTKFile), "r");
-    memset(vkey, 0, RPC2_KEYSIZE);
-    fread(vkey, 1, RPC2_KEYSIZE, tokfile);
-    fclose(tokfile);
-
     CODA_ASSERT(LWP_Init(LWP_VERSION, LWP_MAX_PRIORITY-1, &mylpid) == LWP_SUCCESS);
 
     SFTP_SetDefaults(&sftpi);
@@ -735,16 +721,20 @@ static int U_BindToServer(char *fileserver, RPC2_Handle *RPCid)
  /* Binds to File Server on volume utility port on behalf of uName.
     Sets RPCid to the value of the connection id.    */
 
+    static struct secret_state state = { 0, };
     RPC2_HostIdent hident;
     RPC2_PortIdent pident;
     RPC2_SubsysIdent sident;
     RPC2_EncryptionKey secret;
     long rcode;
+    struct servent *s = coda_getservbyname("codasrv", "udp");
 
     hident.Tag = RPC2_HOSTBYNAME;
     strcpy(hident.Value.Name, fileserver);
-    pident.Tag = RPC2_PORTBYNAME;
-    strcpy(pident.Value.Name, "codasrv");
+
+    pident.Tag = RPC2_PORTBYINETNUMBER;
+    pident.Value.InetPortNumber = s->s_port;
+
     sident.Tag = RPC2_SUBSYSBYID;
     sident.Value.SubsysId = UTIL_SUBSYSID;
 
@@ -754,7 +744,7 @@ static int U_BindToServer(char *fileserver, RPC2_Handle *RPCid)
     bparms.EncryptionType = RPC2_XOR;
     bparms.SideEffectType = SMARTFTP;
 
-    GetSecret(vice_sharedfile(VolTKFile), secret);
+    GetSecret(vice_sharedfile(VolTKFile), secret, &state);
     bparms.SharedSecret = &secret;
 
     LogMsg(9, SrvDebugLevel, stdout, "V_BindToServer: binding to host %s\n",
