@@ -80,8 +80,6 @@ extern "C" {
 #include <lwp/lock.h>
 #include <rpc2/rpc2.h>
 #include <rpc2/rpc2_addrinfo.h>
-#include <map.h>
-#include <portmapper.h>
 #include <rpc2/se.h>
 extern void SFTP_SetDefaults (SFTP_Initializer *initPtr);
 extern void SFTP_Activate (SFTP_Initializer *initPtr);
@@ -94,6 +92,7 @@ extern void SFTP_Activate (SFTP_Initializer *initPtr);
 #include <util.h>
 #include <vice_file.h>
 #include <codaconf.h>
+#include <coda_getservbyname.h>
 #include "update.h"
 #include "getsecret.h"
 
@@ -112,16 +111,15 @@ static char *prefix = NULL;
 static struct timeval  tp;
 static struct timezone tsp;
 
-static char *vicedir = NULL;
-
 void
 ReadConfigFile()
 {
+    const char *vicedir;
+
     /* Load configuration file to get vice dir. */
     codaconf_init("server.conf");
 
-    CODACONF_STR(vicedir, "vicedir", "/vice");
-
+    vicedir = codaconf_lookup("vicedir", "/vice");
     vice_dir_init(vicedir, 0);
 }
 
@@ -162,14 +160,14 @@ InList (char *name)
 
     LogMsg(3, SrvDebugLevel, stdout, "InList Lookup: '%s'", name);
     while (entry) {
-         LogMsg(3, SrvDebugLevel, stdout, "InList strcmp: '%s'", entry->name);
-        if (strcmp(entry->name,name) == 0)
+	LogMsg(3, SrvDebugLevel, stdout, "InList strcmp: '%s'", entry->name);
+	if (strcmp(entry->name,name) == 0)
 	    return 1;
 	entry = entry->next;
     }
     return 0;
 }
-    
+
 int main(int argc, char **argv)
 {
     char    sname[20];
@@ -183,9 +181,8 @@ int main(int argc, char **argv)
     SFTP_Initializer sftpi;
     RPC2_Options options;
     int rc;
-    long portmapid;
     struct stat statbuf;
-    char *miscdir;
+    const char *miscdir;
     int port = 0;
 
     /* process the command line arguments */
@@ -197,20 +194,26 @@ int main(int argc, char **argv)
 	    lwps = atoi(argv[++i]);
 	else if (!strcmp(argv[i], "-port"))
 	    port = atoi(argv[++i]);
-	else if (!strcmp(argv[i], "-p")) {
+	else if (!strcmp(argv[i], "-p"))
 	    prefix = argv[++i];
-	} else if (!strcmp(argv[i], "-q")) {
+	else if (!strcmp(argv[i], "-q")) {
 	    fprintf(stderr, "Old argument -q to update srv\n");
 	    ++i;
 	} else {
 	    fprintf(stderr, "Bad argument %s to update srv\n", 
 		    argv[i]);
-	    fprintf(stderr, "Usage: updatesrv [-p prefix"
-		    "-d (debug level)]) [-l (number of lwps)]\n");
+	    fprintf(stderr, "Usage: updatesrv [-p prefix] [-port port]"
+		    " [-d (debug level)] [-l (number of lwps)]\n");
 	    exit(1);
 	}
     }
 
+    if (!port) {
+	/* use the unused codasrv-se port */
+	struct servent *s = coda_getservbyname("codasrv-se", "udp");
+	port = ntohs(s->s_port);
+    }
+
     ReadConfigFile();
     ReadExportList();
 
@@ -222,7 +225,7 @@ int main(int argc, char **argv)
 
     rc = chdir(miscdir);
     if ( rc ) {
-        snprintf (errmsg, MAXPATHLEN, "Could not chdir to %s",
+	snprintf (errmsg, MAXPATHLEN, "Could not chdir to %s",
 		  vice_sharedfile("misc"));
 	perror(errmsg);
 	exit(1);
@@ -249,10 +252,10 @@ int main(int argc, char **argv)
 
     LogMsg(0, SrvDebugLevel, stdout, "Update Server working directory %s",
 	   prefix);
-    
+
     file = fopen(vice_sharedfile("misc/updatesrv.pid"), "w");
     if ( !file ) {
-        snprintf (errmsg, MAXPATHLEN, "Error in writing %s",
+	snprintf (errmsg, MAXPATHLEN, "Error in writing %s",
 		  vice_sharedfile("misc/updatesrv.pid"));
 	perror(errmsg);
 	exit(1);
@@ -281,23 +284,6 @@ int main(int argc, char **argv)
     CODA_ASSERT(RPC2_Init(RPC2_VERSION, &options, &port1, 6, &tp) == RPC2_SUCCESS);
     RPC2_enableReaping = 1;
 
-    /* register the port with the portmapper */
-    fprintf(stderr, "Attempting to bind with rpc2portmap\n");
-    portmapid = portmap_bind("localhost");
-    if ( !portmapid ) {
-	    fprintf(stderr, "Cannot bind to rpc2portmap; exiting\n");
-	    return 1;
-    }
-    rc = portmapper_client_register_sqsh(portmapid, (RPC2_String)"codaupdate",
-					 0, 17,
-					 ntohs(port1.Value.InetPortNumber));
-
-    if ( rc ) {
-	    fprintf(stderr, "Cannot register with rpc2portmap; exiting, rc = %i \n", rc);
-	    return 1;
-    }
-    RPC2_Unbind(portmapid); 
-
     server.Tag = RPC2_SUBSYSBYID;
     server.Value.SubsysId = SUBSYS_UPDATE;
     CODA_ASSERT(RPC2_Export(&server) == RPC2_SUCCESS);
@@ -306,7 +292,7 @@ int main(int argc, char **argv)
 	sprintf(sname, "ServerLWP-%d", i);
 	CODA_ASSERT(LWP_CreateProcess(ServerLWP, 32 * 1024,
 				      LWP_MAX_PRIORITY - 1,
-				      &i, sname, &serverPid) 
+				      (void *)&i, sname, &serverPid)
 	       == LWP_SUCCESS);
     }
     gettimeofday(&tp, &tsp);
@@ -351,12 +337,13 @@ static long Update_GetKeys(RPC2_Integer *authtype, RPC2_CountedBS *cident,
 			   RPC2_EncryptionKey sharedsecret,
 			   RPC2_EncryptionKey sessionkey)
 {
+    static struct secret_state state = { 0, };
     unsigned int i;
 
     /* Block unauthenticated connections */
     if (!cident) return -1;
 
-    if (GetSecret(vice_sharedfile("db/update.tk"), sharedsecret) == -1)
+    if (GetSecret(vice_sharedfile("db/update.tk"), sharedsecret, &state) == -1)
 	return -1;
 
     memset(sessionkey, 0, RPC2_KEYSIZE);
@@ -408,18 +395,18 @@ static void ServerLWP(void *arg)
 	    RPC2_GetPeerInfo(mycid, &peer);
 
 	    if (peer.SecurityLevel == RPC2_OPENKIMONO) {
-                LogMsg(0, SrvDebugLevel, stdout,
-                       "Receiving unauthenticated request %d, "
-                       "update rpc2 library!", myrequest->Header.Opcode);
+		LogMsg(0, SrvDebugLevel, stdout,
+		       "Receiving unauthenticated request %d, "
+		       "update rpc2 library!", myrequest->Header.Opcode);
 		CODA_ASSERT(peer.RemoteHost.Tag == RPC2_HOSTBYADDRINFO);
 		RPC2_freeaddrinfo(peer.RemoteHost.Value.AddrInfo);
-                RPC2_Unbind(mycid);
-                continue;
-            }
+		RPC2_Unbind(mycid);
+		continue;
+	    }
 
-            LogMsg(1, SrvDebugLevel, stdout,"Worker %d received request %d",
-                   lwpid, myrequest->Header.Opcode);
-            rc = update_ExecuteRequest(mycid, myrequest, 0);
+	    LogMsg(1, SrvDebugLevel, stdout,"Worker %d received request %d",
+		   lwpid, myrequest->Header.Opcode);
+	    rc = update_ExecuteRequest(mycid, myrequest, 0);
 
 	    if (rc)
 	    {
@@ -485,8 +472,8 @@ long UpdateFetch(RPC2_Handle RPCid, RPC2_String FileName,
     *NewTime = 0;
 
     if (!AccessAllowed((char *)name)) {
-        LogMsg(0, SrvDebugLevel, stdout, "Access denied to file %s.", name);
-	rc = CEACCES;
+	LogMsg(0, SrvDebugLevel, stdout, "Access denied to file %s.", name);
+	rc = EACCES;
 	goto Final;
     }
 
@@ -512,7 +499,7 @@ long UpdateFetch(RPC2_Handle RPCid, RPC2_String FileName,
 	strcpy(sid.Value.SmartFTPD.FileInfo.ByName.LocalFileName, name);
 
 	if ((rc = RPC2_InitSideEffect(RPCid, &sid))) {
-	    LogMsg(0, SrvDebugLevel, stdout, 
+	    LogMsg(0, SrvDebugLevel, stdout,
 		   "InitSideEffect failed %s", ViceErrorMsg((int)rc));
 	    if (rc <= RPC2_ELIMIT) {
 		goto Final;
@@ -520,7 +507,7 @@ long UpdateFetch(RPC2_Handle RPCid, RPC2_String FileName,
 	}
 
 	if ((rc = RPC2_CheckSideEffect(RPCid, &sid, SE_AWAITLOCALSTATUS))) {
-	    LogMsg(0, SrvDebugLevel, stdout, 
+	    LogMsg(0, SrvDebugLevel, stdout,
 		   "CheckSideEffect failed %s", ViceErrorMsg((int)rc));
 	    if (rc <= RPC2_ELIMIT) {
 		goto Final;
@@ -532,17 +519,17 @@ Final:
     gettimeofday(&tp, &tsp);
     *CurrentSecs = tp.tv_sec;
     *CurrentUsecs = tp.tv_usec;
-    LogMsg(2, SrvDebugLevel, stdout, 
+    LogMsg(2, SrvDebugLevel, stdout,
 	   "UpdateFetch returns %s newtime is %d at %s",
 	   ViceErrorMsg((int)rc), *NewTime, ctime((time_t*)&tp.tv_sec));
     return(rc);
 }
 
 
-long UpdateNewConnection(RPC2_Handle cid, RPC2_Integer SideEffectType, 
-			 RPC2_Integer SecurityLevel, 
-			 RPC2_Integer EncryptionType, 
-			 RPC2_Integer AuthType, 
+long UpdateNewConnection(RPC2_Handle cid, RPC2_Integer SideEffectType,
+			 RPC2_Integer SecurityLevel,
+			 RPC2_Integer EncryptionType,
+			 RPC2_Integer AuthType,
 			 RPC2_CountedBS *ClientIdent)
 {
     char buf[RPC2_ADDRSTRLEN];
