@@ -45,7 +45,7 @@ void *rec_olist::operator new(size_t size) {
     return(r);
 }
 
-void rec_olist::operator delete(void *deadobj, size_t size) {
+void rec_olist::operator delete(void *deadobj) {
     rvmlib_rec_free(deadobj);
 }
 
