@@ -51,7 +51,9 @@ extern "C" {
 #include "coda_string.h"
 #include <time.h>
 #include <sys/utsname.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <stdarg.h>
 
 #include "util.h"
@@ -73,46 +75,6 @@ int DirDebugLevel = 0;		/* Dir package */
 int AL_DebugLevel = 0;		/* ACL package */
 int AuthDebugLevel = 0;		/* Auth package */
 
-extern int CaseFoldedCmp(char *s1, char *s2)
-    /* same as strcmp() except that case differences are ignored */
-{
-	register int i;
-	register char c1, c2;
-    
-	i = -1;
-	do  {
-		/* Inv: s1[0:i] is identical to s2[0:i] and i is smaller than 
-		   the shorter of s1 and s2 */
-		i += 1;
-		c1 = s1[i];
-		if (islower(c1)) c1 -= ('a' - 'A');
-		c2 = s2[i];
-		if (islower(c2)) c2 -= ('a' - 'A');
-		if (c1 == 0 && c2 ==0) return(0);
-	} while(c1 == c2);
-
-	if (c1 < c2) 
-		return (-1);
-	else 
-		return(1);
-}
-
-int SafeStrCat(char *dest, char *src, int totalspace)
-{
-    if (strlen(dest)+strlen(src) >= totalspace) 
-	    return(-1);
-    strcat(dest,src);
-    return(0);
-}
-
-int SafeStrCpy(char *dest, char *src, int totalspace)
-{
-    if (strlen(src) >= totalspace) return(-1);
-    strcpy(dest,src);
-    return(0);
-}
-
-
 int HashString(char *s, unsigned int size)
 {
 	unsigned int sum;
@@ -175,14 +137,14 @@ void PrintTimeStamp(FILE *f)
 	logs[i].yday = t->tm_yday;
     }
 
-    fprintf(f, "%02d:%02d:%02d ", t->tm_hour, t->tm_min, t->tm_sec);    
+    fprintf(f, "%02d:%02d:%02d ", t->tm_hour, t->tm_min, t->tm_sec);
 }
 
-extern void LogMsg(int msglevel, int debuglevel, FILE *fout, char *fmt,  ...)
+void LogMsg(int msglevel, int debuglevel, FILE *fout, const char *fmt,  ...)
 {
     va_list ap;
 
-    if (debuglevel < msglevel) 
+    if (debuglevel < msglevel)
 	    return;
 
     PrintTimeStamp(fout);
@@ -208,7 +170,7 @@ long int gethostid(void)
 
 #endif
 
-void fdprint(long afd, char *fmt, ...) 
+void fdprint(long afd, const char *fmt, ...)
 {
     va_list ap;
     char buf[240];
@@ -220,7 +182,7 @@ void fdprint(long afd, char *fmt, ...)
 }
 
 /* message to stderr */
-void eprint(char *fmt, ...) 
+void eprint(const char *fmt, ...)
 {
 	va_list ap;
 	char msg[240];
@@ -287,7 +249,7 @@ int UtilHostEq(const char *name1, const char *name2)
     return ret;
 }
 
-void UtilDetach()
+void UtilDetach(void)
 {
     pid_t child; 
     int rc;
