@@ -1,5 +1,3 @@
-/*  $Id: Inet.cc,v 1.4 2006-05-24 20:04:39 jaharkes Exp $	*/
-
 /*
     Copyright (C) 2000 Philip A. Nelson
 
@@ -146,8 +144,8 @@ bool Inet::TcpOpen (char *host, int port)
 }
 
 #if defined(HAVE_SYS_UN_H) && !defined(WIN32)
-// Open a tcp connection to a UNIX file socket 
-bool Inet::TcpOpen (char *SocketPath)
+// Open a tcp connection to a UNIX file socket
+bool Inet::TcpOpen (const char *SocketPath)
 {
   struct sockaddr_un sock;
 
