@@ -1,5 +1,3 @@
-/*	$Id: Inet.h,v 1.3 2006-05-24 20:04:40 jaharkes Exp $	*/
-
 /*
     Copyright (C) 2000 Philip A. Nelson
 
@@ -97,7 +95,7 @@ class Inet {
     bool TcpOpen (char *host, int port);
 
 #if defined(HAVE_SYS_UN_H) && !defined(WIN32)
-    bool TcpOpen (char *socketpath);
+    bool TcpOpen (const char *socketpath);
 #endif
 
     // Open a tcp server socket.  Return false on failure.
