@@ -1,5 +1,3 @@
-/* 	$Id: monitor.cc,v 1.13 2006-05-24 20:04:40 jaharkes Exp $	*/
-
 /* BLURB gpl
  
                             Coda File System
@@ -68,7 +66,7 @@ static void ClearVattr(void *)
 
 static void ClearXfer(void *data)
 {
-  int ix = (int)data;
+  int ix = (long)data;
   if (!XferProg[ix]->active() && XferProg[ix]->visible())
     XferProg[ix]->hide();
 }
@@ -76,17 +74,17 @@ static void ClearXfer(void *data)
 // Run to start at the beginning of the application
 void monitor::Start(void)
 {
-  char *marinerport;
+  const char *marinerport = NULL;
 
   // printf ("monitor::Start\n");
   if (TheMon == NULL) {
     TheMon = this;
   }
 
-  marinerport = codaconf_lookup("marinersocket", "/usr/coda/spool/mariner");
+  CODACONF_STR(marinerport, "marinersocket", "/usr/coda/spool/mariner");
 
   if (!conn.isOpen()) {
-#if  defined(HAVE_SYS_UN_H) && !defined(WIN32)
+#if defined(HAVE_SYS_UN_H) && !defined(WIN32)
     if (!conn.TcpOpen (marinerport))
 #endif
       if (!conn.TcpOpen ("localhost", CODACONPORT)) {
@@ -249,29 +247,14 @@ void monitor::NextLine() {
 	Fl::add_timeout(5, ClearVattr, (void *)NULL);
       }
 
-      else if (strstr(inputline, "Local inconsistent object")) {
+      else if (strstr(inputline, "Local inconsistent object")
+	       || strstr(inputline, "CONFLICT")) {
 	VConfl->color(FL_RED);
 	VConfl->redraw();
 	ConflList->add(inputline,NULL);
 	ConflList->bottomline(ConflList->size());
       }
 
-      else if (strstr(inputline, "fetch::DisconnectFS")) {
-	if (DisFsCount++ == 0) {
-	  VDisConn->color(FL_YELLOW);
-	  VDisConn->redraw();
-	}
-	ConflList->add(inputline,NULL);
-	ConflList->bottomline(ConflList->size());
-      }
-
-      else if (strstr(inputline, "fetch::disconnectfs done")) {
-	if (--DisFsCount == 0) {
-	  VDisConn->color(FL_WHITE);
-	  VDisConn->redraw();
-	}
-      }
-
       else if (strstr(inputline, "pattern to look for")) {
       }
 
@@ -290,11 +273,12 @@ void monitor::NextLine() {
 void monitor::ForceClose()
 {
   /* need to try to restart it ... */
-
-  Fl::remove_fd(conn.FileNo());
+  if (conn.FileNo() >= 0) {
+    Fl::remove_fd(conn.FileNo());
+    conn.Close();
+  }
 
   printf ("ForceClose closed!\n");
-  conn.Close();
   VConn->color(FL_RED);
   VConn->redraw();
 
