@@ -43,7 +43,9 @@ extern "C" {
 #endif
 #include <sys/time.h>
 #include <errno.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <stdio.h>
 #include "coda_string.h"
 #include <struct.h>
@@ -52,7 +54,6 @@ extern "C" {
 
 #include <rpc2/rpc2.h>
 #include <rpc2/se.h>
-#include <rpc2/fail.h>
 #include <rpc2/errors.h>
 
 extern int Fcon_Init(); 
@@ -78,8 +79,6 @@ extern void SFTP_Activate (SFTP_Initializer *initPtr);
 #include "venusrecov.h"
 #include "venusvol.h"
 #include "vproc.h"
-#include "adv_monitor.h"
-#include "adv_daemon.h"
 
 int COPModes = 6;	/* ASYNCCOP2 | PIGGYCOP2 */
 char myHostName[MAXHOSTNAMELEN];
@@ -91,21 +90,10 @@ int sftp_ackpoint = UNSET_AP;
 int sftp_packetsize = UNSET_PS;
 int rpc2_timeflag = UNSET_ST;
 int mrpc2_timeflag = UNSET_MT;
-unsigned long WCThresh = UNSET_WCT;  	/* in Bytes/sec */
-int WCStale = UNSET_WCS;	/* seconds */
 
 extern long RPC2_Perror;
 struct CommQueueStruct CommQueue;
 
-#define MAXFILTERS 32
-
-struct FailFilterInfoStruct {
-	unsigned id;
-	unsigned long host;
-	FailFilterSide side;
-	unsigned char used;
-} FailFilterInfo[MAXFILTERS];
-
 olist *srvent::srvtab;
 char srvent::srvtab_sync;
 olist *connent::conntab;
@@ -134,9 +122,6 @@ void CommInit() {
     else
 	srv_MultiStubWork[0].opengate = mrpc2_timeflag;
 
-    if (WCThresh == UNSET_WCT) WCThresh = DFLT_WCT;
-    if (WCStale == UNSET_WCS) WCStale = DFLT_WCS;
-
     /* Sanity check COPModes. */
     if ( (ASYNCCOP1 && !ASYNCCOP2) ||
 	 (PIGGYCOP2 && !ASYNCCOP2) )
@@ -180,13 +165,6 @@ void CommInit() {
     if (RPC2_Init(RPC2_VERSION, 0, &port1, rpc2_retries, &tv) != RPC2_SUCCESS)
 	CHOKE("CommInit: RPC2_Init failed");
 
-#ifdef USE_FAIL_FILTERS
-    /* Failure package initialization. */
-    memset((void *)FailFilterInfo, 0, (int) (MAXFILTERS * sizeof(struct FailFilterInfoStruct)));
-    Fail_Initialize("venus", 0);
-    Fcon_Init();
-#endif
-
     /* Fire up the probe daemon. */
     PROD_Init();
 }
@@ -468,27 +446,18 @@ connent *conn_iterator::operator()()
  *    contacted (i.e., at initialization), -2 indicates that the server has just NAK'ed an RPC.
  */
 
-#define	SRVRQ_LOCK()
-#define	SRVRQ_UNLOCK()
-#define	SRVRQ_WAIT()	    VprocWait((char *)&srvent::srvtab_sync)
-#define	SRVRQ_SIGNAL()	    VprocSignal((char *)&srvent::srvtab_sync)
-
 void Srvr_Wait() {
-    SRVRQ_LOCK();
     LOG(0, ("WAITING(SRVRQ):\n"));
     START_TIMING();
-    SRVRQ_WAIT();
+    VprocWait((char *)&srvent::srvtab_sync);
     END_TIMING();
     LOG(0, ("WAIT OVER, elapsed = %3.1f\n", elapsed));
-    SRVRQ_UNLOCK();
 }
 
 
 void Srvr_Signal() {
-    SRVRQ_LOCK();
     LOG(10, ("SIGNALLING(SRVRQ):\n"));
-    SRVRQ_SIGNAL();
-    SRVRQ_UNLOCK();
+    VprocSignal((char *)&srvent::srvtab_sync);
 }
 
 
@@ -565,8 +534,7 @@ void PutServer(srvent **spp)
  *    yet a part of MultiRPC.
  */
 
-probeslave::probeslave(ProbeSlaveTask Task, void *Arg, void *Result, char *Sync) :
-	vproc("ProbeSlave", NULL, VPT_ProbeDaemon, 16384) {
+probeslave::probeslave(ProbeSlaveTask Task, void *Arg, void *Result, char *Sync) : vproc("ProbeSlave", NULL, VPT_ProbeDaemon, 32768) {
     LOG(100, ("probeslave::probeslave(%#x): %-16s : lwpid = %d\n", this, name, lwpid));
 
     task = Task;
@@ -933,10 +901,7 @@ srvent::srvent(struct in_addr *Host, RealmId realm)
     connid = -1;
     Xbinding = 0;
     probeme = 0;
-    forcestrong = 0;
-    isweak = 0;
-    bw     = INIT_BW;
-    bwmax  = INIT_BW;
+    bw = INIT_BW;
     lastobs.tv_sec = lastobs.tv_usec = 0;
     VGAPlusSHA_Supported = 0;  /* default is old-style server */
     refcount = 1;
@@ -1015,7 +980,6 @@ int srvent::Connect(RPC2_Handle *cidp, int *authp, uid_t uid, int Force)
 	/* Not already considered down. */
 	MarinerLog("connection::unreachable %s\n", name);
 	Reset();
-	adv_mon.ServerInaccessible(name);
     }
 
     if (code == ETIMEDOUT && VprocInterrupted()) return(EINTR);
@@ -1111,9 +1075,8 @@ void srvent::ServerError(int *codep)
 	/* Reset if TIMED'out or NAK'ed. */
 	switch (*codep) {
 	    case ETIMEDOUT:
-	        MarinerLog("connection::unreachable %s\n", name);
+		MarinerLog("connection::unreachable %s\n", name);
 		Reset();
-		adv_mon.ServerInaccessible(name);
 		break;
 
 	    case ERETRY:
@@ -1141,8 +1104,7 @@ void srvent::ServerUp(RPC2_Handle newconnid)
 	MarinerLog("connection::up %s\n", name);
 	connid = newconnid;
 	VDB->UpEvent(&host);
-	adv_mon.ServerAccessible(name);
-        break;
+	break;
 
     case -1:
 	/* Initial case.  */
@@ -1208,15 +1170,15 @@ long srvent::GetBandwidth(unsigned long *Bandwidth)
 {
     long rc = 0;
     unsigned long oldbw = bw;
-    unsigned long bwmin;
+    unsigned long bwmin, bwmax;
 
-    LOG(1, ("srvent::GetBandwidth (%s) lastobs %ld.%06ld\n", 
+    LOG(1, ("srvent::GetBandwidth (%s) lastobs %ld.%06ld\n",
 	      name, lastobs.tv_sec, lastobs.tv_usec));
-    
+
     /* we don't have a real connid if the server is down or "quasi-up" */
-    if (connid <= 0) 
+    if (connid <= 0)
 	return(ETIMEDOUT);
-    
+
     /* retrieve the bandwidth information from RPC2 */
     if ((rc = RPC2_GetBandwidth(connid, &bwmin, &bw, &bwmax)) != RPC2_SUCCESS)
 	return(rc);
@@ -1226,31 +1188,11 @@ long srvent::GetBandwidth(unsigned long *Bandwidth)
     /* update last observation time */
     RPC2_GetLastObs(connid, &lastobs);
 
-    /* 
-     * Signal if we've crossed the weakly-connected threshold. Note
-     * that the connection is considered strong until proven otherwise.
-     *
-     * The user can block the strong->weak transition using the
-     * 'cfs strong' command (and turn adaptive mode back on with
-     * 'cfs adaptive').
-     */
-    if (!isweak && !forcestrong && bwmax < WCThresh) {
-	isweak = 1;
-	MarinerLog("connection::weak %s\n", name);
-	VDB->WeakEvent(&host);
-        adv_mon.ServerConnectionWeak(name);
-    }
-    else if (isweak && bwmin > WCThresh) {
-	isweak = 0;
-	MarinerLog("connection::strong %s\n", name);
-	VDB->StrongEvent(&host);
-        adv_mon.ServerConnectionStrong(name);
-    }
-	
     *Bandwidth = bw;
-    if (bw != oldbw) {
-	MarinerLog("connection::bandwidth %s %d %d %d\n", name,bwmin,bw,bwmax);
-        adv_mon.ServerBandwidthEstimate(name, *Bandwidth);
+    /* only report new bandwidth estimate if it has changed by more than ~10% */
+    if ((bw > (oldbw + oldbw / 8)) || (bw < (oldbw - oldbw / 8)))
+    {
+	MarinerLog("connection::bandwidth %s %d %d %d\n", name, bwmin,bw,bwmax);
     }
     LOG(1, ("srvent::GetBandwidth (%s) returns %d bytes/sec\n",
 	      name, *Bandwidth));
@@ -1258,33 +1200,6 @@ long srvent::GetBandwidth(unsigned long *Bandwidth)
 }
 
 
-/* 
- * Force server connectivity to strong, or resume with the normal bandwidth
- * adaptive mode (depending on the `on' flag).
- */
-void srvent::ForceStrong(int on) {
-    forcestrong = on;
-
-    /* forced switch to strong mode */
-    if (forcestrong && isweak) {
-	isweak = 0;
-	MarinerLog("connection::strong %s\n", name);
-	VDB->StrongEvent(&host);
-        adv_mon.ServerConnectionStrong(name);
-    }
-
-    /* switch back to adaptive mode */
-    if (!forcestrong && !isweak && bwmax < WCThresh) {
-	isweak = 1;
-	MarinerLog("connection::weak %s\n", name);
-	VDB->WeakEvent(&host);
-        adv_mon.ServerConnectionWeak(name);
-    }
-
-    return;
-}
-
-
 void srvent::print(FILE *f)
 {
     fprintf(f, "%p : %-16s : cid = %d, host = %s, binding = %d, bw = %ld\n",
@@ -1306,7 +1221,8 @@ srvent *srv_iterator::operator()() {
 }
 
 
-#ifndef USE_FAIL_FILTERS
+/* hooks in librpc2 that existed for the old libfail functionality. We use
+ * these leftover hooks to implement cfs disconnect and cfs reconnect */
 extern int (*Fail_SendPredicate)(unsigned char ip1, unsigned char ip2,
 				 unsigned char ip3, unsigned char ip4,
 				 unsigned char color, RPC2_PacketBuffer *pb,
@@ -1337,136 +1253,3 @@ int FailReconnect(int nservers, struct in_addr *hostids)
     return 0;
 }
 
-int FailSlow(unsigned *speedp)
-{
-    return -1;
-}
-#else
-/* *****  Fail library manipulations ***** */
-
-/* 
- * Simulate "pulling the plug". Insert filters on the
- * send and receive sides of venus.
- */
-int FailDisconnect(int nservers, struct in_addr *hostids)
-{
-    int rc, k = 0;
-    FailFilter filter;
-    FailFilterSide side;
-
-    do {
-	srv_iterator next;
-	srvent *s;
-	while ((s = next()))
-	    if (nservers == 0 || s->host.s_addr == hostids[k].s_addr) {
-		/* we want a pair of filters for server s. */
-
-		struct in_addr addr = s->host;    
-		filter.ip1 = ((unsigned char *)&addr)[0];
-		filter.ip2 = ((unsigned char *)&addr)[1];
-		filter.ip3 = ((unsigned char *)&addr)[2];
-		filter.ip4 = ((unsigned char *)&addr)[3];
-		filter.color = -1;
-		filter.lenmin = 0;
-		filter.lenmax = 65535;
-		filter.factor = 0;
-		filter.speed = 0;
-		filter.latency = 0;
-
-		for (int i = 0; i < 2; i++) {
-		    if (i == 0) side = sendSide;
-		    else side = recvSide;
-
-		    /* 
-		     * do we have this filter already?  Note that this only
-		     * checks the filters inserted from Venus.  To check all
-		     * the filters, we need to use Fail_GetFilters.
-		     */
-		    char gotit = 0;
-		    for (int j = 0; j < MAXFILTERS; j++) 
-			if (FailFilterInfo[j].used && 
-			    htonl(FailFilterInfo[j].host) == s->host.s_addr &&
-			    FailFilterInfo[j].side == side) {
-				gotit = 1;
-				break;
-			}
-
-		    if (!gotit) {  /* insert a new filter */
-			int ix = -1;    
-			for (int j = 0; j < MAXFILTERS; j++) 
-			    if (!FailFilterInfo[j].used) {
-				ix = j;
-				break;
-			    }
-
-			if (ix == -1) { /* no room */
-			    LOG(0, ("FailDisconnect: couldn't insert %s filter for %s, table full!\n", 
-				(side == recvSide)?"recv":"send", s->name));
-			    return(ENOBUFS);
-			}
-
-			if ((rc = Fail_InsertFilter(side, 0, &filter)) < 0) {
-			    LOG(0, ("FailDisconnect: couldn't insert %s filter for %s\n", 
-				(side == recvSide)?"recv":"send", s->name));
-			    return(rc);
-			}
-			LOG(10, ("FailDisconnect: inserted %s filter for %s, id = %d\n", 
-			    (side == recvSide)?"recv":"send", s->name, rc));
-
-			FailFilterInfo[ix].id = filter.id;
-			FailFilterInfo[ix].side = side;
-			FailFilterInfo[ix].host = ntohl(s->host.s_addr);
-			FailFilterInfo[ix].used = 1;
-		    }
-		} 
-	    }
-
-    } while (k++ < nservers-1);
-
-    return(0);
-}
-
-/* 
- * Remove fail filters inserted by VIOC_SLOW or VIOC_DISCONNECT.
- * Filters inserted by other applications (ttyfcon, etc) are not
- * affected, since we don't have their IDs. If there is a problem 
- * removing a filter, we print a message in the log and forget 
- * about it. This allows the user to remove the filter using another
- * tool and not have to deal with leftover state in venus.
- */
-int FailReconnect(int nservers, struct in_addr *hostids)
-{
-    int rc, s = 0;
-
-    do {
-	for (int i = 0; i < MAXFILTERS; i++) 
-	    if (FailFilterInfo[i].used &&
-                (nservers == 0 ||
-                 (htonl(FailFilterInfo[i].host) == hostids[s].s_addr))) {
-                if ((rc = Fail_RemoveFilter(FailFilterInfo[i].side,
-                                            FailFilterInfo[i].id)) < 0) {
-		    LOG(0, ("FailReconnect: couldn't remove %s filter, id = %d\n", 
-			(FailFilterInfo[i].side == sendSide)?"send":"recv", 
-			FailFilterInfo[i].id));
-		} else {
-		    LOG(10, ("FailReconnect: removed %s filter, id = %d\n", 
-			(FailFilterInfo[i].side == sendSide)?"send":"recv", 
-			FailFilterInfo[i].id));
-		    FailFilterInfo[i].used = 0;
-		}
-            }
-    } while (s++ < nservers-1);
-
-    return(0);
-}
-
-/* 
- * Simulate a slow network. Insert filters with the
- * specified speed to all known servers.
- */
-int FailSlow(unsigned *speedp) {
-
-    return(-1);
-}
-#endif
-
