@@ -43,7 +43,6 @@ extern "C" {
 /* interfaces */
 #include <callback.h>
 #include <vice.h>
-#include <venus_adv.h>
 
 /* from util */
 #include <olist.h>
@@ -58,7 +57,6 @@ extern "C" {
 #include "fso.h"
 #include "venusrecov.h"
 #include "venus.private.h"
-#include "venusvol.h"
 #include "vproc.h"
 
 /* Forward declarations. */
@@ -70,6 +68,10 @@ class RepOpCommCtxt;
 class mgrpent;
 class probeslave;
 
+/* forward declarations for venusvol.h */
+class volent;
+class repvol;
+
 /* Bogus forward declarations to placate C++! */
 
 extern void ConnPrint();
@@ -78,10 +80,6 @@ extern void ConnPrint(int);
 extern void ServerPrint();
 extern void ServerPrint(FILE *);
 
-extern unsigned long WCThresh;
-
-
-
 /*  *****  Constants  *****  */
 
 const int DFLT_RT = 5;			    /* rpc2 retries */
@@ -122,7 +120,6 @@ const int DFLT_WCS  = 1800;		    /* 30 minutes */
  *  Servers:
  *
 */
-
 class connent {
   friend void CommInit();
   friend void Conn_Wait();
@@ -137,7 +134,7 @@ class connent {
   friend int GetTime(long *, long *);
   friend class vdb;
   friend class volent;
-  friend void repvol::Resolve(void);
+  friend class repvol;
   friend class ClientModifyLog;
   friend class cmlent;
 
@@ -213,7 +210,6 @@ class srvent : private RefCountedObject {
   friend void DownServers(char *, unsigned int *);
   friend void DownServers(int, struct in_addr *, char *, unsigned int *);
   friend void ServerPrint(FILE *);
-  friend long S_GetServerInformation(RPC2_Handle, RPC2_Integer, RPC2_Integer *, ServerEnt *);
   friend class srv_iterator;
   friend class connent;
   friend class mgrpent;
@@ -223,11 +219,10 @@ class srvent : private RefCountedObject {
   friend long VENUS_RevokeWBPermit(RPC2_Handle RPCid, VolumeId Vid);
   friend int FailDisconnect(int, struct in_addr *);
   friend int FailReconnect(int, struct in_addr *);
-  friend int FailSlow(unsigned *);
   friend class userent;
   friend class vproc;
   friend class fsobj;
-  friend void repvol::SetStagingServer(struct in_addr *srvr);
+  friend class repvol;
 
   friend class Realm;
   friend connent *conn_iterator::operator()();
@@ -246,10 +241,8 @@ class srvent : private RefCountedObject {
     RPC2_Handle	connid;		/* The callback connid. */
     unsigned Xbinding : 1;	/* 1 --> BINDING, 0 --> NOT_BINDING */
     unsigned probeme : 1;	/* should ProbeD probe this server? */
-    unsigned forcestrong : 1;	/* the user can force strong connectivity */
-    unsigned isweak : 1;	/* is this server considered weak */
+    unsigned unused : 1;
     unsigned long bw;		/* bandwidth estimate, Bytes/sec */
-    unsigned long bwmax;	/* max bandwidth estimate, Bytes/sec */
     struct timeval lastobs;	/* time of most recent estimate */
   
     /* Constructors, destructors, and private utility routines. */
@@ -278,14 +271,12 @@ class srvent : private RefCountedObject {
 
     long GetLiveness(struct timeval *);
     long GetBandwidth(unsigned long *);
-    void ForceStrong(int on);
     void Reset();
 
     void ServerError(int *);
     void ServerUp(RPC2_Handle);
     int	ServerIsDown() { return(connid == 0); }
     int ServerIsUp() { return(connid != 0); }
-    int ServerIsWeak() { return(connid > 0 && bw <= WCThresh); }
     /* quasi-up != up */
 
     const char *Name(void) { return name; }
@@ -362,7 +353,6 @@ void DownServers(int, struct in_addr *, char *, unsigned int *);
 void CheckServerBW(long);
 int FailDisconnect(int, struct in_addr *);
 int FailReconnect(int, struct in_addr *);
-int FailSlow(unsigned *);
 
 /* comm_daemon.c */
 extern void PROD_Init(void);
@@ -378,16 +368,17 @@ extern struct CommQueueStruct CommQueue;
 
 /*
  * The CommQueue summarizes outstanding RPC traffic for all threads.
- * Threads servicing requests for weakly connected volumes defer to 
- * higher priority threads before using the network.  Note that Venus
- * cannot determine the location of a network bottleneck.  Therefore,
- * it conservatively assumes that all high priority requests are 
- * sources of interference.   Synchronization could be finer, currently 
- * all waiters are awakened instead of the highest priority ones. 
+ * Threads servicing requests defer to higher priority threads before
+ * using the network. Note that Venus cannot determine the location of a
+ * network bottleneck. Therefore, it conservatively assumes that all
+ * high priority requests are sources of interference. Synchronization
+ * could be finer, currently all waiters are awakened instead of the
+ * highest priority ones.
  */
+#define COMM_YIELD 1
 #define START_COMMSYNC()\
 {   vproc *vp = VprocSelf();\
-    if (vp->u.u_vol && vp->u.u_vol->IsWeaklyConnected()) {\
+    if (COMM_YIELD) {\
 	int pri = LWP_MAX_PRIORITY;\
 	while (pri > vp->lwpri) {\
 	    if (CommQueue.count[pri]) { /* anyone bigger than me? */\
@@ -396,7 +387,7 @@ extern struct CommQueueStruct CommQueue;
 		START_TIMING();\
 		VprocWait(&CommQueue.sync);\
 		END_TIMING();\
-                LOG(0, ("WAIT OVER, elapsed = %3.1f\n", elapsed));\
+		LOG(0, ("WAIT OVER, elapsed = %3.1f\n", elapsed));\
 		pri = LWP_MAX_PRIORITY;\
 	    } else {\
 		pri--;\
