@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -34,8 +34,7 @@ class fsdb;
 class fsobj;
 class fso_iterator;
 
-class cmlent;			    /* we have compiler troubles if volume.h is included! */
-class lrdb;
+class cmlent;	    /* we have compiler troubles if volume.h is included! */
 
 #ifdef __cplusplus
 extern "C" {
@@ -59,7 +58,6 @@ extern int global_kernfd;
 /* interfaces */
 #include <user.h>
 #include <vice.h>
-#include <adv_skk.h>
 
 /* from util */
 #include <bstree.h>
@@ -107,22 +105,17 @@ const int UNSET_SSF = -1;
 
 const int CPSIZE = 8;
 
-/* Replica Control Rights. */
-/* Note that we presently do not distinguish between read and write rights. */
-/* We may well do so in the future, however. */
-#define	RC_STATUSREAD	1
-#define	RC_STATUSWRITE	2
-#define	RC_STATUS	(RC_STATUSREAD | RC_STATUSWRITE)
-#define	RC_DATAREAD	4
-#define	RC_DATAWRITE	8
-#define	RC_DATA		(RC_DATAREAD | RC_DATAWRITE)
-
-
 /*  *****  Types  ***** */
 /* Cache stuff was removed here to move to venus.private.h  5/14/92 */
 
 void FSODaemon(void); /* used to be member of class fsdb (Satya 3/31/95) */
 
+#define SERVER_SERVER   1
+#define LOCAL_GLOBAL    2
+#define MIXED_CONFLICT  3
+
+#define FILE_CONFLICT       1
+#define DIRECTORY_CONFLICT  2
 
 /* The (cached) file-system database. */
 class fsdb {
@@ -189,14 +182,14 @@ class fsdb {
 
     /* Constructors, destructors. */
     void *operator new(size_t);
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
     fsdb();
     void ResetTransient();
     ~fsdb() { abort(); }
 
     /* Allocation/Deallocation routines. */
-    fsobj *Create(VenusFid *, int, char *);
+    fsobj *Create(VenusFid *, int, const char *comp, VenusFid *parent);
     int FreeFsoCount();
     int AllocFso(int, fsobj **);
     int GrabFreeFso(int, fsobj **);
@@ -219,8 +212,9 @@ class fsdb {
   public:
     fsobj *Find(const VenusFid *);
     /* rcode arg added for local repair */
-    int Get(fsobj **fso, VenusFid *fid, uid_t uid, int rights, char *comp=NULL,
-	    int *rcode=NULL, int GetInconsistent=0);
+    int Get(fsobj **fso, VenusFid *fid, uid_t uid, int rights,
+	    const char *comp=NULL, VenusFid *parent=NULL, int *rcode=NULL,
+	    int GetInconsistent=0);
     void Put(fsobj **);
     void Flush();
     void Flush(Volid *);
@@ -297,6 +291,7 @@ class CacheFile {
     int  DecRef();             /* returns refcnt, unlinks if refcnt becomes 0 */
 
     void Stat(struct stat *);
+    void Utimes(const struct timeval times[2]);
     void Truncate(long);
     void SetLength(long);
     void SetValidData(long);
@@ -347,7 +342,9 @@ struct FsoFlags {
     unsigned local: 1;				/* local fake fid */
     /*T*/unsigned ckmtpt : 1;			/* mount point needs checked? */
     /*T*/unsigned fetching : 1;			/* fetch in progress? */
-    unsigned padding : 10;
+    unsigned expanded : 1;			/* are we an expanded object */
+    unsigned modified : 1;			/* modified for expansion? */
+    unsigned padding : 8;
 };
 
 enum MountStatus {  NORMAL,
@@ -396,7 +393,6 @@ class fsobj {
   friend class resent;
   friend class mgrpent;
   friend class hdb;
-  friend class lrdb;
   friend class Realm; /* ~Realm */
   friend void RecoverPathName(char *, VenusFid *, ClientModifyLog *, cmlent *);
 
@@ -444,9 +440,9 @@ class fsobj {
     /*T*/int HoardPri;				/* max of priorities of binders */
     /*T*/uid_t HoardVuid;			/* uid of that entry */
     /*T*/dlist *hdb_bindings;			/* list of (bindings to) hdbents referencing this object */
-    /*T*/int FetchAllowed;                      /* Allow a hoard walk to fetch object? */
-    /*T*/int AskingAllowed;                     /* Ask user for hoard fetch advice? */
-
+    /*T*/VnodeId  LocalFid_Vnode;		/* Values of the vnode and */
+    /*T*/Unique_t LocalFid_Unique;		/* uniquifier when this object
+						   had a local fid */
 
     /* MLE linkage. */
     /* T */dlist *mle_bindings;			/* list of (bindings to) mlents referencing this object */
@@ -479,9 +475,9 @@ class fsobj {
     void *operator new(size_t, fso_alloc_t, int); /* for allocation from freelist */
     void *operator new(size_t, fso_alloc_t); /* for allocation from heap */
     void *operator new(size_t); /* dummy to pacify g++ */
-    void operator delete(void *, size_t);
+    void operator delete(void *);
     fsobj(int);
-    fsobj(VenusFid *, char *);
+    fsobj(VenusFid *, const char *);
     void ResetPersistent();
     void ResetTransient();
     fsobj(fsobj&) { abort(); }                          /* not supported! */
@@ -526,32 +522,11 @@ class fsobj {
     void ComputePriority(int Force=0);
     void EnableReplacement();
     void DisableReplacement();
-    void AttachHdbBinding(binding *);
+    binding *AttachHdbBinding(namectxt *);
     void DemoteHdbBindings();
     void DemoteHdbBinding(binding *);
     void DetachHdbBindings();
     void DetachHdbBinding(binding *, int =0);
-    int PredetermineFetchState(int, int);
-    void SetFetchAllowed(HoardFetchState new_state)
-        { FetchAllowed = new_state; }
-    int IsFetchAllowed()
-        { if (FetchAllowed == HF_Fetch) 
-              return 1; 
-          else 
-              return 0; 
-        }
-    void SetAskingAllowed(HoardAskState new_state)
-        { AskingAllowed = new_state; }
-    int IsAskingAllowed()
-        { if (AskingAllowed == HA_Ask)
-            return 1;
-          else
-            return 0;
-        }
-
-    /* advice routines */
-    CacheMissAdvice ReadDisconnectedCacheMiss(vproc *, uid_t);
-    CacheMissAdvice WeaklyConnectedCacheMiss(vproc *, uid_t);
 
     /* MLE Linkage. */
     void AttachMleBinding(binding *);
@@ -567,12 +542,34 @@ class fsobj {
     int IsFakeMtPt() { return(flags.fake && IsMtPt()); }
     int IsFakeMTLink() { return(flags.fake && IsMTLink()); }
 
+    /* expansion related functions */
+    int ExpandObject(void);
+    int CollapseObject(void);
+    int IsExpandedObj(void) { return(flags.expanded); }
+    int IsExpandedDir(void) { return(flags.expanded && IsDir()); }
+    int IsExpandedMTLink(void) { return(flags.expanded && IsMTLink()); }
+    int IsModifiedObj(void) { return(flags.modified); }
+    void SetMtLinkContents(VenusFid *fid);
+
+    /* cmlent expansion related functions */
+    void ExpandCMLEntries(void);
+    void CollapseCMLEntries(void);
+    int HasExpandedCMLEntries(void);
+
+#define LOCALCACHE "_localcache"    /* implies we have a locally cached copy */
+#define LOCALCACHE_HIDDEN ".localcache" /* implies we don't */
+
     /* Local synchronization. */
     void Lock(LockLevel);
     void PromoteLock();
     void DemoteLock();
     void UnLock(LockLevel);
 
+    /* Local-global conflict detection */
+    int IsToBeRepaired(void);
+    uid_t WhoIsLastAuthor(void);
+    int LaunchASR(int, int);
+
     /* Interface to the dir package. */
     void dir_Create(const char *, VenusFid *);
     int dir_Length();
@@ -589,46 +586,32 @@ class fsobj {
 
     /* Private portions of the CFS interface. */
     void LocalStore(Date_t, unsigned long);
-    int ConnectedStore(Date_t, uid_t, unsigned long);
-    int DisconnectedStore(Date_t, uid_t, unsigned long, int);
+    int DisconnectedStore(Date_t, uid_t, unsigned long, int prepend=0);
     void LocalSetAttr(Date_t, unsigned long, Date_t,
 		       uid_t, unsigned short);
-    int ConnectedSetAttr(Date_t, uid_t, unsigned long, Date_t,
-			  uid_t, unsigned short, RPC2_CountedBS *);
     int DisconnectedSetAttr(Date_t, uid_t, unsigned long, Date_t,
-			     uid_t, unsigned short, int);
+			     uid_t, unsigned short, int prepend=0);
     void LocalCreate(Date_t, fsobj *, char *,
 		      uid_t, unsigned short);
-    int ConnectedCreate(Date_t, uid_t, fsobj **,
-			 char *, unsigned short, int);
     int DisconnectedCreate(Date_t, uid_t, fsobj **,
-			    char *, unsigned short, int, int);
+			    char *, unsigned short, int, int prepend=0);
     void LocalRemove(Date_t, char *, fsobj *);
-    int ConnectedRemove(Date_t, uid_t, char *, fsobj *);
-    int DisconnectedRemove(Date_t, uid_t, char *, fsobj *, int);
+    int DisconnectedRemove(Date_t, uid_t, char *, fsobj *, int prepend=0);
     void LocalLink(Date_t, char *, fsobj *);
-    int ConnectedLink(Date_t, uid_t, char *, fsobj *);
-    int DisconnectedLink(Date_t, uid_t, char *, fsobj *, int);
+    int DisconnectedLink(Date_t, uid_t, char *, fsobj *, int prepend=0);
     void LocalRename(Date_t, fsobj *, char *,
 		      fsobj *, char *, fsobj *);
-    int ConnectedRename(Date_t, uid_t, fsobj *,
-			 char *, fsobj *, char *, fsobj *);
     int DisconnectedRename(Date_t, uid_t, fsobj *,
-			    char *, fsobj *, char *, fsobj *, int);
+			    char *, fsobj *, char *, fsobj *, int prepend=0);
     void LocalMkdir(Date_t, fsobj *, char *, uid_t, unsigned short);
-    int ConnectedMkdir(Date_t, uid_t, fsobj **,
-			char *, unsigned short, int);
     int DisconnectedMkdir(Date_t, uid_t, fsobj **,
-			   char *, unsigned short, int, int);
+			   char *, unsigned short, int, int prepend=0);
     void LocalRmdir(Date_t, char *, fsobj *);
-    int ConnectedRmdir(Date_t, uid_t, char *, fsobj *);
-    int DisconnectedRmdir(Date_t, uid_t, char *, fsobj *, int);
+    int DisconnectedRmdir(Date_t, uid_t, char *, fsobj *, int prepend=0);
     void LocalSymlink(Date_t, fsobj *, char *,
 		       char *, uid_t, unsigned short);
-    int ConnectedSymlink(Date_t, uid_t, fsobj **, char *,
-			  char *, unsigned short, int);
     int DisconnectedSymlink(Date_t, uid_t, fsobj **, char *,
-			     char *, unsigned short, int, int);
+			     char *, unsigned short, int, int prepend=0);
     int GetContainerFD(void);
     int LookAside(void);
 
@@ -638,7 +621,7 @@ class fsobj {
     int GetAttr(uid_t, RPC2_BoundedBS * =0);
     int GetACL(RPC2_BoundedBS *, uid_t);
     int Store(unsigned long, Date_t, uid_t);
-    int SetAttr(struct coda_vattr *, uid_t, RPC2_CountedBS * =0);
+    int SetAttr(struct coda_vattr *, uid_t);
     int SetACL(RPC2_CountedBS *, uid_t);
     int Create(char *, fsobj **, uid_t, unsigned short, int);
     int Remove(char *, fsobj *, uid_t);
@@ -655,8 +638,8 @@ class fsobj {
     void Release(int writep);
     int Close(int writep, uid_t uid);
     int Access(int rights, int modes, uid_t);
-    int Lookup(fsobj **, VenusFid *, char *, uid_t, int flags);
-// These are defined in lib-src/kernel-includes/coda.h
+    int Lookup(fsobj **, VenusFid *, const char *, uid_t, int flags, int GetInconsistent=0);
+// These are defined in coda-src/kerndep/coda.h
 // #define CLU_CASE_SENSITIVE	0x01
 // #define CLU_CASE_INSENSITIVE 0x02
 #define CLU_CASE_MASK		0x03
@@ -665,7 +648,7 @@ class fsobj {
     int Readlink(char *, unsigned long, int *, uid_t);
 
     /* Miscellaneous utility routines. */
-    int dir_Lookup(char *, VenusFid *, int);
+    int dir_Lookup(const char *, VenusFid *, int);
     int CheckAcRights(uid_t uid, long rights, int connected);
     void GetVattr(struct coda_vattr *);		/* translate attributes to VFS format */
     void GetFid(VenusFid *f) { *f = fid; }
@@ -686,15 +669,13 @@ class fsobj {
     int	IsMtPt() { return(mvstat == MOUNTPOINT); }      /* covered mount point */
     int	IsMTLink() { return(IsSymLink() && stat.Mode == 0644 && IsNormal()); }
                                                         /* uncovered mount point */
-    int IsBackFetching();			/* fso involved in an ongoing reintegration */
+    int IsVirgin();        /* file which has been created, but not yet stored */
+    int IsBackFetching();  /* fso involved in an ongoing reintegration */
     int SetLastResolved(long t) { lastresolved = t; return(0); }
     int  MakeShadow();
     void RemoveShadow();
     void CacheReport(int, int);
 
-    int /*(secs)*/ EstimatedFetchCost(int =1);  /* 0 = status; 1 = data (default) */
-    void RecordReplacement(int, int);
-
     void print() { print(stdout); }
     void print(FILE *fp) { fflush(fp); print(fileno(fp)); }
     void print(int);
@@ -704,25 +685,19 @@ class fsobj {
     void ListCacheLong(FILE *);
 
     /* local-repair additions */
-    void GetOperationState(int *, int *);                       /*N*/
     cmlent *FinalCmlent(int);                                   /*N*/
-    void SetComp(char *);                                       /*U*/
+    void SetComp(const char *);                                 /*U*/
     const char *GetComp(void);
     void SetLocalObj();						/*T*/
     void UnsetLocalObj();					/*T*/
     int IsLocalObj() { return flags.local; }			/*N*/
     int IsAncestor(VenusFid *);					/*N*/
-    int ReplaceLocalFakeFid();					/*U*/
-    int LocalFakeify();						/*U*/
-    int LocalFakeifyRoot();					/*U*/
 
-    void MixedToGlobal(VenusFid *, VenusFid *, char *);        	/*U*/
-    void MixedToLocal(VenusFid *, VenusFid *, char *);        	/*U*/
-    void GlobalToMixed(VenusFid *, VenusFid *, char *);        	/*U*/
-    void LocalToMixed(VenusFid *, VenusFid *, char *);        	/*U*/
     void DeLocalRootParent(fsobj *, VenusFid *, fsobj *);	/*U*/
     void RecoverRootParent(VenusFid *, char *);			/*U*/
 
+    int SetLocalVV(ViceVersionVector *);
+
     int RepairStore();
     int RepairSetAttr(unsigned long, Date_t, uid_t, unsigned short, RPC2_CountedBS *);
     int RepairCreate(fsobj **, char *, unsigned short, int);
@@ -734,6 +709,8 @@ class fsobj {
     int RepairSymlink(fsobj **, char *, char *, unsigned short, int);
 
     void FetchProgressIndicator(unsigned long offset);
+
+    size_t Size(void) { return stat.Length; }
 };
 
 class fso_iterator : public rec_ohashtab_iterator {
@@ -759,7 +736,7 @@ extern int FSO_SSF;
 extern void FSOInit();
 extern int FSO_PriorityFN(bsnode *, bsnode *);
 extern void UpdateCacheStats(CacheStats *c, enum CacheEvent event, unsigned long blocks);
-extern void PrintCacheStats(char* description, CacheStats *, int);
+extern void PrintCacheStats(const char* description, CacheStats *, int);
 extern void VenusToViceStatus(VenusStat *, ViceStatus *);
 
 /* fso_daemon.c */
@@ -771,10 +748,9 @@ void FSOD_ReclaimFSOs(void);
 #define FSO_RELE(f)	    { (f)->refcnt--; }
 
 /* Some useful state predicates. */
-#define	HOARDING(f)	((f)->vol->state == Hoarding)
-#define	EMULATING(f)	((f)->vol->state == Emulating)
-#define LOGGING(f)      ((f)->vol->state == Logging)
-#define RESOLVING(f)    ((f)->vol->state == Resolving)
+#define UNREACHABLE(f)	((f)->vol->IsUnreachable())
+#define REACHABLE(f)	((f)->vol->IsReachable())
+#define RESOLVING(f)	((f)->vol->IsResolving())
 #define	DIRTY(f)	((f)->flags.dirty)
 #define	HAVESTATUS(f)	((f)->state != FsoRunt)
 #define	STATUSVALID(f)	((f)->IsValid(RC_STATUS))
@@ -789,12 +765,11 @@ void FSOD_ReclaimFSOs(void);
 #define	READING(f)	(((f)->openers - (f)->Writers) > 0)
 #define	WRITING(f)	((f)->Writers > 0)
 #define	EXECUTING(f)	(EXECUTABLE(f) && READING(f) && !k_Purge(&(f)->fid))
-#define	ACTIVE(f)	(WRITING(f) || EXECUTING(f))
+#define	ACTIVE(f)	(WRITING(f) || READING(f)) // was EXECUTING(f)
 #define	BUSY(f)		((f)->refcnt > 0 || EXECUTING(f))
 #define	HOARDABLE(f)	((f)->HoardPri > 0)
-#define	FETCHABLE(f)	(!DYING(f) &&\
-			 (HOARDING(f) || (LOGGING(f) && !DIRTY(f))) &&\
-			 (!HAVESTATUS(f) || !ACTIVE(f)))
+#define	FETCHABLE(f)	(!DYING(f) && REACHABLE(f) && !DIRTY(f) && \
+			 (!HAVESTATUS(f) || !ACTIVE(f)) && !f->IsLocalObj())
 /* we are replaceable whenever we are linked into FSDB->prioq */
 #define	REPLACEABLE(f)	((f)->prio_handle.tree() != 0)
 #define	GCABLE(f)	(DYING(f) && !DIRTY(f) && !BUSY(f))
