@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -39,10 +39,10 @@ extern "C" {
 #include <sys/stat.h>
 #include <struct.h>
 #include <sys/param.h>
+#include <sys/wait.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <dirent.h>
-#include <netdb.h>
 
 #ifdef __cplusplus
 }
@@ -52,9 +52,6 @@ extern "C" {
 #include <vcrcommon.h>
 
 /* from venus */
-#include "advice.h"
-#include "adv_monitor.h"
-#include "adv_daemon.h"
 #include "fso.h"
 #include "hdb.h"
 #include "local.h"
@@ -71,6 +68,7 @@ int FSO_SWT = UNSET_SWT;
 int FSO_MWT = UNSET_MWT;
 int FSO_SSF = UNSET_SSF;
 
+
 /* Call with CacheDir the current directory. */
 void FSOInit() {
     int i; 
@@ -129,54 +127,6 @@ void FSOInit() {
 	    }
 	}
 
-	/* Get rid of garbage in cache dir.
-	   The ifdef's are necessary because the directory operations
-	   have changed completely between 4.3BSD and 4.4BSD  (Satya, 8/12/96)
-        */
-#ifdef __BSD44__
-	DIR *dir;
-	struct dirent *entry;
-
-	dir = opendir(".");
-	if (!dir) CHOKE("FSOInit: opendir");
-
-	char *cwd,*abspath,*eos,*origpath;
-	struct stat statbuf;
-
-	cwd = getwd(NULL);
-	abspath = (char*)malloc(sizeof(char)*MAXPATHLEN);
-	CODA_ASSERT(abspath != NULL);
-
-	abspath = strncpy(abspath, cwd, MAXPATHLEN);
-	abspath = strncat(abspath,"/",2);
-	eos = abspath + strlen(abspath) + 1;
-
-	/* Examine each entry and decide to keep or delete it */
-	while ((entry = readdir(dir)) != NULL)
-	{
-	    *eos = '\0';
-	    /* Don't unlink special files. */
-	    strncpy(eos, entry->d_name, MAXPATHLEN-strlen(cwd));
-	    if (STREQ(entry->d_name, ".") ||
-		STREQ(entry->d_name, "..") ||
-		STREQ(entry->d_name, "lost+found") ||
-		STREQ(abspath, VenusLogFile) ||
-		STREQ(entry->d_name, "pid"))
-		continue;
-
-	    /* Don't unlink cache directories. */
-	    if (stat(entry->d_name, &statbuf) == 0 &&
-		S_ISDIR(statbuf.st_mode))
-		continue;
-
-	    /* Garbage collect everything else. */
-	    ::unlink(entry->d_name);
-	}
-	free(abspath);
-	closedir(dir);
-	free(cwd);
-#endif /* __BSD44__ */
-
 	/* Allocate the fsobj's if requested. */
 	if (InitMetaData) {
 	    /* Do this in a loop to avoid one mongo transaction! */
@@ -254,7 +204,11 @@ void FSOInit() {
 	    fso_iterator next(NL);
 	    fsobj *cf;
 	    while ((cf = next()))
-		cf->SetParent(cf->pfid.Vnode, cf->pfid.Unique);
+		if (!cf->IsExpandedObj())
+		    cf->SetParent(cf->pfid.Vnode, cf->pfid.Unique);
+		else
+		    /* expanded objects need to be pinned down */
+		    FSO_HOLD(cf);
 	}
 
 	/* Recover fsobj <--> cmlent bindings: a grid-like data structure. */
@@ -337,7 +291,8 @@ void UpdateCacheStats(CacheStats *c, enum CacheEvent event, unsigned long blocks
 }
 
 
-void PrintCacheStats(char *description, CacheStats *c, int fd) {
+void PrintCacheStats(const char *description, CacheStats *c, int fd)
+{
     fdprint(fd, "%s ", description);
     for (int i = 0/*HIT*/; i < 10/*REPLACE*/; i++)
 	fdprint(fd, "[%4d : %6d]  ", c->events[i].count, c->events[i].blocks);
@@ -426,26 +381,49 @@ void fsdb::ResetTransient() {
 }
 
 
-void fsdb::operator delete(void *, size_t){
+void fsdb::operator delete(void *){
     abort(); /* how could we possibly get here? */
 }
 
 
 fsobj *fsdb::Find(const VenusFid *key)
 {
+    VenusFid OldFid;
+
     fso_iterator next(NL, key);
     fsobj *f;
     while ((f = next()))
-	if (FID_EQ(key, &f->fid)) return(f);
+	if (FID_EQ(key, &f->fid))
+	    return f;
 
-    return(0);
+    /* If we were looking for a local fid, do a full search because we may have
+     * translated it while the upcall was in transit. */
+    OldFid = *key;
+    if (FID_IsDisco(MakeViceFid(&OldFid)))
+    {
+	fso_iterator full_search(NL);
+	while ((f = full_search()))
+	{
+	    OldFid.Realm  = f->fid.Realm;
+	    OldFid.Volume = f->fid.Volume;
+	    OldFid.Vnode  = f->LocalFid_Vnode;
+	    OldFid.Unique = f->LocalFid_Unique;
+
+	    if (FID_EQ(key, &OldFid)) {
+		k_Replace(&OldFid, &f->fid);
+		return f;
+	    }
+	}
+    }
+    return 0;
 }
 
 
 /* MUST NOT be called from within transaction! */
 /* Caller MUST guarantee that the volume is cached and stable! */
 /* Should priority be an implicit argument? -JJK */
-fsobj *fsdb::Create(VenusFid *key, int priority, char *comp)
+fsobj *fsdb::Create(VenusFid *key, int priority, const char *comp,
+		    VenusFid *parent)
 {
     fsobj *f = 0;
     int rc = 0;
@@ -464,10 +442,32 @@ fsobj *fsdb::Create(VenusFid *key, int priority, char *comp)
 	FSDB->FreeFso(f);
         f = new (FROMFREELIST, priority) fsobj(key, comp);
     }
+
+    if (f && !f->IsRoot()) {
+	if (!parent) {
+	    /* Laboriously scan database to find our parent! */
+	    struct dllist_head *p;
+	    list_for_each(p, f->vol->fso_list) {
+		fsobj *pf = list_entry_plusplus(p, fsobj, vol_handle);
+
+		if (!pf->IsDir() || pf->IsMtPt()) continue;
+		if (!HAVEALLDATA(pf)) continue;
+		if (!pf->dir_IsParent(&f->fid)) continue;
+
+		parent = &pf->fid;
+		break; /* Found! */
+	    }
+	}
+	if (parent)
+	    f->SetParent(parent->Vnode, parent->Unique);
+    }
     Recov_EndTrans(MAXFP);
 
     if (!f)
 	LOG(0, ("fsdb::Create: (%s, %d) failed\n", FID_(key), priority));
+    else
+	LOG(100, ("fsdb::Create: (%s, %d) suceeeded\n", FID_(key), priority));
+
     return(f);
 }
 
@@ -494,44 +494,23 @@ fsobj *fsdb::Create(VenusFid *key, int priority, char *comp)
  * poor user can do about it.
  */
 int fsdb::Get(fsobj **f_addr, VenusFid *key, uid_t uid, int rights,
-	      char *comp, int *rcode, int GetInconsistent)
+	      const char *comp, VenusFid *parent, int *rcode,
+	      int GetInconsistent)
 {
     int getdata = (rights & RC_DATA);
-
-    LOG(100, ("fsdb::Get-mre: key = (%s), uid = %d, rights = %d, comp = %s\n",
-	       FID_(key), uid, rights, comp));
-
-    { 	/* a special check for accessing already localized object */
-	volent *vol = VDB->Find(MakeVolid(key));
-	if (vol && vol->IsReplicated()) {
-	    repvol *vp = (repvol *)vol;
-	    if (!vp->IsUnderRepair(ANYUSER_UID) && vp->HasLocalSubtree()) {
-		lgm_iterator next(LRDB->local_global_map);
-		lgment *lgm;
-		VenusFid *gfid;
-		while ((lgm = next())) {
-		    gfid = lgm->GetGlobalFid();
-		    if (FID_EQ(gfid, key)) {
-			LOG(0, ("fsdb::Get: trying to access localized object %s\n",
-				FID_(key)));
-			return EACCES;
-		    }
-		}
-	    }
-	}
-	VDB->Put(&vol);
-    }
-
     int code = 0;
-    *f_addr = 0;				/* OUT parameter valid on success only. */
+    *f_addr = 0;		     /* OUT parameter valid on success only. */
     vproc *vp = VprocSelf();
 
+    LOG(100, ("fsdb::Get: key = (%s), uid = %d, rights = %d, comp = %s\n",
+	       FID_(key), uid, rights, comp));
+
     /* if (vp->type != VPT_HDBDaemon)
      *  NotifyUserOfProgramAccess(uid, vp->u.u_pid, vp->u.u_pgid, key); */
 
     /* Volume state synchronization. */
     /* If a thread is already "in" one volume, we must switch contexts before entering another. */
-    if (vp->u.u_vol && 
+    if (vp->u.u_vol &&
 	!(vp->u.u_vol->GetRealmId() == key->Realm &&
 	  vp->u.u_vol->GetVolumeId() == key->Volume))
     {
@@ -542,19 +521,22 @@ int fsdb::Get(fsobj **f_addr, VenusFid *key, uid_t uid, int rights,
 	vp->u.u_priority = saved_ctxt.u_priority;
 	vp->u.u_flags = saved_ctxt.u_flags;
 	vp->u.u_pid = saved_ctxt.u_pid;
-        vp->u.u_pgid = saved_ctxt.u_pgid;
+	vp->u.u_pgid = saved_ctxt.u_pgid;
 
 	/* Do the Get on behalf of another volume. */
 	for (;;) {
+	    int retry_call;
+
 	    vp->Begin_VFS(MakeVolid(key), CODA_VGET);
 	    if (vp->u.u_error) break;
 
-	    vp->u.u_error = Get(f_addr, key, uid, rights, comp, rcode,
+	    vp->u.u_error = Get(f_addr, key, uid, rights, comp, parent, rcode,
 				GetInconsistent);
 
 	    if (vp->u.u_error != 0)
 		Put(f_addr);
-	    int retry_call = 0;
+
+	    retry_call = 0;
 	    vp->End_VFS(&retry_call);
 	    if (!retry_call) break;
 	}
@@ -572,7 +554,18 @@ int fsdb::Get(fsobj **f_addr, VenusFid *key, uid_t uid, int rights,
     /* Find the fsobj, or create a fresh one. */
 RestartFind:
     f = Find(key);
+
     if (f == NULL) {
+
+        /* if it's in the local realm and the repair vol, and is a fake root,
+	   it must have left a dangling reference during a collapse */
+        if (FID_IsExpandedDir(key)) {
+
+	    LOG(0, ("Failed to get (%s), probably a collapsed expansion dir!\n", FID_(key)));
+
+	    return(EIO);
+	  }
+
 	/* 
 	 * check if the key is a locally generated fid.  We should never send
 	 * these to the server.  This check is not to be confused with
@@ -580,39 +573,39 @@ RestartFind:
 	 * the object belongs to is the local volume.  yuck.  --lily
 	 */
 	if (FID_IsDisco(MakeViceFid(key))) {
-		LOG(0, ("fsdb::Get: Locally created fid %s not found!\n", 
-			FID_(key)));
-		return ETIMEDOUT;
+	  LOG(0, ("fsdb::Get: Locally created fid %s not found!\n", 
+			  FID_(key)));
+	  return ETIMEDOUT;
 	}
-
-        /* Must ensure that the volume is cached. */
-        volent *v = 0;
-        if (VDB->Get(&v, MakeVolid(key))) {
-            LOG(100, ("Volume not cached and we couldn't get it...\n"));
-            return(ETIMEDOUT);
-        }
-
+	
+	/* Must ensure that the volume is cached. */
+	volent *v = 0;
+	if (VDB->Get(&v, MakeVolid(key))) {
+	  LOG(100, ("Volume not cached and we couldn't get it...\n"));
+	  return(ETIMEDOUT);
+	}
+	
 	/* Retry the find, in case some other thread created the object while we blocked in vdb::Get(). */
 	if (Find(key)) {
-	    VDB->Put(&v);
-	    goto RestartFind;
+	  VDB->Put(&v);
+	  goto RestartFind;
+	}
+	
+	if (v->IsResolving()) {
+	  LOG(0, ("Volume resolving and file not cached, retrying VDB->Get!\n"));
+	  VDB->Put(&v);
+	  return(ERETRY);
+	}
+	
+	/* Cut-out early if volume is unreachable! */
+	if (v->IsUnreachable()) {
+	  LOG(100, ("Volume unreachable and file not cached!\n"));
+	  VDB->Put(&v);
+	  return(ETIMEDOUT);
 	}
 
-        if (v->state == Resolving) {
-            LOG(0, ("Volume resolving and file not cached, retrying VDB->Get!\n"));
-            VDB->Put(&v);
-	    return(ERETRY);
-        }
-
-        /* Cut-out early if volume is disconnected! */
-        if (v->state == Emulating) {
-            LOG(100, ("Volume disconnected and file not cached!\n"));
-            VDB->Put(&v);
-            return(ETIMEDOUT);
-        }
-
 	/* Attempt the create. */
-	f = Create(key, vp->u.u_priority, comp);
+	f = Create(key, vp->u.u_priority, comp, parent);
 
 	/* Release the volume. */
 	VDB->Put(&v);
@@ -620,42 +613,45 @@ RestartFind:
 	if (!f)
 	    return(ENOSPC);
 
-	/* Transform object into fake mtpt if necessary. */
+	/* Transform object into fake mtpt if necessary (for /coda) */
+	/* The first clause catches the /coda root, the second catches
+	   realms as they are demand loaded */
 	if (FID_IsLocalFake(key) || FID_IsFakeRoot(MakeViceFid(key))) {
-	    if (f->Fakeify()) {
+	  LOG(0, ("fsdb::Get: transforming %s (%s) into fake mtpt with Fakeify()\n", f->GetComp(), FID_(&f->fid)));
+	  if (f->Fakeify()) {
 		LOG(0, ("fsdb::Get: can't transform %s (%s) into fake mt pt\n",
-			f->GetComp(), FID_(&f->fid)));
+				f->GetComp(), FID_(&f->fid)));
 		Recov_BeginTrans();
 		f->Kill();
 		Recov_EndTrans(MAXFP);
 		Put(&f);  		 /* will unlock and garbage collect */
 		return(EIO);
-	    }
+	  }
 	}
 	f->DemoteLock();
-    }
-    else {
+  }
+  else {
 	/* Object without status must be matriculating now.  Wait for it to complete. */
 	int curr_matriculation_count = matriculation_count;
-	if (!HAVESTATUS(f)) {
-	    while (curr_matriculation_count == matriculation_count) {
+	if (!HAVESTATUS(f) && !f->IsFake()) {
+	  while (curr_matriculation_count == matriculation_count) {
 		LOG(0, ("WAITING(MATRICULATION): count = %d\n", matriculation_count));
 		START_TIMING();
 		VprocWait(&matriculation_sync);
 		END_TIMING();
 		LOG(0, ("WAIT OVER, elapsed = %3.1f\n", elapsed));
-	    }
-	    goto RestartFind;
+	  }
+	  goto RestartFind;
 	}
-
+	
 	/* Perform GC if necessary. */
 	if (GCABLE(f)) {
-	    Recov_BeginTrans();
-	    f->GC();
-	    Recov_EndTrans(MAXFP);
-	    goto RestartFind;
+	  Recov_BeginTrans();
+	  f->GC();
+	  Recov_EndTrans(MAXFP);
+	  goto RestartFind;
 	}
-
+	
 	/* Read-lock the entry. */
 	f->Lock(RD);
 
@@ -667,17 +663,23 @@ RestartFind:
 	    f->SetComp(comp);
 	    Recov_EndTrans(MAXFP);
 	}
+
+	/* Update parent linkage */
+	if (parent && !f->pfso) {
+	    Recov_BeginTrans();
+	    f->SetParent(parent->Vnode, parent->Unique);
+	    Recov_EndTrans(MAXFP);
+	}
     }
 
     /* Consider fetching status and/or data. */
-    if (!f->IsLocalObj() &&
-	((!getdata && !STATUSVALID(f)) || (getdata && !DATAVALID(f)))) {
+    if ((!getdata && !STATUSVALID(f)) || (getdata && !DATAVALID(f))) {
 	/* Note that we CANNOT fetch, and must use whatever status/data we have, if : */
 	/*     - the file is being exec'ed (or the VM system refuses to release its pages) */
 	/*     - the file is open for write */
 	/*     - the object has been deleted (it must also be open for read at this point) */
-	/*     - the object's volume is disconnected */
- 	/*     - the object's volume is in logging mode and the object is dirty */
+	/*     - the object's volume is unreachable */
+	/*     - the object's volume is reachable, but the object is dirty */
 	if (FETCHABLE(f)) {
 	    f->PromoteLock();
 
@@ -688,295 +690,247 @@ RestartFind:
 		code = f->GetAttr(uid);
 
 		if (rcode) *rcode = code;	/* added for local-repair */
-		/* Conjure a fake directory to represent an inconsistent object. */
-		if (code == EINCONS) { 
-		    LOG(0, ("fsdb::Get: Object inconsistent. (key = <%s>)\n", 
-			    FID_(key)));
-#if 0
-		    userent *u;
-		    char path[MAXPATHLEN];
-		    
-		    f->GetPath(path,1);
-		    GetUser(&u, f->vol->realm, uid);
-		    CODA_ASSERT(u != NULL);
-		    PutUser(&u);
 
-		    /* We notify all users that objects are in conflict because
-		     * it is often the case that uid=-1, so we notify nobody.
-		     * It'd be better if we could notify the user whose
-		     * activities triggered this object to go inconsistent.
-		     * However, that person is difficult to determine and could
-		     * be the hoard daemon.  Notifying everyone seems to be a
-		     * reasonable alternative, if not terribly satisfying. */
-		    /* NotifyUsersObjectInConflict(path, key); */
-#endif
+		/* Mark fsobj in server/server conflict */
+		if (code == EINCONS && !f->IsFake()) {
+		  char path[MAXPATHLEN];
+		  f->GetPath(path, PATH_FULL);
 
-		    k_Purge(&f->fid, 1);
-                    if (f->refcnt > 1) {
-		        /* If refcnt is greater than 1, it means we aren't the
-			 * only one with an active reference to this object.
-			 * If this is the case, then the following Put cannot
-			 * possibly clear all the references to this file.  If
-			 * we were to go ahead and call the Create in this
-			 * situation, we'd get a fatal error ("Create: key
-			 * found"). So, we return ETOOMANYREFS and put an
-			 * informative message in the log rather than allowing
-			 * the fatal error. */
-			f->ClearRcRights();
-			Put(&f);
-			LOG(0, ("fsdb::Get: Object with active reference has gone inconsistent.\n\t Cannot conjure fake directory until object is inactive. (key =  <%s>)\n", FID_(key)));
-			return(ETOOMANYREFS);
-		    }
+		  Recov_BeginTrans();
+		  RVMLIB_REC_OBJECT(f->flags);
+		  f->flags.fake = 1;
+		  Recov_EndTrans(MAXFP);
 
-		    Put(&f);
-		    code = 0;
+		  k_Purge(&f->fid, 0);
 
-		    /* Attempt the create. */
-                    /* N.B. The volume should be explicitly pinned here! 
-                          XXX mre 10/21/94 The volume is pinned prior to fsdb::Get call */
-		    /* N.B. If preceding PUT didn't clear all references,
-		       and if the refcnt test above didn't catch that
-		       the Put wouldn't, we're hosed!  We'll most likely
-		       get a "Create: key found" fatal error. */
-		    f = Create(key, vp->u.u_priority, comp);
-		    if (!f)
-			return(ENOSPC);
+		  LOG(0, ("fsdb::Get: %s (%s) in server/server conflict\n",
+			  path, FID_(key)));
+		  MarinerLog("fsobj::CONFLICT (server/server): %s (%s)\n",
+			     path, FID_(key));
+		} /* s/s conflict objs fall through if(GetInconsistent) */
 
-		    /* 
-		     * Transform object into fake directory.  If that doesn't
-		     * work, return EIO...NOT EINCONS, which will get passed
-		     * back to the user as ENOENT (too alarming).  We must kill
-		     * the object here, otherwise Venus will think it is
-		     * "matriculating" and wait (forever) for it to finish.
-		     */
-		    eprint("%s (%s) inconsistent!", f->GetComp(), FID_(&f->fid));
-		    if (f->Fakeify()) {
-			Recov_BeginTrans();
-			f->Kill();
-			Recov_EndTrans(MAXFP);
-			Put(&f);
-			return(EIO);
-		    }
+		if (code) {
+		    /* wakeup threads that may still be stuck waiting for
+		     * matriculation of this object */
+		    matriculation_count++;
+		    VprocSignal(&matriculation_sync);
 		}
 
-		if (code != 0) {
-                    if (code == ETIMEDOUT)
-                      LOG(100, ("(MARIA) Code is TIMEDOUT after GetAttr...\n"));
-
+		if (code && !(code == EINCONS && GetInconsistent)) {
+		    if (code == EINCONS)
+			LOG(0, ("fsdb::Get: EINCONS after GetAttr\n"));
+		    if (code == ETIMEDOUT)
+			LOG(100, ("fsdb::Get: TIMEDOUT after GetAttr\n"));
 		    Put(&f);
 		    return(code);
 		}
-	   } 
+	   }
 
 	    /* If we want data and we don't have any then fetch new stuff. */
 	    /* we have to re-check FETCHABLE because it may have changed as
 	       a result of the inconsistent object manipulation above. */
-	    if (getdata && FETCHABLE(f) && !HAVEALLDATA(f)) {
-                /* Turn off advice effects for the time being  -Remi 
-                   CacheMissAdvice advice = CoerceToMiss; */
-                CacheMissAdvice advice = FetchFromServers;
-
-                if (f->vol->IsWeaklyConnected()) {
-                    char pathname[MAXPATHLEN];
-                    int hoard_priority = 0;
-
-                    if (f->HoardPri > 0)
-                        hoard_priority = f->HoardPri;
-                    else {
-			f->GetPath(pathname);
-                        hoard_priority = HDB->GetSuspectPriority(MakeVolid(&f->fid), pathname, uid);
-		    }
-
-                    int estimatedCost = f->EstimatedFetchCost();
-                    /* If the fetch will take too long, coerce the request into a miss */
-                    if (f->PredetermineFetchState(estimatedCost, hoard_priority) != 1) {
-                        advice = f->WeaklyConnectedCacheMiss(vp, uid);
-                        if (advice == CoerceToMiss) {
-                            Put(&f);
-                            LOG(0, ("Weak Miss Coersion:\n\tObject:  %s <%s>\n\tEstimated Fetch Cost:  %d seconds\n\tReturn code:  EFBIG\n", 
-                                    comp, FID_(key), estimatedCost));
-                            MarinerLog("Weak Miss Coersion on %s <%s>\n",
-                                       comp, FID_(key));
-                            return(EFBIG);
-                        }
-                    }
-                    /* Otherwise, let fsdb::Get go ahead and fetch the object */
-                }
-
-                int nblocks = BLOCKS(f);
-
-                /* If we haven't got any data yet, allocate enough for the
-                 * whole file. When we have a partial file, we should
-                 * already have reserved enough blocks. */
-                if (f->IsFile() && !HAVEDATA(f)) {
-                    code = AllocBlocks(vp->u.u_priority, nblocks);
-                    if (code != 0) {
-                        Put(&f);
-                        return(code);
-                    }
-                }
-
-                /* Make cache misses non-transparent. */
-                if (advice == CoerceToMiss)
-                    advice = f->ReadDisconnectedCacheMiss(vp, uid);
-                switch (advice) {
-                case FetchFromServers:
-                    LOG(10, ("The advice was to ReadDiscFetch --> Fetching.\n"));
-                    break;
-                case CoerceToMiss:
-                    LOG(0, ("Read Disconnected Miss Coersion:\n\tObject:  %s <%s>\n\tReturn code:  EFBIG\n", 
-                            comp, FID_(key)));
-                    MarinerLog("Read Disconnected Miss Coersion on %s <%s>\n",
-                               comp, FID_(key));
-                    /* We have to release any previously allocated
-                     * cachespace */
-                    FreeBlocks(-nblocks);
-                    Put(&f);
-                    return(ETIMEDOUT);
-                default:
-                    LOG(0, ("The advice was Unrecognized --> Fetching anyway.\n"));
-                    break;
-                }
-
-                /* compensate # blocks for the amount we already have.
-                 * (only used for vmon statistical stuff later on, but
-                 * the fetch will modify f->cf.ValidData) */
-                nblocks -= NBLOCKS(f->cf.ValidData());
+	    if (getdata && FETCHABLE(f) && !f->IsFake() && !HAVEALLDATA(f))
+	    {
+		int nblocks = BLOCKS(f);
 
+		/* If we haven't got any data yet, allocate enough for the
+		 * whole file. When we have a partial file, we should
+		 * already have reserved enough blocks. */
+		if (f->IsFile() && !HAVEDATA(f)) {
+		  code = AllocBlocks(vp->u.u_priority, nblocks);
+		  if (code != 0) {
+			Put(&f);
+			return(code);
+		  }
+		}
+		
+		/* compensate # blocks for the amount we already have.
+		 * (only used for vmon statistical stuff later on, but
+		 * the fetch will modify f->cf.ValidData) */
+		nblocks -= NBLOCKS(f->cf.ValidData());
+		
 		code = 0;
 		/* first try the LookAside cache */
 		if (!f->LookAside()) {
-		    /* Let fsobj::Fetch go ahead and fetch the object */
-		    code = f->Fetch(uid);
+		  /* Let fsobj::Fetch go ahead and fetch the object */
+		  code = f->Fetch(uid);
 		}
+		
+		/* Restart operation in case of inconsistency. */
+		if (code == EINCONS)
+		  code = ERETRY;
 
-                /* Restart operation in case of inconsistency. */
-                if (code == EINCONS)
-                    code = ERETRY;
+		if (code != 0) {
+		  Put(&f);
+		  return(code);
+		}
+	  }
 
-                if (code != 0) {
-                    Put(&f);
-                    return(code);
-                }
-	    }
-
-	    f->DemoteLock();
+	  f->DemoteLock();
 	} else {	/* !FETCHABLE(f) */
-            if (!HOARDING(f) && !LOGGING(f) && !EMULATING(f)) /* = Resolving */
-            {
-                LOG(100, ("(MARIA) TIMEOUT after something...\n"));
-                Put(&f);
-                return(ETIMEDOUT);
+	    if (RESOLVING(f))
+	    {
+		LOG(100, ("(MARIA) TIMEOUT !fetchable and resolving...\n"));
+		Put(&f);
+		return(ETIMEDOUT);
 	    }
 
-            if (!HAVESTATUS(f)) {
-                Put(&f);
-                return(ETIMEDOUT);
-            }
+	    if (!HAVESTATUS(f) && !f->IsFake()) {
+		Put(&f);
+		return(ETIMEDOUT);
+	    }
 
-            /*
-             * Unfortunately, trying to limit access to stale STATUS
-             * won't work because in order to gracefully recover from
-             * the active reference to a now inconsistent object, we
-             * have to be able to close the object.  In order to close
-             * the object, we have to be able to get the STATUS of the
-             * object...  I guess we allow full access to the stale
-             * STATUS, but log that we did so.
-             *
-             *   if (DYING(f)) {
-             *     LOG(0, ("Active reference prevents refetching object!  Providing limited access to stale status!\n"));
-             *     *f_addr = f;
-             *     Put(&f);
-             *     return(ETOOMANYREFS);
-             *   }
-             */
-            if (DYING(f)) 
-                LOG(0, ("Active reference prevents refetching object! "
-                        "Allowing access to stale status! (key = <%s>)\n",
-                        FID_(key)));
-
-            else if (!STATUSVALID(f))
-                LOG(0, ("Allowing access to stale status! (key = <%s>)\n",
+	    /*
+	     * Unfortunately, trying to limit access to stale STATUS
+	     * won't work because in order to gracefully recover from
+	     * the active reference to a now inconsistent object, we
+	     * have to be able to close the object.  In order to close
+	     * the object, we have to be able to get the STATUS of the
+	     * object...  I guess we allow full access to the stale
+	     * STATUS, but log that we did so.
+	     *
+	     *   if (DYING(f)) {
+	     *     LOG(0, ("Active reference prevents refetching object!  Providing limited access to stale status!\n"));
+	     *     *f_addr = f;
+	     *     Put(&f);
+	     *     return(ETOOMANYREFS);
+	     *   }
+	     */
+	    if (DYING(f))
+		LOG(0, ("Active reference prevents refetching object! "
+			"Allowing access to stale status! (key = <%s>)\n",
 			FID_(key)));
 
-            if (getdata) {
-                if (DYING(f)) { 
-                    LOG(0, ("Active reference prevents refetching object! "
+	    else if (!STATUSVALID(f))
+		LOG(0, ("Allowing access to stale status! (key = <%s>)\n",
+			FID_(key)));
+
+	    if (getdata) {
+		if (DYING(f)) {
+		    LOG(0, ("Active reference prevents refetching object! "
 			    "Disallowing access to stale data! (key = <%s>)\n",
 			    FID_(key)));
-                    Put(&f);
-                    return(ETOOMANYREFS);
-                } 
+		    Put(&f);
+		    return(ETOOMANYREFS);
+		}
 
-                if (!HAVEALLDATA(f)) {
-		    int found;
+		if (!HAVEALLDATA(f)) {
+		    int found = 0;
 
 		    /* try the lookaside cache */
-		    f->PromoteLock();
-		    found = f->LookAside();
-		    f->DemoteLock();
+		    if (!f->IsLocalObj() && !f->IsFake()) {
+			f->PromoteLock();
+			found = f->LookAside();
+			f->DemoteLock();
+		    }
 
 		    if (!found) {
 			Put(&f);
 			return(ETIMEDOUT);
-		    }
-                }
+		  }
+		}
 
-                if (!DATAVALID(f))
-                    LOG(0, ("Allowing access to stale data! (key = <%s>)\n",
-			    FID_(key)));
-            }
+		if (!DATAVALID(f) && !f->IsLocalObj())
+		  LOG(0, ("Allowing access to stale data! (key = <%s>)\n",
+				  FID_(key)));
+	  }
 	}
-    }
+  }
 
-    /* Finalize handling of fake objects. */
-    if (!GetInconsistent && f->IsFake() && f->vol->IsReplicated() &&
-        !((repvol *)f->vol)->IsUnderRepair(uid))
-    {
-        repvol *v = (repvol *)f->vol;
-        LOG(1, ("(Puneet)fsdb::Get:Volume (%u) NOT under repair and IsFake(%s)\n",
-		uid, FID_(&f->fid)));
-        char path[MAXPATHLEN];
-        f->GetPath(path, 1);
-        LOG(1, ("(Maria, Puneet) After GetPath, path = %s\n", path));
+  /* Examine the possibility of executing an ASR. */
+  if (!GetInconsistent && f->IsFake() && f->vol->IsReplicated() &&
+      !f->IsExpandedObj())
+  {
+	int ASRInvokable;
+	repvol *v;
+	struct timeval tv;
+	fsobj *realobj;
 
-        /* object is inconsistent - try running the ASR */
-        /* check that ASR time interval has expired */
-#if 0
-        userent *u;
-        GetUser(&u, f->vol->realm, uid);
-#endif
-        struct timeval tv;
-        gettimeofday(&tv, 0);
-        int ASRInvokable = (SkkEnabled && ASRallowed && (vp->type == VPT_Worker) &&
-                            v->IsASRAllowed() && !v->asr_running() &&
-                            ((tv.tv_sec - f->lastresolved) > ASR_INTERVAL) &&
-			    !adv_mon.skkPgid(vp->u.u_pgid) && adv_mon.ConnValid());
-	if (v->asr_running() && vp->u.u_pgid != v->asr_id())
-	  code = EAGAIN; /* bounce out anything which tries to hold kernel locks while repairing */
-        else if (ASRInvokable && (!adv_mon.RequestASRInvokation(v, path, uid))) {
-                gettimeofday(&tv, 0);
-                f->lastresolved = tv.tv_sec;
-                code = ERETRY; /* tell application to retry so that kernel locks get released */
-		/* Actually, the application will get an EAGAIN when the retry bounces out */
+	LOG(0, ("fsdb::Get:Volume NOT under repair and IsFake(%s)\n",
+		FID_(&f->fid)));
+
+	/* At this point, we have Fakeify'd the real object, and f is the
+	 * fake object served in its place. Unfortunately, it is not linked
+	 * into the directory structure and therefore calls to GetPath fail.
+	 * Find the underlying object, and call LaunchASR on that to get
+	 * the required information. */
+
+	realobj = Find(key);
+	if (!realobj) {
+	  LOG(0, ("fsdb::Get:Find failed!\n"));
+	  Put(&f);
+	  return EINCONS;
 	}
-	else code = (f->IsFakeDir() ? EINCONS : ERETRY);
 
-	LOG(1, ("fsdb::Get: returning %d\n", code));
-        Put(&f);
-        return(code);
-    }
+	v = (repvol *)realobj->vol;
+	gettimeofday(&tv, 0);
 
-    /* Update priority. */
-    if (reference)
+	/* Check that:
+	 * 1.) An ASRLauncher path was parsed in venus.conf.
+	 * 2.) This thread is a worker.
+	 * 3.) ASR's are allowed and enabled to execute within this volume.
+	 * 4.) An ASR is not currently running within this volume.
+	 * 5.) The timeout interval for ASR launching has expired.
+	 */
+
+	ASRInvokable = ((ASRLauncherFile != NULL) && (vp->type == VPT_Worker) &&
+					v->IsASRAllowed() && !v->asr_running() &&
+					((tv.tv_sec - realobj->lastresolved) > ASR_INTERVAL) &&
+					v->IsASREnabled() && (ASRPolicyFile != NULL));
+
+	if(ASRLauncherFile == NULL)
+	  LOG(0, ("fsdb::Get: asrlauncher_file not specified in venus.conf!\n"));
+
+	if(ASRPolicyFile == NULL)
+	  LOG(0, ("fsdb::Get: asrpolicy_file not specified in venus.conf!\n"));
+
+	if(vp->type != VPT_Worker)
+	  LOG(0, ("fsdb::Get: Non-worker Thread\n"));
+
+	if(!v->IsASREnabled())
+	  LOG(0, ("fsdb::Get: ASRs disabled by the system at the moment \n"));
+
+	if(v->asr_running())
+	  LOG(0, ("fsdb::Get: ASR already running in this volume\n"));
+
+	if(((tv.tv_sec - realobj->lastresolved) <= ASR_INTERVAL))
+	  LOG(0, ("fsdb::Get: ASR executed too recently for this object\n"
+			  "fsdb::Get: New time: %d\tOld time: %d\tDiff:%d\n",
+			  tv.tv_sec, realobj->lastresolved,
+			  tv.tv_sec - realobj->lastresolved));
+
+	if(!v->IsASRAllowed())
+	  LOG(0, ("fsdb::Get: ASRs disabled in this volume by some user\n"));
+
+	if(v->asr_running() && vp->u.u_pgid != v->asr_pgid())
+	    code = ERETRY;     /* Bounce out anything which tries to hold
+				* kernel locks while repairing. */
+
+	else if (ASRInvokable) { /* Execute ASR. */
+	  LOG(0, ("fsdb::Get: Launching for (%s)... \n", FID_(key)));
+	  if (realobj->LaunchASR(SERVER_SERVER, realobj->IsDir() ? DIRECTORY_CONFLICT : FILE_CONFLICT) == 0)
+		code = ERETRY;  /* wait a short duration and retry */
+	  else
+		code = EINCONS;
+	}
+	else {
+	  LOG(0, ("fsdb::Get: ASR not invokable for %s\n", FID_(key)));
+	  code = EINCONS;
+	}
+
+	Put(&f);
+	return(code);
+  }
+
+  /* Update priority. */
+  if (reference)
 	f->Reference();
-    f->ComputePriority();
+  f->ComputePriority();
 
-    *f_addr = f;
-    return(0);
+  *f_addr = f;
+  return(0);
 }
 
-
 /* MUST NOT be called from within transaction! */
 void fsdb::Put(fsobj **f_addr) {
     if (!(*f_addr)) { LOG(100, ("fsdb::Put: Null FSO\n")); return; }
@@ -1097,43 +1051,56 @@ int fsdb::TranslateFid(VenusFid *OldFid, VenusFid *NewFid)
 			FID_(OldFid)));
 		return(ENOENT);
 	}
-	
+
 	/* Can't handle any case but reintegration. */
 	/* in old versions we would choke too  if (!DIRTY(f)) */
 	if (!HAVESTATUS(f)) {
-		f->print(logFile); 
-		CHOKE("fsdb::TranslateFid: !HAVESTATUS"); 
+		f->print(logFile);
+		CHOKE("fsdb::TranslateFid: !HAVESTATUS");
 	}
 
 	/* Replace the fids in kernel name cache for this object */
 	k_Replace(OldFid, NewFid);
-	
+
 	/* Check that the NewFid is not already known! */
 	fsobj *Newf = Find(NewFid);
 	if (Newf != 0) {
-		f->print(logFile); 
-		Newf->print(logFile); 
-		CHOKE("fsdb::TranslateFid: NewFid found"); 
+		f->print(logFile);
+		Newf->print(logFile);
+		CHOKE("fsdb::TranslateFid: NewFid found");
 	}
-	
+
 	/* Remove OldObject from table. */
-	if (htab.remove(&f->fid, &f->primary_handle) != 
+	if (htab.remove(&f->fid, &f->primary_handle) !=
 	    &f->primary_handle) {
-		f->print(logFile); 
-		CHOKE("fsdb::TranslateFid: old object remove"); 
+		f->print(logFile);
+		CHOKE("fsdb::TranslateFid: old object remove");
 	}
 
+	/* An upcall may already be queued with the old local fid. Or we may
+	 * already be reintegrating before the local fid has been passed back
+	 * to the kernel.
+	 *
+	 * To avoid this race we have to remember the value of OldFid. If it
+	 * was a localfid (reintegration related) it will have the same volume
+	 * id, a special file or directory vnode value, and we only need to
+	 * remember the uniquifier value.
+	 *
+	 * We may need the vnode for repair related objects. */
+	f->LocalFid_Vnode  = OldFid->Vnode;
+	f->LocalFid_Unique = OldFid->Unique;
+
 	/* Change Fid, update dir and reinsert into table. */
 	RVMLIB_REC_OBJECT(f->fid);
 	f->fid = *NewFid;
-	
+
 	/* replace "." and its hardlinks if f is dir */
-	if (f->IsDir() && HAVEALLDATA(f) && !f->IsMtPt()) 
+	if (f->IsDir() && HAVEALLDATA(f) && !f->IsMtPt())
 		f->dir_TranslateFid(OldFid, NewFid);
-	
+
 	/* replace f in the hash table */
 	htab.append(&f->fid, &f->primary_handle);
-	
+
 	/* Update the Parent. */
 	pFid = f->pfid;
 	fsobj *pf = Find(&pFid);
@@ -1142,18 +1109,18 @@ int fsdb::TranslateFid(VenusFid *OldFid, VenusFid *NewFid)
 
 	/* Update the children, if we are a directory. */
 	if (ISDIR(*OldFid) && f->children) {
-            dlist_iterator next(*(f->children));
-            dlink *d;
-            while ((d = next())) { 
-                fsobj *cf = strbase(fsobj, d, child_link);
-                CODA_ASSERT(FID_EQ(&cf->pfid, OldFid));
-                RVMLIB_REC_OBJECT(cf->pfid);
-                cf->pfid = *NewFid;
+	    dlist_iterator next(*(f->children));
+	    dlink *d;
+	    while ((d = next())) {
+		fsobj *cf = strbase(fsobj, d, child_link);
+		CODA_ASSERT(FID_EQ(&cf->pfid, OldFid));
+		RVMLIB_REC_OBJECT(cf->pfid);
+		cf->pfid = *NewFid;
 
-                if (cf->IsDir() && HAVEALLDATA(cf) && !cf->IsMtPt())
-                    cf->dir_TranslateFid(OldFid, NewFid);
-            }
-        }
+		if (cf->IsDir() && HAVEALLDATA(cf) && !cf->IsMtPt())
+		    cf->dir_TranslateFid(OldFid, NewFid);
+	    }
+	}
 	return 0;
 }
 
@@ -1174,12 +1141,15 @@ int fsdb::CallBackBreak(const VenusFid *fid)
 }
 
 void fsdb::ResetUser(uid_t uid) {
-    /* Demote access rights for the user. */
+    /* Clear access rights for the user. */
     fso_iterator next(NL);
     fsobj *f;
+
+    Recov_BeginTrans();
     while ((f = next()))
 	if (f->IsDir())
-	    f->DemoteAcRights(uid);
+	    f->ClearAcRights(uid);
+    Recov_EndTrans(0);
 }
 
 
@@ -1300,8 +1270,6 @@ void fsdb::ReclaimFsos(int priority, int count) {
 	    UpdateCacheStats((f->IsDir() ? &DirDataStats : &FileDataStats),
 			     REPLACE, BLOCKS(f));
 
-        f->RecordReplacement(TRUE, HAVEDATA(f));
-
 	f->Kill();
 	f->GC();
 
@@ -1391,9 +1359,9 @@ int fsdb::AllocBlocks(int priority, int nblocks) {
 
 /* Needn't be called from within transaction. */
 int fsdb::GrabFreeBlocks(int priority, int nblocks) {
-    int free_blocks = FreeBlockCount();
-    if ((free_blocks - nblocks) >= FreeBlockMargin ||
-	 (free_blocks - nblocks) >= 0 && priority >= MarginPri()) {
+    int remaining_blocks = FreeBlockCount() - nblocks;
+    if (remaining_blocks >= FreeBlockMargin ||
+	(remaining_blocks >= 0 && priority >= MarginPri())) {
 	ChangeDiskUsage(nblocks);
 	return(1);
     }
@@ -1421,7 +1389,7 @@ void fsdb::ReclaimBlocks(int priority, int nblocks) {
 	if (ufs_blocks == 0) continue;
 
 	/* Can't reclaim if busy. */
-	if (BUSY(f)) continue;
+	if (BUSY(f) || f->IsLocalObj()) continue;
 
 	/* Reclaim data.  Return if we've got enough. */
 	MarinerLog("cache::Replace [data] %s [%d, %d]\n",
@@ -1429,8 +1397,6 @@ void fsdb::ReclaimBlocks(int priority, int nblocks) {
 	UpdateCacheStats((f->IsDir() ? &DirDataStats : &FileDataStats),
 			 REPLACE, BLOCKS(f));
 
-        f->RecordReplacement(FALSE, TRUE);
-
 	f->DiscardData();
 
 	reclaimed += ufs_blocks;
