@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -47,6 +47,13 @@ extern "C" {
 #include "fso.h"
 #include "venus.private.h"
 
+#ifndef fdatasync
+#define fdatasync(fd) fsync(fd)
+#endif
+
+/* always useful to have a page of zero's ready */
+static char zeropage[4096];
+
 /*  *****  CacheFile Members  *****  */
 
 /* Pre-allocation routine. */
@@ -92,7 +99,7 @@ void CacheFile::Validate()
 void CacheFile::Reset()
 {
     if (access(name, F_OK) == 0 && length != 0) {
-        Recov_BeginTrans();
+	Recov_BeginTrans();
 	Truncate(0);
 	Recov_EndTrans(MAXFP);
     }
@@ -107,7 +114,7 @@ int CacheFile::ValidContainer()
     if (rc) return 0;
 
     int valid =
-#if !defined(DJGPP) && !defined(__CYGWIN32__)
+#ifndef __CYGWIN32__
       tstat.st_uid == (uid_t)V_UID &&
       tstat.st_gid == (gid_t)V_GID &&
       (tstat.st_mode & ~S_IFMT) == V_MODE &&
@@ -134,17 +141,14 @@ void CacheFile::Create(int newlength)
     int tfd;
     struct stat tstat;
     if (mkpath(name, V_MODE | 0100)<0)
-        CHOKE("CacheFile::Create: could not make path for %s", name);
+	CHOKE("CacheFile::Create: could not make path for %s", name);
     if ((tfd = ::open(name, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, V_MODE)) < 0)
 	CHOKE("CacheFile::Create: open failed (%d)", errno);
 
-#ifndef DJGPP
-#ifndef __CYGWIN32__
-    if (::fchown(tfd, (uid_t)V_UID, (gid_t)V_GID) < 0)
-	CHOKE("CacheFile::Create: fchown failed (%d)", errno);
+#ifdef __CYGWIN32__
+    ::chown(name, (uid_t)V_UID, (gid_t)V_GID);
 #else
-    chown(name, (uid_t)V_UID, (gid_t)V_GID);
-#endif
+    ::fchown(tfd, (uid_t)V_UID, (gid_t)V_GID);
 #endif
     if (::ftruncate(tfd, newlength) < 0)
       CHOKE("CacheFile::Create: ftruncate failed (%d)", errno);
@@ -160,8 +164,8 @@ void CacheFile::Create(int newlength)
 }
 
 
-/* 
- * copies a cache file, data and attributes, to a new one.  
+/*
+ * copies a cache file, data and attributes, to a new one.
  */
 int CacheFile::Copy(CacheFile *destination)
 {
@@ -181,23 +185,18 @@ int CacheFile::Copy(char *destname, int recovering)
     struct stat tstat;
 
     if (mkpath(destname, V_MODE | 0100) < 0) {
-        LOG(0, ("CacheFile::Copy: could not make path for %s\n", name));
+	LOG(0, ("CacheFile::Copy: could not make path for %s\n", name));
 	return -1;
     }
     if ((tfd = ::open(destname, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, V_MODE)) < 0) {
 	LOG(0, ("CacheFile::Copy: open failed (%d)\n", errno));
 	return -1;
     }
-#ifndef DJGPP
-    if (::fchmod(tfd, V_MODE) < 0)
-	CHOKE("CacheFile::Copy: fchmod failed (%d)\n", errno);
+    ::fchmod(tfd, V_MODE);
 #ifdef __CYGWIN32__
-    if (::chown(destname, (uid_t)V_UID, (gid_t)V_GID) < 0)
-	CHOKE("CacheFile::ResetCopy: fchown failed (%d)\n", errno);
+    ::chown(destname, (uid_t)V_UID, (gid_t)V_GID);
 #else
-    if (::fchown(tfd, (uid_t)V_UID, (gid_t)V_GID) < 0)
-	CHOKE("CacheFile::Copy: fchown failed (%d)\n", errno);
-#endif
+    ::fchown(tfd, (uid_t)V_UID, (gid_t)V_GID);
 #endif
     if ((ffd = ::open(name, O_RDONLY| O_BINARY, V_MODE)) < 0)
 	CHOKE("CacheFile::Copy: source open failed (%d)\n", errno);
@@ -241,25 +240,45 @@ void CacheFile::Stat(struct stat *tstat)
 }
 
 
+void CacheFile::Utimes(const struct timeval times[2])
+{
+    CODA_ASSERT(::utimes(name, times) == 0);
+}
+
+
 /* MUST be called from within transaction! */
 void CacheFile::Truncate(long newlen)
 {
+    int fd;
+
+    fd = open(name, O_WRONLY | O_BINARY);
+    CODA_ASSERT(fd >= 0 && "fatal error opening container file");
+
+    /* ISR tweak, write zeros to data area before truncation */
+    if (option_isr && newlen < length) {
+	size_t len = sizeof(zeropage), n = length - newlen;
+
+	lseek(fd, newlen, SEEK_SET);
+
+	while (n) {
+	    if (n < len) len = n;
+	    write(fd, zeropage, len);
+	    n -= len;
+	}
+	/* we have to force these writes to disk, otherwise the following
+	 * truncate would simply drop any unwritten data */
+	fdatasync(fd);
+    }
+
     if (length != newlen) {
 	RVMLIB_REC_OBJECT(*this);
 	length = validdata = newlen;
     }
-#ifndef __CYGWIN32__
-    CODA_ASSERT(::truncate(name, length) == 0);
-#else 
-    {
-	int fd = open(name, O_WRONLY | O_BINARY);
-	if ( fd < 0 ) CODA_ASSERT(0);
-	CODA_ASSERT(::ftruncate(fd, length) == 0);
-	close(fd);
-    }
-#endif
-}
 
+    CODA_ASSERT(::ftruncate(fd, length) == 0);
+
+    close(fd);
+}
 
 /* MUST be called from within transaction! */
 void CacheFile::SetLength(long newlen)
