@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -43,6 +43,7 @@ extern "C" {
 /* interfaces */
 #include <vice.h>
 #include <lka.h>
+#include <struct.h>
 
 #ifdef __cplusplus
 }
@@ -185,12 +186,12 @@ int fsobj::Fetch(uid_t uid)
 
     LOG(10, ("fsobj::Fetch: (%s), uid = %d\n", GetComp(), uid));
 
-    CODA_ASSERT(!IsLocalObj());
+    CODA_ASSERT(!IsLocalObj() && !IsFake());
 
     /* Sanity checks. */
     {
 	/* Better not be disconnected or dirty! */
-	FSO_ASSERT(this, (HOARDING(this) || (LOGGING(this) && !DIRTY(this))));
+	FSO_ASSERT(this, (REACHABLE(this) && !DIRTY(this)));
 
 	/* We never fetch data if we don't already have status. */
 	if (!HAVESTATUS(this))
@@ -199,10 +200,6 @@ int fsobj::Fetch(uid_t uid)
 	/* We never fetch data if we already have some. */
 	if (HAVEALLDATA(this))
 	    { print(logFile); CHOKE("fsobj::Fetch: HAVEALLDATA"); }
-
-	/* We never fetch data for fake objects. */
-	if (IsFake())
-	    { print(logFile); CHOKE("fsobj::Fetch: IsFake"); }
     }
 
     int code = 0;
@@ -549,19 +546,7 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
     /* Sanity checks. */
     {
 	/* Better not be disconnected or dirty! */
-	FSO_ASSERT(this, (HOARDING(this) || (LOGGING(this) && !DIRTY(this))));
-
-	if (IsFake()) {
-	    FSO_ASSERT(this, acl == 0);
-
-	    /* We never fetch fake directory without having status and data. */
-	    if (IsFakeDir() && !HAVEALLDATA(this))
-		{ print(logFile); CHOKE("fsobj::GetAttr: IsFakeDir && !HAVEALLDATA"); }
-
-	    /* We never fetch fake mtpts (covered or uncovered). */
-	    if (IsFakeMtPt() || IsFakeMTLink())
-		{ print(logFile); CHOKE("fsobj::GetAttr: IsFakeMtPt || IsFakeMTLink"); }
-	}
+	FSO_ASSERT(this, (REACHABLE(this) && !DIRTY(this)));
     }
 
     int code = 0;
@@ -742,7 +727,7 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 		    int numVFlags = 0;
 
 		    for (i = 0; i < VSG_MEMBERS; i++)
-			if (m->rocc.hosts[i].s_addr != 0)
+			if (m->rocc.hosts[i].s_addr != 0) {
 			    if (numVFlags == 0) {
 				/* unset, copy in one response */
 				ARG_UNMARSHALL_BS(VFlagvar, VFlagBS, i);
@@ -755,6 +740,7 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 				for (int j = 0; j < numPiggyFids; j++)
 				    VFlags[j] &= VFlagvar_bufs[i].SeqBody[j];
 			    }
+			}
 
 		    LOG(10, ("fsobj::GetAttr: ValidateAttrs (%s), %d fids sent, %d checked\n",
 			      GetComp(), numPiggyFids, numVFlags));
@@ -800,6 +786,7 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 					pobj->GetComp(), FID_(&FAVs[i].Fid)));
 				
 				if (REPLACEABLE(pobj) && !BUSY(pobj)) {
+				  LOG(1, ("fsobj::GetAttr: Killing (%s), REPLACEABLE and !BUSY\n", pobj->GetComp()));
 				    Recov_BeginTrans();
 				    pobj->Kill(0);
 				    Recov_EndTrans(MAXFP);
@@ -819,10 +806,11 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 				 * is a side-effect.
 				 */
 				if (HAVEDATA(pobj) && !ACTIVE(pobj) &&
-				    !pobj->IsFakeDir() && !DIRTY(pobj))
+				    !pobj->IsFakeDir() &&
+				    !pobj->IsExpandedDir() && !DIRTY(pobj))
 				{
 				    Recov_BeginTrans();
-				    UpdateCacheStats((IsDir() ? &FSDB->DirDataStats 
+				    UpdateCacheStats((IsDir() ? &FSDB->DirDataStats
 						      : &FSDB->FileDataStats),
 						     REPLACE, BLOCKS(pobj));
 				    pobj->DiscardData();
@@ -953,9 +941,9 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 		       printbuf);
 	    }
 
-
+#if 0 /* XXX: How can this be right? */
 	    /* Handle successful validation of fake directory! */
-	    if (IsFakeDir()) {
+	    if (IsFakeDir() || IsExpandedDir()) { /* XXX:? Adam 5/17/05 */
 		LOG(0, ("fsobj::GetAttr: (%s) validated fake directory\n",
 			FID_(&fid)));
 
@@ -966,6 +954,7 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 		code = ERETRY;
 		goto RepExit;
 	    }
+#endif
 
 	    /* Handle failed validations. */
 	    if (HAVESTATUS(this) && VV_Cmp(&status.VV, &stat.VV) != VV_EQ) {
@@ -1022,9 +1011,6 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 
 RepExit:
 	if (m) m->Put();
-	if (IsFakeDir() && code == EINCONS) {
-	    code = 0;
-	}
 	switch(code) {
 	    case 0:
 		if (asy_resolve)
@@ -1036,9 +1022,10 @@ RepExit:
 		break;
 
 	    case EINCONS:
-		Recov_BeginTrans();
-		Kill();
-		Recov_EndTrans(CMFP);
+	      /* We used to kill inconsistent objects, but that is not a
+	       * useful thing to do anymore, as the object now simply has
+	       * its attributes changed and functions as a .localcache
+	       * object in server/server conflict expansions. -- Adam */
 		break;
 
 	    case ENXIO:
@@ -1157,15 +1144,16 @@ NonRepExit:
 	PutConn(&c);
     }
 
-    if (code != 0) {
-	Recov_BeginTrans();
-	/* Demote or discard existing status. */
-	if (HAVESTATUS(this) && code != ENOENT)
-		Demote();
-	else
-		Kill();
-	Recov_EndTrans(DMFP);
+    if (code && (code != EINCONS)) {
+      Recov_BeginTrans();
+      /* Demote or discard existing status. */
+      if (HAVESTATUS(this) && code != ENOENT)
+	Demote();
+      else
+	  Kill();
+      Recov_EndTrans(DMFP);
     }
+
     return(code);
 }
 
@@ -1174,7 +1162,7 @@ int fsobj::GetACL(RPC2_BoundedBS *acl, uid_t uid)
 {
     LOG(10, ("fsobj::GetACL: (%s), uid = %d\n", GetComp(), uid));
 
-    if (IsFake() || IsLocalObj()) {
+    if (IsLocalObj()) {
 	/* Just read/lookup rights for System:AnyUser */
 	const char *fakeacl = "1\n0\nSystem:AnyUser\t9\n";
 	acl->SeqLen = strlen(fakeacl) + 1;
@@ -1208,227 +1196,28 @@ void fsobj::LocalStore(Date_t Mtime, unsigned long NewLength)
 }
 
 
-int fsobj::ConnectedStore(Date_t Mtime, uid_t uid, unsigned long NewLength)
+int fsobj::DisconnectedStore(Date_t Mtime, uid_t uid, unsigned long NewLength,
+			     int prepend)
 {
-    FSO_ASSERT(this, HOARDING(this));
-
-    int code = 0, fd = -1;
-    char prel_str[256];
-    sprintf(prel_str, "store::Store %%s [%ld]\n", NBLOCKS(NewLength));
-
-    /* Status parameters. */
-    ViceStatus status;
-    VenusToViceStatus(&stat, &status);
-    {
-	/* Temporary!  Until RPC interface is fixed!  -JJK */
-	status.Date = Mtime;
-    }
-
-    /* COP2 Piggybacking. */
-    char PiggyData[COP2SIZE];
-    RPC2_CountedBS PiggyBS;
-    PiggyBS.SeqLen = 0;
-    PiggyBS.SeqBody = (RPC2_ByteSeq)PiggyData;
-
-    /* Set up the SE descriptor. */
-    SE_Descriptor dummysed;
-    memset(&dummysed, 0, sizeof(SE_Descriptor));
-    SE_Descriptor *sed = 0;
-    {
-	/* Must be a file! */
-	sed = &dummysed;
-	sed->Tag = SMARTFTP;
-	struct SFTP_Descriptor *sei = &sed->Value.SmartFTPD;
-	sei->TransmissionDirection = CLIENTTOSERVER;
-	sei->hashmark = 0;
-	sei->SeekOffset = 0;
-	sei->ByteQuota = -1;
-
-        /* and open a safe fd to the containerfile */
-        fd = data.file->Open(O_RDONLY);
-
-        sei->Tag = FILEBYFD;
-        sei->FileInfo.ByFD.fd = fd;
-    }
-
-    /* VCB arguments */
-    RPC2_Integer VS = 0;
-    CallBackStatus VCBStatus = NoCallBack;
-    RPC2_CountedBS OldVS;
-    OldVS.SeqLen = 0;
-    OldVS.SeqBody = 0;
-
-    if (vol->IsReplicated()) {
-	ViceStoreId sid;
-	mgrpent *m = 0;
-	int asy_resolve = 0;
-        repvol *vp = (repvol *)vol;
-
-	/* Acquire an Mgroup. */
-	code = vp->GetMgrp(&m, uid, (PIGGYCOP2 ? &PiggyBS : 0));
-	if (code != 0) goto RepExit;
-
-	/* The COP1 call. */
-	long cbtemp; cbtemp = cbbreaks;
-	vv_t UpdateSet;
-
-	Recov_BeginTrans();
-	sid = vp->GenerateStoreId();
-	Recov_EndTrans(MAXFP);
-	{
-	    /* Make multiple copies of the IN/OUT and OUT parameters. */
-	    int ph_ix; unsigned long ph;
-            ph = ntohl(m->GetPrimaryHost(&ph_ix)->s_addr);
-
-	    vp->PackVS(VSG_MEMBERS, &OldVS);
-
-	    ARG_MARSHALL(IN_OUT_MODE, ViceStatus, statusvar, status, VSG_MEMBERS);
-	    ARG_MARSHALL(IN_OUT_MODE, SE_Descriptor, sedvar, *sed, VSG_MEMBERS);
-	    ARG_MARSHALL(OUT_MODE, RPC2_Integer, VSvar, VS, VSG_MEMBERS);
-	    ARG_MARSHALL(OUT_MODE, CallBackStatus, VCBStatusvar, VCBStatus, VSG_MEMBERS);
-
-	    /* Make the RPC call. */
-	    CFSOP_PRELUDE(prel_str, comp, fid);
-	    MULTI_START_MESSAGE(ViceStore_OP);
-	    code = (int) MRPC_MakeMulti(ViceStore_OP, ViceStore_PTR,
-					VSG_MEMBERS, m->rocc.handles,
-					m->rocc.retcodes, m->rocc.MIp, 0, 0,
-					MakeViceFid(&fid), statusvar_ptrs,
-					NewLength, ph, &sid, &OldVS,
-					VSvar_ptrs, VCBStatusvar_ptrs,
-					&PiggyBS, sedvar_bufs);
-	    MULTI_END_MESSAGE(ViceStore_OP);
-	    CFSOP_POSTLUDE("store::store done\n");
-
-	    free(OldVS.SeqBody);
-	    /* Collate responses from individual servers and decide what to do next. */
-	    code = vp->Collate_COP1(m, code, &UpdateSet);
-	    MULTI_RECORD_STATS(ViceStore_OP);
-	    if (code == EASYRESOLVE) { asy_resolve = 1; code = 0; }
-	    if (code != 0) goto RepExit;
-
-	    /* Collate volume callback information */
-	    if (cbtemp == cbbreaks)
-		vp->CollateVCB(m, VSvar_bufs, VCBStatusvar_bufs);
-
-	    /* Finalize COP2 Piggybacking. */
-	    if (PIGGYCOP2)
-		vp->ClearCOP2(&PiggyBS);
-
-	    /* Manually compute the OUT parameters from the mgrpent::Store() call! -JJK */
-	    int dh_ix; dh_ix = -1;
-	    (void)m->DHCheck(0, ph_ix, &dh_ix);
-	    ARG_UNMARSHALL(statusvar, status, dh_ix);
-	    {
-		long bytes = sedvar_bufs[dh_ix].Value.SmartFTPD.BytesTransferred;
-		LOG(10, ("(Multi)ViceStore: stored %d bytes\n", bytes));
-		if (bytes != (long)status.Length) {
-		    print(logFile);
-		    CHOKE("fsobj::Store: bytes mismatch (%d, %d)",
-			bytes, status.Length);
-		}
-	    }
-	}
-
-	/* Do Store locally. */
-	Recov_BeginTrans();
-	LocalStore(Mtime, NewLength);
-	UpdateStatus(&status, &UpdateSet, uid);
-	Recov_EndTrans(CMFP);
-	if (ASYNCCOP2) ReturnEarly();
-
-	/* Send the COP2 message or add an entry for piggybacking. */
-        (void)vp->COP2(m, &sid, &UpdateSet);
-
-RepExit:
-	if (m) m->Put();
-	switch(code) {
-	    case 0:
-		if (asy_resolve)
-		    vp->ResSubmit(0, &fid);
-		break;
-
-	    default:
-		/* Simulate a disconnection, to be followed by reconnection/reintegration. */
-		Recov_BeginTrans();
-		code = vp->LogStore(Mtime, uid, &fid, NewLength);
-
-		if (code == 0) {
-			LocalStore(Mtime, NewLength);
-			vp->flags.transition_pending = 1;
-		}
-		Recov_EndTrans(DMFP);
-		break;
-	}
-    }
-    else {
-	/* Acquire a Connection. */
-	connent *c;
-	ViceStoreId Dummy;      /* ViceStore needs an address for indirection */
-        volrep *vp = (volrep *)vol;
-	code = vp->GetConn(&c, uid);
-	if (code != 0) goto NonRepExit;
-
-	/* Make the RPC call. */
-	CFSOP_PRELUDE(prel_str, comp, fid);
-	UNI_START_MESSAGE(ViceStore_OP);
-	code = (int) ViceStore(c->connid, MakeViceFid(&fid), &status,
-			       NewLength, 0, &Dummy, &OldVS, &VS, &VCBStatus,
-			       &PiggyBS, sed);
-	UNI_END_MESSAGE(ViceStore_OP);
-	CFSOP_POSTLUDE("store::store done\n");
-
-	/* Examine the return code to decide what to do next. */
-	code = vp->Collate(c, code);
-	UNI_RECORD_STATS(ViceStore_OP);
-	if (code != 0) goto NonRepExit;
-
-	{
-	    long bytes = sed->Value.SmartFTPD.BytesTransferred;
-	    LOG(10, ("ViceStore: stored %d bytes\n", bytes));
-	    if (bytes != (long)status.Length) {
-		print(logFile);
-		CHOKE("fsobj::Store: bytes mismatch (%d, %d)",
-		    bytes, status.Length);
-	    }
-	}
-
-	/* Do Store locally. */
-	Recov_BeginTrans();
-	LocalStore(Mtime, NewLength);
-	UpdateStatus(&status, 0, uid);
-	Recov_EndTrans(CMFP);
-
-NonRepExit:
-	PutConn(&c);
-    }
-    
-    data.file->Close(fd);
-
-    return(code);
-}
-
-int fsobj::DisconnectedStore(Date_t Mtime, uid_t uid, unsigned long NewLength, int Tid)
-{
-    FSO_ASSERT(this, (EMULATING(this) || LOGGING(this)));
-
     int code = 0;
+    repvol *rv;
 
-    if (!vol->IsReplicated()) {
-	code = ETIMEDOUT;
-	goto Exit;
-    }
+    if (!vol->IsReplicated())
+	return ETIMEDOUT;
+    rv = (repvol *)vol;
 
     Recov_BeginTrans();
 	/* Failure to log a store would be most unpleasant for the user! */
 	/* Probably we should try to guarantee that it never happens (e.g., by reserving a record at open). */
-    code = ((repvol *)vol)->LogStore(Mtime, uid, &fid, NewLength, Tid);
+    code = rv->LogStore(Mtime, uid, &fid, NewLength, prepend);
     
-    if (code == 0)
+    if (code == 0 && prepend == 0)
+            /* It's already been updated if we're 'prepending',
+	     * which basically means it is a repair-related operation,
+	     * and doing it again would trigger an assertion. */
 	    LocalStore(Mtime, NewLength);
     Recov_EndTrans(DMFP);
 
-Exit:
     return(code);
 }
 
@@ -1438,14 +1227,7 @@ int fsobj::Store(unsigned long NewLength, Date_t Mtime, uid_t uid)
 
     int code = 0;
 
-    int conn, tid;
-    GetOperationState(&conn, &tid);
-    if (conn == 0) {
-	code = DisconnectedStore(Mtime, uid, NewLength, tid);
-    }
-    else {
-	code = ConnectedStore(Mtime, uid, NewLength);
-    }
+    code = DisconnectedStore(Mtime, uid, NewLength);
 
     if (code != 0) {
 	Recov_BeginTrans();
@@ -1485,57 +1267,157 @@ void fsobj::LocalSetAttr(Date_t Mtime, unsigned long NewLength,
         stat.Date = Mtime;
 	memset(VenusSHA, 0, SHA_DIGEST_LENGTH);
     }
-    if (NewDate != (Date_t)-1) stat.Date = NewDate;
+    if (NewDate != (Date_t)-1) {
+	stat.Date = NewDate;
+
+	/* if this is an open file, the time will be set to the mtime of the
+	 * container file when the fd is closed. So we also update the mtime
+	 * of the container file so that this will become the mtime (as long
+	 * as there are no further writes). */
+	if (IsFile() && flags.owrite) {
+	    struct timeval times[2];
+	    times[0].tv_sec  = times[1].tv_sec  = NewDate;
+	    times[0].tv_usec = times[1].tv_usec = 0;
+	    data.file->Utimes(times);
+	}
+    }
     if (NewOwner != VA_IGNORE_UID) stat.Owner = NewOwner;
     if (NewMode != VA_IGNORE_MODE) stat.Mode = NewMode;
 
     UpdateCacheStats((IsDir() ? &FSDB->DirAttrStats : &FSDB->FileAttrStats),
-                     WRITE, NBLOCKS(sizeof(fsobj)));
+		     WRITE, NBLOCKS(sizeof(fsobj)));
 }
 
-int fsobj::ConnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
-			     Date_t NewDate, uid_t NewOwner,
-                             unsigned short NewMode, RPC2_CountedBS *acl)
+int fsobj::DisconnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
+			       Date_t NewDate, uid_t NewOwner,
+			       unsigned short NewMode, int prepend)
 {
-    FSO_ASSERT(this, HOARDING(this));
+    int code = 0;
+    repvol *rv;
+
+    if (!vol->IsReplicated())
+	return ETIMEDOUT;
+    rv = (repvol *)vol;
+
+    Recov_BeginTrans();
+    RPC2_Integer tNewMode = (short)NewMode;	    /* sign-extend!!! */
+
+    CODA_ASSERT(vol->IsReplicated());
+    code = rv->LogSetAttr(Mtime, uid, &fid, NewLength, NewDate, NewOwner,
+			  (RPC2_Unsigned)tNewMode, prepend);
+    if (code == 0 && prepend == 0)
+            /* It's already been updated if we're 'prepending',
+	     * which basically means it is a repair-related operation,
+	     * and doing it again would trigger an assertion. */
+	    LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
+    Recov_EndTrans(DMFP);
+
+    return(code);
+}
+
+int fsobj::SetAttr(struct coda_vattr *vap, uid_t uid)
+{
+	Date_t NewDate = (Date_t) -1;
+	unsigned long NewLength = (unsigned long) -1;
+	uid_t NewOwner = VA_IGNORE_UID;
+	unsigned short NewMode = VA_IGNORE_MODE;
+
+	LOG(10, ("fsobj::SetAttr: (%s), uid = %d\n", GetComp(), uid));
+	VPROC_printvattr(vap);
+
+	if ( vap->va_size != VA_IGNORE_SIZE )
+		NewLength = vap->va_size;
+
+	if ( (vap->va_mtime.tv_sec != VA_IGNORE_TIME1) &&
+	      (vap->va_mtime.tv_sec != (time_t)stat.Date))
+		NewDate = vap->va_mtime.tv_sec;
+
+        if (vap->va_uid != VA_IGNORE_UID && vap->va_uid != stat.Owner)
+		NewOwner = vap->va_uid;
+
+	if (vap->va_mode != VA_IGNORE_MODE) {
+            /* Only retain the actual user/group/other permission bits */
+            vap->va_mode &= (S_IRWXU | S_IRWXG | S_IRWXO);
+            if (vap->va_mode != stat.Mode)
+                NewMode= vap->va_mode;
+        }
+
+	/* When we are truncating to zero length, should create any missing
+	 * container files */
+	if (!NewLength && !HAVEDATA(this)) {
+	    Recov_BeginTrans();
+	    RVMLIB_REC_OBJECT(data.file);
+	    RVMLIB_REC_OBJECT(cf);
+	    data.file = &cf;
+            data.file->Create();
+	    Recov_EndTrans(MAXFP);
+	}
+
+	/* Only update cache file when truncating and open for write! */
+	if (NewLength != (unsigned long)-1 && WRITING(this)) {
+		Recov_BeginTrans();
+		data.file->Truncate((unsigned) NewLength);
+		Recov_EndTrans(MAXFP);
+		NewLength = (unsigned long)VA_IGNORE_SIZE;
+	}
+
+	/* Avoid performing action where possible. */
+	if (NewLength == (unsigned long)-1 && NewDate == (Date_t)-1 &&
+	    NewOwner == VA_IGNORE_UID && NewMode == VA_IGNORE_MODE)
+	{
+		return(0);
+	}
+
+	int code = 0;
+	Date_t Mtime = Vtime();
+
+	code = DisconnectedSetAttr(Mtime, uid, NewLength, NewDate,
+				   NewOwner, NewMode);
+
+	if (code != 0) {
+		Demote();
+	}
+	return(code);
+}
+
+
+int fsobj::SetACL(RPC2_CountedBS *acl, uid_t uid)
+{
+    LOG(10, ("fsobj::SetACL: (%s), uid = %d\n", GetComp(), uid));
+
+    if (!REACHABLE(this))
+	return ETIMEDOUT;
+
+    if (IsFake() || IsLocalObj())
+	/* Ignore attempts to set ACLs on fake objects. */
+	return EPERM;
+
+    if (DIRTY(this))
+	return EBUSY;
+
+    /* Since we cannot log this operation in the CML we should not be
+     * disconnected. The server also responds with a new status block that
+     * includes new access rights, so the object should not have pending
+     * operations in the log. */
+    CODA_ASSERT(acl != 0);
 
     int code = 0;
-    int setacl = (acl != 0);
-    const char *prel_str=setacl?"store::setacl %s\n"  :"store::setattr %s\n";
-    const char *post_str=setacl?"store::setacl done\n":"store::setattr done\n";
-
-    RPC2_Integer Mask = 0; 
-
-    /* Dummy argument for ACL. */
-    RPC2_CountedBS dummybs;
-    dummybs.SeqLen = 0;
-    if (!setacl)
-	acl = &dummybs;
 
     /* Status parameters. */
     ViceStatus status;
-    VenusToViceStatus(&stat, &status);
+    memset(&status, 0, sizeof(status));
 
-    status.Date = NewDate;
-    status.Owner = NewOwner;
-    status.Mode = NewMode;
-    status.Length = NewLength; 
+    /* setacl really only looks at these two */
+    status.DataVersion = stat.DataVersion;
+    status.VV = stat.VV;
 
-    if (NewDate != (Date_t)-1)
-      Mask |= SET_TIME;
+    /* but we clear these just in case... */
+    status.Date = (Date_t)-1;
+    status.Owner = (uid_t)-1;
+    status.Mode = (unsigned)-1;
+    status.Length = (unsigned)-1;
 
-    if (NewOwner != (uid_t)-1)
-      Mask |= SET_OWNER;
-
-    if (NewMode != (unsigned short)-1)
-      Mask |= SET_MODE;
-
-    if (NewLength != (unsigned long)-1) {
-	Mask |= SET_LENGTH;
-	Mask |= SET_TIME;
-	status.Date = Mtime;
-    }
-    LOG(100,("fsobj::ConnectedSetAttr Mask = %o\n", Mask));
+    LOG(100,("fsobj::ConnectedSetAcl\n"));
 
     /* COP2 Piggybacking. */
     char PiggyData[COP2SIZE];
@@ -1565,7 +1447,7 @@ int fsobj::ConnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
 	vv_t UpdateSet;
 
 	Recov_BeginTrans();
-	sid = vp->GenerateStoreId();
+	Recov_GenerateStoreId(&sid);
 	Recov_EndTrans(MAXFP);
 	{
 	    /* Make multiple copies of the IN/OUT and OUT parameters. */
@@ -1578,41 +1460,24 @@ int fsobj::ConnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
 	    ARG_MARSHALL(OUT_MODE, CallBackStatus, VCBStatusvar, VCBStatus, VSG_MEMBERS)
 
 	    /* Make the RPC call. */
-	    CFSOP_PRELUDE(prel_str, comp, fid);
-	    if (setacl) {
-		MULTI_START_MESSAGE(ViceSetACL_OP);
-		code = (int)MRPC_MakeMulti(ViceSetACL_OP, ViceSetACL_PTR,
-					   VSG_MEMBERS, m->rocc.handles,
-					   m->rocc.retcodes, m->rocc.MIp, 0, 0,
-					   MakeViceFid(&fid), acl,
-					   statusvar_ptrs, ph, &sid, &OldVS,
-					   VSvar_ptrs, VCBStatusvar_ptrs,
-					   &PiggyBS);
-		MULTI_END_MESSAGE(ViceSetACL_OP);
-		CFSOP_POSTLUDE(post_str);
+	    CFSOP_PRELUDE("store::setacl %s\n", comp, fid);
+	    MULTI_START_MESSAGE(ViceSetACL_OP);
+	    code = (int)MRPC_MakeMulti(ViceSetACL_OP, ViceSetACL_PTR,
+				       VSG_MEMBERS, m->rocc.handles,
+				       m->rocc.retcodes, m->rocc.MIp, 0, 0,
+				       MakeViceFid(&fid), acl,
+				       statusvar_ptrs, ph, &sid, &OldVS,
+				       VSvar_ptrs, VCBStatusvar_ptrs,
+				       &PiggyBS);
+	    MULTI_END_MESSAGE(ViceSetACL_OP);
+	    CFSOP_POSTLUDE("store::setacl done\n");
 
-		free(OldVS.SeqBody);
-		/* Collate responses from individual servers and decide what to
-		 * do next. */
-		code = vp->Collate_COP1(m, code, &UpdateSet);
-		MULTI_RECORD_STATS(ViceSetACL_OP);
-	    } else {
-		MULTI_START_MESSAGE(ViceSetAttr_OP);
-		code = (int)MRPC_MakeMulti(ViceSetAttr_OP, ViceSetAttr_PTR,
-					   VSG_MEMBERS, m->rocc.handles,
-					   m->rocc.retcodes, m->rocc.MIp, 0, 0,
-					   MakeViceFid(&fid), statusvar_ptrs,
-					   Mask, ph, &sid, &OldVS, VSvar_ptrs,
-					   VCBStatusvar_ptrs, &PiggyBS);
-		MULTI_END_MESSAGE(ViceSetAttr_OP);
-		CFSOP_POSTLUDE(post_str);
+	    free(OldVS.SeqBody);
+	    /* Collate responses from individual servers and decide what to
+	     * do next. */
+	    code = vp->Collate_COP1(m, code, &UpdateSet);
+	    MULTI_RECORD_STATS(ViceSetACL_OP);
 
-		free(OldVS.SeqBody);
-		/* Collate responses from individual servers and decide what to
-		 * do next. */
-		code = vp->Collate_COP1(m, code, &UpdateSet);
-		MULTI_RECORD_STATS(ViceSetAttr_OP);
-	    }
 	    if (code == EASYRESOLVE) { asy_resolve = 1; code = 0; }
 	    if (code != 0) goto RepExit;
 
@@ -1631,10 +1496,7 @@ int fsobj::ConnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
 	    ARG_UNMARSHALL(statusvar, status, dh_ix);
 	}
 
-	/* Do setattr locally. */
 	Recov_BeginTrans();
-	if (!setacl)
-		LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
 	UpdateStatus(&status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
@@ -1669,36 +1531,22 @@ RepExit:
 	if (code != 0) goto NonRepExit;
 
 	/* Make the RPC call. */
-	CFSOP_PRELUDE(prel_str, comp, fid);
-	if (setacl) {
-	    UNI_START_MESSAGE(ViceSetACL_OP);
-	    code = (int) ViceSetACL(c->connid, MakeViceFid(&fid), acl, &status,
-				    0, &Dummy, &OldVS, &VS, &VCBStatus,
-				    &PiggyBS);
-	    UNI_END_MESSAGE(ViceSetACL_OP);
-	    CFSOP_POSTLUDE("store::setacl done\n");
+	CFSOP_PRELUDE("store::setacl %s\n", comp, fid);
+	UNI_START_MESSAGE(ViceSetACL_OP);
+	code = (int) ViceSetACL(c->connid, MakeViceFid(&fid), acl, &status,
+				0, &Dummy, &OldVS, &VS, &VCBStatus,
+				&PiggyBS);
+	UNI_END_MESSAGE(ViceSetACL_OP);
+	CFSOP_POSTLUDE("store::setacl done\n");
 
-	    /* Examine the return code to decide what to do next. */
-	    code = vp->Collate(c, code);
-	    UNI_RECORD_STATS(ViceSetACL_OP);
-	} else {
-	    UNI_START_MESSAGE(ViceSetAttr_OP);
-	    code = (int) ViceSetAttr(c->connid, MakeViceFid(&fid), &status,
-				     Mask, 0, &Dummy, &OldVS, &VS, &VCBStatus,
-				     &PiggyBS);
-	    UNI_END_MESSAGE(ViceSetAttr_OP);
-	    CFSOP_POSTLUDE("store::setattr done\n");
+	/* Examine the return code to decide what to do next. */
+	code = vp->Collate(c, code);
+	UNI_RECORD_STATS(ViceSetACL_OP);
 
-	    /* Examine the return code to decide what to do next. */
-	    code = vp->Collate(c, code);
-	    UNI_RECORD_STATS(ViceSetAttr_OP);
-	}
 	if (code != 0) goto NonRepExit;
 
 	/* Do setattr locally. */
 	Recov_BeginTrans();
-	if (!setacl)
-		LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
 	UpdateStatus(&status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
@@ -1706,127 +1554,9 @@ NonRepExit:
 	PutConn(&c);
     }
 
-    return(code);
-}
-
-int fsobj::DisconnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength, Date_t NewDate, 
-			       uid_t NewOwner, unsigned short NewMode, int Tid) {
-    FSO_ASSERT(this, (EMULATING(this) || LOGGING(this)));
-
-    int code = 0;
-
-    Recov_BeginTrans();
-    RPC2_Integer tNewMode =	(short)NewMode;	    /* sign-extend!!! */
-
-    CODA_ASSERT(vol->IsReplicated());
-    code = ((repvol *)vol)->LogSetAttr(Mtime, uid, &fid, NewLength, NewDate,
-                                        NewOwner, (RPC2_Unsigned)tNewMode, Tid);
+    /* Cached rights are suspect now! */
     if (code == 0)
-	    LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
-    Recov_EndTrans(DMFP);
-
-    return(code);
-}
-
-int fsobj::SetAttr(struct coda_vattr *vap, uid_t uid, RPC2_CountedBS *acl) 
-{
-	Date_t NewDate = (Date_t) -1;
-	unsigned long NewLength = (unsigned long) -1;
-	uid_t NewOwner = VA_IGNORE_UID;
-	unsigned short NewMode = VA_IGNORE_MODE;
-
-
-	LOG(10, ("fsobj::SetAttr: (%s), uid = %d\n", GetComp(), uid));
-	VPROC_printvattr(vap);
-    
-	if ( vap->va_size != VA_IGNORE_SIZE ) 
-		NewLength = vap->va_size;
-
-	if ( (vap->va_mtime.tv_sec != VA_IGNORE_TIME1) && 
-	      (vap->va_mtime.tv_sec != (time_t)stat.Date))
-		NewDate = vap->va_mtime.tv_sec;
-
-        if (vap->va_uid != VA_IGNORE_UID && vap->va_uid != stat.Owner)
-		NewOwner = vap->va_uid;
-
-	if (vap->va_mode != VA_IGNORE_MODE) {
-            /* Only retain the actual user/group/other permission bits */
-            vap->va_mode &= (S_IRWXU | S_IRWXG | S_IRWXO);
-            if (vap->va_mode != stat.Mode)
-                NewMode= vap->va_mode;
-        }
-
-	/* When we are truncating to zero length, should create any missing
-	 * container files */
-	if (!NewLength && !HAVEDATA(this)) {
-	    Recov_BeginTrans();
-	    RVMLIB_REC_OBJECT(data.file);
-	    RVMLIB_REC_OBJECT(cf);
-	    data.file = &cf;
-            data.file->Create();
-	    Recov_EndTrans(MAXFP);
-	}
-	
-	/* Only update cache file when truncating and open for write! */
-	if (NewLength != (unsigned long)-1 && WRITING(this)) {
-		Recov_BeginTrans();
-		data.file->Truncate((unsigned) NewLength);
-		Recov_EndTrans(MAXFP);
-		NewLength = (unsigned long)VA_IGNORE_SIZE;
-	}
-
-	/* Avoid performing action where possible. */
-	if (NewLength == (unsigned long)-1 && NewDate == (Date_t)-1 &&
-	    NewOwner == VA_IGNORE_UID && NewMode == VA_IGNORE_MODE) {
-		if (acl == 0) return(0);
-	} else {
-		FSO_ASSERT(this, acl == 0);
-	}
-
-	int code = 0;
-	Date_t Mtime = Vtime();
-	
-	int conn, tid;
-	GetOperationState(&conn, &tid);
-	
-	if (conn == 0) {
-                code = DisconnectedSetAttr(Mtime, uid, NewLength, NewDate,
-                                           NewOwner, NewMode, tid);
-	} else {
-		code = ConnectedSetAttr(Mtime, uid, NewLength, NewDate, 
-                                        NewOwner, NewMode, acl);
-	}
-
-	if (code != 0) {
-		Demote();
-	}
-	return(code);
-}
-
-
-int fsobj::SetACL(RPC2_CountedBS *acl, uid_t uid)
-{
-    LOG(10, ("fsobj::SetACL: (%s), uid = %d\n", GetComp(), uid));
-
-    if (!HOARDING(this)) {
-	FSO_ASSERT(this, (EMULATING(this) || LOGGING(this)));
-
-	/* We don't cache ACLs! */
-	return(ETIMEDOUT);
-    }
-
-    if (IsFake() || IsLocalObj())
-	/* Ignore attempts to set ACLs on fake objects. */
-	return(EPERM);
-
-    struct coda_vattr va;
-    va_init(&va);
-    int code = SetAttr(&va, uid, acl);
-
-    if (code == 0) {
-	/* Cached rights are suspect now! */
 	Demote();
-    }
 
     return(code);
 }
@@ -1877,245 +1607,29 @@ void fsobj::LocalCreate(Date_t Mtime, fsobj *target_fso, char *name,
 }
 
 
-int fsobj::ConnectedCreate(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
-			    char *name, unsigned short Mode, int target_pri)
-{
-    FSO_ASSERT(this, HOARDING(this));
-
-    int code = 0;
-    fsobj *target_fso = 0;
-    VenusFid target_fid;
-    RPC2_Unsigned AllocHost = 0;
-
-    /*Status parameters. */
-    ViceStatus parent_status;
-    VenusToViceStatus(&stat, &parent_status);
-    ViceStatus target_status;
-    target_status.Mode = Mode;
-    {
-	/* Temporary!  Until RPC interface is fixed!  -JJK */
-	parent_status.Date = Mtime;
-	target_status.DataVersion = 0;
-	target_status.VV = NullVV;
-    }
-
-    /* COP2 Piggybacking. */
-    char PiggyData[COP2SIZE];
-    RPC2_CountedBS PiggyBS;
-    PiggyBS.SeqLen = 0;
-    PiggyBS.SeqBody = (RPC2_ByteSeq)PiggyData;
-
-    /* VCB Arguments */
-    RPC2_Integer VS = 0;
-    CallBackStatus VCBStatus = NoCallBack;
-    RPC2_CountedBS OldVS; 
-    OldVS.SeqLen = 0;
-    OldVS.SeqBody = 0;
-
-    if (vol->IsReplicated()) {
-	ViceStoreId sid;
-	mgrpent *m = 0;
-	int asy_resolve = 0;
-        repvol *vp = (repvol *)vol;
-
-	/* Allocate a fid for the new object. */
-	code = vp->AllocFid(File, &target_fid, &AllocHost, uid);
-	if (code != 0) goto RepExit;
-
-	/* Allocate the fsobj. */
-	target_fso = FSDB->Create(&target_fid, target_pri, name);
-	if (target_fso == 0) {
-	    UpdateCacheStats(&FSDB->FileAttrStats, NOSPACE,
-			     NBLOCKS(sizeof(fsobj)));
-	    code = ENOSPC;
-	    goto RepExit;
-	}
-	UpdateCacheStats(&FSDB->FileAttrStats, CREATE,
-			 NBLOCKS(sizeof(fsobj)));
-
-	/* Acquire an Mgroup. */
-	code = vp->GetMgrp(&m, uid, (PIGGYCOP2 ? &PiggyBS : 0));
-	if (code != 0) goto RepExit;
-
-	/* The COP1 call. */
-	long cbtemp; cbtemp = cbbreaks;
-	vv_t UpdateSet;
-
-	Recov_BeginTrans();
-	sid = vp->GenerateStoreId();
-	Recov_EndTrans(MAXFP);
-	{
-	    ViceFid target = *MakeViceFid(&target_fid);
-	    ViceFid nullfid = {0,0,0};
-
-	    /* Make multiple copies of the IN/OUT and OUT parameters. */
-	    vp->PackVS(VSG_MEMBERS, &OldVS);
-	    ARG_MARSHALL(IN_OUT_MODE, ViceStatus, target_statusvar, target_status, VSG_MEMBERS);
-	    ARG_MARSHALL(IN_OUT_MODE, ViceFid, targetvar, target, VSG_MEMBERS);
-	    ARG_MARSHALL(IN_OUT_MODE, ViceStatus, parent_statusvar, parent_status, VSG_MEMBERS);
-	    ARG_MARSHALL(OUT_MODE, RPC2_Integer, VSvar, VS, VSG_MEMBERS);
-	    ARG_MARSHALL(OUT_MODE, CallBackStatus, VCBStatusvar, VCBStatus, VSG_MEMBERS);
-
-	    /* Make the RPC call. */
-	    CFSOP_PRELUDE("store::Create %-30s\n", name, NullFid);
-	    MULTI_START_MESSAGE(ViceVCreate_OP);
-	    code = (int) MRPC_MakeMulti(ViceVCreate_OP, ViceVCreate_PTR,
-					VSG_MEMBERS, m->rocc.handles,
-					m->rocc.retcodes, m->rocc.MIp, 0, 0,
-					MakeViceFid(&fid), &nullfid, name,
-					target_statusvar_ptrs, targetvar_ptrs,
-					parent_statusvar_ptrs, AllocHost, &sid,
-					&OldVS, VSvar_ptrs, VCBStatusvar_ptrs,
-					&PiggyBS);
-	    MULTI_END_MESSAGE(ViceVCreate_OP);
-	    CFSOP_POSTLUDE("store::create done\n");
-
-	    free(OldVS.SeqBody);
-	    /* Collate responses from individual servers and decide what to do next. */
-	    code = vp->Collate_COP1(m, code, &UpdateSet);
-	    MULTI_RECORD_STATS(ViceVCreate_OP);
-	    if (code == EASYRESOLVE) { asy_resolve = 1; code = 0; }
-	    if (code != 0) goto RepExit;
-
-	    /* Collate volume callback information */
-	    if (cbtemp == cbbreaks)
-		vp->CollateVCB(m, VSvar_bufs, VCBStatusvar_bufs);
-
-	    /* Finalize COP2 Piggybacking. */
-	    if (PIGGYCOP2)
-		vp->ClearCOP2(&PiggyBS);
-
-	    /* Manually compute the OUT parameters from the mgrpent::Create() call! -JJK */
-	    int dh_ix; dh_ix = -1;
-	    (void)m->DHCheck(0, -1, &dh_ix);
-	    ARG_UNMARSHALL(target_statusvar, target_status, dh_ix);
-	    ARG_UNMARSHALL(targetvar, target, dh_ix);
-	    ARG_UNMARSHALL(parent_statusvar, parent_status, dh_ix);
-	    MakeVenusFid(&target_fid, target_fid.Realm, &target);
-	}
-
-	/* Do Create locally. */
-	Recov_BeginTrans();
-	LocalCreate(Mtime, target_fso, name, uid, Mode);
-	UpdateStatus(&parent_status, &UpdateSet, uid);
-	/* New objects already have a cleared SHA */
-	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
-	Recov_EndTrans(CMFP);
-	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
-	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
-	if (ASYNCCOP2) target_fso->ReturnEarly();
-
-	/* Send the COP2 message or add an entry for piggybacking. */
-        vp->COP2(m, &sid, &UpdateSet);
-
-RepExit:
-	if (m) m->Put();
-	switch(code) {
-	    case 0:
-		if (asy_resolve) {
-		    vp->ResSubmit(0, &fid);
-		    if (target_fso != 0)
-			vp->ResSubmit(0, &target_fso->fid);
-		}
-		break;
-
-	    case ETIMEDOUT:
-	    case ESYNRESOLVE:
-	    case EINCONS:
-		code = ERETRY;
-		break;
-
-	    default:
-		break;
-	}
-    }
-    else {
-	/* Acquire a Connection. */
-	connent *c;
-	ViceStoreId Dummy;                  /* Need an address for ViceCreate */
-	ViceFid nullf = {0, 0, 0};
-        volrep *vp = (volrep *)vol;
-	code = vp->GetConn(&c, uid);
-	if (code != 0) goto NonRepExit;
-
-	/* Make the RPC call. */
-	long cbtemp; cbtemp = cbbreaks;
-	CFSOP_PRELUDE("store::Create %-30s\n", name, NullFid);
-	UNI_START_MESSAGE(ViceVCreate_OP);
-	code = (int) ViceVCreate(c->connid, MakeViceFid(&fid), &nullf,
-				 (RPC2_String)name, &target_status,
-				 MakeViceFid(&target_fid), &parent_status, 0,
-				 &Dummy, &OldVS, &VS, &VCBStatus, &PiggyBS);
-	UNI_END_MESSAGE(ViceVCreate_OP);
-	CFSOP_POSTLUDE("store::create done\n");
-
-	/* Examine the return code to decide what to do next. */
-	code = vp->Collate(c, code);
-	UNI_RECORD_STATS(ViceVCreate_OP);
-	if (code != 0) goto NonRepExit;
-
-	/* Allocate the fsobj. */
-	target_fso = FSDB->Create(&target_fid, target_pri, name);
-	if (target_fso == 0) {
-	    UpdateCacheStats(&FSDB->FileAttrStats, NOSPACE,
-			     NBLOCKS(sizeof(fsobj)));
-	    code = ENOSPC;
-	    goto NonRepExit;
-	}
-	UpdateCacheStats(&FSDB->FileAttrStats, CREATE,
-			 NBLOCKS(sizeof(fsobj)));
-
-	/* Do Create locally. */
-	Recov_BeginTrans();
-	LocalCreate(Mtime, target_fso, name, uid, Mode);
-	UpdateStatus(&parent_status, NULL, uid);
-	target_fso->UpdateStatus(&target_status, NULL, uid);
-	Recov_EndTrans(CMFP);
-	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
-	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
-
-NonRepExit:
-	PutConn(&c);
-    }
-
-    if (code == 0) {
-	*t_fso_addr = target_fso;
-    }
-    else {
-	if (target_fso != 0) {
-	    FSO_ASSERT(target_fso, !HAVESTATUS(target_fso));
-	    Recov_BeginTrans();
-	    target_fso->Kill();
-	    Recov_EndTrans(DMFP);
-	    FSDB->Put(&target_fso);
-	}
-    }
-    return(code);
-}
-
 int fsobj::DisconnectedCreate(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
                               char *name, unsigned short Mode, int target_pri,
-                              int Tid)
+                              int prepend)
 {
-    FSO_ASSERT(this, (EMULATING(this) || LOGGING(this)));
-
     int code = 0;
     fsobj *target_fso = 0;
     VenusFid target_fid;
     RPC2_Unsigned AllocHost = 0;
+    repvol *rv;
 
     if (!vol->IsReplicated()) {
 	code = ETIMEDOUT;
 	goto Exit;
     }
-    
+    rv = (repvol *)vol;
+
     /* Allocate a fid for the new object. */
     /* if we time out, return so we will try again with a local fid. */
-    code = ((repvol *)vol)->AllocFid(File, &target_fid, &AllocHost, uid);
+    code = rv->AllocFid(File, &target_fid, &AllocHost, uid);
     if (code != 0) goto Exit;
 
     /* Allocate the fsobj. */
-    target_fso = FSDB->Create(&target_fid, target_pri, name);
+    target_fso = FSDB->Create(&target_fid, target_pri, name, &fid);
     if (target_fso == 0) {
 	UpdateCacheStats(&FSDB->FileAttrStats, NOSPACE,
 			 NBLOCKS(sizeof(fsobj)));
@@ -2126,10 +1640,14 @@ int fsobj::DisconnectedCreate(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
 		      NBLOCKS(sizeof(fsobj)));
 
     Recov_BeginTrans();
-    code = ((repvol *)vol)->LogCreate(Mtime, uid, &fid, name, &target_fso->fid, Mode, Tid);
+    code = rv->LogCreate(Mtime, uid, &fid, name, &target_fso->fid, Mode,
+			 prepend);
 
-    if (code == 0) {
+    if (code == 0 && prepend == 0) {
 	    /* This MUST update second-class state! */
+            /* It's already been updated if we're 'prepending',
+	     * which basically means it is a repair-related operation,
+	     * and doing it again would trigger an assertion. */
 	    LocalCreate(Mtime, target_fso, name, uid, Mode);
 
 	    /* target_fso->stat is not initialized until LocalCreate */
@@ -2167,17 +1685,8 @@ int fsobj::Create(char *name, fsobj **target_fso_addr,
     Date_t Mtime = Vtime();
     *target_fso_addr = 0;
 
-    int conn, tid;
-    GetOperationState(&conn, &tid);
-
-    if (conn == 0) {
-	code = DisconnectedCreate(Mtime, uid, target_fso_addr,
-				  name, Mode, target_pri, tid);
-    }
-    else {
-	code = ConnectedCreate(Mtime, uid, target_fso_addr,
-			       name, Mode, target_pri);
-    }
+    code = DisconnectedCreate(Mtime, uid, target_fso_addr,
+			      name, Mode, target_pri);
 
     if (code != 0) {
 	Demote();
