@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -39,8 +39,6 @@ extern "C" {
 #include <unistd.h>
 #include <stdlib.h>
 
-#include <netdb.h>
-
 #include <rpc2/rpc2.h>
 #include <vice.h>
 /* from libal */
@@ -94,6 +92,7 @@ int fsobj::Open(int writep, int truncp, struct venus_cnode *cp, uid_t uid)
     FSO_HOLD(this);			/* Pin object until close arrives. */
     openers++;
     if (writep) {
+	FSO_ASSERT(this, IsFile());
 	Writers++;
 	if (!flags.owrite) {
 	    Recov_BeginTrans();
@@ -101,18 +100,20 @@ int fsobj::Open(int writep, int truncp, struct venus_cnode *cp, uid_t uid)
 	    FSDB->owriteq->append(&owrite_handle);
 	    RVMLIB_REC_OBJECT(flags);
 	    flags.owrite = 1;
-	    Recov_EndTrans(((EMULATING(this) || LOGGING(this)) ? DMFP : CMFP));
+	    Recov_EndTrans(DMFP);
 	}
+	/* Do truncate if necessary. */
+	if (truncp) {
+	    struct coda_vattr va;
+	    va_init(&va);
+	    va.va_size = 0;
+	    if ((code = SetAttr(&va, uid)) != 0)
+		goto Exit;
+	}
+	/* set the container file timestamp */
+	data.file->Utimes(NULL);
     }
 
-    /* Do truncate if necessary. */
-    if (truncp && writep) {	/* truncp is acted upon only if writep */
-	struct coda_vattr va; va_init(&va);
-	va.va_size = 0;
-	if ((code = SetAttr(&va, uid)) != 0)
-	    goto Exit;
-   }
-
     /* If object is directory make sure Unix-format contents are valid. */
     if (IsDir()) {
 	if (data.dir->udcf == 0) {
@@ -145,7 +146,7 @@ int fsobj::Open(int writep, int truncp, struct venus_cnode *cp, uid_t uid)
                    the fsobj. */
 		int tfd = ::open(data.dir->udcf->name, O_BINARY | O_RDWR | O_CREAT, V_MODE);
 		if (tfd < 0) CHOKE("fsobj::Open: open");
-#if !defined(DJGPP) && !defined(__CYGWIN32__)
+#ifndef __CYGWIN32__
 		if (::fchmod(tfd, V_MODE) < 0)
 		    CHOKE("fsobj::Open: fchmod");
 		if (::fchown(tfd, (uid_t)V_UID, (gid_t)V_GID) < 0)
@@ -168,8 +169,10 @@ int fsobj::Open(int writep, int truncp, struct venus_cnode *cp, uid_t uid)
 	}
     }
 
-    if (cp)
+    if (cp) {
 	cp->c_cf = IsDir() ? data.dir->udcf : data.file;
+	if (!cp->c_cf) code = EIO;
+    }
 
 Exit:
     if (code != 0) {
@@ -218,12 +221,10 @@ int fsobj::Sync(uid_t uid)
     unsigned long NewLength;
     Date_t NewDate;
     {
-        struct stat tstat;
-        data.file->Stat(&tstat);
-        if (tstat.st_size == 0) tstat.st_mtime = Vtime();
-
-        NewLength = tstat.st_size;
-        NewDate = tstat.st_mtime;
+	struct stat tstat;
+	data.file->Stat(&tstat);
+	NewLength = tstat.st_size;
+	NewDate = tstat.st_mtime;
     }
     int old_blocks = (int) BLOCKS(this);
     int new_blocks = (int) NBLOCKS(NewLength);
@@ -294,7 +295,7 @@ void fsobj::Release(int writep)
                 RVMLIB_REC_OBJECT(stat.Length);
                 stat.Length = 0;	    /* Necessary for blocks maintenance! */
             }
-            Recov_EndTrans(((EMULATING(this) || LOGGING(this)) ? DMFP : CMFP));
+            Recov_EndTrans(DMFP);
         }
     }
 
@@ -323,7 +324,7 @@ int fsobj::Access(int rights, int modes, uid_t uid)
     LOG(10, ("fsobj::Access : (%s, %d, %d), uid = %d\n",
 	      GetComp(), rights, modes, uid));
 
-    int code = 0, disconnected;
+    int code = 0, connected;
 
 #define PRSFS_MUTATE (PRSFS_WRITE | PRSFS_DELETE | PRSFS_INSERT | PRSFS_LOCK)
     /* Disallow mutation of backup, rw-replica, and zombie volumes. */
@@ -352,13 +353,6 @@ int fsobj::Access(int rights, int modes, uid_t uid)
 	LockLevel level;
 	VenusFid parent_fid;
 
-	/* check if the object is GlobalRootObj for a local-fake tree */
-	/* we can safely return 0 here. because if the parent's acl is updated
-	 * during disconnection, then "this" object won't become a global root
-	 * node. */
-	if (LRDB->RFM_IsGlobalRoot(&fid))
-	    return 0;
-
 	/* Check mode bits if necessary. */
 	/* There should be a special case if this user is the creator.
 	   This code used to have a test for `virginity', but only the kernel
@@ -416,16 +410,13 @@ int fsobj::Access(int rights, int modes, uid_t uid)
 	return(code);
     }
 
-    disconnected = EMULATING(this) || (DIRTY(this) && LOGGING(this));
-    code = CheckAcRights(uid, rights, !disconnected);
+    connected = REACHABLE(this) && !DIRTY(this); /* use FETCHABLE here? */
+    code = CheckAcRights(uid, rights, connected);
     if (code != ENOENT)
 	return code;
 
-    if (disconnected)
-	return EACCES;
-
-    /* XXX we might be 'RESOLVING', what will happen then? -JH */
-    FSO_ASSERT(this, (HOARDING(this) || (LOGGING(this) && !DIRTY(this))));
+    /* ENOENT should only be returned when we are connected */
+    FSO_ASSERT(this, connected);
 
     /* We must re-fetch status; rights will be returned as a side-effect. */
     /* Promote the lock level if necessary. */
@@ -442,7 +433,7 @@ int fsobj::Access(int rights, int modes, uid_t uid)
 	if (code != 0) return(code);
     }
 
-    code = CheckAcRights(uid, rights, 1);
+    code = CheckAcRights(uid, rights, 0);
     return (code == 0) ? 0 : EACCES;
 }
 
@@ -450,10 +441,9 @@ int fsobj::Access(int rights, int modes, uid_t uid)
 /* local-repair modification */
 /* inc_fid is an OUT parameter which allows caller to form "fake symlink" if it desires. */
 /* Explicit parameter for TRAVERSE_MTPTS? -JJK */
-int fsobj::Lookup(fsobj **target_fso_addr, VenusFid *inc_fid, char *name,
-		  uid_t uid, int flags)
+int fsobj::Lookup(fsobj **target_fso_addr, VenusFid *inc_fid, const char *name, uid_t uid, int flags, int GetInconsistent)
 {
-    LOG(10, ("fsobj::Lookup: (%s/%s), uid = %d\n", GetComp(), name, uid));
+  LOG(10, ("fsobj::Lookup: (%s/%s), uid = %d, GetInconsistent = %d\n", GetComp(), name, uid, GetInconsistent));
 
     /* We're screwed if (name == "." or ".."). -JJK */
     CODA_ASSERT(!STREQ(name, ".") && !STREQ(name, ".."));
@@ -504,18 +494,10 @@ int fsobj::Lookup(fsobj **target_fso_addr, VenusFid *inc_fid, char *name,
     {
 	int status = RC_STATUS;
 get_object:
-	code = FSDB->Get(&target_fso, &target_fid, uid, status, name);
-
-	if (code) {
-	    if (code == EINCONS && inc_fid != 0) *inc_fid = target_fid;
-
-	    /* If the getattr failed, the object might not exist on all
-	     * servers. This is `fixed' by resolving the parent, but we just
-	     * destroyed the object and RecResolve won't work. That is why we
-	     * submit this directory for resolution. -JH */
-	    if (code == ESYNRESOLVE && vol->IsReplicated())
-		((repvol *)vol)->ResSubmit(&((VprocSelf())->u.u_resblk), &fid);
-
+	code = FSDB->Get(&target_fso, &target_fid, uid, status, name, NULL, NULL, GetInconsistent);
+	if (code && !(GetInconsistent && code == EINCONS)) {
+	    if (code == EINCONS && inc_fid != 0)
+		*inc_fid = target_fid;
 	    goto done;
 	}
 
@@ -554,8 +536,6 @@ get_object:
 
 	    /* If the target is a covered mount point, cross it. */
 	    if (target_fso->IsMtPt()) {
-		LOG(100, ("fsobj::Lookup: crossing mount point\n"));
-
 		/* Get the volume root, and release the mount point. */
 		fsobj *root_fso = target_fso->u.root;
 		root_fso->Lock(RD);
@@ -585,13 +565,14 @@ int fsobj::Readlink(char *buf, unsigned long len, int *cc, uid_t uid)
 	      GetComp(), buf, len, cc, uid));
 
     if (!HAVEALLDATA(this))
-	{ print(logFile); CHOKE("fsobj::Readlink: called without data"); }
+	{ print(logFile); CHOKE("fsobj::Readlink: called without data and isn't fake!"); }
+
     if (!IsSymLink() && !IsMtPt())
-	return(EINVAL);
+	  return(EINVAL);
 
     if (stat.Length > len - 1) {
-	eprint("readlink: contents > bufsize");
-	return(EINVAL);
+	  eprint("readlink: contents > bufsize");
+	  return(EINVAL);
     }
 
     /* Fill in the buffer. */
@@ -600,7 +581,8 @@ int fsobj::Readlink(char *buf, unsigned long len, int *cc, uid_t uid)
     end = ptr + stat.Length;
     out = buf;
     while (ptr != end) {
-	char *subst, *c;
+	const char *subst;
+	char *c;
 	unsigned int tmp_len;
 
 	c = strchr(ptr, '@');
