@@ -202,7 +202,7 @@ class hdb {
 
     /* Constructors, destructors. */
     void *operator new(size_t);
-    void operator delete(void *, size_t);
+    void operator delete(void *);
     hdb();
     void ResetTransient();
     ~hdb() { abort(); }
@@ -229,7 +229,6 @@ class hdb {
     /* Helper Routines hdb::Walk */
     void ValidateCacheStatus(vproc *, int *, int *);
     void ListPriorityQueue();
-    int GetSuspectPriority(Volid *, char *, uid_t);
     void WalkPriorityQueue(vproc *, int *, int *);
     int CalculateTotalBytesToFetch();
     void StatusWalk(vproc *, int *, int *);
@@ -240,8 +239,6 @@ class hdb {
     void SetSolicitAdvice(int uid)
         { LOG(0, ("SetSolicitAdvice: uid=%d\n",uid));
 	  SolicitAdvice = uid; }
-    int MakeAdviceRequestFile(char *);
-    void RequestHoardWalkAdvice();
 
     void SetDemandWalkTime();
     long GetDemandWalkTime();
@@ -289,7 +286,7 @@ class hdbent {
     hdbent(VolumeId, char *, char *, uid_t, int, int, int);
     void ResetTransient();
     ~hdbent();
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
   public:
     void print() { print(stdout); }
@@ -372,7 +369,7 @@ class namectxt {
     namectxt(namectxt&);		/* not supported! */
     int operator=(namectxt&);		/* not supported! */
     ~namectxt();
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
     void Demote(int recursive=0);		/* --> immediate or eventual transition to suspect state */
     void CheckComponent(fsobj *);
@@ -385,7 +382,7 @@ class namectxt {
     void print(int, void * =0);
     void printsuspect(int, int);
     void getpath(char *);
-    void putmsg(int fd, char *reason, int include_modifier);
+    void putmsg(int fd, const char *reason, int include_modifier);
 };
 
 #ifdef	VENUSDEBUG
