@@ -56,101 +56,101 @@ extern "C" {
 /* interfaces */
 #include <cml.h>
 
-/* ********** beginning of cmlent methods ********** */
-/* must be called from within a transaction */
-void cmlent::TranslateFid(VenusFid *global, VenusFid *local)
-{
-    OBJ_ASSERT(this, global && local);
-    LOG(100, ("cmlent::TranslateFid: global = %s local = %s\n",
-	      FID_(global), FID_(local)));
-    VenusFid *Fids[3];
-    GetAllFids(Fids);
-    int count = 0;
-    for (int i = 0; i < 3; i++) {
-        /* Check if Fids[i] is global */
-	if (Fids[i] && FID_EQ(Fids[i], global)) {
-            RVMLIB_REC_OBJECT(*Fids[i]);
-            *Fids[i] = *local;
-            count++;
-	}
-    }
-    LOG(100, ("cmlent::TranslateFid: %d fids have been replaced\n", count));
-}
-
-/* must not be called from within a transaction */
-int cmlent::LocalFakeify()
+/* Find a consistent server replica to act as our 'global'.
+ * All of this is necessary to create these replica objects in fsdb::Get. */
+static int GetGlobalReplica(fsobj **global, VenusFid *fid)
 {
+    VolumeId volumeids[VSG_MEMBERS];
+    volent *vol;
+    repvol *rvol;
     int rc;
-    VenusFid *Fids[3];
-    GetAllFids(Fids);
-    /* 
-     * note that for each cmlent, Fids[0] is always the root fid of the
-     * subtree that are affected by the IFT. the only exception is 
-     * for a rename operation where Fids[2] could be the root fid of
-     * another subtree that is affected by the operation.
-     */
-    VenusFid *fid = Fids[0];
-    OBJ_ASSERT(this, !FID_IsLocalFake(fid));
-    fsobj *root;
-    OBJ_ASSERT(this, root = FSDB->Find(fid));
-    if (DYING(root)) {
-	LOG(100, ("cmlent::LocalFakeify: object %s removed\n",
-		  FID_(fid)));
-	/* it must belong to some local subtree to be repaired */
-	SetRepairFlag();
-	/* prevent this cmlent from being aborted or reintegrated later */
-	return ENOENT;
+    VenusFid replicafid = *fid;
+    vproc *vp = VprocSelf();
+
+    CODA_ASSERT(global);
+    *global = NULL;
+
+    VDB->Get(&vol, MakeVolid(fid));
+    if (!vol) return VNOVOL;
+
+    rc = EVOLUME;
+    if (!vol->IsReplicated())
+	goto unlock_out;
+
+    rvol = (repvol *)vol;
+    rvol->GetVids(volumeids);
+
+    for (int i = 0; i < VSG_MEMBERS; i++) {
+	if (!volumeids[i]) continue;
+
+	replicafid.Volume = volumeids[i];
+	rc = FSDB->Get(global, &replicafid, vp->u.u_uid, RC_DATA);
+	if (rc == 0) break;
+
+	/* any error code is no good, even EINCONS */
+	/* This will probably be an error point when we get to handling
+	 * mixed local/global and server/server conflicts, where EINCONS
+	 * would be an acceptable error return. */
+	if (*global)
+	    FSDB->Put(global);
+
+	continue;
     }
-    rc = root->LocalFakeify();
-    if (rc != 0) return rc;
-    
-    fid = Fids[2];
-    if (!fid || FID_IsLocalFake(fid)) {
-	SetRepairFlag();
-	return (0);
+
+    if (*global) {
+	LOG(0, ("CheckRepair_GetObjects: using global %s -> %s\n",
+		FID_(&replicafid), FID_(fid)));
+
+	/* XXX this isn't safe, we drop the lock, but don't have some other
+	 * refcount to pin the object down. */
+	FSDB->Put(global);
+	rc = VFAIL;
+	*global = FSDB->Find(&replicafid);
     }
-    OBJ_ASSERT(this, root = FSDB->Find(fid));
-    if (DYING(root)) {
-	LOG(100, ("cmlent::LocalFakeify: object %s removed\n",
-		  FID_(fid)));
-	SetRepairFlag();
-	return ENOENT;
-    }
-    rc = root->LocalFakeify();
-    if (rc == 0) {
-	SetRepairFlag();	
-    }
-    return rc;
+
+unlock_out:
+    VDB->Put(&vol);
+
+    return (*global) ? 0 : rc;
 }
 
+/* ********** beginning of cmlent methods ********** */
 /* CheckRepair step 1: check mutation operand(s) */
 static int CheckRepair_GetObjects(const char *operation, VenusFid *fid,
 				  fsobj **global, fsobj **local,
 				  char *msg, int *mcode, int *rcode)
 {
+    vproc *vp = VprocSelf();
     char path[MAXPATHLEN];
     int rc;
 
     LOG(100, ("cmlent::CheckRepair: %s on %s\n", operation, FID_(fid)));
 
-    rc = LRDB->FindRepairObject(fid, global, local);
+    /* Get is used instead of Find to get a useful error code */
+    rc = FSDB->Get(local, fid, vp->u.u_uid, RC_DATA);
     if (rc != 0) {
 	/* figure out what the error was */
-	(*local)->GetPath(path, 1);
+
+        path[0] = '\0';
+	if(*local)
+	  (*local)->GetPath(path, 1);
 	*mcode = MUTATION_MISS_TARGET;
 	*rcode = REPAIR_FAILURE;
 	switch (rc) {
 	case EIO:
 	case ENOENT:
-	    sprintf(msg, "conflict: %s target %s no longer exits on servers", operation, path);
+	    sprintf(msg, "conflict: %s target %s no longer exists on servers",
+		    operation, path);
 	    break;
 	case EINCONS:
-	    sprintf(msg, "conflict: %s target %s in server/server conflict", operation, path);
+	    sprintf(msg, "conflict: %s target %s in server/server conflict",
+		    operation, path);
 	    break;
 	case ESYNRESOLVE:
 	case EASYRESOLVE:
 	case ERETRY:
-	    sprintf(msg, "fetching %s target failed (%d), please re-try!", operation, rc);
+	    sprintf(msg, "fetching %s target failed (%d), please re-try!",
+		    operation, rc);
 	    break;
 	default:
 	    sprintf(msg, "fetching %s target failed (%d)", operation, rc);
@@ -158,7 +158,15 @@ static int CheckRepair_GetObjects(const char *operation, VenusFid *fid,
 	return rc;
     }
 
-    CODA_ASSERT(*global != NULL);
+    FSDB->Put(local); /* release RW lock */
+    *local = FSDB->Find(fid); /* find the object instead */
+
+    rc = GetGlobalReplica(global, fid);
+    if(rc || !(*global)) {
+      LOG(0, ("CheckRepair_GetObjects: Couldn't find a global replica for fid (%s)!\n", FID_(fid)));
+      return rc;
+    }
+
     return 0;
 }
 
@@ -172,9 +180,9 @@ static int CheckRepair_CheckAccess(fsobj *fso, int rights,
 
     ret = fso->CheckAcRights(vp->u.u_uid, rights, 0);
     if (ret == EACCES) {
-	LOG(100, ("cmlent::CheckRepair: acl check failed\n"));
+	LOG(0, ("cmlent::CheckRepair: acl check failed\n"));
 	fso->GetPath(path, 1);
-	sprintf(msg, "conflict: acl check failure on parent %s", path);
+	sprintf(msg, "conflict: acl check failure on parent %s\n", path);
 	*mcode = MUTATION_ACL_FAILURE;
 	*rcode = REPAIR_FAILURE;
 	return -1;
@@ -190,12 +198,12 @@ static int CheckRepair_CheckVVConflict(fsobj *fso, ViceVersionVector *VV,
 
     if (VV_Cmp(fso->VV(), VV) != VV_EQ) {
 	fso->GetPath(path, 1);
-	sprintf(msg, "conflict: target %s updated on servers", path);
+	sprintf(msg, "conflict: target %s updated on servers\n", path);
 	return -1;
     }
     return 0;
 }
- 
+
 /* CheckRepair step 3: check mutation semantic integrity, name/name conflicts */
 static int CheckRepair_CheckNameConflict(fsobj *fso, char *name, int need,
 					 char *msg, int *mcode, int *rcode)
@@ -207,7 +215,7 @@ static int CheckRepair_CheckNameConflict(fsobj *fso, char *name, int need,
     exists = (fso->dir_Lookup(name, &dummy, CLU_CASE_SENSITIVE) == 0);
     if ((!need && exists) || (need && !exists)) {
 	fso->GetPath(path, 1);
-	sprintf(msg, "conflict: target %s/%s %s on servers",
+	sprintf(msg, "conflict: target %s/%s %s on servers\n",
 		path, name, need ? "is missing" : "exists");
 	*mcode = MUTATION_NN_CONFLICT;
 	*rcode = REPAIR_FAILURE;
@@ -222,16 +230,19 @@ static int CheckRepair_CheckNameConflict(fsobj *fso, char *name, int need,
 */
 /* need not be called from within a transaction */
 void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
-{	
+{
     /*
      * this method checks whether the mutation operation of this cmlent can be
      * performed "in commit order" to the global state that is current visible
      * to venus. it has the following OUT parameters:
-     * msg  : a string of diagnostic error messages to be used by the repair tool.
-     * mcode: a mutation error code indicating the nature of local/global conflict.
-     * rcode: a repair error code indicating nature of the needed repair operation.
+     * msg  : a string of diagnostic error messages to be used by the repair
+     *        tool.
+     * mcode: a mutation error code indicating the nature of local/global
+     *        conflict.
+     * rcode: a repair error code indicating nature of the needed repair
+     *        operation.
      */
-    
+
     /* decl of shared local variables */
     fsobj *ParentObj;
     int rc = 0;
@@ -240,7 +251,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
     fsobj *GlobalObjs[3] = {NULL, NULL, NULL};
     fsobj *LocalObjs[3]  = {NULL, NULL, NULL};
     strcpy(msg, "no conflict");
-    
+
     switch (opcode) {
     case CML_Store_OP:
 	/* step 1 */
@@ -249,14 +260,16 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	if (rc) break;
 
 	/* step 2 */
-	OBJ_ASSERT(this, ParentObj = LRDB->GetGlobalParentObj(&GlobalObjs[0]->fid));
+	/* can't use global pfso cause it doesn't exist, nor does pfid */
+	/* if this doesn't work, use local->pfid with global->fid.Volume */
+	OBJ_ASSERT(this, (ParentObj = LocalObjs[0]->pfso)); /*used to be global */
 	rc = CheckRepair_CheckAccess(ParentObj, PRSFS_WRITE, msg, mcode, rcode);
 	if (rc) break;
 
 	/* step 3 */
 	*mcode = MUTATION_VV_CONFLICT;
 	*rcode = REPAIR_OVER_WRITE;
-	rc = CheckRepair_CheckVVConflict(GlobalObjs[0], LocalObjs[0]->VV(),msg);
+	rc = CheckRepair_CheckVVConflict(LocalObjs[0], GlobalObjs[0]->VV(), msg);
 	break;
 
     case CML_Utimes_OP:
@@ -266,14 +279,14 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	if (rc) break;
 
 	/* step 2 */
-	OBJ_ASSERT(this, ParentObj = LRDB->GetGlobalParentObj(&GlobalObjs[0]->fid));
+	OBJ_ASSERT(this, (ParentObj = LocalObjs[0]->pfso));
 	rc = CheckRepair_CheckAccess(ParentObj, PRSFS_WRITE, msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	*mcode = MUTATION_VV_CONFLICT;
 	*rcode = REPAIR_OVER_WRITE;
-	rc = CheckRepair_CheckVVConflict(GlobalObjs[0], LocalObjs[0]->VV(),msg);
+	rc = CheckRepair_CheckVVConflict(LocalObjs[0], GlobalObjs[0]->VV(), msg);
 	break;
 
     case CML_Chown_OP:
@@ -283,15 +296,15 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	if (rc) break;
 
 	/* step 2 */
-	OBJ_ASSERT(this, ParentObj = LRDB->GetGlobalParentObj(&GlobalObjs[0]->fid));
+	OBJ_ASSERT(this, (ParentObj = LocalObjs[0]->pfso));
 	rc = CheckRepair_CheckAccess(ParentObj, PRSFS_ADMINISTER,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	*mcode = MUTATION_VV_CONFLICT;
 	*rcode = REPAIR_OVER_WRITE;
-	rc = CheckRepair_CheckVVConflict(GlobalObjs[0], LocalObjs[0]->VV(),msg);
+	rc = CheckRepair_CheckVVConflict(LocalObjs[0], GlobalObjs[0]->VV(), msg);
 	break;
 
     case CML_Chmod_OP:
@@ -301,14 +314,14 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	if (rc) break;
 
 	/* step 2 */
-	OBJ_ASSERT(this, ParentObj = LRDB->GetGlobalParentObj(&GlobalObjs[0]->fid));
+	OBJ_ASSERT(this, (ParentObj = LocalObjs[0]->pfso));
 	rc = CheckRepair_CheckAccess(ParentObj, PRSFS_WRITE, msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	*mcode = MUTATION_VV_CONFLICT;
 	*rcode = REPAIR_OVER_WRITE;
-	rc = CheckRepair_CheckVVConflict(GlobalObjs[0], LocalObjs[0]->VV(),msg);
+	rc = CheckRepair_CheckVVConflict(LocalObjs[0], GlobalObjs[0]->VV(), msg);
 	break;
 
     case CML_Create_OP:
@@ -322,7 +335,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_INSERT,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	rc = CheckRepair_CheckNameConflict(GlobalObjs[0], (char *)Name, 0,
 					   msg, mcode, rcode);
@@ -340,11 +353,11 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 				    msg, mcode, rcode);
 	if (rc) break;
 
-	/* step2 */
+	/* step 2 */
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_INSERT,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	rc = CheckRepair_CheckNameConflict(GlobalObjs[0], (char *)Name, 0,
 					   msg, mcode, rcode);
@@ -361,7 +374,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_INSERT,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	rc = CheckRepair_CheckNameConflict(GlobalObjs[0], (char *)NewName, 0,
 					   msg, mcode, rcode);
@@ -377,7 +390,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_INSERT,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	rc = CheckRepair_CheckNameConflict(GlobalObjs[0], (char *)Name, 0,
 					   msg, mcode, rcode);
@@ -392,6 +405,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_GetObjects("remove target", &u.u_remove.CFid,
 				    &GlobalObjs[1], &LocalObjs[1],
 				    msg, mcode, rcode);
+	if (rc) break;
 
 	OBJ_ASSERT(this, (LocalObjs[1] == NULL) || DYING(LocalObjs[1]) ||
 		         (LocalObjs[1]->stat.LinkCount > 0));
@@ -400,7 +414,7 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_DELETE,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	*mcode = MUTATION_RU_CONFLICT;
 	*rcode = REPAIR_FORCE_REMOVE;
@@ -423,14 +437,14 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_DELETE,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	if (!(GlobalObjs[1]->dir_IsEmpty())) {
 	    char path[MAXPATHLEN];
 	    GlobalObjs[1]->GetPath(path, 1);
 	    sprintf(msg, "conflict: target %s not empty on servers\n", path);
 	    *mcode = MUTATION_VV_CONFLICT;
-	    *rcode = REPAIR_FAILURE;    
+	    *rcode = REPAIR_FAILURE;
 	    rc = -1;
 	    break;
 	}
@@ -461,11 +475,11 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
 	rc = CheckRepair_CheckAccess(GlobalObjs[0], PRSFS_DELETE,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	rc = CheckRepair_CheckAccess(GlobalObjs[1], PRSFS_INSERT,
 				     msg, mcode, rcode);
 	if (rc) break;
- 
+
 	/* step 3 */
 	rc = CheckRepair_CheckNameConflict(GlobalObjs[0], (char *)Name, 1,
 					   msg, mcode, rcode);
@@ -486,7 +500,22 @@ void cmlent::CheckRepair(char *msg, int *mcode, int *rcode)
     if (!rc)
 	*mcode = *rcode = 0;
 
-    LOG(100, ("cmlent::CheckRepair: mcode = %d rcode = %d msg = %s\n", *mcode, *rcode, msg));
+    LOG(0, ("cmlent::CheckRepair: mcode = %d rcode = %d msg = %s\n", *mcode, *rcode, msg));
+}
+
+
+static int DoRepair_GetObjects(VenusFid *fid, fsobj **global, fsobj **local)
+{
+    int rc;
+
+    LOG(100, ("cmlent::DoRepair: (%s)\n", FID_(fid)));
+
+    *local = FSDB->Find(fid);
+    rc = GetGlobalReplica(global, fid);
+    if (rc) return rc;
+
+    CODA_ASSERT((*global) && (*local)); /* bad news at the moment*/
+    return 0;
 }
 
 /*
@@ -506,14 +535,15 @@ int cmlent::DoRepair(char *msg, int rcode)
     char GlobalPath[MAXPATHLEN], LocalPath[MAXPATHLEN];
     switch (opcode) {
     case CML_Store_OP:
-	{  /* 
-	    * copy the cache file of the local object into the cache file 
+	{  /*
+	    * copy the cache file of the local object into the cache file
 	    * file of the global, then s Store call on the global object.
 	    */
 	    fid = &u.u_store.Fid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
-		sprintf(msg, "can not obtain global mutation objects (%d)", code);
+		sprintf(msg, "can not obtain global mutation objects (%d)\n",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, GObj && LObj && GObj->IsFile() && LObj->IsFile());
@@ -521,57 +551,112 @@ int cmlent::DoRepair(char *msg, int rcode)
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
 
 	    if (!HAVEALLDATA(LObj))
+		CHOKE("DoRepair: Store with no local data!\n");
+
+#if 0 /* old */
+	    /* call on _replicated_ volume (used to be global) */
+	    /* since our VV struct must match global VV, do a SetLocalVV */
+	    code = LObj->SetLocalVV(GObj->VV());
+	    if(code) {
+	      LOG(0, ("cmlent::DoRepair: Updating local VV failed!: %d\n",
+		      code));
+	      sprintf(msg, "store failed(couldn't set VV)\n");
+	      break;
+	    }
+	    code = LObj->RepairStore();
+#else /* new, uses individual VV's */
+	    {
+	      struct in_addr volumehosts[VSG_MEMBERS];
+	      VolumeId volumeids[VSG_MEMBERS];
+	      VenusFid replicafid = *fid;
+	      fsobj *replicas[VSG_MEMBERS];
+	      volent *vol;
+	      vproc *vp = VprocSelf();
+
+	      VDB->Get(&vol, MakeVolid(fid));
+	      CODA_ASSERT(vol && vol->IsReplicated());
+
+	      repvol *rv = (repvol *)vol;
+
+	      rv->GetHosts(volumehosts);
+	      rv->GetVids(volumeids);
+	      VDB->Put(&vol);
+
+	      if (!HAVEALLDATA(LObj))
 		CHOKE("DoRepair: Store with no local data!");
 
-	    /* copy the local-obj cache file into the global-obj cache */
-	    LObj->data.file->Copy(GObj->data.file);
+	      code = 0;
+	      for (int i = 0; (i < VSG_MEMBERS) && !code; i++) {
+		if (!volumehosts[i].s_addr) continue;
+		srvent *s = FindServer(&volumehosts[i]);
+		CODA_ASSERT(s != NULL);
 
-	    /* set the global-obj length to the local-obj length */
-	    GObj->stat.Length = LObj->stat.Length;
-	    code = GObj->RepairStore();
-	    GObj->GetPath(GlobalPath, 1);
+		replicafid.Volume = volumeids[i];
+		code = FSDB->Get(&(replicas[i]), &replicafid,
+				 vp->u.u_uid, RC_DATA);
+		if(code || !replicas[i]) {
+		  LOG(0, ("cmlent::DoRepair: failed fsdb::Get of %s(%d)\n",
+			  FID_(&replicafid), code));
+		  if(replicas[i])
+		    FSDB->Put(&(replicas[i]));
+		  break;
+		}
+
+		/* copy the local-obj cache file into the global-obj cache */
+		LObj->data.file->Copy(replicas[i]->data.file);
+
+		code = replicas[i]->RepairStore();
+		LOG(0, ("cmlent::DoRepair: repair-storing (%s) %s\n",
+			FID_(&replicafid), (code ? "failed" : "succeeded")));
+
+		FSDB->Put(&(replicas[i]));
+	      }
+	    }
+#endif
+	    LObj->GetPath(LocalPath, 1);
 	    if (rcode == REPAIR_OVER_WRITE) {
-		LObj->GetPath(LocalPath, 1);
 		if (code == 0) {
-		    sprintf(msg, "overwrite %s with %s succeeded", GlobalPath, LocalPath);
+		    sprintf(msg, "overwrite %s succeeded\n", LocalPath);
 		} else {
-		    sprintf(msg, "overwrite %s with %s failed(%d)", GlobalPath, LocalPath, code);
+		    sprintf(msg, "overwrite %s failed(%d)\n", LocalPath, code);
 		}
 	    } else {
 		if (code == 0) {
-		    sprintf(msg, "store %s succeeded", GlobalPath);
+		    sprintf(msg, "store %s succeeded\n", LocalPath);
 		} else {
-		    sprintf(msg, "store %s failed(%d)", GlobalPath, code);
+		    sprintf(msg, "store %s failed(%d)\n", LocalPath, code);
 		}
 	    }
 	    break;
 	}
     case CML_Chmod_OP:
-	{   
+	{
 	    fid = &u.u_chmod.Fid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
-		sprintf(msg, "can not obtain global mutation objects (%d)", code);
+		sprintf(msg, "can not obtain global mutation objects (%d)\n", code);
 		break;
 	    }
 	    LOG(100, ("cmlent::DoRepair: do chmod on %s and %s\n",
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
 	    unsigned short NewMode = LObj->stat.Mode;		/* use local new mode */
-	    GObj->stat.Mode = NewMode;			        /* set mode for global-obj */
-	    code = GObj->RepairSetAttr((unsigned long)-1, (unsigned long)-1, 
+	    code = LObj->RepairSetAttr((unsigned long)-1, (unsigned long)-1,
 				       (unsigned short)-1, NewMode,
 				       (RPC2_CountedBS *)NULL);
-	    GObj->GetPath(GlobalPath, 1);
+	    LObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "chmod %s succeeded", GlobalPath);
+		sprintf(msg, "chmod %s succeeded", LocalPath);
 	    } else {
-		sprintf(msg, "chmod %s failed(%d)", GlobalPath, code);
+		sprintf(msg, "chmod %s failed(%d)", LocalPath, code);
 	    }
 	    break;
 	}
     case CML_Chown_OP:
-	{   fid = &u.u_chown.Fid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+	{
+	    fid = &u.u_chown.Fid;
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -580,22 +665,22 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do chown on %s and %s\n",
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
 	    uid_t NewOwner = LObj->stat.Owner; 		/* use local new owner */
-	    GObj->stat.Owner = NewOwner; 	    		/* set for global-obj */
-	    code = GObj->RepairSetAttr((unsigned long)-1, (unsigned long)-1, 
+	    code = LObj->RepairSetAttr((unsigned long)-1, (unsigned long)-1,
 				       NewOwner, (unsigned short)-1,
 				       (RPC2_CountedBS *)NULL);
-	    GObj->GetPath(GlobalPath, 1);
+	    LObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "chown %s succeeded", GlobalPath);
+		sprintf(msg, "chown %s succeeded", LocalPath);
 	    } else {
-		sprintf(msg, "chown %s failed(%d)", GlobalPath, code);
+		sprintf(msg, "chown %s failed(%d)", LocalPath, code);
 	    }
 	    break;
 	}
     case CML_Utimes_OP:
-	{   
+	{
 	    fid = &u.u_chown.Fid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -603,62 +688,69 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, GObj && LObj);
 	    LOG(100, ("cmlent::DoRepair: do utimes on %s and %s\n",
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
-	    Date_t NewDate = LObj->stat.Date;			/* use local date */
-	    GObj->stat.Date = NewDate;	    			/* set time-stamp for global-obj */
-	    code = GObj->RepairSetAttr((unsigned long)-1, NewDate, (unsigned short)-1,
-				       (unsigned short)-1, NULL);
-	    GObj->GetPath(GlobalPath, 1);
+	    Date_t NewDate = LObj->stat.Date;		/* use local date */
+	    code = LObj->RepairSetAttr((unsigned long)-1, NewDate,
+				       (unsigned short)-1, (unsigned short)-1,
+				       NULL);
+	    LObj->GetPath(GlobalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "setattr %s succeeded", GlobalPath);
+		sprintf(msg, "setattr %s succeeded", LocalPath);
 	    } else {
-		sprintf(msg, "setattr %s failed(%d)", GlobalPath, code);
+		sprintf(msg, "setattr %s failed(%d)", LocalPath, code);
 	    }
 	    break;
 	}
     case CML_Create_OP:
 	{   /* do fid replacement after the creation succeeded. */
 	    fid = &u.u_create.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
-		sprintf(msg, "can not obtain global mutation objects (%d)", code);
+		sprintf(msg, "can not obtain global mutation objects (%d)",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, GPObj && LPObj && GPObj->IsDir() && LPObj->IsDir());
 	    LOG(100, ("cmlent::DoRepair: do create on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_create.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != EIO && code != ENOENT) {
-		sprintf(msg, "create semantic re-validation failed (%d)", code);
+		sprintf(msg, "create semantic re-validation failed (%d)",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, LObj != NULL && LObj->IsFile());
 	    unsigned short NewMode = LObj->stat.Mode;
 	    fsobj *target = NULL;
-	    code = GPObj->RepairCreate(&target, (char *)Name, NewMode, FSDB->StdPri());
-	    GPObj->GetPath(GlobalPath, 1);
+	    code = LPObj->RepairCreate(&target, (char *)Name, NewMode,
+				       FSDB->StdPri());
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "create %s/%s succeeded", GlobalPath, (char *)Name);
-		target->UnLock(WR);		/* release write lock on the new object */
-		LRDB->ReplaceRepairFid(&target->fid, &u.u_create.CFid);
+		sprintf(msg, "create %s/%s succeeded", LocalPath,
+			(char *)Name);
+		target->UnLock(WR);  /* release write lock on the new object */
 	    } else {
-		sprintf(msg, "create %s/%s failed(%d)", GlobalPath, (char *)Name, code);
+		sprintf(msg, "create %s/%s failed(%d)", LocalPath,
+			(char *)Name, code);
 	    }
 	    break;
 	}
     case CML_Link_OP:
-	{   
+	{
 	    fid = &u.u_link.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
-		sprintf(msg, "can not obtain global mutation objects (%d)", code);
+		sprintf(msg, "can not obtain global mutation objects (%d)",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, GPObj && LPObj && GPObj->IsDir() && LPObj->IsDir());
 	    LOG(100, ("cmlent::DoRepair: do link on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_link.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -666,19 +758,19 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, GObj && LObj && GObj->IsFile() && LObj->IsFile());
 	    LOG(100, ("cmlent::DoRepair: do link on target %s and %s\n",
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
-	    code = GPObj->RepairLink((char *)Name, GObj);
-	    GPObj->GetPath(GlobalPath, 1);
+	    code = LPObj->RepairLink((char *)Name, GObj);
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "link %s/%s succeeded", GlobalPath, (char *)Name);
+		sprintf(msg, "link %s/%s succeeded", LocalPath, (char *)Name);
 	    } else {
-		sprintf(msg, "link %s/%s failed(%d)", GlobalPath, (char *)Name, code);
+		sprintf(msg, "link %s/%s failed(%d)", LocalPath, (char *)Name, code);
 	    }
 	    break;
 	}
     case CML_MakeDir_OP:
-	{   
+	{
 	    fid = &u.u_link.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -687,7 +779,8 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do mkdir on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_mkdir.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != EIO && code != ENOENT) {
 		sprintf(msg, "mkdir semantic re-validation failed (%d)", code);
 		break;
@@ -695,21 +788,22 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, LObj != NULL && LObj->IsDir());
 	    unsigned short NewMode = LObj->stat.Mode;
 	    fsobj *target = NULL;
-	    code = GPObj->RepairMkdir(&target, (char *)Name, NewMode, FSDB->StdPri());
-	    GPObj->GetPath(GlobalPath, 1);
+	    code = LPObj->RepairMkdir(&target, (char *)Name,
+				      NewMode, FSDB->StdPri());
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "mkdir %s/%s succeeded", GlobalPath, (char *)Name);
-		target->UnLock(WR);			/* relese write lock on the new object */
-		LRDB->ReplaceRepairFid(&target->fid, &u.u_mkdir.CFid);
+		sprintf(msg, "mkdir %s/%s succeeded", LocalPath, (char *)Name);
+		target->UnLock(WR);  /* release write lock on the new object */
 	    } else {
-		sprintf(msg, "mkdir %s/%s failed(%d)", GlobalPath, (char *)Name, code);
+		sprintf(msg, "mkdir %s/%s failed(%d)",
+			LocalPath, (char *)Name, code);
 	    }
 	    break;
 	}
     case CML_SymLink_OP:
-	{   
+	{
 	    fid = &u.u_symlink.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -718,41 +812,44 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do symlink on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_symlink.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != EIO && code != ENOENT) {
-		sprintf(msg, "symlink semantic re-validation failed (%d)", code);
+		sprintf(msg, "symlink semantic re-validation failed (%d)",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, LObj != NULL && LObj->IsSymLink());
 	    unsigned short NewMode = LObj->stat.Mode;
 	    fsobj *target = NULL;
-	    code = GPObj->RepairSymlink(&target, (char *)NewName,
+	    code = LPObj->RepairSymlink(&target, (char *)NewName,
 					(char *)Name, NewMode, FSDB->StdPri());
-	    GPObj->GetPath(GlobalPath, 1);
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
 		sprintf(msg, "symlink %s/%s -> %s succeeded",
-			GlobalPath, (char *)NewName, (char *)Name);
-		target->UnLock(WR);			/* relese write lock on the new object */
-		LRDB->ReplaceRepairFid(&target->fid, &u.u_symlink.CFid);
+			LocalPath, (char *)NewName, (char *)Name);
+		target->UnLock(WR);   /* relese write lock on the new object */
 	    } else {
 		sprintf(msg, "symlink %s/%s -> %s failed(%d)",
-			GlobalPath, (char *)NewName, (char *)Name, code);
+			LocalPath, (char *)NewName, (char *)Name, code);
 	    }
 	    break;
 	}
     case CML_Remove_OP:
-	{   
+	{
 	    fid = &u.u_remove.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
-		sprintf(msg, "can not obtain global mutation objects (%d)", code);
+		sprintf(msg, "can not obtain global mutation objects (%d)",
+			code);
 		break;
 	    }
 	    OBJ_ASSERT(this, GPObj && LPObj && GPObj->IsDir() && LPObj->IsDir());
 	    LOG(100, ("cmlent::DoRepair: do remove on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_remove.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -760,19 +857,19 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, GObj != NULL && (LObj == NULL || DYING(LObj) || LObj->stat.LinkCount > 0));
 	    LOG(100, ("cmlent::DoRepair: do remove on global target %s\n",
 		      FID_(&GObj->fid)));
-	    code = GPObj->RepairRemove((char *)Name, GObj);
-	    GPObj->GetPath(GlobalPath, 1);
+	    code = LPObj->RepairRemove((char *)Name, GObj);
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "remove %s/%s succeeded", GlobalPath, (char *)Name);
+		sprintf(msg, "remove %s/%s succeeded", LocalPath, (char *)Name);
 	    } else {
-		sprintf(msg, "remove %s/%s failed(%d)", GlobalPath, (char *)Name, code);
+		sprintf(msg, "remove %s/%s failed(%d)", LocalPath, (char *)Name, code);
 	    }
 	    break;
 	}
     case CML_RemoveDir_OP:
-	{   
+	{
 	    fid = &u.u_rmdir.PFid;
-	    code = LRDB->FindRepairObject(fid, &GPObj, &LPObj);
+	    code = DoRepair_GetObjects(fid, &GPObj, &LPObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -781,7 +878,8 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do rmdir on parent %s and %s\n",
 		      FID_(&GPObj->fid), FID_(&LPObj->fid)));
 	    fid = &u.u_rmdir.CFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -789,21 +887,21 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, GObj != NULL && (LObj == NULL || DYING(LObj)) && GObj->IsDir());
 	    LOG(100, ("cmlent::DoRepair: do rmdir on global target %s\n",
 		      FID_(&GObj->fid)));
-	    code = GPObj->RepairRmdir((char *)Name, GObj);
-	    GPObj->GetPath(GlobalPath, 1);
+	    code = LPObj->RepairRmdir((char *)Name, GObj);
+	    LPObj->GetPath(LocalPath, 1);
 	    if (code == 0) {
-		sprintf(msg, "rmdir %s/%s succeeded", GlobalPath, (char *)Name);
+		sprintf(msg, "rmdir %s/%s succeeded", LocalPath, (char *)Name);
 	    } else {
-		sprintf(msg, "rmdir %s/%s failed(%d)", GlobalPath, (char *)Name, code);
+		sprintf(msg, "rmdir %s/%s failed(%d)", LocalPath, (char *)Name, code);
 	    }
 	    break;
 	}
     case CML_Rename_OP:
-	{   
+	{
 	    fsobj *GSPObj, *LSPObj;
 	    fsobj *GTPObj, *LTPObj;
 	    fid = &u.u_rename.SPFid;
-	    code = LRDB->FindRepairObject(fid, &GSPObj, &LSPObj);
+	    code = DoRepair_GetObjects(fid, &GSPObj, &LSPObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -812,7 +910,7 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do rename on source parent %s and %s\n",
 		      FID_(&GSPObj->fid), FID_(&LSPObj->fid)));
 	    fid = &u.u_rename.TPFid;
-	    code = LRDB->FindRepairObject(fid, &GTPObj, &LTPObj);
+	    code = DoRepair_GetObjects(fid, &GTPObj, &LTPObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -821,7 +919,8 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    LOG(100, ("cmlent::DoRepair: do rename on target parent %s and %s\n",
 		      FID_(&GTPObj->fid), FID_(&LTPObj->fid)));
 	    fid = &u.u_rename.SFid;
-	    code = LRDB->FindRepairObject(fid, &GObj, &LObj);
+
+	    code = DoRepair_GetObjects(fid, &GObj, &LObj);
 	    if (code != 0) {
 		sprintf(msg, "can not obtain global mutation objects (%d)", code);
 		break;
@@ -829,17 +928,17 @@ int cmlent::DoRepair(char *msg, int rcode)
 	    OBJ_ASSERT(this, GObj && LObj);
 	    LOG(100, ("cmlent::DoRepair: do rename on source object %s and %s\n",
 		      FID_(&GObj->fid), FID_(&LObj->fid)));
-	    code = GTPObj->RepairRename(GSPObj, (char *)Name,
+	    code = LTPObj->RepairRename(GSPObj, (char *)Name,
 					GObj, (char *)NewName,
 					(fsobj *)NULL);
-	    /* 
+	    /*
 	     * note that the target object is always NULL here because a disconnected rename
 	     * operation with target object is always split into a rename without a target
 	     * preceeded by a remove operation for that target object.
 	     */
 	    char SPath[MAXPATHLEN], TPath[MAXPATHLEN];
-	    GSPObj->GetPath(SPath, 1);
-	    GTPObj->GetPath(TPath, 1);
+	    LSPObj->GetPath(SPath, 1);
+	    LTPObj->GetPath(TPath, 1);
 	    if (code == 0) {
 		sprintf(msg, "rename %s/%s -> %s/%s succeeded",
 			SPath, (char *)Name, TPath, (char *)NewName);
@@ -862,6 +961,7 @@ int cmlent::DoRepair(char *msg, int rcode)
     return code;
 }
 
+
 /* need not be called from within a transaction */
 void cmlent::GetLocalOpMsg(char *msg)
 {
@@ -962,80 +1062,8 @@ void cmlent::SetRepairFlag()
     }
 }
 
-/* must not be called from within a transaction */
-void cmlent::SetRepairMutationFlag()
-{
-    if (flags.repair_mutation == 0) {
-	Recov_BeginTrans();
-	RVMLIB_REC_OBJECT(flags);
-	flags.repair_mutation = 1;
-	Recov_EndTrans(MAXFP);
-    }
-}
-
 /* need not be called from within a transaction */
-int cmlent::InLocalRepairSubtree(VenusFid *LocalRootFid)
-{
-    /*
-     * check whether the objects mutated by this cmlent belongs to
-     * the subtree rooted at the object whose fid equals RootFid.
-     */
-    OBJ_ASSERT(this, LocalRootFid && FID_IsLocalFake(LocalRootFid));
-    LOG(100, ("cmlent::InLocalRepairSubtree: LocalRootFid = %s\n",
-	      FID_(LocalRootFid)));
-    VenusFid *Fids[3];
-    fsobj *OBJ;
-    GetAllFids(Fids);
-
-    for (int i = 0; i < 3; i++) {
-	if (!Fids[i]) continue;
-	if (!FID_IsLocalFake(Fids[i])) continue;
-	OBJ_ASSERT(this, OBJ = FSDB->Find(Fids[i]));
-	if (OBJ->IsAncestor(LocalRootFid)) return 1;
-    }
-    
-    return 0;
-}
-
-/* need not be called from within a transaction */
-int cmlent::InGlobalRepairSubtree(VenusFid *GlobalRootFid)
-{
-    /*
-     * check whether this cmlent is a mutation that was performed to mutate
-     * objects contained in the global portion of a fake-subtree, rooted at 
-     * GlobalRootFid. note that this mutation was not performed within the
-     * repair-session for the subtree, otherwise, it could have been set to 
-     * the right flag bits and right tid. note that we do not need to worry about 
-     * this cmlent pointing to deleted objects because they are dirty and will 
-     * not be GCed, so FSDB->Find can alway get the objects we need and OBJ::IsAncestor 
-     * also works on dying objects because parent/child relationship is destroyed 
-     * only at GC time.
-     */
-    OBJ_ASSERT(this, GlobalRootFid && !FID_IsLocalFake(GlobalRootFid));
-    LOG(100, ("cmlent::InGlobalRepairSubtree: GlobalRootFid = %s\n",
-	      FID_(GlobalRootFid)));
-
-    if (flags.to_be_repaired || flags.repair_mutation) {
-	LOG(100, ("cmlent::InGlobalRepairSubtree: repair flag(s) set already\n"));
-	return 0;
-    }
-
-    VenusFid *Fids[3];
-    fsobj *OBJ;
-    GetAllFids(Fids);
-
-    for (int i = 0; i < 3; i++) {
-	if (!Fids[i]) continue;
-	if (FID_IsLocalFake(Fids[i])) break;
-	OBJ = FSDB->Find(Fids[i]);
-	if (!OBJ) continue;
-	if (OBJ->IsAncestor(GlobalRootFid)) return 1;
-    }
-    return 0;
-}
-
-/* need not be called from within a transaction */
-void cmlent::GetVVandFids(ViceVersionVector *vvs[], VenusFid *fids[]) 
+void cmlent::GetVVandFids(ViceVersionVector *vvs[], VenusFid *fids[])
 {
     fids[0] = 0; fids[1] = 0; fids[2] = 0;
     vvs[0] = 0; vvs[1] = 0; vvs[2] = 0;
