@@ -44,7 +44,6 @@ extern "C" {
 #endif
 #include <netinet/in.h>
 #include <errno.h>
-#include <netdb.h>
 #include <stdarg.h>
 #include "coda_string.h"
 #include <unistd.h>
@@ -61,10 +60,7 @@ extern "C" {
 #endif
 
 #include <mkpath.h>
-
-#ifndef INADDR_LOOPBACK
-#define INADDR_LOOPBACK 0x7f000001
-#endif
+#include <coda_getaddrinfo.h>
 
 /* from venus */
 #include "fso.h"
@@ -73,11 +69,10 @@ extern "C" {
 #include "vproc.h"
 #include "worker.h"
 #include "realmdb.h"
+#include "venusvol.h"
 
 const int MarinerStackSize = 65536;
 const int MaxMariners = 25;
-const char MarinerService[] = "venus";
-const char MarinerProto[] = "tcp";
 fd_set MarinerMask;
 int MarinerMaxFD = -1;
 
@@ -130,45 +125,50 @@ void MarinerInit() {
 
 Next:
     if (mariner_tcp_enable) {
-	unsigned short marinerport = htonl(2430);
+	struct RPC2_addrinfo hints, *ai = NULL;
+	int rc;
 
-        /* Look up the well-known CODA mariner service. */
-        struct servent *serventp = getservbyname(MarinerService, MarinerProto);
-        if (serventp) marinerport = serventp->s_port;
-	else eprint("mariner service port lookup failed, using tcp:2430");
+	memset(&hints, 0, sizeof(struct RPC2_addrinfo));
+	hints.ai_family   = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
 
-        /* Socket at which we rendevous with new mariner clients. */
-        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-            eprint("MarinerInit: bogus socket (%d)", sock);
-            goto Done;
-        }
+	rc = coda_getaddrinfo(NULL, "venus", &hints, &ai);
+	if (rc) {
+	    eprint("MarinerInit: failed to resolve loopback address");
+	    goto Done;
+	}
 
-        /* Make address reusable. */
-#ifndef DJGPP
-        setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(int));
-#endif
+	/* Socket at which we rendevous with new mariner clients. */
+	sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+	if (sock < 0) {
+	    eprint("MarinerInit: bogus socket");
+	    RPC2_freeaddrinfo(ai);
+	    goto Done;
+	}
 
-        /* Bind to it. */
-        struct sockaddr_in sin;
-        sin.sin_family = AF_INET;
-        sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-        sin.sin_port = marinerport;
-        if (bind(sock, (struct sockaddr *)&sin, (socklen_t) sizeof(sin)) < 0) {
-            eprint("MarinerInit: bind failed (%d)", errno);
-            close(sock);
-            goto Done;
-        }
+	/* Make address reusable. */
+	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(int));
 
-        /* Listen for requesters. */
-        if (listen(sock, 2) < 0) {
-            eprint("MarinerInit: mariner listen failed (%d)", errno);
-            close(sock);
-            goto Done;
-        }
-        
-        eprint("Mariner: listening on tcp port enabled, "
-               "disable with -noMarinerTcp");
-        mariner::tcp_muxfd = sock;
+	/* Bind to it. */
+	rc = bind(sock, ai->ai_addr, ai->ai_addrlen);
+	RPC2_freeaddrinfo(ai);
+
+	if (rc < 0) {
+	    eprint("MarinerInit: bind failed (%d)", errno);
+	    close(sock);
+	    goto Done;
+	}
+
+	/* Listen for requesters. */
+	if (listen(sock, 2) < 0) {
+	    eprint("MarinerInit: mariner listen failed (%d)", errno);
+	    close(sock);
+	    goto Done;
+	}
+
+	eprint("Mariner: listening on tcp port enabled, "
+	       "disable with -noMarinerTcp");
+	mariner::tcp_muxfd = sock;
     }
 Done:
     /* Allows the MessageMux to distribute incoming messages to us. */
@@ -209,43 +209,12 @@ void MarinerMux(fd_set *mask)
     else if (mariner::nmariners >= MaxMariners)
         ::close(newfd);
     else if (newfd >= 0) {
-#ifndef DJGPP
         if (::fcntl(newfd, F_SETFL, O_NONBLOCK) < 0) {
             eprint("MarinerMux: fcntl failed (%d)", errno);
             ::close(newfd);
         }
-#else
-        if (::__djgpp_set_socket_blocking_mode(newfd, 1) < 0) {
-            eprint("MarinerMux: set nonblock failed (%d)", errno);
-            ::close(newfd);
-        }
-#endif
         else new mariner(newfd);
     }
-
-    /* Dispatch mariners which have pending requests, and kill dying mariners. */
-    mariner_iterator next;
-    mariner *m, *n = NULL;
-    while ((m = n ? n : next())) {
-	n = NULL;
-	if (m->dying) {
-	    n = next();
-	    delete m;
-	    continue;
-	}
-	if (FD_ISSET(m->fd, mask)) {
-	    m->DataReady = 1;
-	    if (m->idle) {
-		if (m->Read() < 0) {
-		    n = next();
-		    delete m;
-		    continue;
-		}
-		m->idle = 0;
-		VprocSignal((char *)m);
-	    }
-	}
-    }
 }
 
 
@@ -297,6 +266,33 @@ void MarinerReport(VenusFid *fid, uid_t uid) {
 	}
 }
 
+void MarinerReportVolState(const char *volname, const char *realm,
+			   const char *state, int cml_entries,
+			   const struct VolFlags *vflags)
+{
+    char buf[CODA_MAXNAMLEN+1];
+    mariner_iterator next;
+    mariner *m;
+    int len;
+
+    len = snprintf(buf, CODA_MAXNAMLEN, "volstate::%s@%s %s %d%s%s%s%s%s",
+		   volname, realm, state, cml_entries,
+		   vflags->unauthenticated ? " unauth" : "",
+		   vflags->reint_conflict ? " conflict" : "",
+		   vflags->asr_running ? " asr" : "",
+		   vflags->reintegrating ? " reint" : "",
+		   vflags->transition_pending ? " transition" : "");
+
+    if (len < 0) return;
+    if (len > CODA_MAXNAMLEN)
+	len = CODA_MAXNAMLEN;
+    buf[len] = '\n'; len++;
+
+    while ((m = next()))
+	if (m->WantVolState())
+	    m->write(buf, len);
+}
+
 
 void PrintMariners() {
     PrintMariners(stdout);
@@ -328,18 +324,14 @@ mariner::mariner(int afd) :
 
     nmariners++;	/* Ought to be a lock protecting this! -JJK */
 
-    DataReady = 0;
     dying = 0;
     logging = 0;
     reporting = 0;
+    want_volstate = 0;
     uid = ANYUSER_UID;
     fd = afd;
     memset(commbuf, 0, MWBUFSIZE);
 
-    FD_SET(fd, &MarinerMask);
-    if (fd > MarinerMaxFD)
-	MarinerMaxFD = fd;
-
     /* Poke main procedure. */
     start_thread();
 }
@@ -362,29 +354,8 @@ mariner::~mariner() {
 
     nmariners--;	/* Ought to be a lock protecting this! -JJK */
 
-    if (fd) {
+    if (fd)
 	::close(fd);
-	FD_CLR(fd, &MarinerMask);
-    }
-}
-
-
-int mariner::Read() {
-    if (!DataReady) CHOKE("mariner::Read: not DataReady!");
-
-    /* Pull the next message out of the socket. */
-    int n = ::read(fd, commbuf, (int)(sizeof(commbuf) - 1));
-    DataReady = 0;
-    if (n <= 0) return(-1);
-
-    /* Erase trailing CR/LF and null-terminate the command. */
-    char lastc = commbuf[--n];
-    if (lastc != '\012' && lastc != '\015') return(-1);
-    while (n > 0 && ((lastc = commbuf[--n]) == '\012' || lastc == '\015'))
-	;
-    commbuf[n + 1] = 0;
-
-    return(0);
 }
 
 
@@ -403,20 +374,43 @@ int mariner::Write(const char *fmt, ...) {
 }
 
 
-void mariner::AwaitRequest() {
+int mariner::AwaitRequest()
+{
+    fd_set fds;
+    unsigned int idx = 0;
+    int n;
+
     idle = 1;
 
-    if (DataReady && !dying) {
-	if (Read() < 0)
-	    dying = 1;
-	else {
-	    idle = 0;
-	    return;
-	}
-    }
+    while (idx < (sizeof(commbuf) - 1) && !dying)
+    {
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
 
-    VprocWait((char *)this);
-    if (dying) CHOKE("mariner::AwaitRequest: signalled while dying!");
+	n = VprocSelect(fd+1, &fds, 0, 0, NULL);
+	if (n < 0) break;
+	if (n == 0 || !FD_ISSET(fd, &fds))
+	    continue;
+
+	n = ::read(fd, &commbuf[idx], 1);
+	if (n < 0 && errno == EAGAIN)
+	    continue;
+
+	if (n <= 0)
+	    return -1;
+
+	if (commbuf[idx] == '\r')
+	    continue;
+
+	if (commbuf[idx] == '\n') {
+	    commbuf[idx] = '\0';
+	    /* end of line seen, return to caller */
+	    break;
+	}
+	idx++;
+    }
+    idle = 0;
+    return 0;
 }
 
 
@@ -439,10 +433,8 @@ void mariner::main(void)
 
     for (;;) {
 	/* Wait for new request. */
-	AwaitRequest();
-
-	/* Sanity check new request. */
-	if (idle) CHOKE("Mariner: signalled but not dispatched!");
+	if (AwaitRequest())
+	    break;
 
 	LOG(100, ("mariner::main: cmd = \"%s\"\n", commbuf));
 
@@ -455,7 +447,8 @@ void mariner::main(void)
 "Commands are:\n"
 "\thelp, debugon, debugoff, dumpcore, quit, rpc2t\n"
 "\tcop <modes>, set:fetch, clear:fetch, reporton <uid>, reportoff\n"
-"\t, fd <fd>pathstat <pathname>, fidstat <fid>, rpc2stat, print <args>\n");
+"\tset:volstate, clear:volstate, fd <fd>pathstat <pathname>\n"
+"\tfidstat <fid>, rpc2stat, print <args>\n");
 	}
 	else if (STREQ(argv[0], "debugon")) {
 	    DebugOn();
@@ -508,6 +501,18 @@ void mariner::main(void)
 	else if (STREQ(argv[0], "reportoff")) {
 	    reporting = 0;
 	}
+	else if (STREQ(argv[0], "set:volstate")) {
+	    repvol_iterator next;
+	    repvol *v;
+
+	    want_volstate = 1;
+
+	    while ((v = next()))
+		v->ReportVolState();
+	}
+	else if (STREQ(argv[0], "clear:volstate")) {
+	    want_volstate = 0;
+	}
 	else if (STREQ(argv[0], "fd") && argc == 2) {
 	    struct stat tstat;
 	    if (::fstat(atoi(argv[1]), &tstat) < 0)
@@ -540,7 +545,7 @@ void mariner::main(void)
 	    Rpc2Stat();
 	}
 	else if (STREQ(argv[0], "print")) {
-	    VenusPrint(fd, argc - 1, &argv[1]);
+	    VenusPrint(fd, argc - 1, (const char **)&argv[1]);
 	}
 	else {
 	    Write("bad mariner command '%-80s'\n", argv[0]);
@@ -587,7 +592,7 @@ void mariner::FidStat(VenusFid *fid) {
 	    break;
 	}
 
-	u.u_error = FSDB->Get(&f, fid, u.u_uid, RC_STATUS, NULL, NULL, 1);
+	u.u_error = FSDB->Get(&f, fid, u.u_uid, RC_STATUS, NULL, NULL, NULL, 1);
 	if (u.u_error) {
 	    Write("fsdb::Get(%s) failed (%d)\n", FID_(fid), u.u_error);
 	    goto FreeLocks;
