@@ -33,6 +33,7 @@ extern "C" {
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <errno.h>
 #include <signal.h>
 #include <unistd.h>
@@ -46,9 +47,9 @@ extern "C" {
 #include "sighand.h"
 #include "venus.private.h"
 #include "venusrecov.h"
+#include "venusvol.h"
 #include "worker.h"
-#include "adv_monitor.h"
-#include "adv_daemon.h"
+#include "fso.h"
 #include "codaconf.h"
 #include "daemonizer.h"
 
@@ -56,6 +57,7 @@ static void SigControl(int);
 static void SigChoke(int);
 static void SigExit(int);
 static void SigMounted(int);
+static void SigASR(int);
 
 int TerminateVenus;
 int mount_done;
@@ -63,10 +65,8 @@ int mount_done;
 void SigInit(void)
 {
     /* Establish/Join our own process group to avoid extraneous signals. */
-#ifndef DJGPP
   if (setpgid(0, 0) < 0)
     eprint("SigInit: setpgid failed (%d)", errno);
-#endif /* !DJGPP */
 
     /* set up the signal handlers */
     struct sigaction sa;
@@ -79,9 +79,6 @@ void SigInit(void)
 #ifdef SIGIO
     sigaction(SIGIO, &sa, NULL);
 #endif
-#ifdef SIGCHLD
-    sigaction(SIGCHLD, &sa, NULL);
-#endif
 #ifdef SIGTSTP
     sigaction(SIGTSTP, &sa, NULL);
 #endif
@@ -147,6 +144,12 @@ void SigInit(void)
      * SIGIOT  == SIGABRT
      * SIGPOLL == SIGIO
      */
+
+    sa.sa_handler = SigASR;
+#ifdef SIGCHLD
+    sigaction(SIGCHLD, &sa, NULL);
+    ASRpid = NO_ASR; 
+#endif
 }
 
 static void SigControl(int sig)
@@ -157,7 +160,6 @@ static void SigControl(int sig)
 
     if (stat(VenusControlFile, &tstat) != 0) {
 	SwapLog();
-	adv_mon.SwapReplacementLog();
         return;
     }
 
@@ -206,10 +208,8 @@ static void SigControl(int sig)
 	LOG(0, ("lwp_debug is now %d.\n", lwp_debug));
     }
 
-    if (STREQ(command, "SWAPLOGS")) {
+    if (STREQ(command, "SWAPLOGS"))
 	SwapLog();
-	adv_mon.SwapReplacementLog();
-    }
 
     if (STREQ(command, "STATSINIT"))
 	StatsInit();
@@ -225,22 +225,18 @@ static void SigControl(int sig)
 
 static void SigChoke(int sig)
 {
-#ifndef DJGPP
     sigset_t mask;
     int pid = getpid();
-#endif
 
     LOG(0, ("*****  FATAL SIGNAL (%d) *****\n", sig));
 
-#ifndef DJGPP
     eprint("Fatal Signal (%d); pid %d becoming a zombie...", sig, pid);
     eprint("You may use gdb to attach to %d", pid);
     MarinerLog("zombie state::pid %d", pid);
 
     /* just in case we still have a parent process waiting for us we don't want
      * to lock up the boot sequence... */
-    if (parent_fd != -1)
-	close(parent_fd);
+    WorkerCloseMuxfd();
 
     /* block all signals, except for INT and TERM (and the non-blockable ones,
      * KILL and STOP) */
@@ -248,7 +244,6 @@ static void SigChoke(int sig)
     sigdelset(&mask, SIGINT);
     sigdelset(&mask, SIGTERM);
     sigsuspend(&mask);
-#endif
 
     SigExit(sig);
 }
@@ -273,3 +268,50 @@ static void SigMounted(int sig)
     gogogo(parent_fd);
 }
 
+static void SigASR(int sig)
+{
+  int pid, child_pid, status, options;
+  repvol *v;
+
+
+  /* Beyond Venus initialization, the only forking occurring within Venus
+   * is a result of ASRLauncher invocation. Thus, every SIGCHLD received is
+   * an ASRLauncher completing execution, and the status is the return code
+   * of success or failure of the repair. */
+
+  if(ASRpid == NO_ASR)
+	return;
+
+  LOG(0, ("Signal Handler(ASR): ASRpid:%d, ASRfid:%s\n", 
+		  ASRpid, FID_(&ASRfid)));
+
+  pid = getpid();
+  status = options = 0;
+
+  child_pid = waitpid(ASRpid, &status, WNOHANG);
+  if(child_pid < 0) { perror("waitpid"); exit(EXIT_FAILURE); }
+  else if(child_pid == ASRpid)
+      LOG(0, ("Signal Handler(ASR): Caught ASRLauncher (%d) with status %d\n", 
+	      child_pid, status));
+  else {
+      LOG(0, ("Signal Handler(ASR): Caught an unknown child!\n"));
+      return;             /* If there are no documented ASR's running, this
+			   * could be the VFSMount double-fork middle child. */
+  }
+
+  v = (repvol *)VDB->Find(MakeVolid(&ASRfid)); 
+  if(v == NULL) {
+      LOG(0, ("Signal Handler(ASR): Couldn't find volume!\n"));
+      return;
+  }
+
+  /* Clear out table entry */
+  ASRpid = NO_ASR;
+
+  /* Unassign Tokens */
+  /* TODO: not easy to do at the moment. */
+
+  /* Unlock volume */
+  v->asr_pgid(NO_ASR);
+  v->unlock_asr();
+}
