@@ -42,10 +42,7 @@ extern "C" {
 #include <netinet/in.h>
 #include <errno.h>
 #include <struct.h>
-#include <netdb.h>
-#ifndef DJGPP
 #include <utmp.h>
-#endif
 #include <pwd.h>
 
 #include <rpc2/rpc2.h>
@@ -58,13 +55,13 @@ extern "C" {
 }
 #endif
 
-
-#include "adv_monitor.h"
+#include <coda_getservbyname.h>
 #include "comm.h"
 #include "hdb.h"
 #include "mariner.h"
 #include "user.h"
 #include "venus.private.h"
+#include "venusvol.h"
 #include "vsg.h"
 #include "worker.h"
 
@@ -90,22 +87,20 @@ userent *Realm::GetUser(uid_t uid)
 	if (Id() == u->realmid && uid == u->GetUid())
 	    return u;
     }
-    return system_anyuser;
+
+    /* allocate a new user entry */
+    u = new userent(Id(), uid);
+    userent::usertab->insert(&u->tblhandle);
+    return u;
 }
 
 int Realm::NewUserToken(uid_t uid, SecretToken *secretp, ClearToken *clearp)
 {
     LOG(100, ("Realm::NewUserToken local uid '%d' for realm '%s'\n", uid,name));
-    userent *u = GetUser(uid);
+    userent *u;
     int ret;
 
-    if (u == system_anyuser) {
-	PutUser(&u);
-
-	/* Create a new entry and initialize it. */
-	u = new userent(Id(), uid);
-	userent::usertab->insert(&u->tblhandle);
-    }
+    u = GetUser(uid);
 
     ret = u->SetTokens(secretp, clearp);
 
@@ -172,10 +167,10 @@ int AuthorizedUser(uid_t thisUser)
 
 int ConsoleUser(uid_t user)
 {
-#if defined(DJGPP) || defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
     return(1);
 
-#elif __linux__
+#elif defined(__linux__)
 #define	CONSOLE	    "tty1"
 
     struct utmp w, *u;
@@ -268,17 +263,14 @@ long userent::SetTokens(SecretToken *asecret, ClearToken *aclear)
     /* Inform the advice monitor that user now has tokens. */
     LOG(100, ("calling TokensAcquired with %d\n",
 	      (clear.EndTimestamp-CLOCK_SKEW)));
-    adv_mon.TokensAcquired((clear.EndTimestamp - CLOCK_SKEW));
-
 
     /* Make dirty volumes "owned" by this user available for reintegration. */
     repvol_iterator next;
     repvol *v;
     while ((v = next())) {
-	if (v->IsReintegratePending() && v->GetCML()->Owner() == uid) {
+	ClientModifyLog *CML = v->GetCML();
+	if (CML->Owner() == uid && CML->count())
 	    v->flags.transition_pending = 1;
-	    v->ClearReintegratePending();
-	}
     }
 
     return(1);
@@ -342,7 +334,6 @@ void userent::Invalidate() {
 
     /* Inform the user */
     eprint("Coda token for user %d has been discarded", uid);
-    adv_mon.TokensExpired();
 
     Reset();
 }
@@ -351,7 +342,7 @@ void userent::Reset()
 {
 LOG(100, ("E userent::Reset()\n"));
     /* Clear the cached access info for the user. */
-    FSDB->ResetUser(uid);
+    // FSDB->ResetUser(uid);
 
     /* Invalidate kernel data for the user. */
     k_Purge(uid);
@@ -405,13 +396,10 @@ int userent::Connect(RPC2_Handle *cid, int *auth, struct in_addr *host)
 	mcid.Tag = RPC2_MGRPBYINETADDR;
 	mcid.Value.InetAddress = *host;
 
+	struct servent *s = coda_getservbyname("codasrv", "udp");
 	RPC2_PortIdent pid;
 	pid.Tag = RPC2_PORTBYINETNUMBER;
-	pid.Value.InetPortNumber = htons(2432);
-
-	struct servent *s = getservbyname("codasrv", "udp");
-	if (s) pid.Value.InetPortNumber = s->s_port;
-	else eprint("getservbyname(codasrv,udp) failed, using 2432/udp");
+	pid.Value.InetPortNumber = s->s_port;
 
 	RPC2_SubsysIdent ssid;
 	ssid.Tag = RPC2_SUBSYSBYID;
@@ -452,13 +440,10 @@ int userent::Connect(RPC2_Handle *cid, int *auth, struct in_addr *host)
 	hid.Tag = RPC2_HOSTBYINETADDR;
 	hid.Value.InetAddress = *host;
 
+	struct servent *s = coda_getservbyname("codasrv", "udp");
 	RPC2_PortIdent pid;
 	pid.Tag = RPC2_PORTBYINETNUMBER;
-	pid.Value.InetPortNumber = htons(2432);
-
-	struct servent *s = getservbyname("codasrv", "udp");
-	if (s) pid.Value.InetPortNumber = s->s_port;
-	else eprint("getservbyname(codasrv,udp) failed, using 2432/udp");
+	pid.Value.InetPortNumber = s->s_port;
 
 	RPC2_SubsysIdent ssid;
 	ssid.Tag = RPC2_SUBSYSBYID;
@@ -565,14 +550,11 @@ int userent::GetWaitForever() {
     return(waitforever);
 }
 
-void userent::SetWaitForever(int state) {
+void userent::SetWaitForever(int state)
+{
     LOG(1, ("userent::SetWaitForever: uid = %d, old_state = %d, new_state = %d\n",
 	     uid, waitforever, state));
 
-    /* Don't allow someone to set the waitforever behaviour for system_anyuser
-     * because it is shared among all anonymous users */
-    if (!tokensvalid) return;
-
     if (state == waitforever) return;
 
     waitforever = state;
@@ -611,7 +593,6 @@ void userent::print(int afd) {
 	end_time[0] = '\0';
     }
 
-    adv_mon.Print(afd);
     fdprint(afd, "Time of last demand hoard walk = %ld\n", DemandHoardWalkTime);
 
     fdprint(afd, "%#08x : uid = %d, wfe = %d, valid = %d, begin = %s, end = %s\n\n",
