@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -34,7 +34,9 @@ extern "C" {
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <netdb.h>
+
+#include "archive.h"
+
 #ifdef __cplusplus
 }
 #endif
@@ -43,8 +45,6 @@ extern "C" {
 #include <vice.h>
 
 /* from venus */
-#include "advice.h"
-#include "adv_daemon.h"
 #include "comm.h"
 #include "fso.h"
 #include "hdb.h"
@@ -81,24 +81,32 @@ VenusFid rootfid;
 long rootnodeid;
 int CleanShutDown;
 int SearchForNOreFind;  // Look for better detection method for iterrupted hoard walks. mre 1/18/93
+int ASRallowed = 1;
 
 /* Command-line/venus.conf parameters. */
-char *consoleFile;
-char *venusRoot;
-char *kernDevice;
-char *realmtab;
-char *CacheDir;
-char *CachePrefix;
+const char *consoleFile;
+const char *venusRoot;
+const char *kernDevice;
+const char *realmtab;
+const char *CacheDir;
+const char *CachePrefix;
 int   CacheBlocks;
 uid_t PrimaryUser = UNSET_PRIMARYUSER;
-char *SpoolDir;
-char *VenusPidFile;
-char *VenusControlFile;
-char *VenusLogFile;
-char *MarinerSocketPath;
+const char *SpoolDir;
+const char *CheckpointFormat;
+const char *VenusPidFile;
+const char *VenusControlFile;
+const char *VenusLogFile;
+const char *ASRLauncherFile;
+const char *ASRPolicyFile;
+const char *MarinerSocketPath;
 int masquerade_port;
 int PiggyValidations;
-
+pid_t ASRpid;
+VenusFid ASRfid;
+uid_t ASRuid;
+int detect_reintegration_retry;
+int option_isr;
 
 #if defined(HAVE_SYS_UN_H) && !defined(__CYGWIN32__)
 int mariner_tcp_enable = 0;
@@ -155,11 +163,6 @@ int main(int argc, char **argv)
     /* Initialize.  N.B. order of execution is very important here! */
     /* RecovInit < VSGInit < VolInit < FSOInit < HDB_Init */
 
-#ifdef DJGPP
-    /* disable debug messages */
-    __djgpp_set_quiet_socket(1);
-#endif
-
 #ifdef __CYGWIN32__
     /* MapPrivate does not work on Cygwin */
     if (MapPrivate) {
@@ -201,16 +204,11 @@ int main(int argc, char **argv)
     MarinerInit();  /* set up mariner socket */
     WorkerInit();   /* open kernel device */
     CallBackInit(); /* set up callback subsystem and create callback server threads */
-    AdviceInit();   /* set up AdSrv and start the advice daemon */
-    LRInit();	    /* set up local-repair database */
 
     /* Get the Root Volume. */
     eprint("Mounting root volume...");
 
     VFSMount();
-#ifdef DJGPP
-    k_Purge();
-#endif
 
     UnsetInitFile();
     eprint("Venus starting...");
@@ -293,8 +291,6 @@ static void Usage(char *argv0)
 " -maxworkers <n>\t\t# of worker threads\n"
 " -maxcbservers <n>\t\t# of callback server threads\n"
 " -maxprefetchers <n>\t\t# of threads servicing prefetch ioctl\n"
-" -weakthresh <n>\t\tstrong/weak threshold (bytes/sec)\n"
-" -weakstale <n>\t\t\twhen estimates become too old (seconds)\n"
 " -retries <n>\t\t\t# of rpc2 retries\n"
 " -timeout <n>\t\t\trpc2 timeout\n"
 " -ws <n>\t\t\tsftp window size\n"
@@ -375,13 +371,6 @@ static void ParseCmdline(int argc, char **argv)
 		i++, COPModes = atoi(argv[i]);
 	    else if (STREQ(argv[i], "-maxworkers"))  /* number of worker threads */
 		i++, MaxWorkers = atoi(argv[i]);
-	    else if (STREQ(argv[i], "-weakthresh")) {   /* Threshold at which to go to weak mode */
-		WCThresh = atoi(argv[++i]);		/* in Bytes/sec */
-	    }
-	    else if (STREQ(argv[i], "-weakstale")) {   /* When estimates become too old */
-		extern int WCStale;
-		WCStale = atoi(argv[++i]);		/* in seconds */
-	    }
 	    else if (STREQ(argv[i], "-maxcbservers")) 
 		i++, MaxCBServers = atoi(argv[i]);
 	    else if (STREQ(argv[i], "-maxprefetchers")) /* max number of threads */
@@ -426,12 +415,6 @@ static void ParseCmdline(int argc, char **argv)
 		i++, FSO_MWT = atoi(argv[i]);
 	    else if (STREQ(argv[i], "-ssf"))         /* short term scale factor */
 		i++, FSO_SSF = atoi(argv[i]);
-	    else if (STREQ(argv[i], "-alpha"))	     /* patience ALPHA value */
-		i++, PATIENCE_ALPHA = atoi(argv[i]);
-	    else if (STREQ(argv[i], "-beta"))	     /* patience BETA value */
-		i++, PATIENCE_BETA = atoi(argv[i]);
-	    else if (STREQ(argv[i], "-gamma"))	     /* patience GAMMA value */
-		i++, PATIENCE_GAMMA = atoi(argv[i]);
 	    else if (STREQ(argv[i], "-primaryuser")) /* primary user of this machine */
 		i++, PrimaryUser = atoi(argv[i]);
 	    else if (STREQ(argv[i], "-von"))
@@ -444,8 +427,6 @@ static void ParseCmdline(int argc, char **argv)
 		mrpc2_timeflag = 0;
 	    else if (STREQ(argv[i], "-SearchForNOreFind"))
 	        SearchForNOreFind = 1;
-	    else if (STREQ(argv[i], "-noskk"))
-	        SkkEnabled = 0;
 	    else if (STREQ(argv[i], "-noasr"))
 	        ASRallowed = 0;
 	    else if (STREQ(argv[i], "-novcb"))
@@ -475,8 +456,6 @@ static void ParseCmdline(int argc, char **argv)
 		done = -1;
 	    }
 	}
-	else
-	    venusRoot = argv[i];   /* default is /coda */
     }
     if (done) exit(done < 0 ? 1 : 0);
 }
@@ -496,12 +475,13 @@ static void DefaultCmdlineParms()
     CODACONF_INT(CacheBlocks,	    "cacheblocks",   40000);
     CODACONF_STR(CacheDir,	    "cachedir",      DFLT_CD);
     CODACONF_STR(SpoolDir,	    "checkpointdir", "/usr/coda/spool");
-    CODACONF_STR(VenusLogFile,	    "logfile",	     "/usr/coda/etc/venus.log");
-    CODACONF_STR(consoleFile,	    "errorlog",      "/usr/coda/etc/console");
+    CODACONF_STR(VenusLogFile,	    "logfile",	     DFLT_LOGFILE);
+    CODACONF_STR(consoleFile,	    "errorlog",      DFLT_ERRLOG);
     CODACONF_STR(kernDevice,	    "kerneldevice",  "/dev/cfs0,/dev/coda/0");
     CODACONF_INT(MapPrivate,	    "mapprivate",     0);
     CODACONF_STR(MarinerSocketPath, "marinersocket", "/usr/coda/spool/mariner");
     CODACONF_INT(masquerade_port,   "masquerade_port", 0);
+    CODACONF_INT(allow_backfetch,   "allow_backfetch", 0);
     CODACONF_STR(venusRoot,	    "mountpoint",     DFLT_VR);
     CODACONF_INT(PrimaryUser,	    "primaryuser",    UNSET_PRIMARYUSER);
     CODACONF_STR(realmtab,	    "realmtab",	      "/etc/coda/realms");
@@ -511,8 +491,11 @@ static void DefaultCmdlineParms()
     CODACONF_INT(rpc2_timeout,	    "RPC2_timeout",   DFLT_TO);
     CODACONF_INT(rpc2_retries,	    "RPC2_retries",   DFLT_RT);
 
-    CODACONF_INT(T1Interval,	    "serverprobe",    150);
-    // used to be 12 minutes
+    CODACONF_INT(T1Interval, "serverprobe", 150); // used to be 12 minutes
+
+    CODACONF_INT(default_reintegration_age,  "reintegration_age",  0);
+    CODACONF_INT(default_reintegration_time, "reintegration_time", 15);
+    default_reintegration_time *= 1000; /* reintegration time is in msec */
 
 #if defined(__CYGWIN32__)
     CODACONF_STR(CachePrefix, "cache_prefix", "/?" "?/C:/cygwin");
@@ -530,10 +513,6 @@ static void DefaultCmdlineParms()
     {
 	if (!CacheFiles) CacheFiles = CacheBlocks / BLOCKS_PER_FILE;
 
-#ifdef DJGPP
-	if (CacheFiles > 1500)
-	    CacheFiles = 1500;
-#endif
 	if (CacheFiles < MIN_CF) {
 	    eprint("Cannot start: minimum number of cache files is %d", MIN_CF);
 	    exit(-1); 
@@ -561,32 +540,48 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CODACONF_STR(VenusPidFile, "pid_file", NULL);
-    {
-#define PIDFILE "/pid"
-	if (!VenusPidFile) {
-	    VenusPidFile = (char *)malloc(strlen(CacheDir)+strlen(PIDFILE)+1);
-	    strcpy(VenusPidFile, CacheDir);
-	    strcat(VenusPidFile, PIDFILE);
-	}
+    CODACONF_STR(VenusPidFile, "pid_file", DFLT_PIDFILE);
+    if (*VenusPidFile != '/') {
+	char *tmp = (char *)malloc(strlen(CacheDir) + strlen(VenusPidFile) + 2);
+	CODA_ASSERT(tmp);
+	sprintf(tmp, "%s/%s", CacheDir, VenusPidFile);
+	VenusPidFile = tmp;
     }
 
-    CODACONF_STR(VenusControlFile, "run_control_file", NULL);
-    {
-#define CTRLFILE "/VENUS_CTRL"
-	if (!VenusControlFile) {
-	    VenusControlFile=(char*)malloc(strlen(CacheDir)+strlen(CTRLFILE)+1);
-	    strcpy(VenusControlFile, CacheDir);
-	    strcat(VenusControlFile, CTRLFILE);
-	}
+    CODACONF_STR(VenusControlFile, "run_control_file", DFLT_CTRLFILE);
+    if (*VenusControlFile != '/') {
+	char *tmp = (char *)malloc(strlen(CacheDir) + strlen(VenusControlFile) + 2);
+	CODA_ASSERT(tmp);
+	sprintf(tmp, "%s/%s", CacheDir, VenusControlFile);
+	VenusControlFile = tmp;
     }
 
-    CODACONF_INT(PiggyValidations, "validateattrs", 21);
+    CODACONF_STR(ASRLauncherFile, "asrlauncher_path", NULL);
+
+    CODACONF_STR(ASRPolicyFile, "asrpolicy_path", NULL);
+
+    CODACONF_INT(PiggyValidations, "validateattrs", 15);
     {
 	if (PiggyValidations > MAX_PIGGY_VALIDATIONS)
 	    PiggyValidations = MAX_PIGGY_VALIDATIONS;
     }
 
+    /* Enable special tweaks for running in a VM
+     * - Write zeros to container file contents before truncation.
+     * - Disable reintegration replay detection. */
+    CODACONF_INT(option_isr, "isr", 0);
+
+    CODACONF_INT(detect_reintegration_retry, "detect_reintegration_retry", 1);
+    if (option_isr) {
+	detect_reintegration_retry = 0;
+    }
+
+    CODACONF_STR(CheckpointFormat,  "checkpointformat", "newc");
+    if (strcmp(CheckpointFormat, "tar") == 0)	archive_type = TAR_TAR;
+    if (strcmp(CheckpointFormat, "ustar") == 0) archive_type = TAR_USTAR;
+    if (strcmp(CheckpointFormat, "odc") == 0)   archive_type = CPIO_ODC;
+    if (strcmp(CheckpointFormat, "newc") == 0)  archive_type = CPIO_NEWC;
+
 #ifdef moremoremore
     char *x = NULL;
     CODACONF_STR(x, "relay", NULL, "127.0.0.1");
@@ -611,11 +606,7 @@ static void CheckInitFile() {
     struct stat tstat;
 
     /* Construct name for INIT file */
-#ifndef DJGPP
-        snprintf(initPath, MAXPATHLEN, "%s/INIT", CacheDir);
-#else
-        sprintf(initPath, "%s/INIT", CacheDir);
-#endif
+    snprintf(initPath, MAXPATHLEN, "%s/INIT", CacheDir);
 
     /* See if it's there */ 
     if (stat(initPath, &tstat) == 0) {
@@ -637,16 +628,12 @@ static void UnsetInitFile() {
     char initPath[MAXPATHLEN];
 
     /* Create the file, if it doesn't already exist */
-#ifndef DJGPP
-         snprintf(initPath, MAXPATHLEN, "%s/INIT", CacheDir);
-#else
-        sprintf(initPath, "%s/INIT", CacheDir);
-#endif
+    snprintf(initPath, MAXPATHLEN, "%s/INIT", CacheDir);
     unlink(initPath);
 }
 
 static void SetRlimits() {
-#if !defined(__CYGWIN32__) && !defined(DJGPP)
+#ifndef __CYGWIN32__
     /* Set DATA segment limit to maximum allowable. */
     struct rlimit rl;
     if (getrlimit(RLIMIT_DATA, &rl) < 0)
