@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -61,13 +61,23 @@ extern "C" {
 #define O_BINARY 0
 #endif
 
+/* Replica Control Rights. */
+/* Note that we presently do not distinguish between read and write rights. */
+/* We may well do so in the future, however. */
+#define	RC_STATUSREAD	1
+#define	RC_STATUSWRITE	2
+#define	RC_STATUS	(RC_STATUSREAD | RC_STATUSWRITE)
+#define	RC_DATAREAD	4
+#define	RC_DATAWRITE	8
+#define	RC_DATA		(RC_DATAREAD | RC_DATAWRITE)
+
 #define	EMULTRSLTS	ETOOMANYREFS	    /* external */
 #define	ESYNRESOLVE	155		    /* internal */
 #define	EASYRESOLVE	156		    /* internal */
 #define	ERETRY		157		    /* internal */
-#define	EVOLUME		158		    /* internal */
-/* The next two are internal, but defined in other modules. */
+/* The next three are internal, but defined in other modules. */
 /*
+#define	EVOLUME		158
 #define	EINCOMPATIBLE	198
 #define	EINCONS		199
 */
@@ -75,20 +85,17 @@ extern "C" {
    an ASR was started and it should block */
 #define EASRSTARTED     200
 
+#define ASR_INTERVAL 300
 
 /*  *****  parameter defaults.  ***** */
-#if defined(DJGPP)
-#define DFLT_VR "N:"                     /* Venus Root */ 
-#define MCFD    16                       /* Michael Callahan File Descriptor? */
-#else
-#define	DFLT_VR	"/coda"			 /* venus root */
-#endif
+#define DFLT_VR "/coda"				/* venus root */
+#define DFLT_CD "/usr/coda/venus.cache"		/* cache directory */
+/* the next two are relative to cachedir if they do not start with '/' */
+#define DFLT_PIDFILE "pid"			/* default pid file */
+#define DFLT_CTRLFILE "VENUS_CTRL"		/* default control file */
 
-#if defined(DJGPP) /* || defined(__CYGWIN32__) Not right now ... */
-#define	DFLT_CD	"C:/usr/coda/venus.cache"    /* Win cache directory */
-#else 
-#define	DFLT_CD	"/usr/coda/venus.cache"	    /* cache directory */
-#endif
+#define DFLT_LOGFILE "/usr/coda/etc/venus.log"	/* venus log file */
+#define DFLT_ERRLOG  "/usr/coda/etc/console"	/* venus error log */
 
 /* rule of thumb */
 const int BLOCKS_PER_FILE = 24;
@@ -107,7 +114,7 @@ const int FREE_FACTOR = 16;
 
 /*  *****  Manifest constants for Venus.  *****  */
 const int NFDS = 32;	/* IOMGR-enforced limit!  Kernel may allocate fds numbered higher than this! */
-#if defined(DJGPP) || defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
 extern uid_t V_UID;    /* UID that the venus process runs under. */
 #else
 const uid_t V_UID = (uid_t)0;	    /* UID that the venus process runs under. */
@@ -152,7 +159,7 @@ typedef void (*PROC_V_UL)(unsigned long);
 
 /*  *****  Debugging macros.  *****  */
 #ifdef	VENUSDEBUG
-#define	LOG(level, stmt)    if (LogLevel >= (level)) dprint stmt
+#define	LOG(level, stmt) do { if (LogLevel >= (level)) dprint stmt; } while(0)
 #else
 #define	LOG(level, stmt)
 #endif /* !VENUSDEBUG */
@@ -284,12 +291,12 @@ struct CacheStats {
 #define CHOKE(me...) choke(__FILE__, __LINE__, ##me)
 
 /*  *****  Declarations for source files without their own headers.  ***** */
-void dprint(char * ...);
-void choke(char *file, int line, char* ...);
+void dprint(const char * ...);
+void choke(const char *file, int line, const char* ...);
 void rds_printer(char * ...);
-void VenusPrint(int, char **);
-void VenusPrint(FILE *, int, char **);
-void VenusPrint(int, int, char **);
+void VenusPrint(int argc, const char **argv);
+void VenusPrint(FILE *, int argc, const char **argv);
+void VenusPrint(int, int argc, const char **argv);
 const char *VenusOpStr(int);
 const char *IoctlOpStr(unsigned char nr);
 const char *VenusRetStr(int);
@@ -309,12 +316,12 @@ void MallocPrint(int);
 void StatsInit();
 void SwapLog();
 void ToggleMallocTrace();
-char *lvlstr(LockLevel);
+const char *lvlstr(LockLevel);
 int GetTime(long *, long *);
 time_t Vtime();
 int FAV_Compare(ViceFidAndVV *, ViceFidAndVV *);
 void DaemonInit();
-void FireAndForget(char *name, void (*f)(void), int interval,
+void FireAndForget(const char *name, void (*f)(void), int interval,
 		   int stacksize=32*1024);
 void RegisterDaemon(unsigned long, char *);
 void DispatchDaemons();
@@ -338,26 +345,42 @@ extern VenusFid rootfid;
 extern int parent_fd;
 extern long rootnodeid;
 extern int CleanShutDown;
-extern char *venusRoot;
-extern char *kernDevice;
-extern char *realmtab;
-extern char *CacheDir;
-extern char *CachePrefix;
+extern const char *venusRoot;
+extern const char *kernDevice;
+extern const char *realmtab;
+extern const char *CacheDir;
+extern const char *CachePrefix;
 extern int CacheBlocks;
-extern char *SpoolDir;
+extern const char *SpoolDir;
 extern uid_t PrimaryUser;
-extern char *VenusPidFile;
-extern char *VenusControlFile;
-extern char *VenusLogFile;
-extern char *consoleFile;
-extern char *MarinerSocketPath;
+extern const char *VenusPidFile;
+extern const char *VenusControlFile;
+extern const char *VenusLogFile;
+extern const char *consoleFile;
+extern const char *MarinerSocketPath;
 extern int   mariner_tcp_enable;
 extern int   allow_reattach;
 extern int   masquerade_port;
 extern int   PiggyValidations;
 extern int   T1Interval;
+extern const char *ASRLauncherFile;
+extern const char *ASRPolicyFile;
+extern int   option_isr;
+extern int   detect_reintegration_retry;
 
 /* spool.cc */
 extern void MakeUserSpoolDir(char *, uid_t);
 
+
+/* ASR misc */
+/* Note: At some point, it would be nice to run ASRs in different
+ * volumes concurrently. This requires replacing the globals
+ * below with a table or other data structure. Due to token
+ * assignment constraints, though, this is not possible as of 06/2006.
+ */
+
+extern pid_t ASRpid;
+extern VenusFid ASRfid;
+extern uid_t ASRuid;
+
 #endif /* _VENUS_PRIVATE_H_ */
