@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -78,9 +78,9 @@ float TransElapsed = 0.0;
 int MapPrivate = 0;
 
 int InitMetaData = UNSET_IMD, InitNewInstance = UNSET_IMD;
-char *VenusLogDevice = NULL;
+const char *VenusLogDevice = NULL;
 unsigned long VenusLogDeviceSize = UNSET_VLDS;
-char *VenusDataDevice = NULL;
+const char *VenusDataDevice = NULL;
 unsigned long VenusDataDeviceSize = UNSET_VDDS;
 int RdsChunkSize = UNSET_RDSCS;
 int RdsNlists = UNSET_RDSNL;
@@ -106,12 +106,9 @@ static const char *VM_RVMADDR = (char *)0x50000000;
  * start address without a problem.
  *
  * Sun/solaris could probably use it as well. I do wonder why sparc-linux is
- * using such an unusual RVM start address. djgpp (win95) is the most likely
- * exception to the rule because we are running in a strange DOS environment */
+ * using such an unusual RVM start address. */
 #elif defined(__linux__) && defined(sparc)
 static const char *VM_RVMADDR = (char *)0xbebd000;
-#elif defined(DJGPP)
-static const char *VM_RVMADDR = (char *)0x03000000;
 #elif defined(sun)
 static const char *VM_RVMADDR = (char *)0x40000000;
 #else
@@ -164,7 +161,6 @@ int RecovVenusGlobals::validate()
     if (!VALID_REC_PTR(recov_VDB)) return(0);
     if (!VALID_REC_PTR(recov_REALMDB)) return(0);
     if (!VALID_REC_PTR(recov_HDB)) return(0);
-    if (!VALID_REC_PTR(recov_LRDB)) return(0);
 
     return(1);
 }
@@ -189,8 +185,8 @@ void RecovVenusGlobals::print(int fd) {
 	    recov_CleanShutDown);
     fdprint(fd, "The following pointers should be between %p and %p:\n",
 	    recov_HeapAddr, recov_HeapAddr + recov_HeapLength);
-    fdprint(fd, "Ptrs = [%p %p %p %p %p], Heap = [%p] HeapLen = %x\n",
-	     recov_FSDB, recov_VDB, recov_HDB, recov_REALMDB, recov_LRDB, 
+    fdprint(fd, "Ptrs = [%p %p %p %p], Heap = [%p] HeapLen = %x\n",
+	     recov_FSDB, recov_VDB, recov_HDB, recov_REALMDB,
 	     recov_HeapAddr, recov_HeapLength);
 
     fdprint(fd, "UUID = %08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x\n",
@@ -199,13 +195,24 @@ void RecovVenusGlobals::print(int fd) {
 	    ntohs(*(unsigned short *)&recov_UUID[6]),
 	    recov_UUID[8], recov_UUID[9], recov_UUID[10], recov_UUID[11],
 	    recov_UUID[12], recov_UUID[13], recov_UUID[14], recov_UUID[15]);
+    fdprint(fd, "StoreId = %d\n", recov_StoreId);
 }
 
 static void RecovNewInstance(void)
 {
     /* We need to initialize the random number generator before first use */
     rpc2_InitRandom();
+
+    Recov_BeginTrans();
+    RVMLIB_REC_OBJECT(rvg->recov_UUID);
+    RVMLIB_REC_OBJECT(rvg->recov_StoreId);
+
     VenusGenID = rpc2_NextRandom(NULL);
+
+    /* server disables replay detection when storeid.uniquifier > INT_MAX */
+    rvg->recov_StoreId =
+	detect_reintegration_retry ? 0 : ((unsigned int)INT_MAX+1);
+    Recov_EndTrans(0);
 }
 
 void RecovInit(void)
@@ -314,7 +321,7 @@ static void Recov_CheckParms()
     }
     else /* !InitMetaData */
     {
-        char *failure = NULL;
+        const char *failure = NULL;
 
         /* Specifying log or data size requires a brain-wipe! */
         if (VenusLogDeviceSize != UNSET_VLDS) {
@@ -355,8 +362,11 @@ fail:
 
 static void Recov_InitRVM()
 {
+    rvm_return_t ret;
+    char *logdev = strdup(VenusLogDevice);
+
     rvm_init_options(&Recov_Options);
-    Recov_Options.log_dev = VenusLogDevice;
+    Recov_Options.log_dev = logdev;
     Recov_Options.truncate = 0;
     //Recov_Options.flags = RVM_COALESCE_TRANS;  /* oooh, daring */
     Recov_Options.flags = RVM_ALL_OPTIMIZATIONS;
@@ -368,7 +378,8 @@ static void Recov_InitRVM()
     if (InitMetaData) /* Initialize log. */
     {
         /* Get rid of any old log */
-        truncate(VenusLogDevice, 0); unlink(VenusLogDevice);
+        truncate(VenusLogDevice, 0);
+	unlink(VenusLogDevice);
 
         /* Pass in the correct parameters so that RVM_INIT can create
          * a new logfile */
@@ -396,7 +407,8 @@ static void Recov_InitRVM()
     }
     eprint("%s size is %ld bytes", VenusLogDevice, VenusLogDeviceSize);
 
-    rvm_return_t ret = RVM_INIT(&Recov_Options);
+    ret = RVM_INIT(&Recov_Options);
+    free(logdev);
     if (ret == RVM_ELOG_VERSION_SKEW) {
 	eprint("Recov_InitRVM: RVM_INIT failed, RVM log version skew");
 	eprint("Venus not started");
@@ -411,6 +423,7 @@ static void Recov_InitRDS()
 {
     rvm_return_t ret;
     rvm_length_t devsize;
+    char *datadev;
 
     devsize = RVM_ROUND_LENGTH_DOWN_TO_PAGE_SIZE(VenusDataDeviceSize);
     Recov_RdsAddr = (char *)VM_RVMADDR;
@@ -441,9 +454,11 @@ static void Recov_InitRDS()
     }
 
     eprint("Initializing RVM data...");
-    rds_zap_heap(VenusDataDevice, RVM_LENGTH_TO_OFFSET(devsize),
-                 Recov_RdsAddr, Recov_RvgLength, Recov_RdsLength,
-                 (unsigned long)RdsNlists,(unsigned long)RdsChunkSize,&ret);
+    datadev = strdup(VenusDataDevice);
+    rds_zap_heap(datadev, RVM_LENGTH_TO_OFFSET(devsize),
+		 Recov_RdsAddr, Recov_RvgLength, Recov_RdsLength,
+		 (unsigned long)RdsNlists,(unsigned long)RdsChunkSize,&ret);
+    free(datadev);
     if (ret != SUCCESS) {
         eprint("Recov_InitRDS: rds_zap_heap failed (%s)", rvm_return(ret));
         exit(-1);
@@ -454,10 +469,14 @@ static void Recov_InitRDS()
 static void Recov_LoadRDS()
 {
     rvm_return_t ret;
+    char *datadev;
+    int detecting_retries;
 
     eprint("Loading RVM data");
-    rds_load_heap(VenusDataDevice, RVM_LENGTH_TO_OFFSET(VenusDataDeviceSize),
-                  &Recov_RvgAddr, &ret);
+    datadev = strdup(VenusDataDevice);
+    rds_load_heap(datadev, RVM_LENGTH_TO_OFFSET(VenusDataDeviceSize),
+		  &Recov_RvgAddr, &ret);
+    free(datadev);
     if (ret != SUCCESS) {
         eprint("Recov_InitRDS: rds_load_heap failed (%s)", rvm_return(ret));
         exit(-1);
@@ -499,24 +518,23 @@ static void Recov_LoadRDS()
         Recov_EndTrans(0);
     }
 
-    if (InitMetaData || InitNewInstance) {
-        Recov_BeginTrans();
-	RVMLIB_REC_OBJECT(VenusGenID);
+    detecting_retries = rvg->recov_StoreId <= INT_MAX;
+    if (InitMetaData || InitNewInstance ||
+        (detect_reintegration_retry && !detecting_retries) || 
+	(!detect_reintegration_retry && detecting_retries))
 	RecovNewInstance();
-        Recov_EndTrans(0);
-    }
 
     /* Plumb the heap here? */
-    if (MallocTrace) {	
-        rds_trace_on(logFile);
-        rds_trace_dump_heap();
+    if (MallocTrace) {
+	rds_trace_on(logFile);
+	rds_trace_dump_heap();
     }
 }
 
 /* Venus transaction handling */
-void Recov_BeginTrans(void)
+void _Recov_BeginTrans(const char file[], int line)
 {
-	rvmlib_begin_transaction(no_restore);
+	_rvmlib_begin_transaction(no_restore, file, line);
 }
 
 void Recov_EndTrans(int time)
@@ -735,3 +753,22 @@ void RecovDaemon(void) {
     }
 }
 
+/* MUST be called from within a transaction */
+void Recov_GenerateStoreId(ViceStoreId *sid)
+{
+    /* VenusGenID, is randomly chosen whenever rvm is reinitialized, it
+     * should be a 128-bit UUID (re-generated whenever rvm is reinitialized).
+     * But that would require changing in the venus-vice protocol to either
+     * add this UUID to every operation, or send it once per (volume-)
+     * connection setup with ViceNewConnectFS. -JH */
+    sid->Host = (RPC2_Unsigned)VenusGenID;
+    sid->Uniquifier = rvg->recov_StoreId;
+
+    /* Avoid overflow past UINT_MAX, server stopped replay detection once
+     * we passed INT_MAX so we stop incrementing */
+    if (rvg->recov_StoreId == UINT_MAX) return;
+
+    RVMLIB_REC_OBJECT(rvg->recov_StoreId);
+    rvg->recov_StoreId++;
+}
+
