@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -50,9 +50,6 @@ class fsdb;
 class vdb;
 class RealmDB;
 class hdb;
-class vmselist;
-class lrdb;
-class vcbdb;
 
 /*  *****  Constants  *****  */
 
@@ -83,7 +80,7 @@ const unsigned long DFLT_MAXTS = 256 * 1024;	/* Maximum Truncate Size */
 const unsigned long UNSET_MAXTS = (unsigned long)-1;
 
 const int RecovMagicNumber = 0x8675309;
-const int RecovVersionNumber = 36;	/* Update this when format changes. */
+const int RecovVersionNumber = 39;	/* Update this when format changes. */
 
 /*  *****  Types  *****  */
 /* local-repair modification */
@@ -98,7 +95,6 @@ struct RecovVenusGlobals {
     vdb	*recov_VDB;		    /* Volume database */
     RealmDB *recov_REALMDB;	    /* Realm database */
     hdb	*recov_HDB;		    /* Hoard database */
-    lrdb *recov_LRDB;		    /* Local repair database */
 
     char *recov_HeapAddr;	    /* Base of recoverable heap */
     unsigned int recov_HeapLength;  /* Length of recoverable heap (in bytes) */
@@ -114,6 +110,7 @@ struct RecovVenusGlobals {
     /* This UUID should be stored in network byte order.
      * "draft-leach-uuids-guids-01.txt". */
     unsigned char recov_UUID[16];
+    unsigned int recov_StoreId;
 
     int validate();
     void print();
@@ -133,9 +130,9 @@ extern int MapPrivate;
 
 extern int InitMetaData, InitNewInstance;
 extern rvm_type_t RvmType;
-extern char *VenusLogDevice;
+extern const char *VenusLogDevice;
 extern unsigned long VenusLogDeviceSize;
-extern char *VenusDataDevice;
+extern const char *VenusDataDevice;
 extern unsigned long VenusDataDeviceSize;
 extern int RdsChunkSize;
 extern int RdsNlists;
@@ -149,7 +146,8 @@ extern unsigned long MAXTS;
 
 /*  *****  Functions  *****  */
 
-void Recov_BeginTrans();
+#define Recov_BeginTrans() _Recov_BeginTrans(__FILE__, __LINE__)
+void _Recov_BeginTrans(const char file[], int line);
 void Recov_EndTrans(int);
 void Recov_SetBound(int);
 void RecovInit();
@@ -162,6 +160,8 @@ void Free_RPC2_String(RPC2_String&);
 void RECOVD_Init(void);
 void RecovDaemon(void);
 
+void Recov_GenerateStoreId(ViceStoreId *sid);
+
 #define	VALID_REC_PTR(rec_ptr)\
     ((char *)(rec_ptr) >= rvg->recov_HeapAddr &&\
      (char *)(rec_ptr) < rvg->recov_HeapAddr + rvg->recov_HeapLength)
