@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -43,7 +43,6 @@ extern "C" {
 #include "coda_string.h"
 #include <unistd.h>
 #include <stdlib.h>
-#include <netdb.h>
 
 #include <rpc2/rpc2.h>
 /* interfaces */
@@ -93,7 +92,7 @@ RPCOpStatistics RPCOpStats;
 /* *****  Private variables  ***** */
 
 static int LogInited = 0;
-static char *VFSOpsNameTemplate[NVFSOPS] = {
+static const char *VFSOpsNameTemplate[NVFSOPS] = {
     "No-Op",
     "No-Op",
     "Root",
@@ -141,7 +140,7 @@ static char *VFSOpsNameTemplate[NVFSOPS] = {
 /* *****  util.c  ***** */
 
 /* Print a debugging message to the log file. */
-void dprint(char *fmt ...) {
+void dprint(const char *fmt ...) {
     va_list ap;
 
     if (!LogInited) return;
@@ -174,7 +173,7 @@ void dprint(char *fmt ...) {
 
 
 /* Print an error message and then exit. */
-void choke(char *file, int line, char *fmt ...) {
+void choke(const char *file, int line, const char *fmt ...) {
     static int dying = 0;
 
     if (!dying) {
@@ -212,19 +211,20 @@ void choke(char *file, int line, char *fmt ...) {
 }
 
 
-void VenusPrint(int argc, char **argv) {
+void VenusPrint(int argc, const char **argv) {
     VenusPrint(stdout, argc, argv);
 }
 
 
-void VenusPrint(FILE *fp, int argc, char **argv) {
+void VenusPrint(FILE *fp, int argc, const char **argv) {
     fflush(fp);
     VenusPrint(fileno(fp), argc, argv);
 }
 
 
 /* local-repair modification */
-void VenusPrint(int fd, int argc, char **argv) {
+void VenusPrint(int fd, int argc, const char **argv)
+{
     int allp = 0;
     int rusagep = 0;
     int recovp = 0;
@@ -242,8 +242,6 @@ void VenusPrint(int fd, int argc, char **argv) {
     int hdbp = 0;
     int vmonp = 0;
     int mallocp = 0;
-    int lrdbp = 0;
-    int vcbdbp = 0;
 
     /* Parse the argv to see what modules should be printed. */
     for (int i = 0; i < argc; i++) {
@@ -264,8 +262,6 @@ void VenusPrint(int fd, int argc, char **argv) {
 	else if (STREQ(argv[i], "hdb")) hdbp++;
 	else if (STREQ(argv[i], "vmon")) vmonp++;
 	else if (STREQ(argv[i], "malloc")) mallocp++;
-	else if (STREQ(argv[i], "lrdb")) lrdbp++;
-	else if (STREQ(argv[i], "vcbdb")) vcbdbp++;
     }
 
     fdprint(fd, "*****  VenusPrint  *****\n\n");
@@ -287,7 +283,6 @@ void VenusPrint(int fd, int argc, char **argv) {
     if (rpcp || allp)     RPCPrint(fd);
     if (hdbp || allp)     if (RecovInited && HDB) HDB->print(fd);
     if (mallocp || allp)  MallocPrint(fd);
-    if (lrdbp || allp)    if (RecovInited && LRDB) LRDB->print(fd);
     fdprint(fd, "************************\n\n");
 }
 
@@ -376,8 +371,7 @@ const char *IoctlOpStr(unsigned char nr)
 	case _VIOC_DISCONNECT:		return("Disconnect");
 	case _VIOC_RECONNECT:		return("Reconnect");
 	case _VIOC_GETPFID:		return("Get Parent Fid");
-	case _VIOC_BEGINML:		return("Begin Modify Logging");
-	case _VIOC_ENDML:		return("End Modify Logging");
+	case _VIOC_WD:			return("Set Reintegration Parameters");
 	case _VIOC_HDB_VERIFY:		return("HDB Verify");
 	case _VIOC_HDB_ENABLE:		return("HDB Enable");
 	case _VIOC_HDB_DISABLE:		return("HDB Disable");
@@ -393,15 +387,15 @@ const char *IoctlOpStr(unsigned char nr)
 	case _VIOC_SET_LOCAL_VIEW:	return("Set Local View");
 	case _VIOC_SET_GLOBAL_VIEW:	return("Set Global View");
 	case _VIOC_SET_MIXED_VIEW:	return("Set Mixed View");
-	case _VIOC_WD_ALL:		return("WD All");
-	case _VIOC_WR_ALL:		return("WR All");
-	case _VIOC_STRONG:		return("Strong");
-	case _VIOC_ADAPTIVE:		return("Adaptive");
+	case _VIOC_WD_ALL:		return("Set Reint Params for All");
+	case _VIOC_SYNCCACHE_ALL:	return("Sync Cache All");
 	case _VIOC_LISTCACHE:		return("List Cache");
 	case _VIOC_GET_MT_PT:		return("Get Mt Pt");
 	case _VIOC_SYNCCACHE:		return("Sync Cache");
 	case _VIOC_REP_CMD:		return("Rep CMD");
 	case _VIOC_UNLOADKERNEL:	return("Unload Kernel");
+	case _VIOC_EXPANDOBJECT:	return("Expand object");
+	case _VIOC_COLLAPSEOBJECT:	return("Collapse object");
 
 	default:	snprintf(buf, 4, "%d", (unsigned int)nr);
 			return(buf);
@@ -476,7 +470,7 @@ void Terminate() {
 void DumpState() {
     if (!LogInited) return;
 
-    char *argv[1];
+    const char *argv[1];
     argv[0] = "all";
     VenusPrint(logFile, 1, argv);
     fflush(logFile);
@@ -744,7 +738,7 @@ void SwapLog()
 }
 
 
-char *lvlstr(LockLevel level)
+const char *lvlstr(LockLevel level)
 {
     switch(level) {
 	case NL:
