@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -32,32 +32,23 @@ listed in the file CREDITS.
 /*
  *    Implementation of the Venus Volume abstraction.
  *
- *    Each volume is always in one of five states.  These states, and
+ *    Each volume is always in one of three states.  These states, and
  *    the next state table are: (there are some caveats, which are
  *    discussed prior to the TakeTransition function)
  *
- *    Hoarding	(H)	(|AVSG| > 0) ? (logv ? L : (CML_Count > 0) ? L : (res_cnt > 0) ? S : H) : E
- *    Resolving	(S)	(|AVSG| > 0) ? (logv ? L : (CML_Count > 0) ? L : H) : E
- *    Emulating	(E)	(|AVSG| > 0) ? (logv ? L : (CML_Count > 0) ? L : (res_cnt > 0) ? S : H) : E
- *    Logging   (L)     (|AVSG| > 0) ? ((res_cnt > 0) ? S : logv ? L : (CML_Count > 0) ? L : H) : E
+ *    Unreachable	(D)	(|AVSG| == 0) ? D : (res_cnt > 0) ? R : W
+ *    Resolving		(R)	(|AVSG| == 0) ? D : (res_cnt > 0) ? R : W
+ *    Reachable		(W)     (|AVSG| == 0) ? D : (res_cnt > 0) ? R : W
  *
- *    State is initialized to Hoarding at startup, unless the volume
- *    is "dirty" in which case it is Emulating.
+ *    State is initialized to Reachable at startup.
  *
- *    Logging state may be triggered by discovery of weak connectivity, or an
- *    application-specific resolution.
- *    All of these are rolled into the flag "logv".  In this state, cache
- *    misses may be serviced, but modify activity is recorded in the CML.
- *    Resolution must be permitted in logging state because references
- *    resulting in cache misses may require it.
- *
- *    Note that non-replicated volumes have only two states, {Hoarding, Emulating}.
+ *    In this state, cache misses may be serviced, but modify activity is
+ *    recorded in the CML. Resolution must be permitted in logging state
+ *    because references resulting in cache misses may require it.
  *
  *    Events which prompt state transition are:
  *       1. Communications state change
  *       2. Begin/End of resolve session
- *       3. End of reintegration session
- *       4. Begin/End modify logging session
  *
  *    Volume state is an attempt to separate the logical state of the
  *    system---represented by the volume state---from the
@@ -89,13 +80,12 @@ listed in the file CREDITS.
  *        - the middle layer, CFS, acquires and performs Vice
  *        operations on fsobjs.  A CFS call uses an fsobj if it is
  *        valid.  An object is valid if it has a callback, is
- *        read-only, or if the volume is currently in disconnected
- *        mode (i.e., state = Emulating).  The CFS call attempts to
+ *        read-only, or if the volume is currently unreachable
+ *        (i.e., state = Unreachable).  The CFS call attempts to
  *        validate the object (by fetching status and/or data) if it
- *        is invalid and the volume is connected.  Mutating CFS calls
- *        are written through to servers if in connected mode.
- *        Mutating operations are recorded on a (per-volume) ModifyLog
- *        if in disconnected mode.  Pending state changes are noticed
+ *        is invalid and the volume is connected.
+ *        Mutating operations are recorded on a (per-volume) ModifyLog.
+ *        Pending state changes are noticed
  *        by the Get{Conn,Mgrp} and certain Collate{...} calls.  They
  *        return ERETRY in such cases, which is to be passed up to the
  *        VFS layer.
@@ -113,14 +103,14 @@ listed in the file CREDITS.
  *        bounds.
  *
  *
- *    A volume in {Hoarding, Emulating, Logging} state may be entered
+ *    A volume in {Unreachable, Reachable} state may be entered
  *    in either Mutating or Observing mode. There may be only one
  *    mutating user in a volume at a time, although that user may have
  *    multiple mutating threads.  Observers do not conflict with each
  *    other, nor with the mutator (if present). Thus, this scheme
  *    differs from classical Shared/Exclusive locking.  Also note that
  *    no user threads, mutating or not, may enter a volume in
- *    {Reintegrating, Resolving} state. These restrictions do not
+ *    {Resolving} state. These restrictions do not
  *    apply to non-rw-replicated volumes, of course.
  * */
 
@@ -152,8 +142,6 @@ extern "C" {
 
 /* from venus */
 #include <parse_realms.h>
-#include "adv_daemon.h"
-#include "advice.h"
 #include "comm.h"
 #include "fso.h"
 #include "local.h"
@@ -177,6 +165,11 @@ unsigned int repvol::allocs = 0;
 unsigned int repvol::deallocs = 0;
 #endif
 
+#define ASR_UID 0 /* XXX: To be changed! */
+
+int default_reintegration_age; /* how long a CML entry must exist before reint*/
+int default_reintegration_time; /* how long a reintegration attempt may take */
+
 /* local-repair modification */
 void VolInit(void)
 {
@@ -284,20 +277,18 @@ int VOL_HashFN(const void *key)
     return volid->Realm + volid->Volume;
 }
 
-static int GetRootVolume(Realm *realm, char **buf)
+static void GetRootVolume(Realm *realm, char **buf)
 {
     connent *c = NULL;
     RPC2_BoundedBS RVN;
     char *rvn = NULL;
-    int code, ret = 0;
-
+    int code;
 
     /* Get the connection. */
     if (realm->GetAdmConn(&c) != 0) {
 	LOG(100, ("GetRootVolume: can't get admin connection for realm %s!\n",
 		  realm->Name()));
 	RPCOpStats.RPCOps[ViceGetRootVolume_OP].bad++;
-	ret = -1;
 	goto err_exit;
     }
 
@@ -319,8 +310,6 @@ static int GetRootVolume(Realm *realm, char **buf)
     code = c->CheckResult(code, 0);
     UNI_RECORD_STATS(ViceGetRootVolume_OP);
 
-    PutConn(&c);
-
     if (!code) {
 	realm->SetRootVolName(rvn);
 	LOG(10, ("GetRootVolume: (%s) received name: %s, code: %d\n",
@@ -330,9 +319,9 @@ static int GetRootVolume(Realm *realm, char **buf)
 err_exit:
     if (rvn)
 	free(rvn);
+    PutConn(&c);
 
     *buf = strdup(realm->GetRootVolName());
-    return ret;
 }
 
 /* Allocate database from recoverable store. */
@@ -365,7 +354,7 @@ void vdb::ResetTransient()
     repvol_hash.SetHFn(VOL_HashFN);
 }
 
-void vdb::operator delete(void *deadobj, size_t len) {
+void vdb::operator delete(void *deadobj) {
     abort(); /* what else? */
 }
 
@@ -569,9 +558,9 @@ int vdb::Get(volent **vpp, Volid *volid)
 #if 1 /* XXX change this to hex after 5.3.9 servers are deployed */
       /* doesn't work as VLDB entries are still hashed by the old
        * volume-id representation */
-    sprintf(volname, "%lu", volid->Volume);
+    sprintf(volname, "%u", volid->Volume);
 #else
-    sprintf(volname, "%#08lx", volid->Volume);
+    sprintf(volname, "%08x", volid->Volume);
 #endif
 
     int err = ENOENT;
@@ -590,11 +579,13 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 {
     int code = 0;
     volent *v = NULL;
-    char *realm_name = NULL;
+    const char *realm_name = NULL;
     Realm *realm;
     char *volname = strdup(name);
     CODA_ASSERT(volname);
 
+    *vpp = 0;
+
     SplitRealmFromName(volname, &realm_name);
 
     if (realm_name) {
@@ -609,10 +600,7 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
      * root volume, try to get it from the servers */
     if (volname[0] == '\0' && realm != prealm) {
 	free(volname);
-	if (GetRootVolume(realm, &volname)) {
-	    code = ENOTCONN;
-	    goto error_exit;
-	}
+	GetRootVolume(realm, &volname);
     }
 
     /* Ivan Popov's suggestion, we just need to allow for long volume
@@ -629,8 +617,6 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 
     LOG(100, ("vdb::Get: volname = %s@%s\n", volname, realm->Name()));
 
-    *vpp = 0;
-
     VolumeInfo volinfo;
 
     for (;;) {
@@ -661,18 +647,21 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
     v = Find(realm, volname);
     if (code == ETIMEDOUT) {
 	/* Completely disconnected case. */
-	if (v) goto Exit;
+	if (v) {
+	    *vpp = v;
+	    goto Exit;
+	}
 
 	goto error_exit;
     }
     if (v && v->GetVolumeId() != volinfo.Vid)
     {
-	eprint("Mapping changed for volume %s@%s (%x --> %x)",
+	eprint("Mapping changed for volume %s@%s (%08x --> %08x)",
 	       volname, realm->Name(), v->GetVolumeId(), volinfo.Vid);
 
 	/* Put a (unique) fakename in the old volent. */
 	char fakename[V_MAXVOLNAMELEN];
-	sprintf(fakename, "%lu", v->GetVolumeId());
+	sprintf(fakename, "%08x", v->GetVolumeId());
 	CODA_ASSERT(Find(realm, fakename) == 0);
 
 	Recov_BeginTrans();
@@ -686,27 +675,24 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 	/* Should we flush the old volent? */
 
 	/* Invalidate HDB entries bound to the volname. XXX */
-
-	/* Forget about the old volent for now, it should disappear
-	 * 'gracefully' once all it's FSO's have been recycled. */
-	v->release();
     }
 
     /* Attempt the create will be mostly a noop if the volume already existed,
      * but it will grow or shrink the volume correctly if the volume
      * replication group has changed. */
-    v = Create(realm, &volinfo, volname);
+    *vpp = Create(realm, &volinfo, volname);
 
-    if (!v) {
+    /* Drop the reference on the volume we found earlier. */
+    if (v) v->release();
+
+    if (!*vpp) {
 	LOG(0, ("vdb::Get: Create (%x@%s, %s) failed\n", volinfo.Vid,
 		realm->Name(), volname));
-	*vpp = NULL;
 	code = EIO;
 	goto error_exit;
     }
 
 Exit:
-    *vpp = v;
     code = 0;
 
 error_exit:
@@ -762,39 +748,6 @@ void vdb::UpEvent(struct in_addr *host)
     VprocSignal(&voldaemon_sync);
 }
 
-void vdb::WeakEvent(struct in_addr *host)
-{
-    LOG(10, ("vdb::WeakEvent: host = %s\n", inet_ntoa(*host)));
-
-    /* Notify each volume of its failure. We only need to notify underlying
-     * replicas, they will notify their replicated parent */
-    volrep_iterator next;
-    volrep *v;
-    while ((v = next()))
-        if (v->IsHostedBy(host))
-            v->WeakMember();
-
-    /* provoke state transitions now */
-    VprocSignal(&voldaemon_sync);
-}
-
-void vdb::StrongEvent(struct in_addr *host)
-{
-    LOG(10, ("vdb::StrongEvent: host = %s\n", inet_ntoa(*host)));
-
-    /* Notify each volume of its failure. We only need to notify underlying
-     * replicas, they will notify their replicated parent */
-    volrep_iterator next;
-    volrep *v;
-    while ((v = next()))
-        if (v->IsHostedBy(host))
-            v->StrongMember();
-
-    /* provoke state transitions now */
-    VprocSignal(&voldaemon_sync);
-}
-
-
 /* MUST be called from within transaction! */
 void vdb::AttachFidBindings()
 {
@@ -805,7 +758,7 @@ void vdb::AttachFidBindings()
 }
 
 
-int vdb::WriteDisconnect(unsigned age, unsigned time)
+int vdb::WriteDisconnect(unsigned int age, unsigned int time)
 {
     repvol_iterator next;
     repvol *v;
@@ -819,14 +772,14 @@ int vdb::WriteDisconnect(unsigned age, unsigned time)
 }
 
 
-int vdb::WriteReconnect()
+int vdb::SyncCache()
 {
     repvol_iterator next;
     repvol *v;
     int code = 0;
 
     while ((v = next())) {
-        code = v->WriteReconnect();
+        code = v->SyncCache();
         if (code) break;
     }
     return(code);
@@ -839,8 +792,7 @@ void vdb::GetCmlStats(cmlstats& total_current, cmlstats& total_cancelled)
     repvol_iterator next;
     repvol *v;
     while ((v = next())) {
-	cmlstats current;
-	cmlstats cancelled;
+	cmlstats current, cancelled;
 	v->CML.IncGetStats(current, cancelled);
 	total_current += current;
 	total_cancelled += cancelled;
@@ -904,17 +856,16 @@ volent::volent(Realm *r, VolumeId volid, const char *volname)
     rvmlib_set_range(name, strlen(volname) + 1);
     strcpy(name, volname);
 
-    flags.reserved = 0;
+    flags.unused1=flags.unused2=flags.unused3=flags.unused4=flags.unused5 = 0;
     flags.has_local_subtree = 0;
-    flags.logv = 0;
-    lc_asr = -1;
+    pgid = NO_ASR;
 }
 
 
 /* local-repair modification */
 void volent::ResetVolTransients()
 {
-    state = Hoarding;
+    state = Reachable;
     observer_count = 0;
     mutator_count = 0;
     waiter_count = 0;
@@ -924,15 +875,16 @@ void volent::ResetVolTransients()
     resolver_count = 0;
     flags.transition_pending = 1;
     flags.demotion_pending = 0;
-    flags.allow_asrinvocation = 1;
+    flags.allow_asrinvocation = 1;  /* user preference, controlled by cfs */
+	flags.enable_asrinvocation = 1; /* whether system is able to launch */
     flags.asr_running = 0;
-    flags.reintegratepending = 0;
     flags.reintegrating = 0;
     flags.repair_mode = 0;		    /* normal mode */
     flags.resolve_me = 0;
-    flags.weaklyconnected = 0;
     flags.available = 1;
     flags.sync_reintegrate = 0;
+    flags.reint_conflict = 0;
+    flags.unauthenticated = 0;
 
     list_head_init(&fso_list);
 
@@ -965,7 +917,7 @@ volent::~volent()
 }
 
 /* MUST be called from within transaction! */
-void volent::operator delete(void *deadobj, size_t len)
+void volent::operator delete(void *deadobj)
 {
     LOG(10, ("volent::operator delete()\n"));
     rvmlib_rec_free(deadobj);
@@ -994,8 +946,11 @@ void volent::release(void)
     int intrans = rvmlib_in_transaction();
 
     if (!intrans) Recov_BeginTrans();
-    if (IsReplicated()) delete (repvol *)this;
-    else		delete (volrep *)this;
+    if (IsReplicated()) {
+	MarinerReportVolState(name, realm->Name(), "deleted", 0, &flags);
+	delete (repvol *)this;
+    } else
+	delete (volrep *)this;
     if (!intrans) Recov_EndTrans(MAXFP);
 }
 
@@ -1006,6 +961,20 @@ int volent::IsReadWriteReplica()
     return (((volrep *)this)->ReplicatedVol() != 0);
 }
 
+void repvol::ReportVolState(void)
+{
+    const char *volstate;
+
+    switch (state) {
+    case Reachable:	volstate = "reachable"; break;
+    case Resolving:	volstate = "resolving"; break;
+    case Unreachable:	volstate = "unreachable"; break;
+    default:		volstate = "unknown"; break;
+    }
+
+    MarinerReportVolState(name, realm->Name(), volstate, CML.count(), &flags);
+}
+
 
 /* See the notes on volume synchronization at the head of this file.
  *
@@ -1075,11 +1044,10 @@ int volent::Enter(int mode, uid_t uid)
      * volumes have as well.
      * We'd like them all to take transitions/demotions first so we can check
      * them en masse.  We risk sticking a real request with this overhead, but
-     * only if a request arrives in the next few (5) seconds!  
+     * only if a request arrives in the next few (5) seconds!
      */
     vproc *vp = VprocSelf();
-    if (VCBEnabled && IsReplicated() && 
-        (state == Hoarding || state == Logging) && 
+    if (VCBEnabled && IsReplicated() && IsReachable() &&
         ((repvol *)this)->WantCallBack())
     {
         repvol *rv = (repvol *)this;
@@ -1105,7 +1073,7 @@ int volent::Enter(int mode, uid_t uid)
 		 */
 		int proc_key = vp->u.u_pgid;
 		while ((excl_count > 0 && proc_key != excl_pgid) ||
-                       state == Resolving ||
+                       IsResolving() ||
                        (IsReplicated() && WriteLocked(&((repvol *)this)->CML_lock)) ||
                        flags.transition_pending) {
 		    if (mode & VM_NDELAY) return (EWOULDBLOCK);
@@ -1120,6 +1088,7 @@ int volent::Enter(int mode, uid_t uid)
 		 */
 		if (IsReplicated()) {
                     repvol *rv = (repvol *)this;
+
 		    /* 
 		     * Claim ownership if the volume is free. 
 		     * The CML lock is used to prevent checkpointers and
@@ -1156,6 +1125,19 @@ int volent::Enter(int mode, uid_t uid)
 			return(0);
 		    }
 
+		    /*
+		     * Allow ASR's in regardless; they cannot be locked out or
+		     * the system will die in deadlock.
+		     */
+
+		    if (rv->asr_running() && uid == ASR_UID) {
+		      mutator_count++;
+		      shrd_count++;
+		      ObtainReadLock(&rv->CML_lock);
+		      return(0);
+		    }
+
+
 		    /* Wait until the volume becomes free again. */
 		    {
 			if (mode & VM_NDELAY) return(EWOULDBLOCK);
@@ -1181,8 +1163,8 @@ int volent::Enter(int mode, uid_t uid)
 		 * transition is pending.
 		 */
 		int proc_key = vp->u.u_pgid;
-		while ((excl_count > 0 && proc_key != excl_pgid) || state == Resolving
-		        || flags.transition_pending) {
+		while ((excl_count > 0 && proc_key != excl_pgid) ||
+		       IsResolving() || flags.transition_pending) {
 		    if (mode & VM_NDELAY) return (EWOULDBLOCK);
 		    LOG(0, ("volent::Enter: observe with proc_key = %d\n",
 			    proc_key));
@@ -1258,31 +1240,37 @@ void volent::Exit(int mode, uid_t uid)
     switch(mode & (VM_MUTATING | VM_OBSERVING | VM_RESOLVING)) {
 	case VM_MUTATING:
 	    {
- 	    if (state == Resolving || mutator_count <= 0)
+	    if (IsResolving() || mutator_count <= 0)
 		{ print(logFile); CHOKE("volent::Exit: mutating"); }
 
 	    mutator_count--;
 	    shrd_count--;
-            if (IsReplicated())
-                ReleaseReadLock(&((repvol *)this)->CML_lock);
-            if (IsReplicated() && mutator_count == 0 &&
-                ((repvol *)this)->GetCML()->count() == 0 &&
-                !((repvol *)this)->IsReintegrating()) {
-		/* Special-case here. */
-                /* If we just cancelled the last log record for a volume that
-                 * was being kept in Emulating state due to auth-token
-                 * absence, we need to provoke a transition! */
-		if (state == Emulating && !flags.transition_pending)
-		    flags.transition_pending = 1;
 
-		((repvol *)this)->GetCML()->owner = UNSET_UID;
+	    if (IsReplicated()) {
+		repvol *rv = (repvol *)this;
+		int count = rv->GetCML()->count();
+
+		ReleaseReadLock(&((repvol *)this)->CML_lock);
+
+		if (mutator_count == 0 && count == 0 && !rv->IsReintegrating())
+		{
+		    /* Special-case here. */
+		    /* If we just cancelled the last log record for a volume that
+		     * was being kept in Unreachable state due to auth-token
+		     * absence, we need to provoke a transition! */
+		    if (IsUnreachable() && !flags.transition_pending)
+			flags.transition_pending = 1;
+
+		    rv->GetCML()->owner = UNSET_UID;
+		}
+		rv->ReportVolState();
 	    }
 	    break;
 	    }
 
 	case VM_OBSERVING:
 	    {
- 	    if (state == Resolving || observer_count <= 0)
+	    if (IsResolving() || observer_count <= 0)
 		{ print(logFile); CHOKE("volent::Exit: observing"); }
 
 	    observer_count--;
@@ -1293,7 +1281,7 @@ void volent::Exit(int mode, uid_t uid)
 	case VM_RESOLVING:
 	    {
 	    CODA_ASSERT(IsReplicated());
-	    if (state != Resolving || resolver_count == 0 || !flags.transition_pending)
+	    if (!IsResolving() || resolver_count == 0 || !flags.transition_pending)
 		{ print(logFile); CHOKE("volent::Exit: resolving"); }
 
 	    resolver_count--;
@@ -1327,68 +1315,49 @@ void volent::TakeTransition()
 {
     CODA_ASSERT(flags.transition_pending && !VOLBUSY(this));
 
-    int size = AVSGsize();
-    repvol *rv = NULL;
-    if (IsReplicated()) rv = (repvol *)this;
+    VolumeStateType nextstate;
+    int avsgsize = AVSGsize();
+    repvol *rv;
 
-    LOG(1, ("volent::TakeTransition: %s, state = %s, |AVSG| = %d, logv = %d\n",
-	     name, PRINT_VOLSTATE(state), size, flags.logv));
-    if (IsReplicated())
-        LOG(1, ("\tCML = %d, Res = %d\n",
-                rv->GetCML()->count(), rv->ResListCount()));
+    if (!IsReplicated()) {
+	LOG(1, ("volent::TakeTransition %s |AVSG| = %d\n", name, avsgsize));
+	state = (avsgsize == 0) ? Unreachable : Reachable;
+	flags.transition_pending = 0;
+	Signal();
+	return;
+    }
+    rv = (repvol *)this;
+
+    LOG(1, ("volent::TakeTransition: %s, state = %s, |AVSG| = %d, "
+	    "CML = %d, Res = %d\n",
+	    name, PRINT_VOLSTATE(state), avsgsize,
+	    rv->GetCML()->count(), rv->ResListCount()));
 
     /* Compute next state. */
-    VolumeStateType nextstate;
+    CODA_ASSERT(IsUnreachable() || IsReachable() || IsResolving());
 
-    CODA_ASSERT(state == Hoarding || state == Emulating || state == Logging ||
-		state == Resolving);
+    nextstate = Reachable;
+    /* if the AVSG is empty we are Unreachable */
+    if (avsgsize == 0)
+	nextstate = Unreachable;
 
-    nextstate = Hoarding;
-    /* if the AVSG is empty we are disconnected */
-    if (size == 0) nextstate = Emulating;
-
-    else if (IsReplicated()) {
-        if (rv->ResListCount())
-            nextstate = Resolving;
-
-        else if (flags.logv || rv->GetCML()->count())
-            nextstate = Logging;
-    }
-
-    /* Special cases here. */
-    /*
-     * 1.  If the volume is transitioning _to_ emulating, any reintegations
-     *     will not be stopped because of lack of tokens.
-     */
-    if (nextstate == Emulating && IsReplicated())
-	rv->ClearReintegratePending();
-
-    /* 2. We refuse to transit to reintegration unless owner has auth tokens.
-     * 3. We force "zombie" volumes to emulation state until they are
-     *    un-zombied. */
-    if (nextstate == Logging && rv->GetCML()->count() > 0) {
-	userent *u = rv->realm->GetUser(rv->GetCML()->Owner());
-	if (!u->TokensValid()) {
-	    rv->SetReintegratePending();
-	    nextstate = Emulating;
-	}
-	PutUser(&u);
-    }
+    else if (rv->ResListCount())
+	nextstate = Resolving;
 
     /* Take corresponding action. */
     state = nextstate;
     flags.transition_pending = 0;
+    rv->ReportVolState();
 
     switch(state) {
-        case Logging:
-	    if (flags.sync_reintegrate)
+	case Reachable:
+	    if (rv->IsSync())
 		rv->Reintegrate();
-	    else if (rv->ReadyToReintegrate()) 
+	    else if (rv->ReadyToReintegrate())
 		::Reintegrate(rv);
-            // Fall through
+	    // Fall through
 
-        case Hoarding:
-        case Emulating:
+	case Unreachable:
 	    Signal();
 	    break;
 
@@ -1442,7 +1411,7 @@ void repvol::DownMember(struct in_addr *host)
 
     ResetStats();
 
-    /* Consider transitioning to Emulating state. */
+    /* Consider transitioning to Unreachable state. */
     if (AVSGsize() == 0) {
 	flags.transition_pending = 1;
         /* Coherence is now suspect */
@@ -1479,7 +1448,7 @@ void repvol::UpMember(void)
 
     LOG(10, ("repvol::UpMember: vid = %08x\n", vid));
 
-    /* Consider transitting to Hoarding state. */
+    /* Consider transitioning to Reachable state. */
     if (AVSGsize() == 1)
 	flags.transition_pending = 1;
 
@@ -1488,111 +1457,21 @@ void repvol::UpMember(void)
     ResetStats();
 }
 
-/* 
- * volent::{Weak,Strong} member.  Cope with a change in 
- * connectivity for a VSG member by  setting the volume
- * weakly connected, and by write-disconnecting or write-reconnecting 
- * the volume.  Note that both of these indications are necessary;
- * while for replicated volumes 
- * 	weakly connected --> write-disconnected 
- * it is not the case that write-disconnected --> weakly connected.
- * In particular, the volume may be write-disconnected for an ASR.
- * Note that write disconnection does not apply to read-only and
- * non-replicated volumes.
- */
-void volrep::WeakMember()
-{
-    flags.weaklyconnected = 1;
-    /* if we are a volume replica notify our replicated parent */
-    if (IsReadWriteReplica()) {
-	Volid volid;
-	volent *v;
-	volid.Realm = realm->Id();
-	volid.Volume = ReplicatedVol();
-        v = VDB->Find(&volid);
-        if (v) {
-            CODA_ASSERT(v->IsReplicated());
-            ((repvol *)v)->WeakMember();
-	    v->release();
-        }
-    }
-}
-
-void repvol::WeakMember()
-{
-    /* Normally a weakmember event implies that WeakVSGSize > 0, however
-     * we might get weak events from VSG servers when using a staging server. */
-    if (!IsWeaklyConnected() && WeakVSGSize() > 0) {
-        WriteDisconnect();
-
-        flags.transition_pending = 1;
-	flags.weaklyconnected = 1;
-    }
-}
-
-void volrep::StrongMember()
-{
-    flags.weaklyconnected = 0;
-    /* if we are a volume replica notify our replicated parent */
-    if (IsReadWriteReplica()) {
-	Volid volid;
-	volent *v;
-	volid.Realm = realm->Id();
-	volid.Volume = ReplicatedVol();
-        v = VDB->Find(&volid);
-        if (v) {
-            CODA_ASSERT(v->IsReplicated());
-            ((repvol *)v)->StrongMember();
-	    v->release();
-        }
-    }
-}
-
-void repvol::StrongMember()
-{
-    /* vsg check is for 0, not 1, because the conn is already strong */
-    if (WeakVSGSize() == 0) {
-        WriteReconnect();
-
-        flags.transition_pending = 1;
-	flags.weaklyconnected = 0;
-    }
-}
-
-int repvol::WriteDisconnect(unsigned age, unsigned time)
+int repvol::WriteDisconnect(unsigned int age, unsigned int hogtime)
 {
     Recov_BeginTrans();
-        RVMLIB_REC_OBJECT(*this);
-        flags.logv = 1;
-        if (age != AgeLimit || time != ReintLimit ||
-            age == V_UNSETAGE || time == V_UNSETREINTLIMIT) {
-            if (age == V_UNSETAGE) 
-                AgeLimit = V_DEFAULTAGE;
-            else 
-                AgeLimit = age;
 
-            if (time == V_UNSETREINTLIMIT) 
-                ReintLimit = V_DEFAULTREINTLIMIT;
-            else 
-                ReintLimit = time*1000;
-        }
+    if (age != V_UNSETAGE && age != AgeLimit) {
+	RVMLIB_REC_OBJECT(AgeLimit);
+	AgeLimit = age;
+    }
+
+    if (hogtime != V_UNSETREINTLIMIT && hogtime != ReintLimit) {
+	RVMLIB_REC_OBJECT(ReintLimit);
+	ReintLimit = hogtime;
+    }
+
     Recov_EndTrans(MAXFP);
-    flags.transition_pending = 1;
-
-    return 0;
-}
-
-
-int repvol::WriteReconnect()
-{
-    Recov_BeginTrans();
-        RVMLIB_REC_OBJECT(*this);
-        flags.logv = 0;
-        AgeLimit = V_UNSETAGE;
-        ReintLimit = V_UNSETREINTLIMIT;
-    Recov_EndTrans(MAXFP);
-    flags.transition_pending = 1;
-
     return 0;
 }
 
@@ -1619,34 +1498,6 @@ int repvol::SyncCache(VenusFid * fid)
     return 0;
 }
 
-void repvol::SetReintegratePending() {
-    flags.reintegratepending = 1;
-    CheckReintegratePending();
-}
-
-
-void repvol::ClearReintegratePending() {
-    flags.reintegratepending = 0;
-    /* if (SkkEnabled) {
-     *    userent *u = realm->GetUser(CML.owner);
-     *    CODA_ASSERT(u != NULL);
-     *    u->NotifyReintegrationEnabled(name);
-     * } */
-}
-
-
-void repvol::CheckReintegratePending() {
-    if (flags.reintegratepending && CML.count() > 0) {
-        eprint("Reintegrate %s pending tokens for uid = %d", name, CML.owner);
-	/* if (SkkEnabled) {
-         *    userent *u = realm->GetUser(CML.owner);
-         *    CODA_ASSERT(u != NULL);
-         *    u->NotifyReintegrationPending(name);
-	 * } */
-    }
-}
-
-
 void volent::Wait()
 {
     waiter_count++;
@@ -1750,13 +1601,14 @@ volrep::~volrep()
     deallocs++;
 #endif
 
-    PutServer(&volserver);
     /* Remove from hash table. */
     Volid volid;
     volid.Realm = realm->Id();
     volid.Volume = vid;
     if (VDB->volrep_hash.remove(&volid, &handle) != &handle)
 	{ print(logFile); CHOKE("volrep::~volrep: htab remove"); }
+
+    PutServer(&volserver);
 }
 
 void volrep::ResetTransient(void)
@@ -1785,7 +1637,7 @@ void repvol::ResetTransient(void)
 
     /* do grab refcounts on the underlying replicas */
     for (int i = 0; i < VSG_MEMBERS; i++)
-        if (volreps[i]) volreps[i]->hold();
+	if (volreps[i]) volreps[i]->hold();
 
     GetHosts(hosts);
     vsg = VSGDB->GetVSG(hosts, GetRealmId());
@@ -1808,7 +1660,7 @@ void repvol::ResetTransient(void)
      * at the servers, but we crashed before we took the fid off of our
      * queue). */
     if (!CleanShutDown &&
-        (FileFids.Count != 0 || DirFids.Count != 0 || SymlinkFids.Count != 0))
+	(FileFids.Count != 0 || DirFids.Count != 0 || SymlinkFids.Count != 0))
     {
 	Recov_BeginTrans();
 	    RVMLIB_REC_OBJECT(FileFids);
@@ -1845,18 +1697,17 @@ repvol::repvol(Realm *r, VolumeId vid, const char *name, volrep *reps[VSG_MEMBER
     memcpy(volreps, reps, VSG_MEMBERS * sizeof(volrep *));
 
     VVV = NullVV;
-    AgeLimit = V_UNSETAGE;
-    ReintLimit = V_UNSETREINTLIMIT;
+    AgeLimit   = default_reintegration_age;  /* seconds */
+    ReintLimit = default_reintegration_time; /* milliseconds */
     reint_id_gen = 100;
     flags.replicated = 1;
 
     /* The uniqifiers should also survive shutdowns, f.i. the server
-       remembers the last sid we successfully reintegrated. And in
-       disconnected mode the unique fids map to unique inodes. The
-       1<<19 shift is to avoid collisions with inodes derived from
+       remembers the last sid we successfully reintegrated. And when
+       the volume is unreachable the unique fids map to unique inodes.
+       The 1<<19 shift is to avoid collisions with inodes derived from
        non-local generated fids -- JH */
     FidUnique = 1 << 19;
-    SidUnique = 0;
 
     ResetTransient();
 
@@ -1874,7 +1725,7 @@ repvol::repvol(Realm *r, VolumeId vid, const char *name, volrep *reps[VSG_MEMBER
 repvol::~repvol()
 {
     LOG(10, ("repvol::~repvol: name = %s, volume = %x, type = ReplicatedVolume\n", name, vid));
-    
+
     int i;
 
 #ifdef VENUSDEBUG
@@ -1888,24 +1739,35 @@ repvol::~repvol()
     if (VDB->repvol_hash.remove(&volid, &handle) != &handle)
 	{ print(logFile); CHOKE("repvol::~repvol: htab remove"); }
 
-    /* Unlink from VSG (if applicable). */
+    /* try to flush any pending COP2 and/or pending resolution requests */
+    flags.transition_pending = 1;
+    TakeTransition();
+
+    /* Unlink from volume replicas (if applicable). */
     for (i = 0; i < VSG_MEMBERS; i++)
-        VDB->Put((volent **)&volreps[i]);
+	VDB->Put((volent **)&volreps[i]);
 
     if (ro_replica)
 	VDB->Put((volent **)&ro_replica);
 
+    /* This should never happen. If there are CML entries, there will be
+     * dirty fsobjs, which will have a reference on the repvol preventing
+     * it's destruction */
     if (CML.count() != 0)
-        CHOKE("volent::~volent: CML not empty");
+	CHOKE("repvol::~repvol: CML not empty");
 
+    /* pending resolution requests should all be dequeued by TakeTransition */
     if (res_list->count() != 0)
-        CHOKE("repvol::~repvol: res_list not empty");
+	CHOKE("repvol::~repvol: res_list not empty");
     delete res_list;
 
+    /* We're about to destroy this repvol. If failed to flush the COP2 entries
+     * to the server, we delete any remaining entries. */
+    ClearCOP2();
     if (cop2_list->count() != 0)
-        CHOKE("volent::~volent: cop2_list not empty");
+	CHOKE("repvol::~repvol: cop2_list not empty");
     delete cop2_list;
-    
+
     vsg->Put();
 }
 
@@ -1914,6 +1776,9 @@ int volrep::GetConn(connent **c, uid_t uid)
     int code = ERETRY;
     *c = 0;
 
+    if (!volserver)
+	return ETIMEDOUT;
+
     while (code == ERETRY && !flags.transition_pending) {
         code = volserver->GetConn(c, uid);
         if (code < 0)
@@ -2059,7 +1924,7 @@ int repvol::AllocFid(ViceDataType Type, VenusFid *target_fid,
     *AllocHost = 0;
 
     /* 
-     * While write-disconnected we usually want to generate a local fid.
+     * While the volume is reachable we usually want to generate a local fid.
      * This defers the latency of contacting the servers for fids until
      * reintegrate time.  However, reintegrators MUST contact the servers
      * for global fids to translate local fids in the CML records being
@@ -2069,11 +1934,11 @@ int repvol::AllocFid(ViceDataType Type, VenusFid *target_fid,
      * a mutator executing during reintegration need not (and should not) 
      * be required to contact the servers.
      */
-    if (state == Emulating || (state == Logging && !force)) {
+    if (IsUnreachable() || (IsReachable() && !force)) {
 	*target_fid = GenerateLocalFid(Type);
     }
     else {
-	VOL_ASSERT(this, (state == Hoarding || (state == Logging && force)));
+	VOL_ASSERT(this, (IsReachable() && force));
 
 	/* COP2 Piggybacking. */
 	char PiggyData[COP2SIZE];
@@ -2090,6 +1955,9 @@ int repvol::AllocFid(ViceDataType Type, VenusFid *target_fid,
 	    /* The Remote AllocFid call. */
 	    {
 		ViceFidRange NewFids;
+		NewFids.Vnode = 0;
+		NewFids.Unique = 0;
+		NewFids.Stride = 0;
 		NewFids.Count = 32;
 
 		/* Make multiple copies of the IN/OUT and OUT parameters. */
@@ -2256,20 +2124,6 @@ int repvol::AVSGsize(void)
     return avsgsize;
 }
 
-int repvol::WeakVSGSize()
-{
-    int count = 0;
-    
-    if (ro_replica)
-	return ro_replica->IsWeaklyConnected() ? 1 : 0;
-
-    for (int i = 0; i < VSG_MEMBERS; i++)
-        if (volreps[i] && volreps[i]->IsWeaklyConnected())
-            count++;
-
-    return(count);
-}
-
 /* volume id's for staging volumes */
 static VolumeId stagingvid = 0xFF000000;
 void repvol::SetStagingServer(struct in_addr *srvr)
@@ -2378,7 +2232,7 @@ VenusFid repvol::GenerateLocalFid(ViceDataType fidtype)
 
 
 /* MUST be called from within a transaction */
-VenusFid repvol::GenerateFakeFid()
+VenusFid volent::GenerateFakeFid()
 {
     VenusFid fid;
     FID_MakeSubtreeRoot(MakeViceFid(&fid), vid, FidUnique);
@@ -2390,31 +2244,13 @@ VenusFid repvol::GenerateFakeFid()
     return(fid);
 }
 
-/* MUST be called from within a transaction */
-ViceStoreId repvol::GenerateStoreId()
-{
-    ViceStoreId sid;
-
-    /* VenusGenID, is randomly chosen whenever rvm is reinitialized, it
-     * should be a 128-bit UUID (re-generated whenever rvm is reinitialized).
-     * But that would require changing in the venus-vice protocol to either
-     * add this UUID to every operation, or send it once per (volume-)
-     * connection setup with ViceNewConnectFS. -JH */
-    sid.Host = (RPC2_Unsigned)VenusGenID;
-
-    RVMLIB_REC_OBJECT(SidUnique);
-    sid.Uniquifier = (RPC2_Unsigned)SidUnique++;
-
-    return(sid);
-}
-
 /* Helper routine to help convert code for (int)(_volent.type) to code
    for (ViceVolumeType)(_VolumeStatus.Type).  This conversion is needed
    by volent::GetVolStat().  When connected, GetVolStat get the type
    from the servers, the code can be {ReadOnly, ReadWrite} ("Backup"
    and "Replicated" are never returned by server
-   (c.f. SetVolumeStatus())).  When disconnected, GetVolStat get the
-   type from volent.
+   (c.f. SetVolumeStatus())).  When the volume is unreachable, GetVolStat
+   gets the type from volent.
    -- Clement
 */
 ViceVolumeType volent::VolStatType(void)
@@ -2430,29 +2266,35 @@ ViceVolumeType volent::VolStatType(void)
 
 /* local-repair modification */
 int volent::GetVolStat(VolumeStatus *volstat, RPC2_BoundedBS *Name,
-		       VolumeStateType *conn_state, int *conflict,
-                       int*cml_count, RPC2_BoundedBS *msg,
-                       RPC2_BoundedBS *motd, uid_t uid, int local_only)
+		       VolumeStateType *conn_state,
+		       unsigned int *age, unsigned int *hogtime,
+		       int *conflict, int *cml_count, uint64_t *cml_bytes,
+		       RPC2_BoundedBS *msg, RPC2_BoundedBS *motd,
+		       uid_t uid, int local_only)
 {
     int code = 0;
 
     LOG(100, ("volent::GetVolStat: vid = %x, uid = %d\n", vid, uid));
 
     *conn_state = state;
-    *conflict = 0;
-    *cml_count = 0;
+    *age = *hogtime = 0;
+    *conflict = *cml_count = 0;
+    *cml_bytes = 0;
     if (IsReplicated()) {
-        repvol *rv = (repvol *)this;
-        rv->CheckLocalSubtree();       /* unset has_local_subtree if possible */
-        *conflict = rv->HasLocalSubtree();
-        *cml_count = rv->GetCML()->count();
-    }
-    if (state == Hoarding && (*conflict || *cml_count > 0)) {
-	LOG(0, ("volent::GetVolStat: Strange! A connected volume 0x%x has "
-		"conflict or cml_count != 0 (=%d)?\n", vid, *cml_count));
+	cmlstats current, cancelled;
+	repvol *rv = (repvol *)this;
+
+	*conflict = rv->ContainUnrepairedCML();
+
+	rv->GetCML()->IncGetStats(current, cancelled, UNSET_TID);
+	*cml_count = current.store_count + current.other_count;
+	*cml_bytes = (size_t)(current.store_size + current.store_contents_size +
+			      current.other_size);
+	*age = rv->AgeLimit;
+	*hogtime = rv->ReintLimit;
     }
 
-    if (IsFake() || state == Emulating || local_only) {
+    if (IsLocalRealm() || IsUnreachable() || local_only) {
 	memset(volstat, 0, sizeof(VolumeStatus));
 	volstat->Vid = vid;
 	volstat->Type = VolStatType();	
@@ -2470,7 +2312,7 @@ int volent::GetVolStat(VolumeStatus *volstat, RPC2_BoundedBS *Name,
 	code = 0;
     }
     else {
-	VOL_ASSERT(this, (state == Hoarding || state == Logging));
+	VOL_ASSERT(this, (IsReachable()));
 
 	if (IsReplicated()) {
 	    /* Acquire an Mgroup. */
@@ -2563,12 +2405,12 @@ int volent::SetVolStat(VolumeStatus *volstat, RPC2_BoundedBS *Name,
 
     int code = 0;
 
-    if (state == Emulating) {
+    if (IsUnreachable()) {
 	/* We do not cache this data! */
 	code = ETIMEDOUT;
     }
     else {
-	VOL_ASSERT(this, (state == Hoarding || state == Logging));  /* let this go in wcc? -lily */
+	VOL_ASSERT(this, (IsReachable()));  /* let this go in wcc? -lily */
 
         /* COP2 Piggybacking. */
 	char PiggyData[COP2SIZE];
@@ -2580,19 +2422,20 @@ int volent::SetVolStat(VolumeStatus *volstat, RPC2_BoundedBS *Name,
 	    /* Acquire an Mgroup. */
 	    mgrpent *m = 0;
 	    vv_t UpdateSet;
-            repvol *vp = (repvol *)this;
+	    repvol *vp = (repvol *)this;
+	    ViceStoreId sid;
 
 	    Recov_BeginTrans();
-            ViceStoreId sid = vp->GenerateStoreId();
+	    Recov_GenerateStoreId(&sid);
 	    Recov_EndTrans(MAXFP);
 
-  	    code = vp->GetMgrp(&m, uid, (PIGGYCOP2 ? &PiggyBS : 0));
+	    code = vp->GetMgrp(&m, uid, (PIGGYCOP2 ? &PiggyBS : 0));
 	    if (code != 0) goto RepExit;
 
 	    {
 		/* Make multiple copies of the IN/OUT and OUT parameters. */
 		int ph_ix; unsigned long ph;
-                ph = ntohl(m->GetPrimaryHost(&ph_ix)->s_addr);
+		ph = ntohl(m->GetPrimaryHost(&ph_ix)->s_addr);
 
 		if (Name->MaxSeqLen > VENUS_MAXBSLEN ||
 		    msg->MaxSeqLen > VENUS_MAXBSLEN ||
@@ -2703,11 +2546,10 @@ void volent::print(int afd)
     fdprint(afd, "%#08x : %-16s : vol = %x @%s\n", (long)this, name, vid,
 	    realm->Name());
 
-    fdprint(afd, "\trefcnt = %d, fsos = %d, logv = %d, weak = %d\n",
-	    refcnt, nfsos, flags.logv, flags.weaklyconnected);
+    fdprint(afd, "\trefcnt = %d, fsos = %d\n", refcnt, nfsos);
     fdprint(afd, "\tstate = %s, t_p = %d, d_p = %d, counts = [%d %d %d %d], repair = %d\n",
 	    PRINT_VOLSTATE(state), flags.transition_pending, flags.demotion_pending,
-	    observer_count, mutator_count, waiter_count, resolver_count, 
+	    observer_count, mutator_count, waiter_count, resolver_count,
 	    flags.repair_mode);
     fdprint(afd, "\tshrd_count = %d and excl_count = %d excl_pgid = %d\n",
 	    shrd_count, excl_count, excl_pgid);
@@ -2730,8 +2572,7 @@ void repvol::print_repvol(int afd)
 	    AgeLimit, ReintLimit);
     fdprint(afd, "\tasr allowed %d, running %d\n", 
 	    flags.allow_asrinvocation, flags.asr_running);
-    fdprint(afd, "\treintegrate pending %d, reintegrating = %d\n", 
-	    flags.reintegratepending, flags.reintegrating);
+    fdprint(afd, "\treintegrating = %d\n", flags.reintegrating);
     fdprint(afd, "\thas_local_subtree = %d, resolve_me = %d\n", 
 	    flags.has_local_subtree, flags.resolve_me);
 
@@ -2767,7 +2608,7 @@ void volent::ListCache(FILE* fp, int long_format, unsigned int valid)
 {
     char mountpath[MAXPATHLEN];
     GetMountPath(mountpath, 0);
-    fprintf(fp, "%s: %lx\n", mountpath, vid);
+    fprintf(fp, "%s: %08x\n", mountpath, vid);
 
     struct dllist_head *p;
     list_for_each(p, fso_list) {
