@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -58,6 +58,7 @@ extern "C" {
 #include <voldefs.h>
 
 /* from venus */
+#include "comm.h"
 #include "venusrecov.h"
 #include "realmdb.h"
 #include "venus.private.h"
@@ -92,9 +93,7 @@ const int MLENTMaxFreeEntries = 32;
 
 const int UNSET_TID = -1;
 
-const unsigned V_DEFAULTAGE = 60;		/* in SECONDS */
 const unsigned V_UNSETAGE = (unsigned)-1;	/* huge */
-const unsigned V_DEFAULTREINTLIMIT = 30000;	/* in MILLESECONDS */
 const unsigned V_UNSETREINTLIMIT = (unsigned)-1;/* huge */
 
 /* Volume-User modes. */
@@ -108,6 +107,10 @@ const unsigned V_UNSETREINTLIMIT = (unsigned)-1;/* huge */
                                            already locked. It's necessary to
                                            keep daemons from getting ``stuck''
                                            on volumes already in use. */
+
+/* Define for 'no ASR running'  */
+#define NO_ASR 0
+
 /*  *****  Types  ***** */
 
 class cmlstats {
@@ -176,13 +179,14 @@ class ClientModifyLog {
     int COP1(char *, int, ViceVersionVector *, int outoforder);
     void UnLockObjs(int);
     void MarkFailedMLE(int);
-    void HandleFailedMLE();
+    void HandleFailedMLE(void);
     void MarkCommittedMLE(RPC2_Unsigned);
     void CancelPending();
     void ClearPending();
+    void ClearToBeRepaired(); /* must not be called within transaction! */
     void CancelStores();
 
-    void GetReintegrateable(int, int *);
+    int GetReintegrateable(int, unsigned long *, int *);
     cmlent *GetFatHead(int);
 
     /* Call to set/clear flags for whether it's safe to cancel frozen entries */
@@ -213,17 +217,23 @@ class ClientModifyLog {
     void IncGetStats(cmlstats&, cmlstats&, int =UNSET_TID); /*N*/
     int IncReallocFids(int);                            /*U*/
     int HaveElements(int);                              /*N*/
+    int DiscardLocalMutation(char *);                   /*U*/
+    void PreserveLocalMutation(char *);                 /*U*/
+    void PreserveAllLocalMutation(char *);              /*U*/
+    void CheckCMLHead(char *msg);                       /*U*/
+    int ListCML(FILE *);                                /*N*/
 };
 
 /* local-repair addition */
 struct CmlFlags {
     unsigned to_be_repaired : 1;
-    unsigned repair_mutation : 1;
+    unsigned unused : 1;
     unsigned frozen : 1;			/* do not cancel */
     unsigned cancellation_pending : 1;		/* once unfrozen */
     /*T*/unsigned failed : 1;			/* offending record */
     /*T*/unsigned committed : 1;		/* committed at server */
-    unsigned reserved : 26;
+    unsigned prepended : 1;
+    unsigned reserved : 25;
 };
 
 
@@ -251,6 +261,7 @@ class cmlent {
     UserId uid;			/* author of operation */
     int tid;			/* local-repair addition */
     CmlFlags flags;		/* local-repair addition */
+    /*T*/int expansions;
 
     /* Discriminant and sub-type specific members. */
     int opcode;
@@ -350,7 +361,7 @@ class cmlent {
     cmlent(ClientModifyLog *, time_t, uid_t, int, int ...);	/* local-repair modification */
     void ResetTransient();
     ~cmlent();
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
     /* Size of an entry */
     long bytes();
@@ -368,7 +379,7 @@ class cmlent {
     int cancelstore();
     int Aged();
     unsigned long ReintTime(unsigned long bw);
-    unsigned long ReintAmount();
+    unsigned long ReintAmount(unsigned long *reint_time);
 
     int Freeze();
     int IsReintegrating();
@@ -381,7 +392,7 @@ class cmlent {
     int DoneSending();
     int GetReintegrationHandle();
     int ValidateReintegrationHandle();
-    int WriteReintegrationHandle();
+    int WriteReintegrationHandle(unsigned long *reint_time);
     int CloseReintegrationHandle(char *, int, ViceVersionVector *);
 
     /* Routines for handling inconsistencies and safeguarding against catastrophe! */
@@ -389,6 +400,8 @@ class cmlent {
     int checkpoint(FILE *);
     void writeops(FILE *);
 
+    void getfids(VenusFid fid[3]);
+
     void AttachFidBindings();
     void DetachFidBindings();
 
@@ -401,18 +414,15 @@ class cmlent {
     void SetTid(int);                                           /*U*/
     int ReintReady();                                           /*U*/
     int ContainLocalFid();                                      /*N*/
-    void TranslateFid(VenusFid *, VenusFid *);                    /*T*/
-    int LocalFakeify();                                         /*U*/
+    void TranslateFid(VenusFid *, VenusFid *);                  /*T*/
     void CheckRepair(char *, int *, int *);                     /*N*/
     int DoRepair(char *, int);                                  /*U*/
     void GetLocalOpMsg(char *);                                 /*N*/
     void SetRepairFlag();                                       /*U*/
     void SetRepairMutationFlag();                               /*U*/
     int IsToBeRepaired() { return flags.to_be_repaired; }       /*N*/
-    int IsRepairMutation() { return flags.repair_mutation; }    /*N*/
-    int InLocalRepairSubtree(VenusFid *);                        /*N*/
-    int InGlobalRepairSubtree(VenusFid *);                       /*N*/
-    void GetVVandFids(ViceVersionVector *[3], VenusFid *[3]);    /*N*/
+    int IsExpanded() { return expansions; }                     /*T*/
+    void GetVVandFids(ViceVersionVector *[3], VenusFid *[3]);   /*N*/
     void GetAllFids(VenusFid *[3]);    				/*N*/
 };
 
@@ -474,7 +484,7 @@ class vdb {
     vdb();
     void ResetTransient();
     ~vdb() { abort(); }
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
     /* Allocation/Deallocation routines. */
     volent *Create(Realm *realm, VolumeInfo *, const char *);
@@ -483,7 +493,6 @@ class vdb {
     void GetDown();
     void FlushCOP2();
     void CheckPoint(unsigned long);
-    void CheckReintegratePending();
     void CheckLocalSubtree();
 
   public:
@@ -495,12 +504,11 @@ class vdb {
 
     void DownEvent(struct in_addr *host);
     void UpEvent(struct in_addr *host);
-    void WeakEvent(struct in_addr *host);
-    void StrongEvent(struct in_addr *host);
 
-    void AttachFidBindings();
-    int WriteDisconnect(unsigned =V_UNSETAGE, unsigned =V_UNSETREINTLIMIT);
-    int WriteReconnect();
+    void AttachFidBindings(void);
+    int WriteDisconnect(unsigned int age=V_UNSETAGE,
+			unsigned int time=V_UNSETREINTLIMIT);
+    int SyncCache(void);
     void GetCmlStats(cmlstats&, cmlstats&);
 
     int CallBackBreak(Volid *);
@@ -516,10 +524,10 @@ class vdb {
 
 
 /* A volume is in exactly one of these states. */
-typedef enum {	Hoarding,
-		Emulating,
-		Logging,
-		Resolving,
+typedef enum {
+    Unreachable = 1,
+    Reachable,
+    Resolving,
 } VolumeStateType;
 
 
@@ -531,21 +539,27 @@ struct VolFlags {
       unsigned replicated : 1;  /* is this a replicated vol or a vol replica */
 /* T*/unsigned transition_pending : 1;
 /* T*/unsigned demotion_pending : 1;
-/*R */unsigned logv : 1;        /* log mutations, allow fetches */
+/*R */unsigned unused1 : 1;
 /*RT*/unsigned allow_asrinvocation : 1; /* asr's allowed in this volume */
 /*RT*/unsigned asr_running : 1; /* only 1 ASR allowed per volume at a time */
 /*R */unsigned has_local_subtree : 1; /* indicating whether this volume contains local subtrees */
-/*RT*/unsigned reintegratepending : 1;	/* are we waiting for tokens? */
+/*RT*/unsigned unused2 : 1;	/* used to be 'reintegration waiting for tokens */
 /*RT*/unsigned reintegrating : 1; /* are we reintegrating now? */
 /*RT*/unsigned repair_mode : 1;	/* 0 --> normal, 1 --> repair */
 /*RT*/unsigned resolve_me: 1;   /* resolve reintegrated objects */
-/*RT*/unsigned weaklyconnected : 1; /* are we weakly connected? */ 
-/*  */unsigned unused1 : 2;
+/*  */unsigned unused3 : 3;
 /*RT*/unsigned sync_reintegrate : 1; /* perform reintegration synchronously*/
-/*  */unsigned unused2 : 2;
-/*V */unsigned readonly : 1;    /* is this a readonly (backup) volume replica */
+/*  */unsigned unused4 : 2;
+/*V */unsigned readonly : 1;   /* is this a readonly (backup) volume replica */
+/*RT*/unsigned enable_asrinvocation : 1; /* asr's enabled in this volume */
 /*VT*/unsigned available : 1;   /* is the server for this volume online? */
-      unsigned reserved : 13;
+      unsigned unused5 : 10;
+      unsigned reint_conflict : 1;  /* set when the head of the CML is marked
+				       as in conflict, should not be used as
+				       authorative information */
+      unsigned unauthenticated : 1; /* set when reintegration fails due to lack
+				       of tokens, should not be used as
+				       authorative information */
 };
 
 
@@ -602,6 +616,8 @@ class volent {
     VolFlags flags;
     /*T*/VolumeStateType state;
 
+    Unique_t FidUnique;
+
     rec_olink handle;			/* link for repvol_hash/volrep_hash */
 
     /* Fso's. */
@@ -612,9 +628,9 @@ class volent {
     /*T*/short observer_count;
     /*T*/short resolver_count;
     /*T*/short shrd_count;		/* for volume pgid locking */
-    /*T*/int lc_asr;            /* last/current ASR run for this volume */
+    /*T*/int pgid;  /* pgid of ASRLauncher and children (0 if none) */
 
-    void operator delete(void *, size_t);
+    void operator delete(void *);
     volent(Realm *r, VolumeId vid, const char *name);
     ~volent();
     void ResetVolTransients();
@@ -649,19 +665,25 @@ class volent {
     int IsBackup() { return (!flags.replicated && flags.readonly); }
     int IsReplicated() { return flags.replicated; }
     int IsReadWriteReplica();
-    int IsHoarding() { return (state == Hoarding); }
-    int IsDisconnected() { return (state == Emulating); }
-    int IsWriteDisconnected() { return (state == Logging); }
-    int IsWeaklyConnected() { return flags.weaklyconnected; }
-    int IsFake() { return (realm == LocalRealm); }
+    int IsUnreachable() { return (state == Unreachable); }
+    int IsReachable() { return (state == Reachable); }
+    int IsResolving() { return (state == Resolving); }
+    int IsLocalRealm() { return (realm == LocalRealm); }
     void GetMountPath(char *, int =1);
     void GetBandwidth(unsigned long *bw);
 
     /* local-repair addition */
+    VenusFid GenerateFakeFid();
     RealmId GetRealmId()   { return realm->Id(); }     /*N*/
     VolumeId GetVolumeId() { return vid; }           /*N*/
     const char *GetName() { return name; }      /*N*/
 
+    fsobj *NewFakeDirObj(const char *comp);
+    fsobj *NewFakeMountLinkObj(VenusFid *fid, const char *comp);
+    int IsRepairVol(void) {
+	return (realm->Id() == LocalRealm->Id() && vid == FakeRepairVolumeId);
+    }
+
     void print() { print(stdout); }
     void print(FILE *fp) { fflush(fp); print(fileno(fp)); }
     void print(int);
@@ -679,9 +701,10 @@ class volrep : public volent {
 
     VolumeId replicated;        /* replicated `parent' volume */
     struct in_addr host;        /* server that has this volume */
-    
-    /* not yet used */
+
 /*T*/srvent *volserver;          /* srvent of the server hosting this volume */
+
+    /* not yet used */
 /*T*/struct dllist_head vollist; /* links volumes to a srvent */
 
     volrep(Realm *r, VolumeId vid, const char *name,
@@ -702,9 +725,7 @@ class volrep : public volent {
     void GetBandwidth(unsigned long *bw);
 
     void DownMember(struct in_addr *host);
-    void UpMember();
-    void WeakMember();
-    void StrongMember();
+    void UpMember(void);
 
     /* Utility routines. */
     void Host(struct in_addr *addr) { *addr = host; }
@@ -740,10 +761,8 @@ class repvol : public volent {
     ClientModifyLog CML;
     struct Lock CML_lock;               /* for synchronization */
 
-    unsigned AgeLimit;			/* min age of log records in SECONDS */
-    unsigned ReintLimit;		/* work limit, in MILLISECONDS */
-    Unique_t FidUnique;
-    RPC2_Unsigned SidUnique;
+    unsigned int AgeLimit;		/* min age of log records in SECONDS */
+    unsigned int ReintLimit;		/* work limit, in MILLISECONDS */
     int reint_id_gen;                   /* reintegration id generator */
     /*T*/int cur_reint_tid;             /* tid of reintegration in progress */
     /*T*/int RecordsCancelled;
@@ -781,9 +800,7 @@ class repvol : public volent {
     void GetBandwidth(unsigned long *bw);
 
     void DownMember(struct in_addr *host);
-    void UpMember();
-    void WeakMember();
-    void StrongMember();
+    void UpMember(void);
 
     int Collate_NonMutating(mgrpent *, int);
     int Collate_COP1(mgrpent *, int, ViceVersionVector *);
@@ -797,73 +814,68 @@ class repvol : public volent {
     void GetHosts(struct in_addr hosts[VSG_MEMBERS]);
     void GetVids(VolumeId out[VSG_MEMBERS]);
     int AVSGsize();
-    int WeakVSGSize();
     int IsHostedBy(const struct in_addr *addr); /* XXX not called? */
     void SetStagingServer(struct in_addr *srvr);
     void Reconfigure(void);
+    int IsSync(void) { return (flags.sync_reintegrate || ReintLimit == 0); }
 
     /* Allocation routines. */
     VenusFid GenerateLocalFid(ViceDataType);
-    VenusFid GenerateFakeFid();
-    ViceStoreId GenerateStoreId();
 
     /* Reintegration routines. */
     void Reintegrate();
     int IncReintegrate(int);
-    int PartialReintegrate(int);
-    void SetReintegratePending();
-    void CheckReintegratePending();
-    int IsReintegratePending() { return flags.reintegratepending; }
-    void ClearReintegratePending();
+    int PartialReintegrate(int, unsigned long *reint_time);
     int IsReintegrating() { return flags.reintegrating; }
     int ReadyToReintegrate();
     int GetReintId();                           /*U*/
     void CheckTransition();                     /*N*/
     void IncAbort(int);                         /*U*/
-    int SyncCache(VenusFid * fid);
+    int SyncCache(VenusFid *fid = NULL);
 
     void RestoreObj(VenusFid *);
     int	CheckPointMLEs(uid_t, char *);
     int LastMLETime(unsigned long *);
     int PurgeMLEs(uid_t);
     void ResetStats() { CML.ResetHighWater(); }
-    int WriteDisconnect(unsigned =V_UNSETAGE, unsigned =V_UNSETREINTLIMIT);
-    int WriteReconnect();
+    int WriteDisconnect(unsigned int age=V_UNSETAGE,
+			unsigned int time=V_UNSETREINTLIMIT);
 
     /* local-repair modifications to the following methods */
     /* Modlog routines. */
-    int LogStore(time_t, uid_t, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
+    int LogStore(time_t, uid_t, VenusFid *, RPC2_Unsigned, int prepend);
     int LogSetAttr(time_t, uid_t, VenusFid *,
-		    RPC2_Unsigned, Date_t, UserId, RPC2_Unsigned, int = UNSET_TID);
-    int LogTruncate(time_t, uid_t, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
-    int LogUtimes(time_t, uid_t, VenusFid *, Date_t, int = UNSET_TID);
-    int LogChown(time_t, uid_t, VenusFid *, UserId, int = UNSET_TID);
-    int LogChmod(time_t, uid_t, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
-    int LogCreate(time_t, uid_t, VenusFid *, char *, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
-    int LogRemove(time_t, uid_t, VenusFid *, char *, const VenusFid *, int, int = UNSET_TID);
-    int LogLink(time_t, uid_t, VenusFid *, char *, VenusFid *, int = UNSET_TID);
+		    RPC2_Unsigned, Date_t, UserId, RPC2_Unsigned, int prepend);
+    int LogTruncate(time_t, uid_t, VenusFid *, RPC2_Unsigned, int prepend);
+    int LogUtimes(time_t, uid_t, VenusFid *, Date_t, int prepend);
+    int LogChown(time_t, uid_t, VenusFid *, UserId, int prepend);
+    int LogChmod(time_t, uid_t, VenusFid *, RPC2_Unsigned, int prepend);
+    int LogCreate(time_t, uid_t, VenusFid *, char *, VenusFid *, RPC2_Unsigned, int prepend);
+    int LogRemove(time_t, uid_t, VenusFid *, char *, const VenusFid *, int, int prepend);
+    int LogLink(time_t, uid_t, VenusFid *, char *, VenusFid *, int prepend);
     int LogRename(time_t, uid_t, VenusFid *, char *,
-		   VenusFid *, char *, VenusFid *, const VenusFid *, int, int = UNSET_TID);
-    int LogMkdir(time_t, uid_t, VenusFid *, char *, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
-    int LogRmdir(time_t, uid_t, VenusFid *, char *, const VenusFid *, int = UNSET_TID);
+		   VenusFid *, char *, VenusFid *, const VenusFid *, int, int prepend);
+    int LogMkdir(time_t, uid_t, VenusFid *, char *, VenusFid *, RPC2_Unsigned, int prepend);
+    int LogRmdir(time_t, uid_t, VenusFid *, char *, const VenusFid *, int prepend);
     int LogSymlink(time_t, uid_t, VenusFid *, char *,
-		    char *, VenusFid *, RPC2_Unsigned, int = UNSET_TID);
+		    char *, VenusFid *, RPC2_Unsigned, int prepend);
     int LogRepair(time_t, uid_t, VenusFid *, RPC2_Unsigned,
-		  Date_t, UserId, RPC2_Unsigned, int = UNSET_TID);
+		  Date_t, UserId, RPC2_Unsigned, int prepend);
     /* local-repair modifications to the above methods */
 
+    /* CML routines */
+    void ListCML(FILE *fp);
+    void PreserveAllLocalMutation(char *msg);
+    void PreserveLocalMutation(char *msg);
+    void DiscardAllLocalMutation(char *msg);
+    void DiscardLocalMutation(char *msg);
 
     /* local-repair */
-    void TranslateCMLFid(VenusFid *, VenusFid *); /*T*/
     void ClearRepairCML();                      /*U*/
     ClientModifyLog *GetCML() { return &CML; }  /*N*/
     int ContainUnrepairedCML();			/*N*/
-    int HasLocalSubtree() { return flags.has_local_subtree; }
-    void CheckLocalSubtree();			/*U*/
 
     /* Repair routines. */
-    int EnableRepair(uid_t, VolumeId *, uid_t *, unsigned long *);
-    int DisableRepair(uid_t);
     int Repair(VenusFid *, char *, uid_t, VolumeId *, int *);
     int ConnectedRepair(VenusFid *, char *, uid_t, VolumeId *, int *);
     int DisconnectedRepair(VenusFid *, char *, uid_t, VolumeId *, int *);
@@ -871,7 +883,7 @@ class repvol : public volent {
 
     /* Resolution routines */
     void Resolve();
-    void ResSubmit(char **, VenusFid *);
+    void ResSubmit(char **, VenusFid *, resent **requeue=NULL);
     int ResAwait(char *);
     int RecResolve(connent *, VenusFid *);
     int ResListCount() { return(res_list->count()); }
@@ -885,6 +897,7 @@ class repvol : public volent {
     cop2ent *FindCOP2(ViceStoreId *);
     void AddCOP2(ViceStoreId *, ViceVersionVector *);
     void ClearCOP2(RPC2_CountedBS *);
+    void ClearCOP2(void);
 
     /* Callback routines */
     int GetVolAttr(uid_t);
@@ -899,19 +912,20 @@ class repvol : public volent {
     int ValidateFSOs();
 
     /* ASR routines */
-    int EnableASR(uid_t);
+    int AllowASR(uid_t);
+    int DisallowASR(uid_t);
+    void EnableASR(uid_t);
     int DisableASR(uid_t);
     int IsASRAllowed() { return flags.allow_asrinvocation; }
+    int IsASREnabled() { return flags.enable_asrinvocation; }
     void lock_asr();
     void unlock_asr();
     int asr_running() { return flags.asr_running; }
-    void asr_id(int);
-    int asr_id() { return lc_asr; }
-
-    /* Repair routines */
-    int IsUnderRepair(uid_t);
+    void asr_pgid(pid_t new_pgid);
+    pid_t asr_pgid() { return pgid; }
 
     void print_repvol(int);
+    void ReportVolState(void);
 };
 
 class volent_iterator : public rec_ohashtab_iterator {
@@ -946,7 +960,7 @@ class cop2ent : public dlink {
     cop2ent(cop2ent&);		/* not supported! */
     int operator=(cop2ent&);	/* not supported! */
     ~cop2ent();
-    void operator delete(void *, size_t);
+    void operator delete(void *);
 
   public:
 #ifdef	VENUSDEBUG
@@ -963,12 +977,13 @@ class cop2ent : public dlink {
 /* Entries representing fids that need to be resolved. */
 class resent : public olink {
   friend void repvol::Resolve();
-  friend void repvol::ResSubmit(char **, VenusFid *);
+  friend void repvol::ResSubmit(char **, VenusFid *, resent **requeue);
   friend int repvol::ResAwait(char *);
 
     VenusFid fid;
     int result;
     int refcnt;
+    int requeues;
 
     resent(VenusFid *);
     resent(resent&);		/* not supported! */
@@ -991,10 +1006,15 @@ class resent : public olink {
 /*  *****  Variables  *****  */
 extern int MLEs;
 extern int LogOpts;
+extern int allow_backfetch;
 extern int vcbbreaks;
 extern char voldaemon_sync;
 extern char VCBEnabled;
 
+/* reintegration parameters, see venusvol.cc */
+extern int default_reintegration_age;
+extern int default_reintegration_time;
+
 /*  *****  Functions/Procedures  *****  */
 
 /* venusvol.c */
@@ -1026,10 +1046,9 @@ extern int PathAltered(VenusFid *, char *, ClientModifyLog *, cmlent *);
     }\
 }
 
-#define	PRINT_VOLSTATE(state)	((state) == Hoarding ? "Hoarding" :\
-				 (state) == Resolving ? "Resolving" :\
-				 (state) == Emulating ? "Emulating" :\
-				 (state) == Logging ? "Logging":\
+#define	PRINT_VOLSTATE(state)	((state) == Resolving ? "Resolving" :\
+				 (state) == Unreachable ? "Unreachable" :\
+				 (state) == Reachable ? "Reachable":\
 				 "???")
 #define	PRINT_VOLMODE(mode)	((mode) & VM_OBSERVING ? "Observing" :\
 				 (mode) & VM_MUTATING ? "Mutating" :\
