@@ -57,7 +57,6 @@ extern "C" {
 #include <sys/types.h>
 #include <stdio.h>
 #include <struct.h>
-#include <netdb.h>
 
 #include <rpc2/errors.h>
 
@@ -71,7 +70,6 @@ extern "C" {
 #include "venus.private.h"
 #include "venusvol.h"
 #include "vproc.h"
-#include "adv_daemon.h"
 
 
 /* must not be called from within a transaction */
@@ -79,6 +77,13 @@ void repvol::Reintegrate()
 {
     LOG(0, ("repvol::Reintegrate\n"));
 
+    if (!ReadyToReintegrate())
+	return;
+
+    /* prevent ASRs from slipping in and adding records we might reintegrate. */
+    if (DisableASR(V_UID))
+	return;
+
     /* 
      * this flag keeps multiple reintegrators from interfering with
      * each other in the same volume.  This is necessary even with 
@@ -86,16 +91,6 @@ void repvol::Reintegrate()
      * iterations of the loop below.  Without the flag, other threads
      * may slip in between commit or abort of a record.
      */
-    if (IsReintegrating())
-	return;
-
-#if 0
-    u = realm->GetUser(CML.owner);
-    CODA_ASSERT(u != NULL);
-    /* if (SkkEnabled) u->NotifyReintegrationActive(name); */
-    PutUser(&u);
-#endif
-
     flags.reintegrating = 1;
 
     /* enter the volume */
@@ -108,8 +103,7 @@ void repvol::Reintegrate()
     v->Begin_VFS(&volid, CODA_REINTEGRATE);
     VOL_ASSERT(this, v->u.u_error == 0);
 
-    /* prevent ASRs from slipping in and leaving records we might reintegrate. */
-    DisableASR(V_UID);
+    ReportVolState();
 
     /* lock the CML to prevent a checkpointer from getting confused. */
     ObtainReadLock(&CML_lock);
@@ -119,6 +113,10 @@ void repvol::Reintegrate()
     CML.CancelStores();
 
     int nrecs, startedrecs, thisTid, code = 0;
+    int stop_loop = 0;
+
+    /* remaining reintegration time (msec) */
+    unsigned long reint_time = ReintLimit;
 
     /* We do the actual reintegration steps in a loop, as we reintegrate in
      * blocks of 100 cmlents. JH */
@@ -127,28 +125,31 @@ void repvol::Reintegrate()
         thisTid = -GetReintId();
         nrecs = 0;
 
-        /*
-         * step 2. Attempt to do partial reintegration for big stores at
-         * the head of the CML.
-         */
-        code = PartialReintegrate(thisTid);
+	/*
+	 * step 2. Attempt to do partial reintegration for big stores at
+	 * the head of the CML.
+	 */
+	code = PartialReintegrate(thisTid, &reint_time);
 
-        /* PartialReintegrate returns ENOENT when there was no CML entry
-         * available for partial reintegration */
-        if (code != ENOENT) {
-            eprint("Reintegrate: %s, partial record, result = %s", 
-                   name, VenusRetStr(code));
-            /* done for now */
-            break;
-        }
-        /* clear the errorcode, ENOENT was not a fatal error. */
-        code = 0;
+	if (code == 0 && IsSync())
+	    continue;
+
+	/* PartialReintegrate returns ENOENT when there was no CML entry
+	 * available for partial reintegration */
+	if (code != ENOENT) {
+	    eprint("Reintegrate: %s, partial record, result = %s",
+		   name, VenusRetStr(code));
+	    /* done for now */
+	    break;
+	}
+	/* clear the errorcode, ENOENT was not a fatal error. */
+	code = 0;
 
         /*
          * step 3.
          * scan the log, gathering records that are ready to to reintegrate.
          */
-        CML.GetReintegrateable(thisTid, &nrecs);
+        stop_loop = CML.GetReintegrateable(thisTid, &reint_time, &nrecs);
 
         /* nothing to reintegrate? jump out of the loop! */
         if (nrecs == 0) break;
@@ -162,7 +163,7 @@ void repvol::Reintegrate()
         startedrecs = CML.count();
         MarinerLog("reintegrate::%s, %d/%d\n", name, nrecs, startedrecs);
 
-        code = IncReintegrate(thisTid);
+	code = IncReintegrate(thisTid);
 
 	/* Log how many entries are left to reintegrate */
 	MarinerLog("reintegrate::%s, 0/%d\n", name, CML.count());
@@ -174,23 +175,21 @@ void repvol::Reintegrate()
      * but we don't want to interfere with trickle reintegration so we test
      * whether a full block has been sent (see also cmlent::GetReintegrateable)
      */
-    } while(code == 0 && ((flags.sync_reintegrate && nrecs) || nrecs == 100));
+    } while (code == 0 && !stop_loop);
 
     flags.reintegrating = 0;
 
-    /* 
+    /* we have to clear sync_reintegrateto avoid recursion when exiting the
+     * volume since that might trigger another volume transition */
+    flags.sync_reintegrate = 0;
+
+    /*
      * clear CML owner if possible.  If there are still mutators in the volume,
      * the owner must remain set (it is overloaded).  In that case the last
      * mutator will clear the owner on volume exit.
      */
-    if (CML.count() == 0  &&  mutator_count == 0)
-        CML.owner = UNSET_UID;
-
-    /* trigger a transition if necessary */
-    if ((CML.count() == 0 || (CML.count() > 0 && !ContainUnrepairedCML()))
-	&& flags.logv == 0 && state == Logging) {
-	flags.transition_pending = 1;
-    }
+    if (CML.count() == 0 && mutator_count == 0)
+	CML.owner = UNSET_UID;
 
     /* if code was non-zero, return EINVAL to End_VFS to force this
        reintegration to inc fail count rather than success count */
@@ -200,14 +199,81 @@ void repvol::Reintegrate()
     EnableASR(V_UID);	
     ReleaseReadLock(&CML_lock);
 
+    ReportVolState();
+
     /* Surrender control of the volume. */
     v->End_VFS();
 
-    /* reset it, 'cause we can't leave errors just laying around */
-    v->u.u_error = 0;
+    /* Check and launch ASRs, if appropriate. */
+    if (code == EINCOMPATIBLE || code == EINCONS)
+    {
+	VenusFid fids[3];
+	int ASRInvokable;
+	fsobj *conflict;
+	repvol *v;
+	struct timeval tv;
+	cml_iterator next(CML, CommitOrder);
+	cmlent *m;
 
-    /* Let user know reintegration has completed */
-    /*   if (SkkEnabled) u->NotifyReintegrationCompleted(name); */
+	/* grab the first cmlent that is flagged as 'to_be_repaired' */
+	while ((m = next()))
+	    if (m->IsToBeRepaired())
+		break;
+
+	if (!m) goto Done;
+
+	m->getfids(fids);
+
+	/* we could also iterate through all returned fids, now we just pick
+	 * the (first) parent directory. */
+	if (FID_EQ(&fids[0], &NullFid))
+	    goto Done;
+
+	conflict = FSDB->Find(&fids[0]);
+	if(!conflict) {
+	    LOG(0, ("repvol::Reintegrate: Couldn't find conflict for ASR!\n"));
+	    goto Done;
+	}
+
+	v = (repvol *)conflict->vol;
+	gettimeofday(&tv, 0);
+
+	/* Check that:
+	 * 1.) An ASRLauncher path was parsed in venus.conf.
+	 * 2.) This thread is a worker.
+	 * 3.) ASR's are allowed to execute within this volume.
+	 * 4.) An ASR is not currently running within this volume.
+	 * 5.) The timeout interval for ASR launching has expired.
+	 */
+	if (!ASRLauncherFile) {
+	    LOG(0, ("ClientModifyLog::HandleFailedMLE: No ASRLauncher "
+		    "specified in venus.conf!\n"));
+	    goto Done;
+	}
+	if (!v->IsASRAllowed()) {
+	    LOG(0, ("ClientModifyLog::HandleFailedMLE: User does not "
+		    "allow ASR execution in this volume.\n"));
+	    goto Done;
+	}
+	if (v->asr_running()) {
+	    LOG(0, ("ClientModifyLog::HandleFailedMLE: ASR already "
+		    "running in this volume.\n"));
+	    goto Done;
+	}
+	if ((tv.tv_sec - conflict->lastresolved) <= ASR_INTERVAL) {
+	    LOG(0, ("ClientModifyLog::HandleFailedMLE: ASR too soon!\n"));
+	    goto Done;
+	}
+
+	ASRInvokable = ASRPolicyFile && v->IsASREnabled();
+	/* Send in any conflict type, it will get changed later. */
+	if (ASRInvokable)  /* Execute ASR. */
+	    conflict->LaunchASR(LOCAL_GLOBAL, DIRECTORY_CONFLICT);
+    }
+
+Done:
+    /* reset any errors, 'cause we can't leave errors just laying around */
+    v->u.u_error = 0;
 }
 
 
@@ -232,8 +298,8 @@ int repvol::IncReintegrate(int tid)
 	return 0;
     }
 
-    /* check if volume state is Logging or not */
-    if (state != Logging) return ETIMEDOUT;	/* it must be Emulating */
+    /* check if volume state is Reachable or not */
+    if (!IsReachable()) return ETIMEDOUT; /* we must be Unreachable or Resolving */
 
     int code = 0;
     int done;
@@ -246,8 +312,7 @@ int repvol::IncReintegrate(int tid)
     long StartBackFetched = BytesBackFetched;
 
     /* Get the NEW CML stats. */
-    cmlstats current;
-    cmlstats cancelled;
+    cmlstats current, cancelled;
     CML.IncGetStats(current, cancelled, tid);	/* get incremental stats */
 
     START_TIMING();
@@ -387,13 +452,8 @@ extern struct timeval *VprocRetryBeta;
 		CML.CheckPoint(0);
 		ObtainReadLock(&CML_lock);
 
-		CML.CancelPending();       
-		/*
-		 * localize or abort the record if this is not a local repair
-		 * session.  Local repair does its own error handling.
-		 */
-		if (tid != LRDB->GetRepairSessionTid())
-		    CML.HandleFailedMLE();
+		CML.CancelPending();
+		CML.HandleFailedMLE();
 
 		break;
 	    }
@@ -435,7 +495,7 @@ extern struct timeval *VprocRetryBeta;
  * Reintegrate some portion of the store record at the head
  * of the log.
  */
-int repvol::PartialReintegrate(int tid)
+int repvol::PartialReintegrate(int tid, unsigned long *reint_time)
 {
     cmlent *m;
     int code = 0;
@@ -474,8 +534,8 @@ int repvol::PartialReintegrate(int tid)
 
     /* send some file data to the server */
     {
-	while (!m->DoneSending() && (code == 0)) 
-	    code = m->WriteReintegrationHandle();
+	while (!m->DoneSending() && (code == 0))
+	    code = m->WriteReintegrationHandle(reint_time);
 
 	if (code != 0) goto CheckResult;
     }
@@ -496,13 +556,6 @@ int repvol::PartialReintegrate(int tid)
     }
 
 CheckResult:
-    /* 
-     * if this record was being partially reintegrated, but was
-     * not sent completely, then the partial results are invalid.
-     */
-    if (m->HaveReintegrationHandle() && !m->DoneSending())
-	m->ClearReintegrationHandle();		/* start over! */
-
     /* allow log optimizations to go through. */
     cur_reint_tid = UNSET_TID;
 
@@ -519,6 +572,7 @@ CheckResult:
 	    LOG(0, ("volent::PartialReintegrate: committed\n"));
 
 	    CML.ClearPending();
+	    code = 0;
 	} else {
 	    /* allow an incompletely sent record to be cancelled. */
 	    CML.CancelPending();
@@ -540,6 +594,7 @@ CheckResult:
     case EINCOMPATIBLE:
     default:
 	/* non-retryable failures -- see IncReintegrate for comments */
+	m->flags.failed = 1;
 
 	LOG(0, ("volent::PartialReintegrate: fail code = %d\n", code));
 	CML.print(logFile);
@@ -567,8 +622,11 @@ CheckResult:
  */
 int repvol::ReadyToReintegrate()
 {
-    int ready = 0;
-    userent *u = realm->GetUser(CML.owner); /* if CML is non-empty, u != 0 */
+    userent *u; 
+    cml_iterator next(CML, CommitOrder);
+    cmlent *m;
+    int rc;
+
 
     /* 
      * we're a bit draconian about ASRs.  We want to avoid reintegrating
@@ -578,64 +636,68 @@ int repvol::ReadyToReintegrate()
      * is Venus-wide, so the check is correct but more conservative than 
      * we would like.
      */
-    if (IsWriteDisconnected() && 
-	(CML.count() > 0) && u->TokensValid() &&
-	!asr_running() && !IsReintegrating()) {
-	    cmlent *m;
-	    cml_iterator next(CML, CommitOrder);
+    if (IsReintegrating() || !IsReachable() || CML.count() == 0)
+	return 0;
 
-	    while ((m = next()) && m->ReintReady()) 
-		ready++;
+    u = realm->GetUser(CML.owner); /* if CML is non-empty, u != 0 */
+    flags.unauthenticated = !u->TokensValid();
+    PutUser(&u);
+
+    m = next();
+    if (!m) {
+	LOG(0, ("repvol::ReadyToReintegrate: failed to find head of the CML\n"));
+	// Should we trigger an assertion? This should never happen because
+	// CML.count is non-zero and we are not yet reintegrating.
+	return 0;
     }
 
-    PutUser(&u);
-    return(ready);
+    rc = m->ReintReady();
+    flags.reint_conflict = (rc == EINCONS);
+
+    if (flags.unauthenticated) {
+	MarinerLog("Reintegrate %s pending tokens for uid = %d\n", name, CML.owner);
+	eprint("Reintegrate %s pending tokens for uid = %d", name, CML.owner);
+    }
+
+    if (flags.transition_pending || asr_running() || flags.unauthenticated || rc) {
+	ReportVolState();
+	return 0;
+    }
+
+    /* volume state will be reported when reintegration actually starts */
+    return 1;
 }
 
 
 /* need not be called from within a transaction */
 int cmlent::ReintReady()
 {
-    /* check volume state */
     repvol *vol = strbase(repvol, log, CML);
-    if (!(vol->IsWriteDisconnected())) {
-	LOG(100, ("cmlent::ReintReady: not write-disconnected\n"));
-	return 0;
-    }
-
-    if (vol->flags.transition_pending) {
-	LOG(100, ("cmlent::ReintReady: transition_pending\n"));
-	return 0;
-    }
 
     /* check if its repair flag is set */
-    if (flags.to_be_repaired || flags.repair_mutation) {
-	LOG(100, ("cmlent::ReintReady: this is a repair related cmlent\n"));
-	return 0;
+    if (!vol->IsSync() && IsToBeRepaired()) {
+	LOG(0, ("cmlent::ReintReady: this is a repair related cmlent\n"));
+	return EINCONS;
     }
 
     if (ContainLocalFid()) {
-	LOG(100, ("cmlent::ReintReady: contains local fid\n"));
+	LOG(0, ("cmlent::ReintReady: contains local fid\n"));
 	/* set its to_be_repaired flag */
 	SetRepairFlag();
-	return 0;
+	return EINCONS;
     }
 
     /* check if this record is part of a transaction (IOT, etc.) */
     if (tid > 0) {
-	LOG(100, ("cmlent::ReintReady: transactional cmlent\n"));
-	return 0; 
+	LOG(0, ("cmlent::ReintReady: transactional cmlent\n"));
+	return EALREADY;
     }
 
-    /* if vol staying write disconnected, check age. does not apply to ASRs */
-    /* nor when returning from writeback */
-    if (!(vol->asr_running()) && vol->flags.logv && !Aged() && 
-	!vol->flags.sync_reintegrate) {
+    if (!Aged()) {
 	LOG(100, ("cmlent::ReintReady: record too young\n"));
-	return 0; 
+	return EAGAIN;
     }
-
-    return 1;
+    return 0;
 }
 
 
@@ -733,10 +795,11 @@ void reintegrator::main(void)
 
     for (;;) {
 	if (idle) CHOKE("reintegrator::main: signalled but not dispatched!");
-	if (!u.u_vol) CHOKE("reintegrator::main: no volume!");
 
 	/* Do the reintegration. */
-	((repvol *)u.u_vol)->Reintegrate();
+	if (u.u_vol && u.u_vol->IsReplicated())
+	    ((repvol *)u.u_vol)->Reintegrate();
+
 	seq++;
 	idle = 1;
 
