@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -39,7 +39,6 @@ extern "C" {
 #include "coda_string.h"
 #include <unistd.h>
 #include <stdlib.h>
-#include <netdb.h>
 #include <fcntl.h>
 
 #include <rpc2/rpc2.h>
@@ -94,57 +93,22 @@ extern "C" {
  *          * rw-replicas
  * */
 
-
-/*
-  BEGIN_HTML
-  <a name="enablerepair"><strong> mark the volume to be in a repair state </strong></a>
-  END_HTML
-*/
-/* AVSG is returned in rwvols. */
-/* LockUids and LockWSs parameters are deprecated! */
-int repvol::EnableRepair(uid_t uid, VolumeId *RWVols,
-                         uid_t *LockUids, unsigned long *LockWSs)
-{
-    LOG(100, ("volent::EnableRepair: vol = %x, uid = %d\n", vid, uid));
-    int code = 0, i;
-
-    /* Place volume in "repair mode." */
-    if (flags.repair_mode != 1)
-	flags.repair_mode = 1;
-
-    /* RWVols, LockUids, and LockWSs are OUT parameters. */
-    memset(LockUids, 0, VSG_MEMBERS * sizeof(uid_t));
-    memset(LockWSs, 0, VSG_MEMBERS * sizeof(unsigned long));
-    memset(RWVols, 0, VSG_MEMBERS * sizeof(VolumeId));
-    for (i = 0; i < VSG_MEMBERS; i++)
-        if (volreps[i]) RWVols[i] = volreps[i]->GetVolumeId();
-
-    return(code);
-}
-
 /* local-repair modification */
 /* Attempt the Repair. */
 int repvol::Repair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 		    VolumeId *RWVols, int *ReturnCodes)
 {
-    LOG(100, ("volent::Repair: fid = %s, file = %s, uid = %d\n",
+    LOG(0, ("volent::Repair: fid = %s, file = %s, uid = %d\n",
 	       FID_(RepairFid), RepairFile, uid));
-    switch (state) {
-    case Hoarding:
-	return ConnectedRepair(RepairFid, RepairFile, uid, RWVols, ReturnCodes);
-    case Logging:
-	if (1 /* to be replaced by a predicate for not being issued by ASR */)
-	    return ConnectedRepair(RepairFid, RepairFile, uid, RWVols, ReturnCodes);
-	else
-	    return DisconnectedRepair(RepairFid, RepairFile, uid, RWVols, ReturnCodes);
-    case Emulating:
+    if (IsUnreachable())
 	return ETIMEDOUT;
-    case Resolving:
-	return ERETRY;
-    default: CHOKE("volent::Repair: bogus volume state %d", state);
-    }
 
-    return -1;
+    if (IsResolving())
+	return ERETRY;
+
+    if (1 /* to be replaced by a predicate for not being issued by ASR */)
+	return ConnectedRepair(RepairFid, RepairFile, uid, RWVols, ReturnCodes);
+    return DisconnectedRepair(RepairFid, RepairFile, uid, RWVols, ReturnCodes);
 }
 
 /* Translate RepairFile to cache entry if "REPAIRFILE_BY_FID." */
@@ -161,7 +125,7 @@ static int GetRepairF(char *RepairFile, uid_t uid, fsobj **RepairF)
 	return 0;
 
     /* strrchr should succeed now because sscanf succeeded. */
-    char *realmname = strrchr(RepairFile, '@')+2;
+    char *realmname = strrchr(RepairFile, '@')+1;
     Realm *realm = REALMDB->GetRealm(realmname);
     RepairFileFid.Realm = realm->Id();
 
@@ -169,10 +133,14 @@ static int GetRepairF(char *RepairFile, uid_t uid, fsobj **RepairF)
 
     realm->PutRef();
 
-    if (code) return code;
+    if (code) {
+      LOG(0, ("GetRepairF: fsdb::Get failed with code: %d\n", code));
+      return code;
+    }
 
     if (!(*RepairF)->IsFile()) {
 	FSDB->Put(RepairF);
+	LOG(0, ("GetRepairF: wasn't a file!\n"));
 	return EINVAL;
     }
     return 0;
@@ -183,12 +151,13 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 {
     int code = 0, i, j, fd = -1, localFake = 0;
     int *LCarr = NULL; /* repLC, mvLC */
-    fsobj *RepairF = NULL, *global = NULL, *local = NULL;
-    VenusFid gfid, lfid, *rFid = NULL;
+    fsobj *RepairF = NULL, *local = NULL;
+    VenusFid *rFid = NULL;
     VenusFid *fidarr = NULL; /* entryFid, mvFid, mvPFid */
     struct listhdr *hlist = NULL, *l = NULL;
     dlist CMLappends;
-    
+    ViceStoreId sid;
+
     memset(ReturnCodes, 0, VSG_MEMBERS * sizeof(int));
     memset(RWVols, 0, VSG_MEMBERS * sizeof(VolumeId));
     for (i = 0; i < VSG_MEMBERS; i++)
@@ -198,60 +167,50 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
     {
 	fsobj *f = NULL;
 
-	code = FSDB->Get(&f, RepairFid, uid, RC_STATUS);
-	if (!(code == 0 && f->IsFakeDir()) && code != EINCONS) {
+	code = FSDB->Get(&f, RepairFid, uid, RC_STATUS, NULL, NULL, NULL, 1);
+
+	CODA_ASSERT(f); //would be ridiculous to fail here
+	if (code || (!f->IsFake() && !f->IsToBeRepaired())) {
 	    if (code == 0) {
-		eprint("Repair: %s (%s) consistent", f->GetComp(), FID_(RepairFid));
+		eprint("Repair: %s (%s) consistent\n", f->GetComp(), FID_(RepairFid));
+		LOG(0,("repvol::Repair: %s (%s) consistent\n", f->GetComp(), FID_(RepairFid)));
 		code = EINVAL;	    /* XXX -JJK */
 	    }
+	    LOG(0, ("repvol::Repair: %s (%s) fsdb::Get failed with code %d\n",
+		   f->GetComp(), FID_(RepairFid), code));
 	    FSDB->Put(&f);
 	    return(code);
 	}
 
-	/* Check for local repair expansion */
-	if ((code == 0) && FAKEROOTFID(*RepairFid)) {
-	    localFake = 1;
-	    code = f->Lookup(&global, NULL, "global", uid,
-			     CLU_CASE_SENSITIVE | CLU_TRAVERSE_MTPT);
-	    if (code != 0) {
-		FSDB->Put(&f);
-		return(code);
-	    }
-	    code = f->Lookup(&local, NULL, "local", uid,
-			     CLU_CASE_SENSITIVE | CLU_TRAVERSE_MTPT);
-	    if (code != 0) {
-		FSDB->Put(&global);
-		FSDB->Put(&f);
-		return(code);
-	    }
-	    LOG(100, ("Local-Repair expansion, got (%s)\n", 
-		      FID_(&global->fid)));
-	    gfid = global->fid;
-	    lfid = local->fid;
-	    rFid = &gfid;
-	}
-	else /* normal repair, proceed as usual */
-	    rFid = RepairFid; 
+	localFake = f->IsToBeRepaired();
 
+	rFid = RepairFid;
 	FSDB->Put(&f);
     }
 
+    LOG(0, ("repvol::Repair: (%s) inconsistent!\n", FID_(RepairFid)));
+
     /* Flush all COP2 entries. */
-    /* This would NOT be necessary if ViceRepair took a "PiggyCOP2" parameter! */
+    /* This would NOT be necessary if ViceRepair took
+     * a "PiggyCOP2" parameter! */
     {
 	code = FlushCOP2();
-	if (code != 0) { FSDB->Put(&local); FSDB->Put(&global); return(code); }
+	if (code != 0) {
+	  LOG(0,("repvol::ConnectedRepair: FlushCOP2 failed with code %d!\n",
+		 code));
+	  return(code);
+	}
     }
 
     code = GetRepairF(RepairFile, uid, &RepairF);
     if (code) {
-	FSDB->Put(&local);
-	FSDB->Put(&global);
-	return code;
+      LOG(0, ("repvol::ConnectedRepair: GetRepairF failed with code %d!\n",
+	      code));
+      return code;
     }
 
     Recov_BeginTrans();
-    ViceStoreId sid = GenerateStoreId();
+    Recov_GenerateStoreId(&sid);
     Recov_EndTrans(MAXFP);
 
     mgrpent *m = 0;
@@ -259,7 +218,13 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 
     /* Acquire an Mgroup. */
     code = GetMgrp(&m, uid);
-    if (code != 0) goto Exit;
+    if (code != 0) {
+      LOG(0, ("repvol::ConnectedRepair: GetMgrp failed with code %d!\n",
+	      code));
+      goto Exit;
+    }
+
+    LOG(0, ("repvol::Repair: (%s) attempting COP1!\n", FID_(RepairFid)));
 
     /* The COP1 call. */
     vv_t UpdateSet;
@@ -298,6 +263,7 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 	    struct stat tstat;
 	    if (::stat(RepairFile, &tstat) < 0) {
 		code = errno;
+		LOG(0, ("repvol::Repair: (%s) Failed stat of RepairFile (%s)!\n", FID_(RepairFid), RepairFile));
 		goto Exit;
 	    }
 
@@ -322,6 +288,8 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 #endif
 		default:
 		    code = EINVAL;
+		    LOG(0, ("repvol::Repair: (%s) invalid Vnode type!\n",
+			    FID_(RepairFid)));
 		    goto Exit;
 	    }
 	}
@@ -357,26 +325,37 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
         if (RepairF) fd = RepairF->data.file->Open(O_RDONLY);
         else         fd = open(RepairFile, O_RDONLY, (int)V_MODE);
 
+	if(fd < 0) {
+	  LOG(0, ("repvol::Repair: (%s) failed opening fixfile (%s)!\n",
+		  FID_(RepairFid), RepairFile));
+	  goto Exit;
+	}
+
         sei->Tag = FILEBYFD;
         sei->FileInfo.ByFD.fd = fd;
 
-	/* For directory conflicts only! (for file conflicts, there is no fixfile)
-	 * If localhost is specified in fixfile, get info for pruning CML entries  
-	 * Must do this here, since later would get errno 157 (Resource temporarily unavailable) */
-	if (ISDIR(*RepairFid)) 
+	/* For directory conflicts only! (for file conflicts, there is no
+	 * fixfile)  If localhost is specified in fixfile, get info for
+	 * pruning CML entries.  Must do this here, since later would get
+	 * errno 157/ERETRY (Resource temporarily unavailable) */
+
+	LOG(0, ("repvol::Repair: (%s) attempting fixfile parse\n",
+		FID_(RepairFid)));
+	if (ISDIR(*RepairFid))
 	{
 	    int hcount;
 	    struct repair *rep_ent;
 
-	    /* parse input file and obtain internal rep  */
+	    /* parse input file and obtain internal rep */
 	    if (repair_getdfile(fd, &hcount, &hlist) < 0) {
 		code = errno; /* XXXX - Could use a more meaningful return code here */
+		LOG(0, ("repvol::Repair: (%s) repair_getdfile failed: %d, fd = %d\n", FID_(RepairFid), code, fd));
 		goto Exit;
 	    }
 
 	    for (i = 0; i < hcount; i++)
-		if (IS_LOCAL_VID(hlist[i].replicaId))
-		    { l = &(hlist[i]); break; }
+	      if ((hlist[i].replicaFid.Volume == RepairFid->Volume))
+		{ l = &(hlist[i]); break; } /* localhost */
 
 	    if (l != NULL) {
 		/* found localhost in fixfile */
@@ -387,6 +366,8 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 		    fidarr = (VenusFid *)calloc((l->repairCount * 3), sizeof(VenusFid));
 		    if ((LCarr == NULL) || (fidarr == NULL)) {
 			code = ENOMEM;
+			LOG(0, ("repvol::Repair: (%s) LCarr calloc failed!\n",
+				FID_(RepairFid)));
 			goto Exit;
 		    }
 		}
@@ -399,11 +380,21 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 		    {
 			fsobj *e = NULL;
 
-			CODA_ASSERT(local != NULL);
+			/* we can get data because (l != NULL) => _localcache exists */
+			code = FSDB->Get(&local, RepairFid, uid, RC_DATA,
+					 NULL, NULL, NULL, 1);
+			if(code || !local) {
+			  LOG(0, ("Repair: FSDB->Get(%s) error: %d", FID_(RepairFid),
+				  rep_ent[i].name));
+			  if(local)
+			    FSDB->Put(&local);
+			  goto Exit;
+			}
+
 			code = local->Lookup(&e, NULL, rep_ent[i].name, uid,
 					     CLU_CASE_SENSITIVE);
-			if (code != 0) {
-			    LOG(15, ("Repair: local(%s)->Lookup(%s) error", 
+			if (code) {
+			    LOG(0, ("Repair: local(%s)->Lookup(%s) error",
 				     FID_(&local->fid), rep_ent[i].name));
 			    goto Exit;
 			}
@@ -415,7 +406,7 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 			    code = local->Lookup(&e, NULL, rep_ent[i].newname,
 						 uid, CLU_CASE_SENSITIVE);
 			    if (code != 0) {
-				LOG(15, ("Repair: (%s)->Lookup(%s) error", 
+				LOG(0, ("Repair: (%s)->Lookup(%s) error",
 					 FID_(&local->fid), rep_ent[i].newname));
 				goto Exit;
 			    }
@@ -424,15 +415,14 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 			    LCarr[(l->repairCount + i)] = e->stat.LinkCount;
 			    FSDB->Put(&e);
 			}
+			FSDB->Put(&local);
 		    }
-
 		}
-
 	    }
 	}
 
 	if (::lseek(fd, 0, SEEK_SET) != 0)
-	    { code = errno; goto Exit; }
+	  { code = errno; goto Exit; }
 
 	/* Make multiple copies of the IN/OUT and OUT parameters. */
 	ARG_MARSHALL(IN_OUT_MODE, ViceStatus, statusvar, status, VSG_MEMBERS);
@@ -444,6 +434,8 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
   repair actions </strong> </a>
   END_HTML
 */
+	LOG(0, ("repvol::Repair: (%s) Attempting RPC call\n",
+		FID_(RepairFid)));
 
 	/* Make the RPC call. */
 	MarinerLog("store::Repair (%s)\n", FID_(rFid));
@@ -460,7 +452,11 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 	code = Collate_COP1(m, code, &UpdateSet);
 	if (code == EASYRESOLVE) { asy_resolve = 1; code = 0; }
 	MULTI_RECORD_STATS(ViceRepair_OP);
-	if (code != 0 && code != ESYNRESOLVE) goto Exit;
+	if (code != 0 && code != ESYNRESOLVE) {
+	  LOG(0, ("repvol::Repair: (%s) Collate_COP1 failed: %d!\n",
+		  FID_(RepairFid), code));
+	  goto Exit;
+	}
 
 	/* Collate ReturnCodes. */
 	struct in_addr VSGHosts[VSG_MEMBERS];
@@ -481,111 +477,128 @@ int repvol::ConnectedRepair(VenusFid *RepairFid, char *RepairFile, uid_t uid,
 	    }
 	if (HostCount != m->rocc.HowMany)
 	    CHOKE("volent::Repair: collate failed");
-	if (code != 0) goto Exit;
+	if (code != 0) {
+	  LOG(0, ("repvol::Repair: (%s) can you even get here? code:%d!\n",
+		  FID_(RepairFid), code));
+	  goto Exit;
+	}
     }
 
     /* For directory conflicts only! (for file conflicts, there is no fixfile)
      * Prune CML entries if localhost is specified in fixfile */
-    if (ISDIR(*RepairFid)) 
-    {
+    if (ISDIR(*RepairFid) && (l != NULL)) {
 	time_t modtime;
 	int rc;
 	struct repair *rep_ent;
 
-	if (l != NULL) {
-	    /* found localhost in fixfile */
-	    rep_ent = l->repairList;
+	LOG(0, ("repvol::Repair: (%s) pruning CML from fixfile!\n",
+		FID_(RepairFid)));
 
-	    rc = time(&modtime);
-	    if (rc == (time_t)-1) {
-	      code = errno;
-	      goto Exit;
-	    }
+	/* found localhost in fixfile */
+	rep_ent = l->repairList;
 
-	    for (unsigned int i = 0; i < l->repairCount; i++) {
+	rc = time(&modtime);
+	if (rc == (time_t)-1) {
+	  LOG(0, ("repvol::Repair: (%s) time() failed!\n",
+		  FID_(RepairFid)));
+	  code = errno;
+	  goto Exit;
+	}
 
-		switch (rep_ent[i].opcode) {
+	for (unsigned int i = 0; i < l->repairCount; i++) {
 
-		case REPAIR_REMOVEFSL: /* Remove file or (hard) link */
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogRemove(modtime, uid, rFid, rep_ent[i].name,
-				     &(fidarr[i]), LCarr[i], UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
-		case REPAIR_REMOVED:   /* Remove dir */
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogRmdir(modtime, uid, rFid, rep_ent[i].name,
-				    &(fidarr[i]), UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
-		case REPAIR_SETMODE:
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogChmod(modtime, uid, rFid, 
-				    (RPC2_Unsigned)rep_ent[i].parms[0], UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
-		case REPAIR_SETOWNER: /* Have to be a sys administrator for this */
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogChown(modtime, uid, rFid, 
-				    (UserId)rep_ent[i].parms[0], UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
-		case REPAIR_SETMTIME:
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogUtimes(modtime, uid, rFid, 
-				     (Date_t)rep_ent[i].parms[0], UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
-		case REPAIR_RENAME:
-		    Recov_BeginTrans();
-		    CML.cancelFreezes(1);
-		    code = LogRename(modtime, uid, rFid, rep_ent[i].name, 
-				     &(fidarr[((2 * l->repairCount) + i)]), 
-				     rep_ent[i].name, &(fidarr[i]), 
-				     &(fidarr[(l->repairCount + i)]), 
-				     LCarr[(l->repairCount + i)], UNSET_TID);
-		    CML.cancelFreezes(0);
-		    Recov_EndTrans(CMFP);
-		    break;
+	  switch (rep_ent[i].opcode) {
 
-		/* These should never occur in the fixfile for the local replica */
-		case REPAIR_CREATEF: 	 /* Create file */
-		case REPAIR_CREATED:	 /* Create directory */
-		case REPAIR_CREATES:	 /* Create sym link */
-		case REPAIR_CREATEL:	 /* Create (hard) link */
-		case REPAIR_SETACL:	 /* Set rights */
-		case REPAIR_SETNACL:	 /* Set negative rights */
-		    LOG(1, ("Unexpected local repair command code (%d)", rep_ent[i].opcode));
-		    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
-		    break;
+	  case REPAIR_REMOVEFSL: /* Remove file or (hard) link */
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogRemove(modtime, uid, rFid, rep_ent[i].name,
+			     &(fidarr[i]), LCarr[i], UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
+	  case REPAIR_REMOVED:   /* Remove dir */
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogRmdir(modtime, uid, rFid, rep_ent[i].name,
+			    &(fidarr[i]), UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
+	  case REPAIR_SETMODE:
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogChmod(modtime, uid, rFid,
+			    (RPC2_Unsigned)rep_ent[i].parms[0], UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
+	  case REPAIR_SETOWNER: /* Have to be a sys administrator for this */
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogChown(modtime, uid, rFid,
+			    (UserId)rep_ent[i].parms[0], UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
+	  case REPAIR_SETMTIME:
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogUtimes(modtime, uid, rFid,
+			     (Date_t)rep_ent[i].parms[0], UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
+	  case REPAIR_RENAME:
+	    Recov_BeginTrans();
+	    CML.cancelFreezes(1);
+	    code = LogRename(modtime, uid, rFid, rep_ent[i].name,
+			     &(fidarr[((2 * l->repairCount) + i)]),
+			     rep_ent[i].name, &(fidarr[i]),
+			     &(fidarr[(l->repairCount + i)]),
+			     LCarr[(l->repairCount + i)], UNSET_TID);
+	    CML.cancelFreezes(0);
+	    Recov_EndTrans(CMFP);
+	    break;
 
-		case REPAIR_REPLICA: 
-		    LOG(1, ("Unexpected REPAIR_REPLICA -- truncated fixfile?"));
-		    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
-		    break;
+	    /* These should never occur in the fixfile for the local replica */
+	  case REPAIR_CREATEF: 	 /* Create file */
+	  case REPAIR_CREATED:	 /* Create directory */
+	  case REPAIR_CREATES:	 /* Create sym link */
+	  case REPAIR_CREATEL:	 /* Create (hard) link */
+	  case REPAIR_SETACL:	 /* Set rights */
+	  case REPAIR_SETNACL:	 /* Set negative rights */
+	    LOG(0, ("Unexpected local repair command code (%d)", rep_ent[i].opcode));
+	    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
+	    break;
 
-		default:
-		    LOG(1, ("Unknown local repair command code (%d)", rep_ent[i].opcode));
-		    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
-		    break;
-		}
-		
-		/* if (code != 0) goto Exit; */
-		CODA_ASSERT(code == 0);
-	    }
+	  case REPAIR_REPLICA:
+	    LOG(0, ("Unexpected REPAIR_REPLICA -- truncated fixfile?"));
+	    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
+	    break;
+
+	  default:
+	    LOG(0, ("Unknown local repair command code (%d)", rep_ent[i].opcode));
+	    code = EINVAL; /* XXXX - Could use a more meaningful return code here */
+	    break;
+	  }
+
+	  /* we still want to send a COP2 to finalize the repair of the
+	   * server-server conflict. */
+	  // if (code != 0) goto Exit;
 	}
     }
+    else if (!ISDIR(*RepairFid) && localFake) { /* local/global file rep */
+      char msg[2048];
+      /* Walk the cml here, unsetting to_be_repaired on everything.
+       * A 'cfs forcereintegrate' should then succeed/clear the conflict,
+       * if it doesn't happen on its own. */
+      CODA_ASSERT(!CML.DiscardLocalMutation(msg));
+      CML.ClearToBeRepaired();
+    }
 
+
+    LOG(0, ("repvol::Repair: (%s) sending COP2!\n", FID_(RepairFid)));
     /* Send the COP2 message.  Don't Piggy!  */
     (void)COP2(m, &sid, &UpdateSet, 1);
 
@@ -596,32 +609,29 @@ Exit:
     }
 
     if (m) m->Put();
-    FSDB->Put(&local);
-    FSDB->Put(&global);
     FSDB->Put(&RepairF);
 
     if (LCarr != NULL) free(LCarr);
     if (fidarr != NULL) free(fidarr);
     if (hlist != NULL) free(hlist);
 
-    if ((code == 0) && !localFake) {
+    if ((code == 0) && !localFake) { /* successful server/server */
 	/* Purge the fake object. */
 	fsobj *f = FSDB->Find(RepairFid);
 	if (f != 0) {
 	    f->Lock(WR);
 	    Recov_BeginTrans();
-	        f->Kill();
+	    f->flags.fake = 0; /* so we can refetch status before death?? */
+	    f->Kill();
 	    Recov_EndTrans(MAXFP);
 	    FSDB->Put(&f);
-
-	    /* Ought to flush its descendents too! */
 	}
-
 	/* Invoke an asynchronous resolve for directories. */
 	if (ISDIR(*RepairFid)) {
 	    ResSubmit(0, RepairFid);
 	}
     }
+
     if (code ==	ESYNRESOLVE) code = EMULTRSLTS;	/* "multiple results" */
 
     return(code);
@@ -636,8 +646,8 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
     ViceStatus status;
     vproc *vp = VprocSelf();
     CODA_ASSERT(vp);
-    
-    LOG(10, ("volent::DisConnectedRepair: fid = (%s), file = %s, uid = %d\n",
+
+    LOG(0, ("volent::DisconnectedRepair: fid = (%s), file = %s, uid = %d\n",
 	     FID_(RepairFid), RepairFile, uid));
 
     VenusFid tpfid;
@@ -646,7 +656,7 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
     memset(ReturnCodes, 0, VSG_MEMBERS * sizeof(int));
     memset(RWVols, 0, VSG_MEMBERS * sizeof(VolumeId));
     for (i = 0; i < VSG_MEMBERS; i++)
-        if (volreps[i]) RWVols[i] = volreps[i]->GetVolumeId();
+	if (volreps[i]) RWVols[i] = volreps[i]->GetVolumeId();
 
     /* Verify that RepairFid is a file fid */
     /* can't repair directories while disconnected */
@@ -658,28 +668,39 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
 
     /* Verify that RepairFid is inconsistent. */
     {
-	fsobj *f = 0;
-	code = FSDB->Get(&f, RepairFid, uid, RC_STATUS);
-	if (!(code == 0 && f->IsFakeDir()) && code != EINCONS) {
+	fsobj *f = NULL;
+
+	code = FSDB->Get(&f, RepairFid, uid, RC_STATUS, NULL, NULL, NULL, 1);
+
+	CODA_ASSERT(f);
+	if (code || (!f->IsFake() && !f->IsToBeRepaired())) {
 	    if (code == 0) {
-		eprint("DisconnectedRepair: %s (%s) consistent",
+		eprint("DisconnectedRepair: %s (%s) consistent\n",
 		       f->GetComp(), FID_(RepairFid));
-		code = EINVAL;	    /* XXX */
+		LOG(0, ("DisconnectedRepair: %s (%s) consistent\n",
+			f->GetComp(), FID_(RepairFid)));
+		code = EINVAL;	    /* XXX -JJK */
 	    }
+	    else
+	      LOG(0, ("DisconnectedRepair: %s (%s) Get failed with code %d\n",
+		      f->GetComp(), FID_(RepairFid), code));
 	    FSDB->Put(&f);
 	    return(code);
 	}
+
 	/* save the fid of the parent of the inconsistent object */
 	tpfid.Vnode = f->pfid.Vnode;
 	tpfid.Unique = f->pfid.Unique;
 	FSDB->Put(&f);
     }
+
+    LOG(0, ("DisconnectedRepair: (%s) inconsistent!\n", FID_(RepairFid)));
     /* check rights - can user write the file to be repaired */
     {
-	LOG(100, ("DisconnectedRepair: Going to check access control (%s)\n",
+	LOG(0, ("DisconnectedRepair: Going to check access control (%s)\n",
 		  FID_(&tpfid)));
 	if (!tpfid.Vnode) {
-	    LOG(10, ("DisconnectedRepair: Parent fid is NULL - cannot check access control\n"));
+	    LOG(0, ("DisconnectedRepair: Parent fid is NULL - cannot check access control\n"));
 	    return(EACCES);
 	}
 	/* check the parent's rights for write permission*/
@@ -688,21 +709,21 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
 	if (code == 0) {
 	    code = parentf->Access(PRSFS_WRITE, C_A_W_OK, vp->u.u_uid);
 	    if (code) {
-		LOG(10, ("DisconnectedRepair: Access disallowed (%s)\n",
+		LOG(0, ("DisconnectedRepair: Access disallowed (%s)\n",
 			 FID_(&tpfid)));
 		FSDB->Put(&parentf);
 		return(code);
 	    }
 	}
 	else {
-	    LOG(100, ("DisconnectedRepair: Couldn't get parent (%s)\n",
+	    LOG(0, ("DisconnectedRepair: Couldn't get parent (%s)\n",
 		      FID_(&tpfid)));
 	    if (parentf) FSDB->Put(&parentf);
 	    return(code);
 	}
 	FSDB->Put(&parentf);
     }
-    LOG(100, ("DisconnectedRepair: going to check repair file %s\n", RepairFile));
+    LOG(0, ("DisconnectedRepair: checking repair file %s\n", RepairFile));
     code = GetRepairF(RepairFile, uid, &RepairF);
     if (code) return code;
 
@@ -736,6 +757,8 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
 	/* set up status block */
 	memset((void *)&status, 0, (int)sizeof(ViceStatus));
 	if (RepairF != 0) {
+	  LOG(0, ("DisconnectedRepair: RepairF found! (%s)\n",
+		  FID_(&RepairF->fid)));
 	    status.Length = RepairF->stat.Length;
 	    status.Date = RepairF->stat.Date;
 	    status.Owner = RepairF->stat.Owner;
@@ -744,26 +767,30 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
 	    status.VnodeType = RepairF->stat.VnodeType;
 	}
 	else {
+	  LOG(0, ("DisconnectedRepair: RepairF not found!\n"));
 	    struct stat tstat;
 	    if (::stat(RepairFile, &tstat) < 0) {
 		code = errno;
+		LOG(0, ("DisconnectedRepair: (%s) stat failed\n",
+			FID_(RepairFid)));
 		goto Exit;
 	    }
 
 	    status.Length = (RPC2_Unsigned)tstat.st_size;
 	    status.Date = (Date_t)tstat.st_mtime;
-	    RPC2_Integer se_uid = (short)tstat.st_uid;		/* sign-extend uid! */
+	    RPC2_Integer se_uid = (short)tstat.st_uid;	/* sign-extend uid! */
 	    status.Owner = (UserId)se_uid;
 	    status.Mode = (RPC2_Unsigned)tstat.st_mode & 0777;
 	    status.LinkCount = (RPC2_Integer)tstat.st_nlink;
-	    if (tstat.st_mode & S_IFMT != S_IFREG) {
+	    if ((tstat.st_mode & S_IFMT) != S_IFREG) {
 		code = EINVAL;
+		LOG(0, ("DisconnectedRepair: (%s) not a regular file\n",
+			FID_(RepairFid)));
 		goto Exit;
 	    }
 	}
 	status.DataVersion = (FileVersion)1;	  /* Anything but -1? -JJK */
 	status.VV = tvv;
-
     }
 
     /* fake the call */
@@ -771,44 +798,50 @@ int repvol::DisconnectedRepair(VenusFid *RepairFid, char *RepairFile,
 	/* first kill the fake directory if it exists */
 	fsobj *f = FSDB->Find(RepairFid);
 	if (f != 0) {
-	    LOG(0, ("DisconnectedRepair: Going to kill %s\n", FID_(RepairFid)));
+	    LOG(0, ("DisconnectedRepair: Going to kill %s, refcnt: %d\n",
+		    FID_(&f->fid), f->refcnt));
+
+	    if(f->IsExpandedObj())
+	      f->CollapseObject(); /* drop FSO_HOLD ref */
+
 	    f->Lock(WR);
 	    Recov_BeginTrans();
-	       f->Kill();
-	     Recov_EndTrans(MAXFP);
+	    f->Kill();
+	    Recov_EndTrans(MAXFP);
 
 	    if (f->refcnt > 1) {
-		/* Put isn't going to release the object so can't call create
-		 * Instead of failing, put an informative message on console
-		 * and ask user to retry */
-		f->ClearRcRights();
-		FSDB->Put(&f);
-		LOG(0, ("DisconnectedRepair: (%s) has active references - cannot repair\n",
-			FID_(RepairFid)));
-		code = ERETRY;
-		goto Exit;
+	      LOG(0, ("DisconnectedRepair: (%s) has %d active references - cannot repair\n", FID_(RepairFid), f->refcnt));
+	      /* Put isn't going to release the object so can't call create
+	       * Instead of failing, put an informative message on console
+	       * and ask user to retry */
+	      f->ClearRcRights();
+	      FSDB->Put(&f);
+	      code = ERETRY;
+	      goto Exit;
 	    }
 	    FSDB->Put(&f);
 	    /* Ought to flush its descendents too? XXX -PK */
 	}
 	/* attempt the create now */
-	LOG(100, ("DisconnectedRepair: Going to create %s\n", FID_(RepairFid)));
+	LOG(0, ("DisconnectedRepair: Going to create %s\n", FID_(RepairFid)));
 	/* need to get the priority from the vproc pointer */
-	f = FSDB->Create(RepairFid, vp->u.u_priority, (char *)NULL);
+	f = FSDB->Create(RepairFid, vp->u.u_priority, NULL, NULL);
 			/* don't know the component name */
 	if (f == 0) {
 	    UpdateCacheStats(&FSDB->FileAttrStats, NOSPACE, NBLOCKS(sizeof(fsobj)));
+	    LOG(0, ("DisconnectedRepair: Create failed (%s)\n",
+		    FID_(RepairFid)));
 	    code = ENOSPC;
-	    goto Exit; 
+	    goto Exit;
 	}
 
 	Date_t Mtime = Vtime();
 
-	LOG(100, ("DisconnectedRepair: Going to call LocalRepair(%s)\n",
+	LOG(0, ("DisconnectedRepair: Going to call LocalRepair(%s)\n",
 		  FID_(RepairFid)));
 	Recov_BeginTrans();
 	   code = LogRepair(Mtime, uid, RepairFid, status.Length, status.Date,
-			    status.Owner, status.Mode);
+			    status.Owner, status.Mode, 1);
 	   /*
 	    * LogRepair puts a ViceRepair_OP record into the CML and it
 	    * will be reintegrated to the servers at the end of the ASR
@@ -873,7 +906,7 @@ int repvol::LocalRepair(fsobj *f, ViceStatus *status, char *fname, VenusFid *pfi
 
 	int srcfd = open(fname, O_RDONLY | O_BINARY, V_MODE);
 	CODA_ASSERT(srcfd);
-	LOG(100, ("LocalRepair: Going to open %s\n", f->data.file->Name()));
+	LOG(0, ("LocalRepair: Going to open %s\n", f->data.file->Name()));
         int tgtfd = open(f->data.file->Name(), O_WRONLY|O_TRUNC|O_BINARY,
                          V_MODE);
 	CODA_ASSERT(tgtfd>0);
@@ -893,7 +926,7 @@ int repvol::LocalRepair(fsobj *f, ViceStatus *status, char *fname, VenusFid *pfi
 	CODA_ASSERT(rc == 0);
 
 	if ((unsigned long)stbuf.st_size != status->Length) {
-	    LOG(10, ("LocalRepair: Length mismatch - actual stored %u bytes, expected %u bytes\n",
+	    LOG(0, ("LocalRepair: Length mismatch - actual stored %u bytes, expected %u bytes\n",
 		     stbuf.st_size, status->Length));
 	    return(EIO); 	/* XXX - what else can we return? */
 	}
@@ -903,42 +936,19 @@ int repvol::LocalRepair(fsobj *f, ViceStatus *status, char *fname, VenusFid *pfi
     /* set the flags of the object before returning */
     f->flags.dirty = 1;
 
-    return(0);
-}    
-
-int repvol::DisableRepair(uid_t uid)
-{
-    LOG(100, ("volent::DisableRepair: vol = %x, uid = %d\n", vid, uid));
-
-    if (IsUnderRepair(uid))
-	flags.repair_mode = 0;
-    else
-	LOG(0, ("volent::DisableRepair: %x not under repair\n", vid));
-
     return(0);
 }
 
-
-/* If (uid == ANYUSER_UID) the enquiry is taken to be "does anyone on the WS have the volume under repair". */
-int repvol::IsUnderRepair(uid_t uid)
-{
-    LOG(100, ("repvol::IsUnderRepair: vol = %x, uid = %d\n", vid, uid));
-
-    return (flags.repair_mode);
-}
-
-/* Enable ASR invocation for this volume */
-int repvol::EnableASR(uid_t uid)
+/* Enable ASR invocation for this volume (as a volume service)  */
+void repvol::EnableASR(uid_t uid)
 {
     LOG(100, ("repvol::EnableASR: vol = %x, uid = %d\n", vid, uid));
 
     /* Place volume in "repair mode." */
-    if (flags.allow_asrinvocation != 1) {
-	flags.allow_asrinvocation = 1;
-	(void)k_Purge();  /* we should be able to do this on a volume/user basis! */
-    }
-    
-    return(0);
+    if (IsASREnabled())
+	LOG(0, ("volent::EnableASR: ASR for %x already enabled", vid));
+
+    flags.enable_asrinvocation = 1;
 }
 
 int repvol::DisableASR(uid_t uid)
@@ -948,14 +958,39 @@ int repvol::DisableASR(uid_t uid)
     if (asr_running())
 	return EBUSY;
 
-    if (!IsASRAllowed()) {
-	LOG(100, ("volent::DisableASr: ASR for %x already disabled", vid));
-    }
-    else {
-	LOG(0, ("volent::DisableASR: disabling asr for %x\n", vid));
-	flags.allow_asrinvocation = 0; 
-	(void)k_Purge();     /* we should be able to do this on a volume/user basis! */
-    }
+    if (!IsASREnabled())
+	LOG(0, ("volent::DisableASR: ASR for %x already disabled", vid));
+
+    flags.enable_asrinvocation = 0;
+
+    return 0;
+}
+
+/* Allow ASR invocation for this volume (this is user's permission). */
+int repvol::AllowASR(uid_t uid)
+{
+    LOG(100, ("repvol::AllowASR: vol = %x, uid = %d\n", vid, uid));
+
+    /* Place volume in "repair mode." */
+    if (IsASRAllowed())
+	LOG(0, ("volent::AllowASR: ASR for %x already allowed", vid));
+
+    flags.allow_asrinvocation = 1;
+
+    return 0;
+}
+
+int repvol::DisallowASR(uid_t uid)
+{
+    LOG(100, ("repvol::DisallowASR: vol = %x, uid = %d\n", vid, uid));
+
+    if (asr_running())
+	return EBUSY;
+
+    if (!IsASRAllowed())
+	LOG(0, ("volent::DisallowASR: ASR for %x already disallowed", vid));
+
+    flags.allow_asrinvocation = 0;
 
     return 0;
 }
@@ -971,8 +1006,7 @@ void repvol::unlock_asr() {
     flags.asr_running = 0;
 }
 
-void repvol::asr_id(int id) {
+void repvol::asr_pgid(pid_t new_pgid) {
     CODA_ASSERT(flags.asr_running == 1);
-    lc_asr = id;
+    pgid = new_pgid;
 }
-
