@@ -54,6 +54,7 @@ extern "C" {
 #include "venusvol.h"
 #include "vproc.h"
 
+#define MAX_REQUEUE 3
 
 #ifdef VENUSDEBUG
 int resent::allocs = 0;
@@ -95,6 +96,8 @@ void repvol::Resolve()
 	connent *c = 0;
 
 	{
+	    LOG(0, ("repvol::Resolve: Resolving (%s)\n", FID_(&r->fid)));
+
 	    /* Get an Mgroup. */
 	    code = GetMgrp(&m, V_UID);
 	    if (code != 0) goto HandleResult;
@@ -102,9 +105,9 @@ void repvol::Resolve()
 	    /* Pick a coordinator and get a connection to it. */
 	    struct in_addr *phost = m->GetPrimaryHost();
 	    CODA_ASSERT(phost->s_addr != 0);
-            srvent *s = GetServer(phost, GetRealmId());
+	    srvent *s = GetServer(phost, GetRealmId());
 	    code = s->GetConn(&c, ANYUSER_UID);
-            PutServer(&s);
+	    PutServer(&s);
 	    if (code != 0) goto HandleResult;
 
 	    /* Make the RPC call. */
@@ -117,35 +120,35 @@ void repvol::Resolve()
 	    /* Examine the return code to decide what to do next. */
 	    code = Collate(c, code);
 	    UNI_RECORD_STATS(ViceResolve_OP);
+
+	    LOG(0, ("repvol::Resolve: Resolving (non-hinted) (%s) returned "
+		    "code %d\n", FID_(&r->fid), code));
 	}
 
 	/* Demote the object (if cached) */
 	f = FSDB->Find(&r->fid);
 	if (f) f->Demote();
 
-	if (code == VNOVNODE) {
-	    VenusFid *pfid;
-	    if (f) {
-		pfid = &(f->pfid);
-		if (!FID_EQ(pfid, &NullFid) && FID_VolEQ(pfid, &r->fid) &&
-		    (pfid->Vnode != r->fid.Vnode) ) {
-
-		    LOG(10,("Resolve: Submitting parent for resolution\n"));
-		    ResSubmit(NULL, pfid);
-
-		    /* We shouldn't resubmit ourselves as this might lead to an
-		     * endless loop. Hopefully the hoard/getattr that triggered
-		     * the resolution will loop around and retry. --JH */
-		    // ResSubmit(NULL, r->fid);
-		}
-	    } else
-		LOG(10,("Resolve: Couldn't find current object\n"));
+	if (code) {
+	    if (code == VNOVNODE && f)
+	    {
+		/* Retrying resolve on parent is also a "hint" of sorts. */
+		LOG(0, ("Resolve: Submitting parent (%s) for resolution, "
+			"failed on its child (%s)\n",
+			FID_(&f->pfid), FID_(&r->fid)));
+		ResSubmit(NULL, &f->pfid, &r);
+	    }
+	    else
+	    {
+		/* General failure case (can't win them all). */
+		LOG(0,("Resolve: Failed on (%s)\n", FID_(&r->fid)));
+	    }
 	}
 
 HandleResult:
 	PutConn(&c);
-        if (m) m->Put();
-	r->HandleResult(code);
+	if (m) m->Put();
+	if (r) r->HandleResult(code);
     }
 
 Exit:
@@ -169,37 +172,55 @@ Exit:
 
 
 /* Asynchronous resolve is indicated by NULL waitblk. */
-void repvol::ResSubmit(char **waitblkp, VenusFid *fid)
+void repvol::ResSubmit(char **waitblkp, VenusFid *fid, resent **requeue)
 {
-    VOL_ASSERT(this, fid->Realm == realm->Id() && fid->Volume == vid);
+    if (requeue && !(*requeue)->requeues)
+	return;
+
+    if (fid->Realm != realm->Id() || fid->Volume != vid ||
+	(requeue && FID_EQ(fid, &(*requeue)->fid)))
+	*fid = NullFid;
 
     /* Create a new resolver entry for the fid, unless one already exists. */
+    if (!FID_EQ(fid, &NullFid))
     {
 	olist_iterator next(*res_list);
-	resent *r = 0;
+	resent *r = NULL;
 	while ((r = (resent *)next()))
-	    if (FID_EQ(&r->fid, fid)) break;
-	if (r == 0) {
+	    if (FID_EQ(&r->fid, fid))
+		break;
+
+	if (r == NULL) {
 	    r = new resent(fid);
 	    res_list->append(r);
 	}
 
 	/* Bump refcnt and set up pointer to resent in case of SYN resolve. */
-	if (waitblkp != 0) {
+	if (waitblkp) {
 	    r->refcnt++;
 	    *(resent **)waitblkp = r;
 	}
     }
 
-    /* Force volume state transition at next convenient point. */
-    flags.transition_pending = 1;
+    /* Append requeue'd entries at the end of the list (this way they are
+     * guaranteed to be behind whatever entry we just found or created) */
+    if (requeue) {
+	if (FID_EQ(fid, &NullFid))
+	     (*requeue)->requeues = 0;
+	else (*requeue)->requeues--;
+
+	res_list->append(*requeue);
+	*requeue = NULL;
+    } else {
+	/* Force volume state transition at next convenient point. */
+	flags.transition_pending = 1;
+    }
 
     /* Demote the object (if cached). */
     fsobj *f = FSDB->Find(fid);
     if (f) f->Demote();
 }
 
-
 /* Wait for resolve to complete. */
 int repvol::ResAwait(char *waitblk)
 {
@@ -225,6 +246,7 @@ resent::resent(VenusFid *Fid) {
     fid = *Fid;
     result = -1;
     refcnt = 0;
+    requeues = MAX_REQUEUE;
 
 #ifdef VENUSDEBUG
     allocs++;
