@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -37,7 +37,6 @@ extern "C" {
 
 #include <unistd.h>
 #include <stdlib.h>
-#include <netdb.h>
 
 #include <math.h>
 #include <lwp/lwp.h>
@@ -59,6 +58,10 @@ extern "C" {
 #include "vproc.h"
 #include "worker.h"
 
+/* Compatibility with rpc2 < 2.6 */
+#ifndef EVOLUME
+#define EVOLUME CEVOLUME
+#endif
 
 olist vproc::tbl;
 int vproc::counter;
@@ -119,7 +122,8 @@ void VprocPreamble(void *arg)
 
     /* This lock is released by start_thread, as soon as the initialization has
      * finalized (aka. this->lwpid has been set */
-    ObtainWriteLock(init_lock);  /* we never have to give it up. */
+    if (init_lock)
+	ObtainWriteLock(init_lock);  /* we never have to give it up. */
 
     /* VPROC rock permits mapping back to vproc object. */
     PROCESS x;
@@ -146,6 +150,8 @@ void VprocPreamble(void *arg)
      * from vproc. Others should pass a function pointer to indicate the entry
      * point when initializing a new vproc */
     vp->main();
+    if (init_lock)
+	delete vp;
 }
 
 
@@ -163,7 +169,7 @@ vproc *VprocSelf() {
 }
 
 
-void VprocWait(char *addr) {
+void VprocWait(const void *addr) {
 #ifdef VENUSDEBUG
     {
 	/* Sanity-check: vproc must not context-switch in mid-transaction! */
@@ -180,7 +186,7 @@ void VprocWait(char *addr) {
 }
 
 
-void VprocMwait(int wcount, char **addrs) {
+void VprocMwait(int wcount, const void **addrs) {
 #ifdef VENUSDEBUG
     {
 	/* Sanity-check: vproc must not context-switch in mid-transaction! */
@@ -197,7 +203,7 @@ void VprocMwait(int wcount, char **addrs) {
 }
 
 
-void VprocSignal(char *addr, int yield) {
+void VprocSignal(const void *addr, int yield) {
 #ifdef VENUSDEBUG
     if (yield) {
 	/* Sanity-check: vproc must not context-switch in mid-transaction! */
@@ -355,13 +361,12 @@ void PrintVprocs(int fd) {
     fdprint(fd, "\n");
 }
 
-vproc::vproc(char *n, PROCBODY f, vproctype t, int stksize, int priority)
+vproc::vproc(const char *n, PROCBODY f, vproctype t, int stksize, int priority)
 {
     /* Initialize the data members. LWPid is filled in as a side effect of
      * LWP operations in start_thread. */
     lwpid = NULL;
-    name = new char[strlen(n) + 1];
-    strcpy(name, n);
+    name = strdup(n);
     func = f;
     vpid = counter++;
     memset((void *)&rvm_data, 0, (int) sizeof(rvm_perthread_t));
@@ -418,7 +423,7 @@ void vproc::start_thread(void)
 	    CHOKE("VprocInit: IOMGR_Initialize failed (%d)", iomgrrc);
 
 	initialized = 1;
-	VprocPreamble((void *)&init_lock);
+	VprocPreamble(NULL);
     }
 }
 
@@ -453,7 +458,7 @@ vproc::~vproc() {
     if (LWP_DestroyProcess((PROCESS)lwpid) != LWP_SUCCESS)
 	CHOKE("vproc::~vproc: LWP_DestroyProcess failed");
 
-    delete [] name;
+    free(name);
 }
 
 /* local-repair modification */
@@ -553,7 +558,7 @@ void vproc::Begin_VFS(Volid *volid, int vfsop, int volmode)
     u.u_vfsop = vfsop;
 
     if (u.u_volmode == VM_MUTATING && u.u_vol->IsReplicated() &&
-	u.u_vol->IsWriteDisconnected())
+	!u.u_vol->IsUnreachable())
     {
 	struct timeval delay = { 1, 0 };
 	int free_fsos, free_mles, free_blocks;
@@ -593,8 +598,8 @@ wait_for_reintegration:
 
     /* Kick out non-ASR processes if an ASR is running */
     if (u.u_vol->IsReplicated() && (vfsop != CODA_RESOLVE) && 
-	(((repvol *)u.u_vol)->asr_running() && 
-	 (u.u_pgid != ((repvol *)u.u_vol)->asr_id())))
+		(((repvol *)u.u_vol)->asr_running() && 
+		 (u.u_pgid != ((repvol *)u.u_vol)->asr_pgid())))
       u.u_error = EAGAIN;
     else /* Attempt to enter the volume. */
       u.u_error = u.u_vol->Enter(u.u_volmode, u.u_uid);
@@ -605,14 +610,27 @@ wait_for_reintegration:
 
 /* local-repair modification */
 /* Retryp MUST be non-null in order to do any form of waiting. */
-void vproc::End_VFS(int *retryp) {
+void vproc::End_VFS(int *retryp)
+{
     LOG(1, ("vproc::End_VFS(%s): code = %d\n",
 	     VenusOpStr(u.u_vfsop), u.u_error));
 
     if (retryp) *retryp = 0;
 
-    /* Exit the volume. */
     if (u.u_vol == 0) goto Exit;
+
+    /* sync reintegrate whenever we create or destroy an object. Ignore open
+     * because those don't result in action until close, and setattr because
+     * those are mostly cosmetic and don't affect access to the object */
+    if (u.u_volmode == VM_MUTATING && u.u_vol->IsReplicated() &&
+	u.u_vfsop != CODA_OPEN && u.u_vfsop != CODA_SETATTR &&
+	u.u_error == 0)
+    {
+	if (((repvol *)u.u_vol)->IsSync())
+	    u.u_vol->flags.transition_pending = 1;
+    }
+
+    /* Exit the volume. */
     u.u_vol->Exit(u.u_volmode, u.u_uid);
 
     /* Handle synchronous resolves. */
@@ -648,8 +666,11 @@ void vproc::End_VFS(int *retryp) {
 	{ u.u_error = EINTR; goto Exit; }
 
     /* Caller should be prepared to retry! */
-    if (!retryp)
-	{ if (u.u_error == ERETRY) u.u_error = EWOULDBLOCK; goto Exit; }
+    if (!retryp) {
+	if (u.u_error == ERETRY)
+	    u.u_error = EWOULDBLOCK;
+	goto Exit;
+    }
 
     switch(u.u_error) {
 	case ERETRY:
@@ -814,21 +835,21 @@ void VPROC_printvattr(struct coda_vattr *vap)
 	}
 }
 
-long FidToNodeid(VenusFid *fid) 
+long FidToNodeid(VenusFid *fid)
 {
 	if (FID_EQ(fid, &NullFid))
 		CHOKE("FidToNodeid: null fid");
 
-#ifdef __BSD44__
-    /* Venus Root.  Use the mount point's nodeid. */
+#if defined(__FreeBSD__) || defined(__NetBSD__)
+	/* Venus Root.  Use the mount point's nodeid. */
 	if (FID_EQ(fid, &rootfid))
 	    return(rootnodeid);
+#endif
 
 	/* Other volume root.  We need the relevant mount point's fid,
-           but we don't know what that is! */
-#endif
+	   but we don't know what that is! */
 	if (FID_IsVolRoot(fid)) {
-		LOG(0, ("FidToNodeid: called for volume root (%x.%x)!!!\n", 
+		LOG(0, ("FidToNodeid: called for volume root (%x.%x)!!!\n",
 			fid->Realm, fid->Volume));
 	}
 
