@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -39,13 +39,7 @@ extern "C" {
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#if 0
-#ifdef __NetBSD__
-#define __attribute__(x)    /* dummied out because of machine/segments.h */
-#include <sys/user.h>
-#undef __attribute__
-#endif /* __NetBSD__ */
-#endif
+#include <stdint.h>
 
 #include <coda.h>
 
@@ -212,7 +206,8 @@ class vproc : public olink {
     unsigned interrupted : 1;
     struct vcbevent *ve;
 
-    vproc(char *, PROCBODY, vproctype, int =VPROC_DEFAULT_STACK_SIZE, int =LWP_NORMAL_PRIORITY);
+    vproc(const char *, PROCBODY, vproctype, int =VPROC_DEFAULT_STACK_SIZE,
+	  int =LWP_NORMAL_PRIORITY);
     vproc(vproc&);		// not supported
     int operator=(vproc&);	// not supported
     virtual ~vproc();
@@ -225,12 +220,10 @@ class vproc : public olink {
     void End_VFS(int * =0);
 
     /* The vproc interface: mostly matching kernel requests.  */
-    void mount(char *, void *);
-    void unmount();
     void root(struct venus_cnode *);
     void statfs(struct coda_statfs *);
     void sync();
-    void vget(struct venus_cnode *, struct cfid *);
+    void vget(struct venus_cnode *, struct cfid *, int what=RC_STATUS);
     void open(struct venus_cnode *, int);
     void close(struct venus_cnode *, int);
     void ioctl(struct venus_cnode *, unsigned char nr, struct ViceIoctl *, int);
@@ -282,11 +275,10 @@ extern void VprocInit();
 extern void Rtry_Wait();
 extern void Rtry_Signal();
 extern vproc *FindVproc(int);
-extern void VprocPreamble(struct Lock *);
 extern vproc *VprocSelf();
-extern void VprocWait(char *);
-extern void VprocMwait(int, char **);
-extern void VprocSignal(char *, int=0);
+extern void VprocWait(const void *);
+extern void VprocMwait(int, const void **);
+extern void VprocSignal(const void *, int=0);
 extern void VprocSleep(struct timeval *);
 extern void VprocYield();
 extern int VprocSelect(int, fd_set *, fd_set *, fd_set *, struct timeval *);
