@@ -42,7 +42,6 @@ extern "C" {
 #include <auth2.h>
 #include <prs.h>
 #include <rpc2/rpc2.h>
-#include <rpc2/fail.h>
 #include <lka.h>
 #include <vice.h>
 
@@ -62,8 +61,7 @@ extern "C" {
 #include "venus.private.h"
 #include "venusvol.h"
 #include "vproc.h"
-#include "worker.h" 
-#include "adv_daemon.h"
+#include "worker.h"
 #include "realmdb.h"
 
 
@@ -91,14 +89,14 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 	    fsobj *f = 0;
 
 	    int volmode = (nr == _VIOCSETAL || nr == _VIOC_ADD_MT_PT ||
-			   nr == _VIOC_AFS_DELETE_MT_PT || nr == _VIOC_SETVV) ?
-			   VM_MUTATING : VM_OBSERVING;
+			   nr == _VIOC_AFS_DELETE_MT_PT || nr == _VIOC_SETVV)
+	      ? VM_MUTATING : VM_OBSERVING;
 	    int rcrights = RC_STATUS;
 	    if (nr == _VIOC_ADD_MT_PT || nr == _VIOC_AFS_DELETE_MT_PT)
 		rcrights |= RC_DATA;
 
 	    for (;;) {
-		Begin_VFS(fid, CODA_IOCTL, volmode);
+	        Begin_VFS(fid, CODA_IOCTL, volmode);
 		if (u.u_error) break;
 
 		u.u_error = FSDB->Get(&f, fid, u.u_uid, rcrights);
@@ -228,7 +226,7 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 
 			/* Disallow special names. */
 			verifyname(link_name, NAME_NO_DOTS | NAME_NO_CONFLICT);
-                        if (u.u_error) break;
+			if (u.u_error) break;
 
 			/* Verify that parent is a directory. */
 			if (!f->IsDir()) { u.u_error = ENOTDIR; break; }
@@ -269,6 +267,12 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 			 * mount a specific object in the fake repair volume.
 			 *
 			 * -JH
+			 *
+			 * Additionally, collapsing an expanded object that
+			 * was previously a mount point modifies the old mount-
+			 * link to start with '$'. This is to avoid problems
+			 * with ignoring dangling symlinks within
+			 * TryToCover, and is never visible to the user. - Adam
 			 */
 			/* make it a 'magic' mount name */
 			snprintf(contents, CODA_MAXNAMLEN,  "#%s.", arg);
@@ -280,6 +284,11 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 					       0644, FSDB->StdPri());
 			if (u.u_error) { break; }
 			/* set vattr fields? */
+
+			/* Send a downcall to the kernel to get rid of any
+			 * negative name cache entries for the newly created
+			 * object */
+			k_Purge(fid, 0);
 			break;
 			}
 
@@ -289,8 +298,8 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 			char *target_name = (char *) data->in;
 
 			/* Disallow deletion of special names. */
-                        verifyname(target_name, NAME_NO_DOTS | NAME_NO_CONFLICT);
-                        if (u.u_error) break;
+			verifyname(target_name, NAME_NO_DOTS | NAME_NO_CONFLICT);
+			if (u.u_error) break;
 
 			/* Verify that parent is a directory. */
 			if (!f->IsDir())
@@ -303,35 +312,37 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 
 			/* Verify that target is a mount point (either valid or dangling). */
 			if (!target_fso->IsMtPt() && !target_fso->IsMTLink()) {
-                            FSDB->Put(&target_fso);
-                            u.u_error = ENOTDIR;
-                            break;
-                        }
+			    FSDB->Put(&target_fso);
+			    u.u_error = ENOTDIR;
+			    break;
+			}
 
 			/* Verify that we have delete permission for the parent. */
 			u.u_error = f->Access(PRSFS_DELETE, C_A_F_OK, u.u_uid);
 			if (u.u_error) {
-                            FSDB->Put(&target_fso);
-                            break;
-                        }
+			    FSDB->Put(&target_fso);
+			    break;
+			}
 
 			/* We only remove MTLinks, not valid MtPts! */
 			if (target_fso->IsMtPt()) {
 			    Recov_BeginTrans();
-                            fsobj *root_fso = target_fso->u.root;
-                            FSO_ASSERT(target_fso,
-                                       root_fso && root_fso->u.mtpoint == target_fso);
-                            root_fso->UnmountRoot();
-                            target_fso->UncoverMtPt();
-                            Recov_EndTrans(MAXFP);
+			    fsobj *root_fso = target_fso->u.root;
+			    FSO_ASSERT(target_fso,
+				       root_fso && root_fso->u.mtpoint == target_fso);
+			    root_fso->UnmountRoot();
+			    target_fso->UncoverMtPt();
+			    Recov_EndTrans(MAXFP);
 			}
 
 			/* Do the remove. */
 			f->PromoteLock();
 			target_fso->PromoteLock();
 			u.u_error = f->Remove(target_name, target_fso, u.u_uid);
-			k_Purge(&target_fso->fid, 1);
 
+			/* Get rid of anything cached in the kernel relating to
+			 * the removed volume and it's children */
+			k_Purge(&target_fso->fid, 1);
 
 			FSDB->Put(&target_fso);
 			break;
@@ -342,13 +353,13 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 			fsobj *target_fso = 0;
 			char *target_name = (char *) data->in;
 			int out_size = 0;	/* needed since data->out_size is a short! */
-                        verifyname(target_name, NAME_NO_DOTS);
+			verifyname(target_name, NAME_NO_DOTS);
 
 			/* Verify that parent is a directory. */
 			if (!f->IsDir()) {
-                            u.u_error = ENOTDIR;
-                            break;
-                        }
+			    u.u_error = ENOTDIR;
+			    break;
+			}
 
 			/* Get the target object. */
 			u.u_error = f->Lookup(&target_fso, NULL, target_name,
@@ -371,9 +382,9 @@ void vproc::do_ioctl(VenusFid *fid, unsigned char nr, struct ViceIoctl *data)
 							 CODA_MAXPATHLEN,
 							 &out_size, u.u_uid);
 			if (u.u_error) {
-                            FSDB->Put(&target_fso);
-                            break;
-                        }
+			    FSDB->Put(&target_fso);
+			    break;
+			}
 
 			/* Make link a proper string. */
 			data->out_size = out_size;
@@ -443,48 +454,146 @@ O_FreeLocks:
 
 	/* Object-based. Allowing access to inconsistent objects. */
 	case _VIOC_ENABLEREPAIR:
+	case _VIOC_DISABLEREPAIR:
+        case _VIOC_EXPANDOBJECT:
+        case _VIOC_COLLAPSEOBJECT:
+        case _VIOC_REPAIR:
 	case _VIOC_FLUSHASR:
 	case _VIOC_GETFID:
 	    {
 	    fsobj *f = 0;
 
+	    int volmode =
+	      (nr == _VIOC_EXPANDOBJECT || nr == _VIOC_ENABLEREPAIR ||
+	       nr == _VIOC_COLLAPSEOBJECT || nr == _VIOC_DISABLEREPAIR ||
+	       nr == _VIOC_REPAIR)
+	      ? VM_MUTATING : VM_OBSERVING;
+
+	    int rcrights = RC_STATUS;
+
+	    if (nr == _VIOC_EXPANDOBJECT || nr == _VIOC_ENABLEREPAIR ||
+		nr == _VIOC_COLLAPSEOBJECT || nr == _VIOC_DISABLEREPAIR ||
+		nr == _VIOC_REPAIR)
+	      rcrights |= RC_DATA;
+
 	    for (;;) {
-		Begin_VFS(fid, CODA_IOCTL, VM_OBSERVING);
+		Begin_VFS(fid, CODA_IOCTL, volmode);
 		if (u.u_error) break;
 
 		u.u_error = FSDB->Get(&f, fid, u.u_uid, RC_STATUS,
-				      NULL, NULL, 1);
-		if (u.u_error) goto OI_FreeLocks;
+				      NULL, NULL, NULL, 1);
+		if (u.u_error && u.u_error != EINCONS) goto OI_FreeLocks;
 
 		switch(nr) {
+
 		case _VIOC_ENABLEREPAIR:
-		    {
-		    /* Try to enable target volume for repair by this user. */
-		    VolumeId  *RWVols  = (VolumeId *)data->out;
-		    uid_t    *LockUids = (uid_t *)&(RWVols[VSG_MEMBERS]);
-		    unsigned long *LockWSs =
-			(unsigned long *)&(LockUids[VSG_MEMBERS]);
-		    char      *endp     = (char *)&(LockWSs[VSG_MEMBERS]);
-
-		    /* actually EnableRepair operates on the volume, but we
-		     * also check if the object is really inconsistent */
-		    if (f->IsFake() && f->vol->IsReplicated())
-                        u.u_error =
-			    ((repvol *)f->vol)->EnableRepair(u.u_uid,
-							     RWVols,
-							     LockUids,
-							     LockWSs);
-                    else
-                        u.u_error = EOPNOTSUPP;
-
-		    data->out_size = (endp - data->out);
+		case _VIOC_EXPANDOBJECT:
+		  {
+		    f->PromoteLock();
+		    u.u_error = f->ExpandObject();
 
 		    /* Make sure the kernel drops the symlink */
 		    (void)k_Purge(fid, 1);
 		    (void)k_Purge(&f->pfid, 1);
 		    break;
+		  }
+
+		case _VIOC_DISABLEREPAIR:
+		  u.u_error = 0;
+		case _VIOC_COLLAPSEOBJECT:
+		  {
+		      f->PromoteLock();
+		      u.u_error = f->CollapseObject();
+
+		      /* Make sure the kernel drops the subtree */
+		      (void)k_Purge(&f->fid, 1);
+		      (void)k_Purge(&f->pfid, 1);
+		      break;
+		  }
+
+		  /*
+		    BEGIN_HTML
+		    <a name="dorepair"><strong> dorepair handler </strong></a>
+		    END_HTML
+		  */
+		case _VIOC_REPAIR:
+		  {
+		    if (f->IsLocalObj()) {
+		      int rc;
+		      fsobj *fakedir = f;
+
+		      /* find the expanded object */
+		      f = NULL;
+		      rc = fakedir->Lookup(&f, NULL, LOCALCACHE, u.u_uid,
+				    CLU_CASE_SENSITIVE | CLU_TRAVERSE_MTPT, 1);
+		      if(rc)
+			rc = fakedir->Lookup(&f, NULL, LOCALCACHE_HIDDEN,
+			   u.u_uid, CLU_CASE_SENSITIVE | CLU_TRAVERSE_MTPT, 1);
+		      if(rc) {
+			LOG(0, ("VIOC_REPAIR: Lookup() failed for LOCALCACHE:%d\n", rc));
+			break;
+		      }
+		      else
+			CODA_ASSERT(f);
+
+		      LOG(0, ("VIOC_REPAIR: called on expanded directory (%s)! redirecting to localcache (%s)\n", FID_(&fakedir->fid), FID_(&f->fid)));
+
+		      FSDB->Put(&fakedir);
 		    }
 
+#ifdef TIMING
+		    gettimeofday(&u.u_tv1, 0); u.u_tv2.tv_sec = 0;
+#endif
+		    /* whether we need this depends on the granularity of repair calls */
+		    volent *v = 0;
+		    if ((u.u_error = VDB->Get(&v, MakeVolid(&f->fid)))) break;
+
+		    int entered = 0;
+		    if ((u.u_error = v->Enter(volmode, u.u_uid)) != 0)
+		      CODA_ASSERT("V_FreeLocks!\n");//goto V_FreeLocks;
+		    entered = 1;
+
+		    /* Try to repair target object. */
+#define	RepairFile  ((char *)data->in)
+#define	startp	    (data->out)
+#define	RWVols	    ((VolumeId *)(startp))
+#define	ReturnCodes ((int *)(RWVols + VSG_MEMBERS))
+#define	endp	    ((char *)(ReturnCodes + VSG_MEMBERS))
+		    data->out_size = (endp - startp);
+
+                    u.u_error = EOPNOTSUPP;
+
+		    if(v->IsReplicated()) {
+		      VenusFid fid = f->fid;
+		      LOG(0, ("VIOC_REPAIR calling repvol::Repair (%s)\n",
+			      FID_(&f->fid)));
+		      FSDB->Put(&f);
+		      u.u_error = ((repvol *)v)->Repair(&fid, RepairFile,
+							u.u_uid, RWVols,
+							ReturnCodes);
+		    }
+		    else
+		      LOG(0, ("VIOC_REPAIR: non-replicated volume!\n"));
+
+		    LOG(0, ("VIOC_REPAIR: repvol::Repair returns %d\n",
+			    u.u_error));
+#undef	RepairFile
+#undef	startp
+#undef	RWVols
+#undef	ReturnCodes
+#undef	endp
+		    if (entered) v->Exit(volmode, u.u_uid);
+		    float elapsed = 0.0;
+#ifdef TIMING
+
+		    gettimeofday(&u.u_tv2, 0);
+		    elapsed = SubTimes(&(u.u_tv2), &(u.u_tv1));
+#endif
+
+		    VDB->Put(&v);
+		    break;
+		  }
+
 		case _VIOC_FLUSHASR:
 		    /* This function used to be built around FSDB->Find, which
 		     * did no locking. Now we use FSDB->Get, which does do
@@ -513,7 +622,7 @@ O_FreeLocks:
 			VenusFid mtptfid = f->u.mtpoint->fid;
 			FSDB->Put(&f);
 			u.u_error = FSDB->Get(&f, &mtptfid, u.u_uid, RC_STATUS,
-					      NULL, NULL, 1);
+					      NULL, NULL, NULL, 1);
 			if (u.u_error) break;
 		    }
 
@@ -538,7 +647,8 @@ O_FreeLocks:
 		    }
 		}
 OI_FreeLocks:
-		FSDB->Put(&f);
+		if(f)
+		  FSDB->Put(&f);
 		int retry_call = 0;
 		End_VFS(&retry_call);
 		if (!retry_call) break;
@@ -551,18 +661,17 @@ OI_FreeLocks:
 	case _VIOCSETVOLSTAT:
 	case _VIOCWHEREIS:
 	case _VIOC_FLUSHVOLUME:
-	case _VIOC_DISABLEREPAIR:
-	case _VIOC_REPAIR:
 	case _VIOC_GETSERVERSTATS:
 	case _VIOC_CHECKPOINTML:
 	case _VIOC_PURGEML:
-	case _VIOC_BEGINML:
-	case _VIOC_ENDML:
+	case _VIOC_WD:
 	case _VIOC_ENABLEASR:
 	case _VIOC_DISABLEASR: 
 	case _VIOC_LISTCACHE_VOLUME:
 	case _VIOC_SYNCCACHE:
 	case _VIOC_REDIR:
+	case _VIOC_REP_CMD:
+
 	    {
 #ifdef TIMING
  	    gettimeofday(&u.u_tv1, 0); u.u_tv2.tv_sec = 0;
@@ -570,7 +679,7 @@ OI_FreeLocks:
 	    volent *v = 0;
 	    if ((u.u_error = VDB->Get(&v, MakeVolid(fid)))) break;
 
-	    int volmode = ((nr == _VIOC_REPAIR || nr == _VIOC_PURGEML) ?
+	    int volmode = ((nr == _VIOC_PURGEML || nr == _VIOC_REP_CMD) ?
 			   VM_MUTATING : VM_OBSERVING);
 	    int entered = 0;
 	    if ((u.u_error = v->Enter(volmode, u.u_uid)) != 0)
@@ -669,7 +778,7 @@ OI_FreeLocks:
 		    if (u.u_error) break;
 
 		    /* Format is (status, name, conn_state, conflict,
-		       cml_count, offlinemsg, motd) */
+		       cml_count, offlinemsg, motd, age, hogtime) */
 		    /* First we make sure we won't overflow data->out */
 		    if ( (sizeof(VolumeStatus) + /* volstat */
 			 strlen(name) + 1 + /* name */
@@ -724,7 +833,7 @@ OI_FreeLocks:
 
 		    /* Volume name. */
                     char name[V_MAXVOLNAMELEN];
-		    unsigned long namelen = strlen(cp) + 1;
+		    unsigned int namelen = strlen(cp) + 1;
 		    if (namelen >= V_MAXVOLNAMELEN) { u.u_error = EINVAL; break; }
 		    strcpy(name, cp);
 
@@ -744,7 +853,7 @@ OI_FreeLocks:
 
 		    /* Offline message for this volume. */
 		    char offlinemsg[256];
-		    unsigned long offlinemsglen = strlen(cp) + 1;
+		    unsigned int offlinemsglen = strlen(cp) + 1;
 		    if (offlinemsglen > 256) { u.u_error = EINVAL; break; }
 		    strcpy(offlinemsg, cp);
 		    RPC2_BoundedBS OfflineMsg;
@@ -755,7 +864,7 @@ OI_FreeLocks:
 
 		    /* Message of the day for this volume. */
 		    char motd[256];
-		    unsigned long motdlen = strlen(cp) + 1;
+		    unsigned int motdlen = strlen(cp) + 1;
 		    if (motdlen >= 256) { u.u_error = EINVAL; break; }
 		    strcpy(motd, cp);
 		    RPC2_BoundedBS MOTD;
@@ -805,53 +914,6 @@ OI_FreeLocks:
 		    break;
 		    }
 
-		case _VIOC_DISABLEREPAIR:
-		    {
-		    /* Disable repair of target volume by this user. */
-                    if (v->IsReplicated())
-                        u.u_error =
-                            ((repvol *)v)->DisableRepair(u.u_uid);
-                    else
-                        u.u_error = EOPNOTSUPP;
-
-		    (void)k_Purge(fid, 1);
-
-		    break;
-		    }
-/*
-  BEGIN_HTML
-  <a name="dorepair"><strong> dorepair handler </strong></a>
-  END_HTML
-*/
-		case _VIOC_REPAIR:
-		    {
-		    /* Try to repair target object. */
-#define	RepairFile  ((char *)data->in)
-#define	startp	    (data->out)
-#define	RWVols	    ((VolumeId *)(startp))
-#define	ReturnCodes ((int *)(RWVols + VSG_MEMBERS))
-#define	endp	    ((char *)(ReturnCodes + VSG_MEMBERS))
-		    data->out_size = (endp - startp);
-                    u.u_error = EOPNOTSUPP;
-                    if (v->IsReplicated())
-                        u.u_error = ((repvol *)v)->Repair(fid, RepairFile,
-                                       u.u_uid, RWVols, ReturnCodes);
-
- 	            LOG(0, ("MARIA: VIOC_REPAIR calls volent::Repair which returns %d\n",u.u_error));
-		    /* We don't have the object so can't provide a pathname
-		     * if ((SkkEnabled) && (u.u_error == 0)) {
-		     *   NotifyUsersObjectConsistent("???UNKNOWN???",fid);
-		     * } */
-		    
-#undef	RepairFile
-#undef	startp
-#undef	RWVols
-#undef	ReturnCodes
-#undef	endp
-
-		    break;
-		    }
-
 		case _VIOC_GETSERVERSTATS:
 		    {
 		    /* Extract the host array from the vsgent or volent as appropriate. */
@@ -898,7 +960,7 @@ OI_FreeLocks:
                         u.u_error = ((repvol *)v)->PurgeMLEs(u.u_uid);
 		    break;
 		    }
-     	        case _VIOC_BEGINML:
+		case _VIOC_WD:
 		    {
 		    /* 
 		     * Begin logging mutations to this volume. 
@@ -906,47 +968,32 @@ OI_FreeLocks:
 		     * fetches may be performed but mutations are logged.
 		     */
 		    char *startp = (char *) data->in;
-#define agep ((unsigned *)(startp))
-#define timep ((unsigned *)(agep + 1))
+#define agep ((unsigned int *)(startp))
+#define timep ((unsigned int *)(agep + 1))
                     u.u_error = EOPNOTSUPP;
                     if (v->IsReplicated())
                         u.u_error = ((repvol *)v)->WriteDisconnect(*agep, *timep); 
-		    if (u.u_error == 0) 
-			eprint("Logging updates to volume %s", v->GetName());
-
+#undef timep
+#undef agep
 		    break;
 		    }
-   	        case _VIOC_ENDML:
-		    {
-		    /* 
-		     * Stop logging mutations to this volume, and
-		     * schedule it for reintegration if appropriate.
-		     */
-                    u.u_error = EOPNOTSUPP;
-                    if (v->IsReplicated())
-                        u.u_error = ((repvol *)v)->WriteReconnect();    
-		    if (u.u_error == 0) 
-			eprint("Propagating updates to volume %s (|CML| = %d)", 
-			       v->GetName(), ((repvol *)v)->GetCML()->count());
-		    break;
-		    }
-	      case _VIOC_ENABLEASR:
+		case _VIOC_ENABLEASR:
 		    {			
                         u.u_error = EOPNOTSUPP;
                         if (v->IsReplicated())
-                            u.u_error = ((repvol *)v)->EnableASR(u.u_uid);
+                            u.u_error = ((repvol *)v)->AllowASR(u.u_uid);
 			break;
 		    }
-	      case _VIOC_DISABLEASR:
+		case _VIOC_DISABLEASR:
 		    {
                         u.u_error = EOPNOTSUPP;
                         if (v->IsReplicated())
-                            u.u_error = ((repvol *)v)->DisableASR(u.u_uid);
+                            u.u_error = ((repvol *)v)->DisallowASR(u.u_uid);
 			break;
 		    }
 
-	      case _VIOC_SYNCCACHE:
-              {
+		case _VIOC_SYNCCACHE:
+			{
                   u.u_error = EOPNOTSUPP;
                   if (v->IsReplicated()) {
 		      v->Exit(volmode, u.u_uid);
@@ -956,37 +1003,317 @@ OI_FreeLocks:
 		  break;
               }
 
-	      case _VIOC_REDIR:
+		case _VIOC_REDIR:
 	      {
-		  struct in_addr staging_server;
-		  if (data->in_size != (int)sizeof(struct in_addr)) {
+			struct in_addr staging_server;
+			if (data->in_size != (int)sizeof(struct in_addr)) {
 		      u.u_error = EINVAL; break;
-		  }
-		  if (!v->IsReplicated()) {
+			}
+			if (!v->IsReplicated()) {
 		      u.u_error = EINVAL; break;
+			}
+			
+			staging_server = *(struct in_addr *)data->in;
+			((repvol *)v)->SetStagingServer(&staging_server);
+			break;
+	      }
+
+	    case _VIOC_REP_CMD:
+	      {
+		int rep_cmd;
+		CODA_ASSERT(sscanf((char *) data->in, "%d", &rep_cmd) == 1);
+		switch (rep_cmd) {
+/*
+  BEGIN_HTML
+  <a name="beginrepair"><strong> beginrepair handler </strong></a>
+  END_HTML
+*/
+		  case REP_CMD_BEGIN:
+		    {
+		      /* This ioctl only figures out what type of conflict
+		       * we are dealing with and verifies the object is
+		       * expanded correctly. No mutations are performed.
+		       * The idea of 'beginning repair' is historical, and
+		       * there is no problem calling this on the same directory
+		       * or volume many times without an 'endrepair', or
+		       * attempting any form of repair without calling this
+		       * first. */
+		      /*
+		       *      1 - Local/Global repair session
+		       *      2 - Server/Server repair session
+		       *      3 - Both Local/Global and Server/Server
+		       */
+
+		      int code = -1, rc;
+		      fsobj *dir = NULL, *localcache = NULL;
+		      char *msg;
+
+		      msg = (char *)data->out;
+		      if(!msg) {
+			LOG(0, ("REP_CMD_BEGIN: (%s) bad data->out parameter\n",
+				FID_(fid)));
+			code = -1;
+			goto BEGIN_cleanup;
+		      }
+
+		      dir = FSDB->Find(fid);
+		      if(!dir) {
+			LOG(0, ("REP_CMD_BEGIN: (%s) <= this object missing!\n",
+				FID_(fid)));
+			code = -1;
+			goto BEGIN_cleanup;
+		      }
+
+		      if(!dir->IsLocalObj()) {
+			LOG(0, ("REP_CMD_BEGIN: (%s) not an expanded dir\n",
+				FID_(fid)));
+			code = -1;
+			goto BEGIN_cleanup;
+		      }
+
+		      rc = dir->Lookup(&localcache, fid, LOCALCACHE_HIDDEN,
+				       u.u_uid, CLU_CASE_SENSITIVE |
+				       CLU_TRAVERSE_MTPT, 1);
+
+		      if(!localcache)
+			rc = dir->Lookup(&localcache, fid, LOCALCACHE,
+					 u.u_uid, CLU_CASE_SENSITIVE |
+					 CLU_TRAVERSE_MTPT, 1);
+		      if(!localcache || (rc && rc != EINCONS)) {
+			LOG(0, ("REP_CMD_BEGIN: (%s) failed finding localcache "
+				"object.. bad news.\n", FID_(fid)));
+			code = -1;
+			goto BEGIN_cleanup;
+		      }
+
+		      if(localcache->IsFake())
+			code = 2;
+
+		      if(localcache->IsToBeRepaired()) {
+			if(code == 2)
+			  code = 3;
+			else
+			  code = 1;
+		      }
+
+		    BEGIN_cleanup:
+		      if(localcache)
+			FSDB->Put(&localcache);
+
+		      sprintf(msg, "%d", code);
+		      data->out_size = (short)sizeof((char *) data->out);
+
+		      u.u_error = 0;
+		      break;
+		    }
+/*
+  BEGIN_HTML
+  <a name="endrepair"><strong> endrepair handler </strong></a>
+  END_HTML
+*/
+		  case REP_CMD_END:
+		    {
+		      /*
+		       * This ioctl literally does nothing. The only applicable
+		       * use might be for unfreezing volumes if mechanisms
+		       * are implemented to freeze state upon server-server
+		       * conflict discovery.
+		       *
+		       * Even then, that might be better suited to the
+		       * _VIOC_REPAIR ioctl, which figures out if any repair
+		       * actually occurred and if it was successful.
+		       */
+		      char *msg;
+
+		      msg = (char *)data->out;
+		      if(!msg) {
+			u.u_error = EINVAL;
+			break;
+		      }
+
+		      u.u_error = EOPNOTSUPP;
+		      if(v->IsReplicated()) {
+			sprintf(msg, "no action performed\n");
+			u.u_error = 0;
+		      }
+
+		      data->out_size = (short)strlen(msg) + 1;
+		      break;
+		    }
+/*
+  BEGIN_HTML
+  <a name="checklocal"><strong> checklocal handler </strong></a>
+  END_HTML
+*/
+		  case REP_CMD_CHECK:
+		  {
+		    char *msg;
+		    msg = (char *)data->out;
+		    if(!msg)
+		      u.u_error = EINVAL;
+		    else
+		      u.u_error = EOPNOTSUPP;
+		    if(msg && v->IsReplicated()) {
+		      ClientModifyLog *cml = ((repvol *)v)->GetCML();
+		      cml->CheckCMLHead(msg);
+		      u.u_error = 0;
+		    }
+		    else
+		      sprintf(msg, "not a replicated volume\n");
+
+		    data->out_size = (short)strlen(msg) + 1;
+		    break;
 		  }
 
-		  staging_server = *(struct in_addr *)data->in;
-		  ((repvol *)v)->SetStagingServer(&staging_server);
+/*
+  BEGIN_HTML
+  <a name="preservelocal"><strong> preservelocal handler </strong></a>
+  END_HTML
+*/
+		case REP_CMD_PRESERVE:
+		  {
+		    char *msg;
+		    msg = (char *)data->out;
+		    if(!msg) {
+		      u.u_error = EINVAL;
+		      break;
+		    }
+
+		    msg[0] = '\0';
+		    u.u_error = EOPNOTSUPP;
+		    if(v->IsReplicated()) {
+		      ClientModifyLog *cml = ((repvol *)v)->GetCML();
+		      u.u_error = 0;
+		      cml->PreserveLocalMutation(msg);
+		    }
+
+		    data->out_size = (short)strlen(msg) + 1;
+		    break;
+		  }
+
+/*
+  BEGIN_HTML
+  <a name="preservealllocal"><strong> preservealllocal handler </strong></a>
+  END_HTML
+*/
+		case REP_CMD_PRESERVE_ALL:
+		  {
+		    char *msg;
+		    msg = (char *)data->out;
+		    if(!msg) {
+		      u.u_error = EINVAL;
+		      break;
+		    }
+
+		    u.u_error = EOPNOTSUPP;
+		    if(v->IsReplicated()) {
+		      ClientModifyLog *cml = ((repvol *)v)->GetCML();
+		      CODA_ASSERT(cml);
+		      if(!cml) {
+			sprintf(msg, "no client modify log on this volume\n");
+			data->out_size = (short)strlen(msg) + 1;
+			u.u_error = EINVAL;
+			break;
+		      }
+		      u.u_error = 0;
+		      cml->PreserveAllLocalMutation(msg);
+		    }
+
+		    data->out_size = (short)strlen(msg) + 1;
+		    break;
+		  }
+
+/*
+  BEGIN_HTML
+  <a name="discardlocal"><strong> discardlocal handler </strong></a>
+  END_HTML
+*/
+		case REP_CMD_DISCARD:
+		  {
+		    char *msg;
+		    msg = (char *)data->out;
+		    if(!msg) {
+		      u.u_error = EINVAL;
+		      break;
+		    }
+
+		    u.u_error = EOPNOTSUPP;
+		    if(v->IsReplicated()) {
+		      ClientModifyLog *cml = ((repvol *)v)->GetCML();
+		      CODA_ASSERT(cml);
+		      if(!cml) {
+			sprintf(msg, "no client modify log on this volume\n");
+			data->out_size = (short)strlen(msg) + 1;
+			u.u_error = EINVAL;
+			break;
+		      }
+		      u.u_error = cml->DiscardLocalMutation(msg);
+		    }
+
+		    data->out_size = (short)strlen(msg) + 1;
+		    break;
+		  }
+/*
+  BEGIN_HTML
+  <a name="discardalllocal"><strong> discardalllocal handler </strong></a>
+  END_HTML
+*/
+		case REP_CMD_DISCARD_ALL:
+		  {
+		    /* Not supported. Use PURGEML instead. */
+		    data->out_size = 0;
+		    u.u_error = EOPNOTSUPP;
+		    break;
+		  }
+/*
+  BEGIN_HTML
+  <a name="listlocal"><strong> listlocal handler </strong></a>
+  END_HTML
+*/
+		case REP_CMD_LIST:
+		  {
+		    /* list local mutations belonging to this session */
+		    char fpath[CODA_MAXPATHLEN];
+		    int dummy;
+
+		    sscanf((char *) data->in, "%d %s", &dummy, fpath);
+
+		    u.u_error = EOPNOTSUPP;
+		    if(v->IsReplicated()) {
+		      FILE *fp = fopen(fpath, "w");
+		      u.u_error = EINVAL; /* EBADF even though not a fd? */
+		      if (fp) {
+			ClientModifyLog *cml = ((repvol *)v)->GetCML();
+			CODA_ASSERT(cml);
+			int count = cml->ListCML(fp);
+			fprintf(fp, "%d entries total in the modify log for volume %s\n", count, v->GetName());
+			fflush(fp);
+			fclose(fp);
+			u.u_error = 0;
+		      }
+		    }
+		    break;
+		  }
+		default:
+		  eprint("bogus REP_CMD(%d)", rep_cmd);
 		  break;
+		}
 	      }
 	    }
 
-V_FreeLocks:
-	    if (entered) v->Exit(volmode, u.u_uid);
- 	    float elapsed = 0.0;
+	    V_FreeLocks:
+	      if (entered) v->Exit(volmode, u.u_uid);
+	      float elapsed = 0.0;
 #ifdef TIMING
- 	    
-	    gettimeofday(&u.u_tv2, 0);
-	    elapsed = SubTimes(&(u.u_tv2), &(u.u_tv1));
+
+	      gettimeofday(&u.u_tv2, 0);
+	      elapsed = SubTimes(&(u.u_tv2), &(u.u_tv1));
 #endif
-
-	    VDB->Put(&v);
-	    }
+	      VDB->Put(&v);
 	    if (u.u_error == ERETRY)
-		u.u_error = EWOULDBLOCK;
+		  u.u_error = EWOULDBLOCK;
 	    return;
-
+	    }
 	/* FS-based. */
 	case _VIOCSETTOK:
 	case _VIOCGETTOK:
@@ -1010,14 +1337,11 @@ V_FreeLocks:
 	case _VIOC_TRUNCATELOG:
 	case _VIOC_DISCONNECT:
 	case _VIOC_RECONNECT:
-	case _VIOC_STRONG:
-	case _VIOC_ADAPTIVE:
 	case _VIOC_LISTCACHE:
 	case _VIOC_GET_MT_PT:
 	case _VIOC_WD_ALL:
-	case _VIOC_WR_ALL:
-	case _VIOC_REP_CMD:
-	case _VIOC_UNLOADKERNEL:	
+	case _VIOC_SYNCCACHE_ALL:
+	case _VIOC_UNLOADKERNEL:
 	case _VIOC_LOOKASIDE:
 	    {
 	    switch(nr) {
@@ -1100,9 +1424,9 @@ V_FreeLocks:
 		    {
 		    /* Format of data is (len, secret, len, clear) */
 		    char *startp = (char *) data->in;
-#define secretlen ((long *)(startp))
+#define secretlen ((uint32_t *)(startp))
 #define secretp ((SecretToken *)(secretlen + 1))
-#define clearlen ((long *)(secretp + 1))
+#define clearlen ((uint32_t *)(secretp + 1))
 #define clearp ((ClearToken *)(clearlen + 1))
 #define realmp ((char *)(clearp + 1))
 /*
@@ -1132,9 +1456,9 @@ V_FreeLocks:
 		    {
 		    /* Format of data is (len, secret, len, clear) */
 		    char *startp = (char *) data->out;
-#define secretlen ((long *)(startp))
+#define secretlen ((uint32_t *)(startp))
 #define secretp ((SecretToken *)(secretlen + 1))
-#define clearlen ((long *)(secretp + 1))
+#define clearlen ((uint32_t *)(secretp + 1))
 #define clearp ((ClearToken *)(clearlen + 1))
 #define endp ((char *)(clearp + 1)) 
 		    Realm *realm = REALMDB->GetRealm(data->in);
@@ -1160,6 +1484,7 @@ V_FreeLocks:
 		    Realm *realm = REALMDB->GetRealm(data->in);
 		    userent *ue = realm->GetUser(u.u_uid);
 		    ue->Invalidate();
+		    FSDB->ResetUser(u.u_uid);
 		    PutUser(&ue);
 		    realm->PutRef();
 
@@ -1200,8 +1525,8 @@ V_FreeLocks:
 		    if (u.u_uid != V_UID)
 			{ u.u_error = EACCES; break; }
 
-		    long on;
-		    memcpy(&on, data->in, sizeof(long));
+		    uint32_t on;
+		    memcpy(&on, data->in, sizeof(uint32_t));
 		    on &= 0xff;
 		    if (on) DebugOn(); else DebugOff();
 
@@ -1411,221 +1736,19 @@ V_FreeLocks:
 		    break;
 		    }
 
-		/* cfs strong: Force strong connectivity to all servers */
-	        case _VIOC_STRONG:
-		    {
-		    srv_iterator next;
-		    srvent *s;
-		    while ((s = next()))
-			s->ForceStrong(1);
-		    break;
-		    }
-		/* cfs adaptive: Allow dynamic adaptation of connectivity */
-	        case _VIOC_ADAPTIVE:
-		    {
-		    srv_iterator next;
-		    srvent *s;
-		    while ((s = next()))
-			s->ForceStrong(0);
-		    break;
-		    }
-
 		case _VIOC_WD_ALL:
 		    {
 		    char *startp = (char *) data->in;
-#define agep ((unsigned *)(startp))
-#define timep ((unsigned *)(agep + 1))
+#define agep ((unsigned int *)(startp))
+#define timep ((unsigned int *)(agep + 1))
 		    u.u_error = VDB->WriteDisconnect(*agep, *timep);
-		    if (u.u_error == 0) 
-			eprint("Logging updates to all volumes");
 		    break;
 		    }
-   	        case _VIOC_WR_ALL:
+		case _VIOC_SYNCCACHE_ALL:
 		    {
-		    u.u_error = VDB->WriteReconnect();
-		    if (u.u_error == 0) 
-			eprint("Propagating updates to all volumes");
+		    u.u_error = VDB->SyncCache();
 		    break;
-		    }    	        
-		case _VIOC_REP_CMD:
-		    {
-			int rep_cmd;
-			CODA_ASSERT(sscanf((char *) data->in, "%d", &rep_cmd) == 1);
-			switch (rep_cmd) {
-			case REP_CMD_BEGIN:
-			    {
-				/* begin a repair session */
-				int mode, dummy;
-				fsobj *obj = FSDB->Find(fid);
-				CODA_ASSERT(obj);
-				sscanf((char *) data->in, "%d %d", &dummy, &mode);
-				LRDB->BeginRepairSession(&obj->fid,
-							 mode ? REP_SCRATCH_MODE : REP_DIRECT_MODE, 
-							 (char *) data->out);
-				data->out_size = (short)sizeof((char *) data->out);
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="endrepair"><strong> endrepair handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_END:
-			    {			
-				int commit, dummy;
-				VenusFid *squirrelFid;
-
-				CODA_ASSERT(LRDB);
-				squirrelFid = LRDB->RFM_LookupGlobalRoot(LRDB->repair_root_fid);
-
-				sscanf((char *) data->in, "%d %d", &dummy, &commit);
-				LRDB->EndRepairSession(commit, (char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				if (strncmp((char *) data->out, "repair session completed", strlen("repair session completed")) == 0) {
-
-				  LOG(0, ("MARIA:  End local repair successful\n"));
-				  /* We don't have the object so can't provide a pathname
-				   * if (SkkEnabled) {
-				   *   NotifyUsersObjectConsistent("???UNKNOWN???", squirrelFid);
-				   * } */
-				} else {
-				  LOG(0, ("MARIA:  End local repair failed\n"));
-				}
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="checklocal"><strong> checklocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_CHECK:
-			    {
-				LRDB->ContinueRepairSession((char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="preservelocal"><strong> preservelocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_PRESERVE:
-			    {
-				LRDB->PreserveLocalMutation((char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="preservealllocal"><strong> preservealllocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_PRESERVE_ALL:
-			    {
-				LRDB->PreserveAllLocalMutation((char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="discardlocal"><strong> discardlocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_DISCARD:
-			    {
-				LRDB->DiscardLocalMutation((char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="discardalllocal"><strong> discardalllocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_DISCARD_ALL:
-			    {
-				LRDB->DiscardAllLocalMutation((char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="listlocal"><strong> listlocal handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_LIST:
-			    {  
-				/* list local mutations belonging to this session */
-				int dummy;
-				char fpath[CODA_MAXPATHLEN];
-				sscanf((char *) data->in, "%d %s", &dummy, fpath);
-				if (LRDB->repair_root_fid == NULL) {
-				    sprintf((char *) data->out, "no repair session going on\n");
-				    u.u_error = ENOENT;
-				} else {
-				    FILE *fp = fopen(fpath, "w");
-				    if (fp == NULL) {
-					u.u_error = ENOENT;
-					sprintf((char *) data->out, "can not open %s\n", fpath);
-				    } else {
-					LRDB->ListCML(LRDB->repair_root_fid, fp);
-					sprintf((char *) data->out, "local mutations are:\n");
-					fflush(fp);
-					fclose(fp);
-					u.u_error = 0;
-				    }
-				}
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="setlocalview"><strong> setlocalview handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_LOCAL_VIEW:
-			    {
-				LRDB->SetSubtreeView(SUBTREE_LOCAL_VIEW, (char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="setglobalview"><strong> setglobalview handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_GLOBAL_VIEW:
-			    {
-				LRDB->SetSubtreeView(SUBTREE_GLOBAL_VIEW, (char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-/*
-  BEGIN_HTML
-  <a name="setmixedview"><strong> setmixedview handler </strong></a>
-  END_HTML
-*/
-			case REP_CMD_MIXED_VIEW:
-			    {
-				LRDB->SetSubtreeView(SUBTREE_MIXED_VIEW, (char *) data->out);
-				data->out_size = (short)strlen((char *) data->out) + 1;
-				u.u_error = 0;
-				break;
-			    }
-			default:
-			    eprint("bogus REP_CMD(%d)", rep_cmd);
-			}
-			break;
-		    }		    
+		    }
 		}
 	    }
 	    if (u.u_error == ERETRY)
