@@ -89,18 +89,6 @@ extern "C" {
 
 /* ***** VFS Operations  ***** */
 
-void vproc::mount(char *path, void *data) {
-    LOG(1, ("vproc::mount: path = %s\n", path));
-    u.u_error = EOPNOTSUPP;
-}
-
-
-void vproc::unmount() {
-    LOG(1, ("vproc::unmount\n"));
-    u.u_error = EOPNOTSUPP;
-}
-
-
 void vproc::root(struct venus_cnode *vpp) {
     LOG(1, ("vproc::root\n"));
 
@@ -120,8 +108,8 @@ void vproc::statfs(struct coda_statfs *sfs) {
 }
 
 
-void vproc::vget(struct venus_cnode *vpp, struct cfid *cfidp) {
-
+void vproc::vget(struct venus_cnode *vpp, struct cfid *cfidp, int what)
+{
     LOG(1, ("vproc::vget: fid = %s, nc = %x\n", FID_(&cfidp->cfid_fid),
 	    u.u_nc));
 
@@ -130,11 +118,21 @@ void vproc::vget(struct venus_cnode *vpp, struct cfid *cfidp) {
     if (u.u_nc && LogLevel >= 100)
 	u.u_nc->print(logFile);
 
+    /* return early if we're called to prefetch data */
+    if (type == VPT_Worker && (what & RC_DATA))
+    {
+	worker *w = (worker *)this;
+	union outputArgs *out = (union outputArgs *)w->msg->msg_buf;
+	out->coda_vget.Fid = *VenusToKernelFid(&cfidp->cfid_fid);
+	out->coda_vget.vtype = CDT_UNKNOWN;
+	w->Return(w->msg, sizeof(struct coda_vget_out));
+    }
+
     for (;;) {
 	Begin_VFS(&cfidp->cfid_fid, CODA_VGET);
 	if (u.u_error) break;
 
-	u.u_error = FSDB->Get(&f, &cfidp->cfid_fid, u.u_uid, RC_STATUS);
+	u.u_error = FSDB->Get(&f, &cfidp->cfid_fid, u.u_uid, what);
 	if (u.u_error) {
 	    if (u.u_error == EINCONS) {
 		u.u_error = 0;
@@ -304,7 +302,15 @@ void vproc::getattr(struct venus_cnode *cp, struct coda_vattr *vap)
 
 	/* Get the object. */
 	u.u_error = FSDB->Get(&f, &cp->c_fid, u.u_uid, RC_STATUS);
-	if (u.u_error) goto FreeLocks;
+
+	/* Check if it is in conflict. */
+	if(!u.u_error && f &&
+	   ((f->IsToBeRepaired() && !f->HasExpandedCMLEntries()) ||
+	    (f->IsFake() && !f->IsExpandedObj())))
+	  u.u_error = EINCONS;
+
+	if (u.u_error)
+	  goto FreeLocks;
 
 	/* No rights required to get attributes? -JJK */
 
@@ -320,11 +326,11 @@ FreeLocks:
 
     /* do we need to do this here? -JH */
     if (u.u_error == EINCONS) {
-	u.u_error = 0;
+        u.u_error = 0;
 
 	/* Make a "fake" vattr block for the inconsistent object. */
 	va_init(vap);
-	vap->va_mode = 0444;
+	vap->va_mode = 0644;
 	vap->va_type = FTTOVT(SymbolicLink);
 	vap->va_uid = V_UID;
 	vap->va_gid = V_GID;
@@ -401,14 +407,14 @@ void vproc::setattr(struct venus_cnode *cp, struct coda_vattr *vap) {
 	if (vap->va_size != 0 && vap->va_size != VA_IGNORE_SIZE)
 	    rcrights |= RC_DATA;
 
-	/* When we are write-disconnected, the setattr would 'dirty' the object
+	/* When the volume is reachable, the setattr would 'dirty' the object
 	 * and block a data fetch until the CML has been reintegrated. To avoid
 	 * having an inaccessible object we have to make sure to fetch the
 	 * data as well. */
 	volent *v = 0;
 	u.u_error = VDB->Get(&v, MakeVolid(&cp->c_fid));
 	if (u.u_error) goto FreeLocks;
-	if (v->IsWriteDisconnected())
+	if (v->IsReachable())
 	    rcrights |= RC_DATA;
 	VDB->Put(&v);
 
@@ -416,8 +422,10 @@ void vproc::setattr(struct venus_cnode *cp, struct coda_vattr *vap) {
 	u.u_error = FSDB->Get(&f, &cp->c_fid, u.u_uid, rcrights);
 	if (u.u_error) goto FreeLocks;
 
-	/* Symbolic links are immutable. */
-	if (f->IsSymLink() || f->IsMtPt())
+	/* Symbolic links are immutable, but we should allow mtime changes. */
+	if ((f->IsSymLink() || f->IsMtPt()) &&
+	    (vap->va_mode != VA_IGNORE_MODE || vap->va_uid != VA_IGNORE_UID ||
+	     vap->va_size != VA_IGNORE_SIZE))
 	    { u.u_error = EINVAL; goto FreeLocks; }
 
 	/* Permission checks. */
@@ -441,11 +449,28 @@ void vproc::setattr(struct venus_cnode *cp, struct coda_vattr *vap) {
 		if (f->stat.Owner != (uid_t)vap->va_uid)
 		    { u.u_error = EACCES; goto FreeLocks; }
 #endif
+		/* returning EACCESS seems more correct here, but this actually
+		 * happens often enough to be annoying. Programs often create a
+		 * file, write some data, fchown, fchmod and close it last.
+		 * Getting an EACCESS on the chown can be a little disturbing,
+		 * especially since Coda in most ways really doesn't care about
+		 * ownership, and happily resets it to the 'author' of the file
+		 * when the file is closed... */
+		if (f->IsVirgin()) {
+		    u.u_error = 0; /* EACCESS */
+		    goto FreeLocks;
+		}
+
 		u.u_error = f->Access(PRSFS_ADMINISTER, C_A_F_OK, u.u_uid);
 		if (u.u_error) goto FreeLocks;
 	    }
 
 	    /* truncate, ftruncate */
+
+	    /* cancel out resize to existing length */
+	    if (vap->va_size != VA_IGNORE_SIZE && vap->va_size == f->Size())
+		vap->va_size = VA_IGNORE_SIZE;
+
 	    if (vap->va_size != VA_IGNORE_SIZE) {
 		if (!f->IsFile()) {
 		    u.u_error = EISDIR; 
@@ -603,7 +628,8 @@ void vproc::lookup(struct venus_cnode *dcp, char *name,
 
 	/* Set OUT parameter. */
 	MAKE_CNODE2(*cp, target_fso->fid, FTTOVT(target_fso->stat.VnodeType));
-	if (target_fso->IsFake() || target_fso->IsMTLink())
+	if (target_fso->IsToBeRepaired() || target_fso->IsFake()
+	    || target_fso->IsMTLink())
 	    cp->c_flags |= C_INCON;
 
 FreeLocks:
@@ -656,13 +682,6 @@ void vproc::create(struct venus_cnode *dcp, char *name, struct coda_vattr *vap,
 	if (!parent_fso->IsDir())
 	    { u.u_error = ENOTDIR; goto FreeLocks; }
 
-	/* Get the target object (if it exists). */
-	if (LRDB->repair_root_fid &&
-	    (parent_fso->IsLocalObj() ||
-	     LRDB->RFM_IsRootParent(&parent_fso->fid)))
-	    /* cross mount-point when under local/global repair */
-	    flags |= CLU_TRAVERSE_MTPT;
-
 	u.u_error = parent_fso->Lookup(&target_fso, NULL, name, u.u_uid, flags);
 	if (u.u_error == 0) {
 	    FSDB->Put(&parent_fso);	    /* avoid deadlock! */
@@ -961,18 +980,24 @@ void vproc::rename(struct venus_cnode *spcp, char *name,
 	 */
 	{
 	    fsobj *f = (SameParent ? t_parent_fso : s_parent_fso);
-#ifdef DJGPP
-	    int flags = CLU_CASE_INSENSITIVE;
-#else
 	    int flags = CLU_CASE_SENSITIVE;
-#endif
 	    u.u_error = f->Lookup(&s_fso, NULL, name, u.u_uid, flags);
 	    if (u.u_error) goto FreeLocks;
 
 	    if (s_fso->IsMtPt() || s_fso->IsMTLink())
 		{ u.u_error = EINVAL; goto FreeLocks; }
 
-	    if (s_fso->IsDir()) {
+	    /* Lookup only makes sure the status is cached, the rename will
+	     * dirty the object because it changes the parent pointer. We
+	     * cannot fetch data for dirty objects which prevents us from
+	     * accessing it until the rename has been reintegrated, To avoid
+	     * the access problem we either have to allow fetching data for
+	     * dirty objects (hard, because we aren't sure why the object is
+	     * considered dirty and right now a fetch would clobber the cached
+	     * status or we have to make sure we have cached data before the
+	     * object is renamed. */
+	    //if (s_fso->IsDir())
+	    {
 		VenusFid s_fid = s_fso->fid;
 		FSDB->Put(&s_fso);
 
@@ -1198,10 +1223,15 @@ void vproc::rmdir(struct venus_cnode *dcp, char *name)
 	if (target_fso->IsRoot())
 	    { target_fso->print(logFile); CHOKE("vproc::rmdir: target is root"); }
 
-	/* Verify that it is a directory (mount points are an exception). */
-	if (!target_fso->IsDir() || target_fso->IsMtPt())
+	/* Verify that it is a directory */
+	if (!target_fso->IsDir())
 	    { u.u_error = ENOTDIR; goto FreeLocks; }
 
+	/* Mount points have to be removed by an administrator with
+	 * cfs rmmount */
+	if (target_fso->IsMtPt())
+	    { u.u_error = EPERM; goto FreeLocks; }
+
 	/* Must have data for the target. */
 	VenusFid target_fid; target_fid = target_fso->fid;
 	FSDB->Put(&target_fso);
@@ -1316,10 +1346,15 @@ void vproc::readlink(struct venus_cnode *cp, struct coda_string *string)
 
 	/* Get the object. */
 	u.u_error = FSDB->Get(&f, &cp->c_fid, u.u_uid, RC_DATA);
+
+	if(!u.u_error && f &&
+	   ((f->IsToBeRepaired() && !f->HasExpandedCMLEntries()) ||
+	    (f->IsFake() && !f->IsExpandedObj())))
+	  u.u_error = EINCONS;
 	if (u.u_error) goto FreeLocks;
 
 	/* Verify that it is a symlink. */
-	if (!f->IsSymLink())
+	if (f && !f->IsSymLink())
 	    { u.u_error = EINVAL; goto FreeLocks; }
 
 	/* Verify that we have read permission for it. */
