@@ -38,7 +38,7 @@ extern "C" {
 #include <sys/stat.h>
 #include <netinet/in.h>
 
-#if !defined(__CYGWIN32__) && !defined(DJGPP)
+#ifndef __CYGWIN32__
 #include <sys/syscall.h>
 #include <sys/mount.h>
 #endif
@@ -71,10 +71,6 @@ extern "C" {
 
 #include <vice.h>
 
-#ifdef DJGPP
-#include <relay.h>
-#endif
-
 #ifdef __CYGWIN__
 #include <windows.h>
 #endif
@@ -84,7 +80,7 @@ extern "C" {
 #endif
 
 /* Darwin uses the same venus-kernel interface as BSD */
-#if defined(__APPLE__) && defined(__MACH__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || (defined(__APPLE__) && defined(__MACH__))
 #define __BSD44__
 #endif
 
@@ -101,6 +97,7 @@ extern "C" {
 #include "worker.h"
 
 #include "nt_util.h"
+#include "getpeereid.h"
 
 
 extern int venus_relay_addr;
@@ -116,10 +113,8 @@ olist worker::FreeMsgs;
 olist worker::QueuedMsgs;
 olist worker::ActiveMsgs;
 
-#ifdef VENUSDEBUG
 int msgent::allocs = 0;
 int msgent::deallocs = 0;
-#endif
 
 const int WorkerStackSize = 131072;
 
@@ -159,70 +154,102 @@ msgent *FindMsg(olist& ol, u_long seq) {
 }
 
 
-int MsgRead(msgent *m) 
+msgent::msgent()
 {
-#ifdef DJGPP
-	size_t cc = read_relay(m->msg_buf);
-
-#elif defined(__CYGWIN32__)
-        DWORD size;  
-        size_t cc = read(worker::muxfd, (char *)&size, sizeof(size)); 
-	CODA_ASSERT(size <= VC_MAXMSGSIZE);
-	cc = read(worker::muxfd, m->msg_buf, (int)size);
-#else
-	size_t cc = read(worker::muxfd, m->msg_buf, (int) (VC_MAXMSGSIZE));
-#endif
-	if (cc < sizeof(struct coda_in_hdr)) 
-		return(-1);
-
-	return(0);
-}
-
-
-size_t MsgWrite(char *buf, int size) 
-{
-#ifdef DJGPP
-	 return write_relay(buf, size);
-#elif defined(__CYGWIN32__)
-	 return nt_msg_write(buf, size);
-#else 
-	return write(worker::muxfd, buf, size);
-#endif
-}
-
-
-msgent::msgent() 
-{
-#ifdef VENUSDEBUG
     allocs++;
-#endif
 }
 
 
-msgent::~msgent() 
+msgent::~msgent()
 {
-#ifdef VENUSDEBUG
     deallocs++;
-#endif
 }
 
 
-msg_iterator::msg_iterator(olist& ol) : olist_iterator(ol) 
+msg_iterator::msg_iterator(olist& ol) : olist_iterator(ol)
 {
 }
 
 
-msgent *msg_iterator::operator()() 
+msgent *msg_iterator::operator()()
 {
     return((msgent *)olist_iterator::operator()());
 }
 
+msgent *AllocMsgent(void)
+{
+    msgent *m = (msgent *)worker::FreeMsgs.get();
+    if (!m) m = new msgent;
+    m->return_fd = -1;
+    return m;
+}
+
+/* read a msg from the given socket */
+void ReadUpcallMsg(int fd, size_t size)
+{
+    msgent *m = AllocMsgent();
+    ssize_t len;
+    uid_t euid = 0;
+    gid_t egid;
+    int rc;
+
+    CODA_ASSERT(size <= VC_MAXMSGSIZE);
+    len = read(fd, m->msg_buf, size);
+
+    if (len < (ssize_t)sizeof(struct coda_in_hdr))
+    {
+	eprint("Failed to read upcall");
+	worker::FreeMsgs.append(m);
+	return;
+    }
+
+    if (fd != worker::muxfd) {
+	rc = getpeereid(fd, &euid, &egid);
+	if (rc) {
+	    eprint("Unable to check peer credentials");
+	    /* disconnect peer? */
+	}
+
+	if (euid != 0) { /* root can already do whatever it wants */
+	    ((union inputArgs *)m->msg_buf)->ih.uid = euid;
+	}
+    }
+
+    m->return_fd = fd;
+    DispatchWorker(m);
+}
+
+ssize_t WriteDowncallMsg(int fd, const char *buf, size_t size)
+{
+    if (fd == -1)
+	return size;
+
+    CODA_ASSERT(size <= VC_MAXMSGSIZE);
+    if (fd != worker::muxfd) {
+	char hdr[11];
+	int len;
+	len = sprintf(hdr, "down: %d\n", size);
+	len -= write(fd, hdr, len);
+	CODA_ASSERT(len == 0);
+    }
+#ifdef __CYGWIN32__
+    else
+	return nt_msg_write(buf, size);
+#endif
+    return write(fd, buf, size);
+}
+
+ssize_t MsgWrite(const char *buf, size_t size)
+{
+    return WriteDowncallMsg(worker::muxfd, buf, size);
+}
+
 
 /* test if we can open the kernel device and purge the cache,
    BSD systems like to purge that cache */
 void testKernDevice() 
 {
-#if defined(DJGPP) || defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
 	return;
 #else
 	int fd = -1;
@@ -361,8 +388,24 @@ void VFSMount()
 	    goto child_done;
 
 #ifdef __BSD44__ /* BSD specific mount */
+	error = -1;
 	/* Issue the VFS mount request. */
-	error = mount("coda", venusRoot, 0, kernDevice);
+
+#ifdef HAVE_NMOUNT
+	{ /* FreeBSD-6.x introduced a new mount syscall */
+	    struct iovec md[6];
+	    md[0].iov_base = (char *)"fstype"; md[0].iov_len = sizeof("fstype");
+	    md[1].iov_base = (char *)"coda";   md[1].iov_len = sizeof("coda");
+	    md[2].iov_base = (char *)"fspath"; md[2].iov_len = sizeof("fspath");
+	    md[3].iov_base = venusRoot;        md[3].iov_len = strlen(venusRoot) + 1;
+	    md[4].iov_base = (char *)"from";   md[4].iov_len = sizeof("from");
+	    md[5].iov_base = kernDevice;       md[5].iov_len = strlen(kernDevice) + 1;
+	    error = nmount(md, 6, 0);
+	}
+#endif
+			
+	if (error < 0)
+	    error = mount("coda", venusRoot, 0, kernDevice);
 	if (error < 0)
 	    error = mount("cfs", venusRoot, 0, kernDevice);
 #if defined(__FreeBSD__) && !defined(__FreeBSD_version)
@@ -385,11 +428,11 @@ void VFSMount()
 	if (!error) {
 	    FILE *fd = setmntent("/etc/mtab", "a");
 	    struct mntent ent;
-	    if (fd) { 
-		ent.mnt_fsname = "coda";
-		ent.mnt_dir    = venusRoot;
-		ent.mnt_type   = "coda";
-		ent.mnt_opts   = "rw,noatime,nosuid,nodev";
+	    if (fd) {
+		ent.mnt_fsname = (char *)"coda";
+		ent.mnt_dir    = (char *)venusRoot;
+		ent.mnt_type   = (char *)"coda";
+		ent.mnt_opts   = (char *)"rw,noatime,nosuid,nodev";
 		ent.mnt_freq   = 0;
 		ent.mnt_passno = 0;
 		addmntent(fd, &ent);
@@ -469,19 +512,6 @@ child_done:
     }
 #endif
 
-#ifdef DJGPP
-    int res;
-    eprint ("Mounting on %s", venusRoot);
-    res = mount_relay(venusRoot);
-    if (res)
-	    eprint("Mount OK");
-    else{
-	    eprint ("Mount failed");
-	    close_relay();
-	    exit(0);
-    }
-#endif
-
 #ifdef __CYGWIN32__
     /* Mount by starting another thread. */
     eprint ("Mounting on %s", venusRoot);
@@ -491,41 +521,8 @@ child_done:
     Mounted = 1;
 }
 
-int VFSUnload()
+void VFSUnmount()
 {
-#ifdef DJGPP
-  	int i = 0, res;
-	do {
-		i++;
-		eprint ("UNLOAD result %d.", res = unload_vxd("CODADEV"));    
-    
-	} while (i < 10 && res == 0);
-	if (i==10 && res ==0) return -1;
-#endif
-	return 0;
-}
-
-void VFSUnmount() 
-{
-#ifdef DJGPP
-    int res;
-    
-    if (!Mounted) return;
-
-    res = unmount_relay();
-    eprint (res ? "Unmount OK" : "Unmount failed");
-
-    res = close_relay();
-    eprint (res ? "Close relay OK" : "Close relay failed");
-
-    res = VFSUnload();
-    eprint (res == 0 ? "Kernel module unloaded" :
-	               "Kernel module could not be unloaded");
-
-    if (res == 0) KernelFD = -1;	
-
-    return;
-#endif
     /* Purge the kernel cache so that all cnodes are (hopefully) released. */
     k_Purge();
 
@@ -610,7 +607,7 @@ void VFSUnmount()
 
 
 int k_Purge() {
-    size_t size;
+    ssize_t size;
 
     if (KernelFD == -1) return(1);
 
@@ -636,7 +633,7 @@ int k_Purge() {
 
 
 int k_Purge(VenusFid *fid, int severely) {
-    size_t size;
+    ssize_t size;
 
     if (KernelFD == -1) return(1);
 
@@ -666,7 +663,7 @@ int k_Purge(VenusFid *fid, int severely) {
     }	
 
     /* Send the message. */
-    if (MsgWrite((char *)&msg, (int) size) != size) {
+    if (MsgWrite((char *)&msg, size) != size) {
 	retcode = errno;
 	LOG(0, ("k_Purge: %s, message write fails: errno %d\n", 
 	      msg.oh.opcode == CODA_PURGEFID ? "CODA_PURGEFID" :
@@ -690,7 +687,7 @@ int k_Purge(VenusFid *fid, int severely) {
 
 int k_Purge(uid_t uid)
 {
-    size_t size;
+    ssize_t size;
 
     if (KernelFD == -1) return(1);
 
@@ -735,7 +732,8 @@ int k_Replace(VenusFid *fid_1, VenusFid *fid_2) {
     msg.NewFid = *VenusToKernelFid(fid_2);
 	
     /* Send the message. */
-    if (MsgWrite((char *)&msg, sizeof (struct coda_replace_out)) != sizeof (struct coda_replace_out))
+    ssize_t size = sizeof(struct coda_replace_out);
+    if (MsgWrite((char *)&msg, size) != size)
 	CHOKE("k_Replace: message write");
 
     LOG(0, ("k_Replace: returns 0\n"));
@@ -759,14 +757,7 @@ void WorkerInit()
             exit(-1);
         }
 
-#ifdef DJGPP
-    if (!init_relay()){
-	    LOG(0, ("init_relay failed.\n"));
-	    exit(-1);
-    }
-    worker::muxfd = MCFD;
-    dprint("WorkerInit: muxfd = %d\n", worker::muxfd);
-#elif defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
     int sd[2];
     if (socketpair(AF_LOCAL, SOCK_STREAM, 0, sd)) {
 	    eprint("WorkerInit: socketpair() returns %d", errno);
@@ -828,7 +819,12 @@ int WorkerCloseMuxfd(void)
     if (KernelFD)
 	ret = close(worker::muxfd);
 
-    worker::muxfd = KernelFD = -1;
+    /* just in case we still have a parent process waiting for us we don't want
+     * to lock up the boot sequence... */
+    if (parent_fd != -1)
+	close(parent_fd);
+
+    worker::muxfd = KernelFD = parent_fd = -1;
 
     return ret;
 }
@@ -928,22 +924,23 @@ void DispatchWorker(msgent *m) {
     worker::QueuedMsgs.append(m);
 }
 
-
 void WorkerMux(fd_set *mask)
 {
-    /* Get a free buffer and read a message from the kernel into it. */
-    msgent *fm = (msgent *)worker::FreeMsgs.get();
-    if (!fm) fm = new msgent;
-    if (MsgRead(fm) < 0) {
-	eprint("WorkerMux: worker read error");
-	worker::FreeMsgs.append(fm);
-	return;
-    }
+    size_t size = VC_MAXMSGSIZE;
 
-    DispatchWorker(fm);
+#ifdef __CYGWIN32__
+    /* CYGWIN uses a stream oriented socket for kernel messages, there are no
+     * message boundaries and we need to read the size of the message first. */
+    DWORD msg_size;
+    ssize_t len;
+    len = read(worker::muxfd, (char *)&msg_size, sizeof(msg_size));
+    CODA_ASSERT(len == sizeof(msg_size));
+    size = msg_size;
+#endif
+
+    ReadUpcallMsg(worker::muxfd, size);
 }
 
-
 time_t GetWorkerIdleTime() {
     /* Return 0 if any call is in progress. */
     worker_iterator next;
@@ -1084,7 +1081,10 @@ void worker::Resign(msgent *msg, int size) {
 }
 
 
-void worker::Return(msgent *msg, size_t size) {
+void worker::Return(msgent *msg, size_t size)
+{
+    size_t cc;
+
     if (returned)
 	CHOKE("worker::Return: already returned!");
 
@@ -1106,22 +1106,24 @@ void worker::Return(msgent *msg, size_t size) {
 #endif
 
     /* There is no reply to an interrupted operation. */
-    if (!interrupted) {
-	size_t cc = MsgWrite(msg->msg_buf, size);
-	int errn = errno;
-	if (cc != size) {
-	    eprint("worker::Return: message write error %d (op = %d, seq = %d), wrote %d of %d bytes\n",
-		   errno, ((union outputArgs*)msg->msg_buf)->oh.opcode,
-		   ((union outputArgs*)msg->msg_buf)->oh.unique, cc, size);  
+    if (interrupted)
+	goto out;
 
-	    /* Guard against a race in which the kernel is signalling us, but we entered this */
-	    /* block before the signal reached us.  In this case the error code from the MsgWrite */
-	    /* will be ESRCH.  No other error code is legitimate. */
-	    if (errn != ESRCH) CHOKE("worker::Return: errno (%d) from MsgWrite", errno);
-	    interrupted = 1;
-	}
+    cc = WriteDowncallMsg(msg->return_fd, msg->msg_buf, size);
+    if (cc != size) {
+	int err = errno;
+	eprint("worker::Return: message write error %d (op = %d, seq = %d), wrote %d of %d bytes\n",
+	       errno, ((union outputArgs*)msg->msg_buf)->oh.opcode,
+	       ((union outputArgs*)msg->msg_buf)->oh.unique, cc, size);
+
+	/* Guard against a race in which the kernel is signalling us, but we
+	 * entered this block before the signal reached us. In this case the
+	 * error code from WriteDowncallMsg will be ESRCH. No other error code
+	 * is legitimate. */
+	if (err != ESRCH) CHOKE("worker::Return: errno (%d) from WriteDowncallMsg",err);
+	interrupted = 1;
     }
-
+out:
     returned = 1;
 }
 
@@ -1142,6 +1144,7 @@ void worker::main(void)
 
     for (;;) {
 	openfd = -1;
+
 	/* Wait for new request. */
 	AwaitRequest();
 
@@ -1229,7 +1232,7 @@ void worker::main(void)
 		unsigned char type = _IOC_TYPE(cmd);
 		unsigned char nr = _IOC_NR(cmd);
 
-		data.in = (char *)in + (int)in->coda_ioctl.data;
+		data.in = (char *)in + (intptr_t)in->coda_ioctl.data;
 		data.in_size = 0;
 		data.out = outbuf;	/* Can't risk overcopying. Sigh. -dcs */
 		data.out_size =	0;
@@ -1274,7 +1277,7 @@ void worker::main(void)
 
 		out->coda_ioctl.len = data.out_size;
 		out->coda_ioctl.data = (char *)(sizeof (struct coda_ioctl_out));
-		memcpy((char *)out + (int)out->coda_ioctl.data, data.out, data.out_size);
+		memcpy((char *)out + (intptr_t)out->coda_ioctl.data, data.out, data.out_size);
 		if (nr == _VIOCPREFETCH)
 		    worker::nprefetchers--;
 
@@ -1372,7 +1375,7 @@ void worker::main(void)
 	    case CODA_OPEN_BY_PATH:
 		{
 		LOG(100, ("CODA_OPEN_BY_PATH: u.u_pid = %d u.u_pgid = %d\n", u.u_pid, u.u_pgid)); 
-#if defined(DJGPP) || defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
 		char *slash;
 #endif
                 /* Remember some info for dealing with interrupted open calls */
@@ -1390,7 +1393,7 @@ void worker::main(void)
                     out->coda_open_by_path.path = begin - (char *)out;
                     sprintf(begin, "%s%s/%s", CachePrefix, CacheDir, 
                             vtarget.c_cf->Name());
-#if defined(DJGPP) || defined(__CYGWIN32__)
+#ifdef __CYGWIN32__
                     slash = begin;
                     for (slash = begin ; *slash ; slash++ ) {
                         if ( *slash == '/' ) 
@@ -1486,7 +1489,7 @@ void worker::main(void)
 		struct cfid fid;
 		fid.cfid_len = (unsigned short)sizeof(VenusFid);
 		KernelToVenusFid(&fid.cfid_fid, &in->coda_vget.Fid);
-		vget(&vtarget, &fid);
+		vget(&vtarget, &fid, RC_DATA);
 
 		if (u.u_error == 0) {
 		    out->coda_vget.Fid = *VenusToKernelFid(&vtarget.c_fid);
@@ -1519,6 +1522,7 @@ void worker::main(void)
         out->oh.result = u.u_error;
         Resign(msg, size);
 
+	/* extra case added to deal with dangling references to a collapse */
         if (opcode == CODA_OPEN_BY_FD && openfd != -1)
 	    vtarget.c_cf->Close(openfd);
 
@@ -1538,11 +1542,10 @@ void worker::main(void)
                  * harm done, I guess. But why not just call close directly?
                  * -- DCS */
                 /* Fashion a CLOSE message. */
-                msgent *fm = (msgent *)worker::FreeMsgs.get();
-                if (!fm) fm = new msgent;
+                msgent *fm = AllocMsgent();
                 union inputArgs *dog = (union inputArgs *)fm->msg_buf;
 
-                dog->coda_close.ih.unique = (u_long)-1;
+                dog->coda_close.ih.unique = (unsigned)-1;
                 dog->coda_close.ih.opcode = CODA_CLOSE;
                 dog->coda_close.Fid = saveFid;
                 dog->coda_close.flags = saveFlags;
