@@ -46,22 +46,23 @@ extern "C" {
 #include "fso.h"
 #include "vproc.h"
 
-int WorkerCloseMuxfd(void);
-
 class msgent;
 class msg_iterator;
 class worker;
 class worker_iterator;
 
+int WorkerCloseMuxfd(void);
+void ReadUpcallMsg(int fd, size_t size);
+
 const int DFLT_MAXWORKERS = 20;
 const int UNSET_MAXWORKERS = -1;
 const int DFLT_MAXPREFETCHERS = 1;
 
 class msgent : public olink {
   friend msgent *FindMsg(olist&, u_long);
-  friend int MsgRead(msgent *);
-  friend size_t MsgWrite(char *, int);
   friend worker *FindWorker(u_long);
+  friend msgent *AllocMsgent(void);
+  friend void ReadUpcallMsg(int fd, size_t size);
   friend void DispatchWorker(msgent *);
   friend int IsAPrefetch(msgent *);
   friend class worker;
@@ -74,14 +75,14 @@ class msgent : public olink {
   friend void WorkerMux(fd_set *mask);
 
     char msg_buf[VC_MAXMSGSIZE];
-    msgent();
-    virtual ~msgent();
+    int return_fd;
 
   public:
-#ifdef	VENUSDEBUG
+    msgent();
+    ~msgent();
+
     static int allocs;
     static int deallocs;
-#endif /* VENUSDEBUG */
 };
 
 class msg_iterator : public olist_iterator {
@@ -96,11 +97,13 @@ class worker : public vproc {
   friend worker *FindWorker(u_long);
   friend worker *GetIdleWorker();
   friend void DispatchWorker(msgent *);
+  friend msgent *AllocMsgent(void);
+  friend void ReadUpcallMsg(int fd, size_t size);
+  friend ssize_t WriteDowncallMsg(int fd, const char *buf, size_t size);
+  friend ssize_t MsgWrite(const char *msg, size_t size);
   friend void WorkerMux(fd_set *mask);
   friend time_t GetWorkerIdleTime();
   friend void PrintWorkers(int);
-  friend int MsgRead(msgent *);
-  friend size_t MsgWrite(char *, int);
   friend int WorkerCloseMuxfd(void);
   friend void VFSMount();
   friend class vproc;
@@ -148,15 +151,12 @@ extern int KernelFD;
 
 
 extern msgent *FindMsg(olist&, u_long);
-extern int MsgRead(msgent *);
-extern size_t MsgWrite(char *, int);
 extern int k_Purge();
 extern int k_Purge(VenusFid *, int =0);
 extern int k_Purge(uid_t);
 extern int k_Replace(VenusFid *, VenusFid *);
 extern void VFSMount();
 extern void VFSUnmount();
-extern int VFSUnload();
 extern void WorkerInit();
 extern worker *FindWorker(u_long);
 extern worker *GetIdleWorker();
