@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -278,13 +278,9 @@ Exit:
     return(errorCode);
 }
 
-/*
-  ViceResolve: Resolve the diverging replicas of an object
-*/
-long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid) 
+static long ViceResolveOne(vrent *vre, int reson, ViceFid *Fid, DirFid *HintFid)
 {
     int errorCode = 0;
-    VolumeId VSGVolnum;
     ViceVersionVector VV;
     ResStatus rstatus;
     unsigned long hosts[VSG_MEMBERS];
@@ -292,17 +288,9 @@ long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
     long resError = 0;
     long lockerror = 0;
     int logsizes = 0;
-    int reson = 0;
-    VolumeId tmpvid = Fid->Volume;
     int j;
 
-    if ( !Fid ) { 
-	    SLog(0, "ViceResolve: I was handed NULL Fid");
-	    CODA_ASSERT(0);
-    }
-
-    SLog(2,  "ViceResolve(%d, %s)", cid, FID_(Fid));
-    VSGVolnum = Fid->Volume;
+    SLog(2,  "ViceResolveOne(%s)", FID_(Fid));
 
     if (pathtiming && probingon && (!(ISDIR(*Fid)))) {
 	    FileresTPinfo = new timing_path(MAXPROBES);
@@ -310,20 +298,7 @@ long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
     }
 
     /* get an mgroup */
-    vrent *vre = VRDB.find(VSGVolnum);
-    if (!vre) {
-	    SLog(0, "ViceResolve: Can't find replicated volume (%s)", FID_(Fid));
-	    errorCode = EINVAL;
-	    goto FreeGroups;
-    }
     vre->GetHosts(hosts);
-
-    if (!XlateVid(&tmpvid)) {
-	    SLog(0,  "ViceResolve: Couldn't xlate vid %x", tmpvid);
-	    errorCode = EINVAL;
-	    goto FreeGroups;
-    }
-    reson = GetResFlag(tmpvid);
     if (GetResMgroup(&mgrp, hosts)){
 	    /* error getting mgroup */
 	    errorCode = EINVAL;
@@ -338,49 +313,51 @@ long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
 
     for (j = 0; j < VSG_MEMBERS; j++) {
 	    RPC2_Handle Handle = mgrp->rrcc.handles[j];
-	    RPC2_Integer rc; 
+	    RPC2_Integer rc;
 
-	    if ( !mgrp->rrcc.hosts[j] ) 
+	    if ( !mgrp->rrcc.hosts[j] )
 		    continue;
 
 	    rc = Res_LockAndFetch(Handle, Fid, FetchStatus, VVvar_ptrs[j],
 				  rstatusvar_ptrs[j], logsizesvar_ptrs[j]);
 	    mgrp->rrcc.retcodes[j] = rc;
     }
-				    
-    // delete hosts from mgroup where rpc failed 
+
+    // delete hosts from mgroup where rpc failed
     (void)mgrp->CheckResult();
 
-    // delete hosts from mgroup where rpc succeeded but call returned error 
+    // delete hosts from mgroup where rpc succeeded but call returned error
     lockerror = CheckResRetCodes(mgrp->rrcc.retcodes, mgrp->rrcc.hosts, hosts);
     errorCode = mgrp->GetHostSet(hosts);
 
-    // call resolve on object  only if no locking errors 
-    if ( errorCode ) 
-	    goto UnlockExit;
-    if ( lockerror )
+    // call resolve on object, only if no locking errors
+    if ( errorCode || lockerror )
 	    goto UnlockExit;
 
     if (ISDIR(*Fid)) {
 	    SLog(9,  "ViceResolve: Going to call DirResolve");
 	    if (reson) {
-                resError = RecovDirResolve(mgrp, Fid, VVvar_ptrs,
-                                           rstatusvar_ptrs,
-                                           (int *)logsizesvar_bufs);
-            } else {
-                resError = OldDirResolve(mgrp, Fid, VVvar_ptrs);
+		resError = RecovDirResolve(mgrp, Fid, VVvar_ptrs,
+					   rstatusvar_ptrs,
+					   (int *)logsizesvar_bufs, HintFid);
+	    } else {
+		resError = OldDirResolve(mgrp, Fid, VVvar_ptrs);
 	    }
     } else {
 	    SLog(9, "ViceResolve: Going to call Fileresolve");
 	    resError = FileResolve(mgrp, Fid, VVvar_ptrs);
     }
 
-    
+
  UnlockExit:
-    // reget the host set - want to unlock wherever we locked volume 
-    vre->GetHosts(hosts);
+    /* reget the host set - want to unlock wherever we locked the volume */
+    /* but we shouldn't unlock if there was a lock error, we might otherwise
+     * unlock a lock we already had taken from another worker thread */
+    if (!lockerror) {
+	vre->GetHosts(hosts);
+    }
     mgrp->GetHostSet(hosts);
-    MRPC_MakeMulti(UnlockVol_OP, UnlockVol_PTR, VSG_MEMBERS, 
+    MRPC_MakeMulti(UnlockVol_OP, UnlockVol_PTR, VSG_MEMBERS,
 		   mgrp->rrcc.handles, mgrp->rrcc.retcodes,
 		   mgrp->rrcc.MIp, 0, 0, Fid->Volume);
 
@@ -389,8 +366,8 @@ long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
 	PutResMgroup(&mgrp);
     PROBE(FileresTPinfo, COORDENDVICERESOLVE);
     if (lockerror) {
-	    SLog(0,  "ViceResolve:Couldnt lock volume %x at all accessible servers",
-		 Fid->Volume);
+	    SLog(0, "ViceResolve: Couldnt lock volume %x at all accessible "
+		    "servers", Fid->Volume);
 	if ( lockerror != VNOVNODE ) 
 	    lockerror = EWOULDBLOCK;
 	return(lockerror);
@@ -404,6 +381,52 @@ long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
     return(resError);
 }
 
+/*
+  ViceResolve: Resolve the diverging replicas of an object
+*/
+#define MAX_HINTS 5
+long FS_ViceResolve(RPC2_Handle cid, ViceFid *Fid)
+{
+    DirFid hints[MAX_HINTS];
+    int i = 1, j;
+    long err;
+
+    int reson = GetResFlag(Fid->Volume);
+    vrent *vre = VRDB.find(Fid->Volume);
+    if (!vre) {
+	SLog(0, "ViceResolve: Can't find replicated volume (%s)", FID_(Fid));
+	return EINVAL;
+    }
+
+    memset(hints, 0, MAX_HINTS * sizeof(DirFid));
+    FID_VFid2DFid(Fid, &hints[0]);
+    while (i < MAX_HINTS)
+    {
+	err = ViceResolveOne(vre, reson, Fid, &hints[i]);
+
+	/* loop avoidance */
+	for (j = 0; j < i; j++)
+	    if (hints[i].Vnode  == hints[j].Vnode &&
+		hints[i].Unique == hints[j].Unique)
+		break;
+
+	if (err != EINCONS || j != i || !(hints[i].Vnode || hints[i].Unique))
+	    break;
+
+	FID_DFid2VFid(&hints[i++], Fid);
+    }
+    /* If the last resolve was successful (or a total loss), there is no
+     * need to re-resolve the last object */
+    if (!err || err == VNOVNODE) i--;
+
+    while (i-- > 0)
+    {
+	FID_DFid2VFid(&hints[i], Fid);
+	err = ViceResolveOne(vre, reson, Fid, NULL);
+    }
+    return err;
+}
+
 /*
   BEGIN_HTML
   <a name="ViceSetVV"><strong>Sets the version vector for an object</strong></a> 
@@ -510,14 +533,14 @@ long FS_ViceRepair(RPC2_Handle cid, ViceFid *Fid, ViceStatus *status,
 	   but vnode with a  READLOCK because later on we 
 	   will get it with a WRITELOCK */
 	ov = AddVLE(*vlist, Fid);
-	if ( ISDIR(*Fid) ) 
+	if ( ISDIR(*Fid) )
 		ov->d_inodemod = 1;
-	errorCode = GetFsObj(Fid, &volptr, &ov->vptr, 
-			     READ_LOCK, SHARED_LOCK, 
+	errorCode = GetFsObj(Fid, &volptr, &ov->vptr,
+			     READ_LOCK, SHARED_LOCK,
 			     1, 0, ov->d_inodemod);
-	if ( errorCode ) 
+	if ( errorCode )
 		goto FreeLocks;
-	
+
 	volindex = V_volumeindex(volptr);
 	if (ov->vptr->disk.type == vFile ||
 	    ov->vptr->disk.type == vSymlink)
@@ -662,12 +685,12 @@ static int PerformFileRepair(vle *ov, Volume *volptr, VolumeId VSGVolnum,
 	CodaBreakCallBack(0, Fid, VSGVolnum);
     }
 
-    if (ov->vptr->disk.inodeNumber)
-	ov->f_sinode = ov->vptr->disk.inodeNumber;
-    
+    if (ov->vptr->disk.node.inodeNumber)
+	ov->f_sinode = ov->vptr->disk.node.inodeNumber;
+
     {
 	Vnode *vptr = ov->vptr;
-	vptr->disk.inodeNumber = ov->f_finode;
+	vptr->disk.node.inodeNumber = ov->f_finode;
 	vptr->disk.cloned = 0;	// invoke COW - added 9/30/92 - Puneet Kumar
 	vptr->disk.length = status->Length;
 	vptr->disk.owner = status->Owner;
@@ -695,7 +718,7 @@ int GetMyRepairList(ViceFid *Fid, struct listhdr *replicaList, int
     int	i;
     int found = -1;
     for (i = 0; i < replicaCount; i++)
-	if (memcmp((const void *)&(replicaList[i].replicaId), (const void *) &(Fid->Volume), (int)sizeof(VolumeId)))
+	if (memcmp((const void *)&(replicaList[i].replicaFid.Volume), (const void *) &(Fid->Volume), (int)sizeof(VolumeId)))
 	    continue;
 	else
 	    /* found an entry */
@@ -1300,12 +1323,12 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 
 		/* create the inode */
 		cv->vptr->disk.dataVersion = 1;
-		cv->f_finode = icreate((int)V_device(volptr), (int)V_id(volptr),
-				       (int)cv->vptr->vnodeNumber, 
-				       (int)cv->vptr->disk.uniquifier,
-				       (int)cv->vptr->disk.dataVersion);
+		cv->f_finode = icreate(V_device(volptr), V_id(volptr),
+				       cv->vptr->vnodeNumber,
+				       cv->vptr->disk.uniquifier,
+				       cv->vptr->disk.dataVersion);
 		CODA_ASSERT(cv->f_finode > 0);
-		cv->vptr->disk.inodeNumber = cv->f_finode;
+		cv->vptr->disk.node.inodeNumber = cv->f_finode;
 
 		/* set the delete flag to true - for abort case */
 		cv->vptr->delete_me = 1;
@@ -1359,12 +1382,12 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		
 		/* create the inode */
 		cv->vptr->disk.dataVersion = 1;
-		cv->f_finode = icreate((int)V_device(volptr), (int)V_id(volptr),
-				       (int)cv->vptr->vnodeNumber, 
-				       (int)cv->vptr->disk.uniquifier,
-				       (int)cv->vptr->disk.dataVersion);
+		cv->f_finode = icreate(V_device(volptr), V_id(volptr),
+				       cv->vptr->vnodeNumber,
+				       cv->vptr->disk.uniquifier,
+				       cv->vptr->disk.dataVersion);
 		CODA_ASSERT(cv->f_finode > 0);
-		cv->vptr->disk.inodeNumber = cv->f_finode;
+		cv->vptr->disk.node.inodeNumber = cv->f_finode;
 
 		/* set the delete flag to true - for abort case */
 		cv->vptr->delete_me = 1;
@@ -1406,8 +1429,8 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		    int tblocks = (int) -nBlocks(cv->vptr->disk.length);
 		    CODA_ASSERT(AdjustDiskUsage(volptr, tblocks) == 0);
 		    *deltablocks += tblocks;
-		    cv->f_sinode = cv->vptr->disk.inodeNumber;
-		    cv->vptr->disk.inodeNumber = 0;
+		    cv->f_sinode = cv->vptr->disk.node.inodeNumber;
+		    cv->vptr->disk.node.inodeNumber = 0;
 		}
 	    }
 	    break;
@@ -1497,9 +1520,9 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		*deltablocks += tblocks;
 
 		if (tv && tv->vptr->disk.type != vDirectory) {
-		    tv->f_sinode = tv->vptr->disk.inodeNumber;
-		    tv->vptr->disk.inodeNumber = 0;
-		}		
+		    tv->f_sinode = tv->vptr->disk.node.inodeNumber;
+		    tv->vptr->disk.node.inodeNumber = 0;
+		}
 
 		// make sure both parents are inconsistent
 		if (!IsIncon(sdv->vptr->disk.versionvector))
@@ -1738,13 +1761,13 @@ static int GetRepairObjects(Volume *volptr, vle *ov, dlist *vlist,
 	vle *v;
 	while ((v = (vle *)next())) {
 	    SLog(10,  "GetRepairObjects: acquiring (%s)", FID_(&v->fid));
-	    if ((errorCode = GetFsObj(&v->fid, &volptr, &v->vptr, 
-				     WRITE_LOCK, SHARED_LOCK, 1, 0, 
+	    if ((errorCode = GetFsObj(&v->fid, &volptr, &v->vptr,
+				     WRITE_LOCK, SHARED_LOCK, 1, 0,
 				     v->d_inodemod)))
 		return(errorCode);
-	}	
+	}
     }
-    
+
     return(errorCode);
 }
 
@@ -1765,7 +1788,7 @@ int GetSubTree(ViceFid *fid, Volume *volptr, dlist *vlist) {
 	if ((errorCode = GetFsObj(fid, &volptr, &vptr,
 				 READ_LOCK, NO_LOCK, 1, 0, 1)) )
 	    goto Exit;
-        
+
 	CODA_ASSERT(vptr->disk.type == vDirectory);
     }
 	
@@ -2061,8 +2084,8 @@ int PerformTreeRemoval(PDirEntry de, void *data)
 		    nblocks = (int)-nBlocks(cv->vptr->disk.length);
 		    CODA_ASSERT(AdjustDiskUsage(pkdparm->volptr, nblocks) == 0);
 		    *(pkdparm->blocks) += nblocks;
-		    cv->f_sinode = cv->vptr->disk.inodeNumber;
-		    cv->vptr->disk.inodeNumber = 0;
+		    cv->f_sinode = cv->vptr->disk.node.inodeNumber;
+		    cv->vptr->disk.node.inodeNumber = 0;
 		}
 
 		//spool log record for resolution 
@@ -2124,12 +2147,12 @@ START_TIMING(COP2_Total);
 	/* called before the flag is cleared by the user */
 	/* No need to lock the volume, because this doesnt change the file structure */
 	for (i = 0; i < nfids; i++) {
-	    errorCode = GetFsObj(&cpe->fids[i], &volptr, &vptrs[i], WRITE_LOCK, 
+	    errorCode = GetFsObj(&cpe->fids[i], &volptr, &vptrs[i], WRITE_LOCK,
 				 NO_LOCK, 1, 1, 0);
 	    /* Don't complain about vnodes that were deleted by COP1 */
 	    if (errorCode == VNOVNODE)
 		errorCode = 0;
-	    
+
 	    if (errorCode)
 		break;
 	}
@@ -2186,6 +2209,11 @@ static int GetResFlag(VolumeId Vid)
     int error = 0;
     Volume *volptr = 0;
 
+    if (!XlateVid(&Vid)) {
+	SLog(0,  "GetResFlag: Couldn't xlate vid %x", Vid);
+	return 0;
+    }
+
     if (!AllowResolution) return(0);
     if ((error = GetVolObj(Vid, &volptr, VOL_NO_LOCK, 0, 0))) {
 	SLog(0,  "GetResFlag:: GetVolObj failed (%d) for %x",
@@ -2256,8 +2284,7 @@ void NewCOP1Update(Volume *volptr, Vnode *vptr,
 
     /* Look up the index of this host. */
     int ix = vre->index(ThisHostAddr);
-    if (ix < 0) 
-	    Die("COP1Update: this host not found!");
+    if (ix < 0) Die("COP1Update: this host not found!");
 
     SLog(2, "COP1Update: Fid = (%x),(%x.%x.%x), StoreId = (%x.%x)",
 	 V_groupId(volptr), V_id(volptr), vptr->vnodeNumber, 
@@ -2296,13 +2323,11 @@ static void COP2Update(Volume *volptr, Vnode *vptr,
 {
     /* Look up the VRDB entry. */
     vrent *vre = VRDB.find(V_groupId(volptr));
-    if (!vre) 
-	    Die("COP2Update: VSG not found!");
+    if (!vre) Die("COP2Update: VSG not found!");
 
     /* Look up the index of this host. */
     int ix = vre->index(ThisHostAddr);
-    if (ix < 0) 
-	    Die("COP2Update: this host not found!");
+    if (ix < 0) Die("COP2Update: this host not found!");
 
     SLog(2,  "COP2Update: Fid = (%x),(%x.%x.%x)",
 	 V_groupId(volptr), V_id(volptr), vptr->vnodeNumber, 
