@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -659,15 +659,17 @@ static int ValidateReintegrateParms(RPC2_Handle RPCid, VolumeId *Vid,
 	}
 
 	list_head_init(&r->reint_log);
-	r->opcode = ntohl(*((RPC2_Integer *)_ptr++));
-	r->Mtime = ntohl(*((Date_t *)_ptr++));
 	r->Name[0] = r->Name[1] = NULL;
-
 	for (int i = 0; i < 3; i++) {
 	    r->Fid[i] = NullFid;
 	    r->VV[i] = NullVV;
 	}
 
+	r->opcode = ntohl(*(RPC2_Integer *)_ptr);
+	_ptr = (PARM *)((char *)_ptr + sizeof(RPC2_Integer));
+	r->Mtime = ntohl(*(Date_t *)_ptr);
+	_ptr = (PARM *)((char *)_ptr + sizeof(Date_t));
+
 	SLog(100,  "ValidateReintegrateParms: [B] Op = %d, Mtime = %d",
 		r->opcode, r->Mtime);
 
@@ -1036,15 +1038,23 @@ static int ValidateReintegrateParms(RPC2_Handle RPCid, VolumeId *Vid,
      * uniquifier field from the storeid of the record.  Return it
      * in the index variable; saves a parameter for this special case.
      */
-    if (!list_empty(rlog))
+    if (!list_empty(rlog) && check_reintegration_retry)
     {
 	int i = 0;
 	rle *r = list_entry(rlog->next, struct rle, reint_log);
 
-	while (i < (*volptr)->nReintegrators) {
+	/* uniquifier	[0, INT_MAX]	    -> replay detection
+	 *		(INT_MAX, UINT_MAX] -> no replay detection
+	 *
+	 * This way a client can disable replay detection by initializing the
+	 * counter to INT_MAX+1, but still be incrementing the uniquifier
+	 * value in case we are talking to older servers. */
+	while (r->sid.Uniquifier <= INT_MAX && i < (*volptr)->nReintegrators)
+	{
 	    if ((r->sid.Host == (*volptr)->reintegrators[i].Host) &&
-		((long)r->sid.Uniquifier - (long)(*volptr)->reintegrators[i].Uniquifier <= 0)) {
-		SLog(0, "ValidateReintegrateParms: Already seen id %u < %u)",
+		(r->sid.Uniquifier <= (*volptr)->reintegrators[i].Uniquifier))
+	    {
+		SLog(0, "ValidateReintegrateParms: Already seen id %u <= %u",
 		     r->sid.Uniquifier, (*volptr)->reintegrators[i].Uniquifier);
 		errorCode = VLOGSTALE;
 		index = (*volptr)->reintegrators[i].Uniquifier;
@@ -1059,14 +1069,14 @@ static int ValidateReintegrateParms(RPC2_Handle RPCid, VolumeId *Vid,
 	SLog(0, "ValidateReintegrateParms: Handle = (%d,%d,%d)",
 	     RHandle->BirthTime, RHandle->Device, RHandle->Inode);
 
-	/*  Currently, if an RHandle is supplied, the log must consist
-	 * of only one new store record.  (The store record is sent
-	 * only by old Venii.)  Verify that is the case.  */
+	/* Currently, if an RHandle is supplied, the log must consist
+	 * of only one store record. Verify that is the case. */
+	errorCode = EINVAL;
+	if (rlog_len == 1)
         {
-		CODA_ASSERT(rlog_len == 1);
-		
-		struct rle *r = list_entry(rlog->next, struct rle, reint_log);
-		CODA_ASSERT(r->opcode == CML_Store_OP);
+	    struct rle *r = list_entry(rlog->next, struct rle, reint_log);
+	    if (r->opcode == CML_Store_OP)
+		errorCode = 0; 
 	}
     }
 
@@ -1363,9 +1373,11 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			goto Exit;
 		    }
 		    /* Perform. */
+		    SLog(9, "CML_Store %s", FID_(&v->fid));
+
 		    if (v->f_finode == 0) {
 			/* First StoreData; record pre-reintegration inode. */
-			v->f_sinode = v->vptr->disk.inodeNumber;
+			v->f_sinode = v->vptr->disk.node.inodeNumber;
 		    }
 		    else {
 			/* Nth StoreData; discard previous inode. */
@@ -1458,6 +1470,7 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 		    }
 
 		    /* Perform. */
+		    SLog(9, "CML_SetAttr %s", FID_(&v->fid));
 #if 0
 		    //if (r->u.u_truncate.Length != v->vptr->disk.length)
 		    //   truncp = 1;
@@ -1501,7 +1514,7 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 		    if (c_inode) {
 			CODA_ASSERT(v->f_sinode == 0);
 			v->f_sinode = c_inode;
-			v->f_finode = v->vptr->disk.inodeNumber;
+			v->f_finode = v->vptr->disk.node.inodeNumber;
 			truncp = 0;
 		    }
 
@@ -1509,8 +1522,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 		    if (truncp) {
 			SLog(3,  "CheckSemanticsAndPerform: noting truncation (%s, %d, %d), (%d, %d)",
 			     FID_(&v->fid), v->f_tinode, v->f_tlength,
-			     v->vptr->disk.inodeNumber, v->vptr->disk.length);
-			v->f_tinode = v->vptr->disk.inodeNumber;
+			     v->vptr->disk.node.inodeNumber, v->vptr->disk.length);
+			v->f_tinode = v->vptr->disk.node.inodeNumber;
 			v->f_tlength = v->vptr->disk.length;
 		    }
 
@@ -1551,6 +1564,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			    parent_v->d_reintstale = 1;
 
 			/* Perform. */
+			SLog(9, "CML_Create %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 			HandleWeakEquality(volptr, parent_v->vptr, &r->VV[0]);
 			errorCode = PerformCreate(client, VSGVolnum, volptr,
 						  parent_v->vptr,
@@ -1612,6 +1627,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			parent_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_Remove %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 		    int tblocks = 0;
 		    HandleWeakEquality(volptr, parent_v->vptr, &r->VV[0]);
 		    HandleWeakEquality(volptr, child_v->vptr, &r->VV[1]);
@@ -1644,8 +1661,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			*blocks += deltablocks;
 
 			child_v->f_sid = NullSid;
-			child_v->f_sinode = child_v->vptr->disk.inodeNumber;
-			child_v->vptr->disk.inodeNumber = 0;
+			child_v->f_sinode = child_v->vptr->disk.node.inodeNumber;
+			child_v->vptr->disk.node.inodeNumber = 0;
 
 			/* Cancel previous truncate. */
 			SLog(3,  "CheckSemanticsAndPerform: cancelling truncate (%s, %d, %d)",
@@ -1677,6 +1694,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			parent_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_Link %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 		    HandleWeakEquality(volptr, parent_v->vptr, &r->VV[0]);
 		    HandleWeakEquality(volptr, child_v->vptr, &r->VV[1]);
 		    errorCode = PerformLink(client, VSGVolnum, volptr,
@@ -1756,11 +1775,15 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 		    if (VV_Cmp(&Vnode_vv(sd_v->vptr), &r->VV[0]) != VV_EQ)
 			sd_v->d_reintstale = 1;
 
-		    if (!SameParent && 
+		    if (!SameParent &&
 			(VV_Cmp(&Vnode_vv(td_v->vptr), &r->VV[1]) != VV_EQ))
 			    td_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_Rename %s/%s -> %s/%s",
+			 FID_(&sd_v->fid), FID_(&s_v->fid),
+			 FID_(&td_v->fid), TargetExists ? FID_(&t_v->fid):"-");
+
 		    HandleWeakEquality(volptr, sd_v->vptr, &r->VV[0]);
 		    if (!SameParent)
 			HandleWeakEquality(volptr, td_v->vptr, &r->VV[1]);
@@ -1810,8 +1833,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 
 			if (t_v->vptr->disk.type != vDirectory) {
 			    t_v->f_sid = NullSid;
-			    t_v->f_sinode = t_v->vptr->disk.inodeNumber;
-			    t_v->vptr->disk.inodeNumber = 0;
+			    t_v->f_sinode = t_v->vptr->disk.node.inodeNumber;
+			    t_v->vptr->disk.node.inodeNumber = 0;
 			}
 		    }
 		    }
@@ -1839,6 +1862,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			parent_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_MakeDir %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 		    HandleWeakEquality(volptr, parent_v->vptr, &r->VV[0]);
 		    errorCode = PerformMkdir(client, VSGVolnum, volptr,
 					     parent_v->vptr, child_v->vptr,
@@ -1906,6 +1931,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			parent_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_RemoveDir %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 		    int tblocks = 0;
 		    HandleWeakEquality(volptr, parent_v->vptr, &r->VV[0]);
 		    HandleWeakEquality(volptr, child_v->vptr, &r->VV[1]);
@@ -1964,6 +1991,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 			parent_v->d_reintstale = 1;
 
 		    /* Perform. */
+		    SLog(9, "CML_SymLink %s/%s", FID_(&parent_v->fid), FID_(&child_v->fid));
+
 		    CODA_ASSERT(child_v->f_finode == 0);
 		    child_v->f_finode = icreate(V_device(volptr), V_id(volptr),
 					       child_v->vptr->vnodeNumber,
@@ -2023,13 +2052,14 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 
     /* Make sure we don't get killed by accident */
     he = client->VenusId;
-    ObtainReadLock(&he->lock);
     if (!he || he->id == 0) {
 	errorCode = RPC2_FAIL;	/* ??? -JJK */
 	index = -1;
-	goto LockExit;
+	goto Exit;
     }
 
+    ObtainWriteLock(&he->lock);
+
     /* Now do bulk transfers. */
     {
 	struct dllist_head *p;
@@ -2070,12 +2100,9 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
                 }
 
                 if ( errorCode < RPC2_ELIMIT ) {
-                    /* We have to release the lock, because
-                     * CLIENT_CleanUpHost wants to grab an exclusive lock */
-                    ReleaseReadLock(&he->lock);
                     CLIENT_CleanUpHost(he);
                     index = -1;
-                    goto Exit;
+                    goto LockExit;
                 }
 
                 if (errorCode) {
@@ -2101,9 +2128,8 @@ START_TIMING(Reintegrate_CheckSemanticsAndPerform);
 	}
     }
 
-    /* If we still have the hosttable entry locked, release it now */
 LockExit:
-    ReleaseReadLock(&he->lock);
+    ReleaseWriteLock(&he->lock);
 
 Exit:
     if (Index)  
@@ -2307,7 +2333,8 @@ int AddChild(Volume **volptr, dlist *vlist, ViceFid *Did,
     switch(errorCode) {
 	case 0:
 	    vle = AddVLE(*vlist, &Fid);
-	    vle->d_inodemod = 1;
+	    if (ISDIR(Fid))
+		vle->d_inodemod = 1;
 	    break;
 
 	case ENOENT:
@@ -2356,13 +2383,7 @@ static int AddParent(Volume **volptr, dlist *vlist, ViceFid *Fid) {
     int errorCode = 0;
     Vnode *vptr = 0;
 
-    /* Get volptr. */
     /* We assume that volume has already been locked in exclusive mode! */
-    if (*volptr == 0) {
-	int ret = GetVolObj(Fid->Volume, volptr, VOL_NO_LOCK, 0, 0);
-	CODA_ASSERT(ret == 0);
-    }
-
     /* Child must NOT have just been alloc'ed, else this will deadlock! */
     if ((errorCode = GetFsObj(Fid, volptr, &vptr, READ_LOCK, 
 			     VOL_NO_LOCK, 0, 0, 0)))
