@@ -114,10 +114,10 @@ void cpent::print(int fd) {
 
 }
 
-int coppendhashfn(void *a)
+intptr_t coppendhashfn(void *a)
 {
     ViceStoreId *sid = (ViceStoreId *)a;
-    return(sid->Host + sid->Uniquifier);
+    return (sid->Host + sid->Uniquifier);
 }
 
 void cpman_func(void *arg)
@@ -126,7 +126,7 @@ void cpman_func(void *arg)
     cpm->func(0);
 }
 
-cpman::cpman(char *n): objects(COPHASHSIZE, coppendhashfn)
+cpman::cpman(const char *n): objects(COPHASHSIZE, coppendhashfn)
 {
     name = new char[strlen(n) + 1];
     strcpy(name, n);
