@@ -186,8 +186,7 @@ static int GetRawStatistics(SmonStatistics *);
 
 /*  *****  External variables  *****  */
 
-char *SmonHost = DFLT_MONDHOST;	/* may be overridden from command line */
-
+const char *SmonHost = DFLT_MONDHOST; /* may be overridden from command line */
 int SmonPort = DFLT_MONDPORT;	/* may be overridden from command line */	    
 /*  *****  Smon  *****  */
 
