@@ -51,7 +51,6 @@ extern "C" {
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <netinet/in.h>
-#include <netdb.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <net/if.h>
@@ -164,8 +163,11 @@ long FS_TokenExpired(RPC2_Handle RPCid)
     errorCode = RPC2_GetPrivatePointer(RPCid, &rock);
     client = (ClientEntry *)rock;
 
-    if (!errorCode && client)
+    if (!errorCode && client) {
+	ObtainWriteLock(&client->VenusId->lock);
 	CLIENT_CleanUpHost(client->VenusId);
+	ReleaseWriteLock(&client->VenusId->lock);
+    }
 
     return(RPC2_NAKED);
 }
@@ -260,6 +262,41 @@ long FS_ViceGetVolumeInfo(RPC2_Handle RPCid, RPC2_String VolName, VolumeInfo *In
     return(errorCode);
 }
 
+/*
+ViceGetVolumeLocation: Used to get the location (host:port) of a volume replica
+*/
+long FS_ViceGetVolumeLocation(RPC2_Handle RPCid, VolumeId Vid,
+			      RPC2_BoundedBS *HostPort)
+{
+    const char *addr;
+    size_t len;
+    int	err = 0;
+
+    SLog(1,"ViceGetVolumeLocation volume = %08x", Vid);
+    HostPort->SeqLen = 0;
+
+    addr = VGetVolumeLocation(Vid);
+    if (!addr) {
+	/* do we need more detailed errors? VNOVOL/ISREPLICATED/EWOULDBENICE? */
+	err = VNOSERVICE;
+	goto errout;
+    }
+
+    len = strlen(addr);
+    if (len >= HostPort->MaxSeqLen) {
+	err = ENAMETOOLONG;
+	goto errout;
+    }
+
+    strcpy((char *)HostPort->SeqBody, addr);
+    HostPort->SeqLen = len;
+
+errout:
+    SLog(2, "ViceGetVolumeLocation: Volume %08x, location %s, err %s",
+	 Vid, addr ? addr : "", ViceErrorMsg(err));
+    return(err);
+}
+
 
 /*
   ViceGetVolumeStatus: Get the status of a particular volume
@@ -317,7 +354,7 @@ long FS_ViceGetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
     vfid.Unique = 1;
 
     // get the root vnode even if it is inconsistent
-    if ((errorCode = GetFsObj(&vfid, &volptr, &vptr, 
+    if ((errorCode = GetFsObj(&vfid, &volptr, &vptr,
 			     READ_LOCK, VOL_NO_LOCK, 1, 0, 0))) {
 	goto Final;
     }
@@ -382,7 +419,7 @@ void PerformSetQuota(ClientEntry *client, VolumeId VSGVolnum, Volume *volptr, Vn
 
 /*
   BEGIN_HTML
-  <a name="ViceSetVolumeStatus"><strong>Set the status(e.g. quota) for a volume </strong></a> 
+  <a name="ViceSetVolumeStatus"><strong>Set the status(e.g. quota) for a volume </strong></a>
   END_HTML
 */
 long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *status, RPC2_BoundedBS *name, RPC2_BoundedBS *offlineMsg, RPC2_BoundedBS *motd, RPC2_Unsigned PrimaryHost, ViceStoreId *StoreId, RPC2_CountedBS *PiggyCOP2)
@@ -420,7 +457,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
     }
 
     {
-	if ((errorCode = ValidateParms(RPCid, &client, &ReplicatedOp, &vid, 
+	if ((errorCode = ValidateParms(RPCid, &client, &ReplicatedOp, &vid,
 				      PiggyCOP2, NULL)))
 	    goto Final;
     }
@@ -439,7 +476,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
 	errorCode = EINVAL;
 	goto Final;
     }
-    
+
 
     vfid.Volume = vid;
     vfid.Vnode = ROOTVNODE;
@@ -452,7 +489,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
 
 
     v = AddVLE(*vlist, &vfid);
-    if ((errorCode = GetFsObj(&vfid, &volptr, &v->vptr, READ_LOCK, 
+    if ((errorCode = GetFsObj(&vfid, &volptr, &v->vptr, READ_LOCK,
 			     VOL_NO_LOCK, 0, 0, 0))) {
 	goto Final;
     }
@@ -477,7 +514,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
 
     if(status->MaxQuota > -1) {
         oldquota = V_maxquota(volptr);
-	PerformSetQuota(client, VSGVolnum, volptr, v->vptr, &vfid, 
+	PerformSetQuota(client, VSGVolnum, volptr, v->vptr, &vfid,
 			(int)status->MaxQuota, ReplicatedOp, StoreId);
     }
 
@@ -491,7 +528,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
 	strcpy(V_motd(volptr), (char *)motd->SeqBody);
 
     // Only spool a log entry if the quota was set.
-    if (oldquota > -1) 
+    if (oldquota > -1)
 	if (ReplicatedOp && !errorCode) {
 	    SLog(1, "ViceSetVolumeStatus: About to spool log record, oldquota = %d, new quota = %d\n", oldquota, status->MaxQuota);
 	    if ((errorCode = SpoolVMLogRecord(vlist, v, volptr, StoreId,
@@ -502,7 +539,7 @@ long FS_ViceSetVolumeStatus(RPC2_Handle RPCid, VolumeId vid, VolumeStatus *statu
 
  Final:
 
-    if (!errorCode) 
+    if (!errorCode)
         SetVolumeStatus(status, name, offlineMsg, motd, volptr);
 
     PutObjects((int) errorCode, volptr, VOL_EXCL_LOCK, vlist, 0, 1, TRUE);
@@ -643,16 +680,13 @@ long FS_ViceGetTime(RPC2_Handle RPCid, RPC2_Unsigned *seconds,
 		}
 	    }
 
-	    ReleaseWriteLock(&client->VenusId->lock);
-	    /* we don't need the lock here anymore, because MakeCallBackConn
-	     * does it's own locking */
-
 	    if (!client->VenusId->id && !client->DoUnbind) {
 		SLog(0, "GetTime: Building callback conn to %s:%d.",
 		     inet_ntoa(client->VenusId->host),
 		     ntohs(client->VenusId->port));
 		errorCode = CLIENT_MakeCallBackConn(client);
 	    }
+	    ReleaseWriteLock(&client->VenusId->lock);
 	}
 
 	SLog(2, "GetTime returns %d, %d, errorCode %d", 
@@ -826,7 +860,7 @@ static void SetVolumeStats(ViceStatistics *stats)
 	}
 }
 
-#ifdef __BSD44__
+#if defined(__FreeBSD__) || defined(__NetBSD__)
 
 #ifdef	__NetBSD__
 #define KERNEL "/netbsd"
@@ -971,7 +1005,7 @@ static void SetSystemStats(ViceStatistics *stats)
     stats->SysTime = (resource.ru_stime.tv_sec * 100 +
 		      resource.ru_stime.tv_usec / 10000);
 
-#ifdef __BSD44__
+#if defined(__FreeBSD__) || defined(__NetBSD__)
     SetSystemStats_bsd44(stats);
 #endif
 #ifdef __linux__
@@ -1013,15 +1047,14 @@ long FS_ViceNewConnectFS(RPC2_Handle RPCid, RPC2_Unsigned ViceVersion,
 	return(RPC2_FAIL);
     }
 
+    ObtainWriteLock(&client->VenusId->lock);
+
     if (ViceVersion != VICE_VERSION) {
 	CLIENT_CleanUpHost(client->VenusId);
+	ReleaseWriteLock(&client->VenusId->lock);
 	return(RPC2_FAIL);
     }
 
-    /* we need a lock, because we cannot do concurrent RPC2 calls on the same
-     * connection */
-    ObtainWriteLock(&client->VenusId->lock);
-
     /* attempt to send a callback message to this host */
     if (client->VenusId->id != 0) {
 	errorCode = CallBack(client->VenusId->id, (ViceFid *)&NullFid);
@@ -1032,10 +1065,6 @@ long FS_ViceNewConnectFS(RPC2_Handle RPCid, RPC2_Unsigned ViceVersion,
 	}
     }			
 
-    ReleaseWriteLock(&client->VenusId->lock);
-    /* we don't need the lock here anymore, because MakeCallBackConn
-     * does it's own locking */
-
     /* set up a callback channel if there isn't one for this host */
     if (client->VenusId->id == 0) {
 	SLog(0, "Building callback conn.");
@@ -1045,6 +1074,8 @@ long FS_ViceNewConnectFS(RPC2_Handle RPCid, RPC2_Unsigned ViceVersion,
     if (errorCode)
 	CLIENT_CleanUpHost(client->VenusId);
 
+    ReleaseWriteLock(&client->VenusId->lock);
+
     SLog(2, "FS_ViceNewConnectFS returns %s", ViceErrorMsg((int) errorCode));
     
     return(errorCode);
