@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -25,7 +25,6 @@ extern void CLIENT_GetWorkStats(int *, int *, unsigned int);
 extern void CLIENT_PrintClients();
 extern void CLIENT_CallBackCheck();
 int CLIENT_MakeCallBackConn(ClientEntry *Client);
-int CLIENT_MakeWriteBackConn(HostTable * VenusId);
 
 char *ViceErrorMsg(int errorCode);
 
@@ -38,6 +37,6 @@ extern void BreakCallBack(HostTable *, ViceFid *);
 extern void DeleteVenus (HostTable *);
 extern void DeleteFile (ViceFid *);
 extern int InitCallBack ();
-extern int CheckWriteBack(ViceFid * Fid);
 
-extern int NoWritebackConn;
+extern int check_reintegration_retry;
+
