@@ -64,7 +64,6 @@ extern "C" {
 #include <rpc2/rpc2.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <netdb.h>
 #include <netinet/in.h>
 
 #include <histo.h>
@@ -406,11 +405,11 @@ void BreakCallBack(HostTable *client, ViceFid *afid) {
     qsort(helist, nhents, sizeof(HostTable *), order_helist);
 
     for (int i = 0; i < nhents; i++) {
-	ObtainReadLock(&helist[i]->lock);
+	ObtainWriteLock(&helist[i]->lock);
 
 	/* Recheck! The callback connection might have been destroyed */
 	if (!helist[i]->id) {
-	    ReleaseReadLock(&helist[i]->lock);
+	    ReleaseWriteLock(&helist[i]->lock);
 	    helist[i] = NULL;
 	}
 	else
@@ -422,8 +421,10 @@ void BreakCallBack(HostTable *client, ViceFid *afid) {
 
     /* assuming no multicast in call below (MCast parameter NULL) */
     if (nhosts > 0) {
+	/* make sure we abort long running callback break attempts */
+	struct timeval tout = { 60, 0 };
 	MRPC_MakeMulti(CallBack_OP, CallBack_PTR, nhosts, cidlist, rclist,
-		       NULL, NULL, NULL, afid);
+		       NULL, NULL, &tout, afid);
 
 	for (int i = 0, nhosts = 0; i < nhents; i++) {
 	    if (!helist[i]) continue;
@@ -434,13 +435,13 @@ void BreakCallBack(HostTable *client, ViceFid *afid) {
 	    /* if a file callback, delete any volume callbacks */
 	    if (!aVCB)  DeleteCallBack(helist[i], &vFid);
 
-	    /* let go of the lock */
-	    ReleaseReadLock(&helist[i]->lock);
-
 	    /* recursively calls DeleteVenus */
 	    if (rclist[nhosts] < RPC2_ELIMIT)  
 		CLIENT_CleanUpHost(helist[i]);
 
+	    /* let go of the lock */
+	    ReleaseWriteLock(&helist[i]->lock);
+
 	    nhosts++;
 	}
     }
@@ -737,7 +738,7 @@ void PrintCallBackState(FILE *fp)
 	{
 	int i;
             for (i = 0;  i < numVolumes; i++) {
-	        fprintf(fp, "\t0x%lx  %d FEs (%d VEs), %d CBEs (%d VCBEs)\n",
+	        fprintf(fp, "\t%08x  %d FEs (%d VEs), %d CBEs (%d VCBEs)\n",
 		        CBStats[i].volid, CBStats[i].FEs, CBStats[i].VEs,
 		        CBStats[i].CBEs, CBStats[i].VCBEs);
 	    
@@ -777,13 +778,13 @@ static void GetCallBacks(ViceFid *fid, FILE *fp)
 
 static void GetCallBacks(VolumeId vid, FILE *fp) 
 {
-    fprintf(fp, "Printing callbacks for 0x%lx\n", vid);
+    fprintf(fp, "Printing callbacks for %08x\n", vid);
 
     /* print volume callbacks first */
     ViceFid fid = NullFid; fid.Volume = vid;
     struct FileEntry *tfe = FindEntry(&fid);
     if (tfe) {
-	fprintf(fp, "VID 0x%lx  Volume Callback\n", vid);
+	fprintf(fp, "VID %08x  Volume Callback\n", vid);
 	for (struct CallBackEntry *tcbe = tfe->callBacks; tcbe; tcbe = tcbe->next) 
 	    PrintCBE(tcbe, fp);
     }
@@ -800,7 +801,7 @@ static void GetCallBacks(VolumeId vid, FILE *fp)
 	    }
         }
     }
-    fprintf(fp, "End of callbacks for %lx\n", vid);
+    fprintf(fp, "End of callbacks for %08x\n", vid);
 }
 
 // print all the callbacks for given fid.
