@@ -109,6 +109,8 @@ extern "C" {
 #define _VIOC_FLUSHVOLUME	(37)	/* flush whole volume's data */
 #define _VIOC_LISTCACHE_VOLUME	(39)	/* list volume's cached status */
 #define _VIOC_LOOKASIDE         (40) /* Add or remove  cache lookaside databases (Satya, 1/2003) */
+#define _VIOC_EXPANDOBJECT	(41) /* Turn fso into fake directory to expose underlying replicas */
+#define _VIOC_COLLAPSEOBJECT	(42) /* Collapse an expanded fso */
 
 /* These were defined with numbers that wrapped around the 8-bit size of the nr
  * component in the ioctl */
@@ -157,8 +159,8 @@ extern "C" {
 #define _VIOC_RECONNECT         (CFS_IOCTL_BASE + 25) /* Heal previous partition */
 /*#define _VIOC_SLOW		(CFS_IOCTL_BASE + 26)  * Slow the network down */
 #define _VIOC_GETPFID		(CFS_IOCTL_BASE + 27) /* Get fid of object's parent */
-#define _VIOC_BEGINML           (CFS_IOCTL_BASE + 28) /* Log updates to a volume */
-#define _VIOC_ENDML             (CFS_IOCTL_BASE + 29) /* Stop logging (write back) */
+#define _VIOC_WD                (CFS_IOCTL_BASE + 28) /* set write-disconnect parameters for a volume */
+/* #define _VIOC_ENDML          (CFS_IOCTL_BASE + 29) */
 
 #define _VIOC_HDB_VERIFY	(CFS_IOCTL_BASE + 30) /* Compare hoard database to cache contents */
 /*#define _VIOC_BWHINT		(CFS_IOCTL_BASE + 31)  * Give a bandwidth hint */
@@ -179,10 +181,10 @@ extern "C" {
 #define _VIOC_SET_LOCAL_VIEW    (CFS_IOCTL_BASE + 43) /* Set local repair session view */
 #define _VIOC_SET_GLOBAL_VIEW   (CFS_IOCTL_BASE + 44) /* Set global repair session view */
 #define _VIOC_SET_MIXED_VIEW    (CFS_IOCTL_BASE + 45) /* Set mixed repair session view */
-#define _VIOC_WD_ALL 		(CFS_IOCTL_BASE + 46) /* Write-disconnect all volumes */
-#define _VIOC_WR_ALL     	(CFS_IOCTL_BASE + 47) /* Write-reconnect all volumes */
-#define _VIOC_STRONG		(CFS_IOCTL_BASE + 48)	/* Force strong connectivity */
-#define _VIOC_ADAPTIVE		(CFS_IOCTL_BASE + 49)	/* Allow bandwidth adaptation driven connectivity */
+#define _VIOC_WD_ALL 		(CFS_IOCTL_BASE + 46) /* set write-disconnect parameters on all volumes */
+#define _VIOC_SYNCCACHE_ALL    	(CFS_IOCTL_BASE + 47) /* force reintegrate all volumes */
+/*#define _VIOC_STRONG		(CFS_IOCTL_BASE + 48)  * Force strong connectivity */
+/*#define _VIOC_ADAPTIVE	(CFS_IOCTL_BASE + 49)  * Allow bandwidth adaptation driven connectivity */
 #define _VIOC_LISTCACHE		(CFS_IOCTL_BASE + 50)	/* List cache status */
 #define _VIOC_GET_MT_PT		(CFS_IOCTL_BASE + 51)	/* Get mount point path from volume id */
 
