@@ -52,7 +52,6 @@ extern "C" {
 
 void ForceUnlockVol(VolumeId );
 
-class lqman;
 class lq_iterator;
 class lqent;
 
@@ -66,7 +65,7 @@ class lqman {
     int func(void);
 
   public:
-    lqman(char * ="anonymous lqman");
+    lqman(const char *name = "anonymous lqman");
     ~lqman();
     void add(lqent *);
     void remove(lqent *);
@@ -85,7 +84,7 @@ class lq_iterator : public dlist_iterator {
 
 class lqent : public dlink{
   friend class lqman;
-  friend int GetVolObj(VolumeId, Volume **, int, int, int);
+  friend int GetVolObj(VolumeId, Volume **, int, int, unsigned);
   friend void PutVolObj(Volume **, int, int);
   friend long ViceLockVol(RPC2_Handle, VolumeId, UserId *, RPC2_Unsigned *);
   friend long ViceUnlockVol(RPC2_Handle, VolumeId);
