@@ -54,10 +54,9 @@ extern "C" {
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#ifdef __BSD44__
-#include <fstab.h>
-#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <netinet/in.h>
 #include <sys/wait.h>
 #include "coda_string.h"
@@ -104,8 +103,11 @@ int ThisServerId = -1;	/* this server id, as found in  .../db/servers */
 bit32 HostAddress[N_SERVERIDS];	/* Assume host addresses are 32 bits */
 int VInit;		/* Set to 1 when the volume package is initialized */
 int HInit;		/* Set to 1 when the volid hash table is  initialized */
-char *VSalvageMessage =	  /* Common message used when the volume goes off line */
-"Files in this volume are currently unavailable; call operations";
+const char *VSalvageMessage =/* Common message used when volume goes off line */
+    "Files in this volume are currently unavailable; call operations";
+
+const char *Server_FQDN[N_SERVERIDS];	/* DNS host name (with optional port) */
+/* something like "codaserverN.foo.bar:2432" */
 
 /*
   VolumeHashTable: Hash table used to store pointers to the Volume structure
@@ -116,7 +118,7 @@ char *VSalvageMessage =	  /* Common message used when the volume goes off line *
 #define VOLUME_HASH(volumeId) (volumeId&(VOLUME_HASH_TABLE_SIZE-1))
 static Volume *VolumeHashTable[VOLUME_HASH_TABLE_SIZE];
 
-extern void dump_storage(int level, char *s);
+extern void dump_storage(int level, const char *s);
 extern void VBumpVolumeUsage(Volume *vp);
 extern int VCheckVLDB();
 extern int InSkipVolumeList(VolumeId, VolumeId *, int);
@@ -319,12 +321,12 @@ void VInitVolumePackage(int nLargeVnodes, int nSmallVnodes, int DoSalvage)
 		VLog(0, "Forcing Volume %x Offline", header.id);
 		VForceOffline(vp);
 	    } else {
+		if (V_type(vp) == readwriteVolume) {
 		    /* initialize the RVM log vm structures */
-		    if (V_RVMResOn(vp)) {
-			    V_VolLog(vp)->ResetTransients(V_id(vp));
-			    extern olist ResStatsList;
-			    ResStatsList.insert((olink *)V_VolLog(vp)->vmrstats);
-		    }
+		    V_VolLog(vp)->ResetTransients(V_id(vp));
+		    extern olist ResStatsList;
+		    ResStatsList.insert((olink *)V_VolLog(vp)->vmrstats);
+		}
 	    }
 	    VPutVolume(vp);
 	}
@@ -364,7 +366,7 @@ void VDisconnectFS() {
     FSYNC_clientFinis();
 }
 
-void VInitThisHost(char *host)
+void VInitThisHost(const char *host)
 {
     if (ThisHost) free(ThisHost);
 
@@ -381,10 +383,10 @@ void VInitThisHost(char *host)
 
 /* must be called before calling VInitVolumePackage!! */
 /* Find the server id */
-void VInitServerList(char *host) 
+void VInitServerList(const char *host)
 {
     char line[200];
-    char *serverList = SERVERLISTPATH;
+    const char *serverList = SERVERLISTPATH;
     FILE *file;
 
     memset(HostAddress, 0, sizeof(bit32) * N_SERVERIDS);
@@ -403,11 +405,12 @@ void VInitServerList(char *host)
     }
 
     while (fgets(line, sizeof(line), file) != NULL) {
-        char sname[50];
-        unsigned int sid;
+	char sname[51];
+	unsigned int sid;
+	int i;
 	long netaddress;
 
-	if (sscanf(line, "%s%u", sname, &sid) == 2) {
+	if (sscanf(line, "%50s %u", sname, &sid) == 2) {
 	    if (sid >= N_SERVERIDS) {
 		VLog(0, "Host %s is assigned a bogus server number (%x) in %s. Exit.",
 		     sname, sid, serverList);
@@ -420,11 +423,23 @@ void VInitServerList(char *host)
 		exit(1);
 	    }
 	    /* make sure we don't get duplicate ids */
-	    if (HostAddress[sid]) {
-		VLog(0, "Warning: host %s is using a an already assigned "
-			"server-id (%lu) in %s. Exit", sname, sid, serverList);
+	    if (Server_FQDN[sid]) {
+		VLog(0, "Fatal: unable to map server-id %d to host %s,\n"
+		     "\tas it is already assigned to host %s",
+		     sid, sname, Server_FQDN[sid]);
 		exit(1);
 	    }
+	    Server_FQDN[sid] = strdup(sname);
+
+	    /* strip trailing :NNNN for the legacy code */
+	    for (i = strlen(sname) - 1; i >= 0; i--) {
+		if (sname[i] == ':') {
+		    sname[i] = '\0';
+		    break;
+		}
+		if (sname[i] < '0' || sname[i] > '9')
+		    break;
+	    }
 
 	    struct hostent *hostent = gethostbyname(sname);
 	    if (!hostent || hostent->h_length != sizeof(struct in_addr) ||
@@ -525,6 +540,37 @@ void VGetVolumeInfo(Error *ec, char *key, VolumeInfo *info)
     return;
 }
 
+const char *VGetVolumeLocation(VolumeId vid)
+{
+    struct vldb *vldp;
+    char key[11]; /* sizeof(#MAX_UINT) + 1 */
+    const char *location;
+    int serverid;
+
+    /* sigh, the VLDB is indexed based on the ascii representation of the id */
+    snprintf(key, 10, "%u", vid);
+
+    vldp = VLDBLookup(key);
+    if (!vldp) {
+	VLog(9, "VGetVolumeLocation: VLDBLookup for %08x failed", vid);
+	return NULL; /* VNOVOL */
+    }
+
+    if (vldp->nServers != 1) {
+	VLog(9, "VGetVolumeLocation: %08x seems to be replicated", vid);
+	return NULL; /* VNOVOL (nservers == 0) | ISREPLICATED (nservers > 1) */
+    }
+
+    serverid = vldp->serverNumber[0];
+    /* Maybe at some point we need to some hook when we migrate volumes? */
+    /* serverid = FSYNC_CheckRelocationSite(vid); ?? */
+
+    location = Server_FQDN[serverid];
+
+    VLog(9, "VGetVolumeLocation: %08x located at %s", location);
+    return location;
+}
+
 static void VListVolume(char **buf, unsigned int *buflen,
 				unsigned int *offset, Volume *vp)
 {
@@ -665,7 +711,7 @@ void VShutdown() {
 		VLog(0, "VShutdown: Taking volume %s(0x%x) offline...",
 		     V_name(vp), V_id(vp));
 	    if (vp)
-	        VOffline(vp, "File server was shut down");
+		VOffline(vp, "File server was shut down");
 	    VLog(0, "... Done");
 	    rvmlib_end_transaction(flush, &(camstatus));
 	    p = p->hashNext;
@@ -1119,9 +1165,8 @@ void VPutVolume(Volume *vp)
 		LWP_SignalProcess((char *)VPutVolume);
 	}
     }
-    else 
-	VLog(1, "VPutVolume (%x): NO-OP since vp->nUsers = %d", 
-	     V_id(vp), vp->nUsers + 1);
+    else VLog(1, "VPutVolume (%x): NO-OP since vp->nUsers = %d",
+	      V_id(vp), vp->nUsers + 1);
 }
 
 /* Force the volume offline, set the salvage flag.  No further references to */
@@ -1137,7 +1182,7 @@ void VForceOffline(Volume *vp)
     VLog(0, "Volume %x forced offline:  it needs to be salvaged!", V_id(vp));
     V_inUse(vp) = 0;
     VLog(1, "VForceOffline: setting V_inUse(%x) = 0 and writing out voldiskinfo", V_id(vp));
-    vp->goingOffline = 0;    
+    vp->goingOffline = 0;
     V_needsSalvaged(vp) = 1;
     VUpdateVolume(&error, vp);
     LWP_SignalProcess((char *)VPutVolume);
@@ -1147,7 +1192,7 @@ void VForceOffline(Volume *vp)
    the inUse bit turned off.  A copy of the header is maintained in memory,
    however (which is why this is VOffline, not VDetach).
  */
-void VOffline(Volume *vp, char *message)
+void VOffline(Volume *vp, const char *message)
 {
     Error error;
     VolumeId vid = V_id(vp);
@@ -1622,7 +1667,7 @@ static void VScanUpdateList()
     /* Be careful with this code, since it works with interleaved calls to AddToVolumeUpdateList */
     for (i = gap = 0; i<nUpdatedVolumes; i++) {
 	VLog(29, "ScanUpdateList: Going to call VGetVolume ");
-        vp = VGetVolume(&error, UpdateList[i-gap] = UpdateList[i]);
+	vp = VGetVolume(&error, UpdateList[i-gap] = UpdateList[i]);
 	VLog(29, "ScanUpdateList: Just returned from VGetVolume");
 	if (error) {
 	    VLog(29, "ScanUpdateList: Error %d in VGetVolume", error);
@@ -1679,20 +1724,20 @@ static int GetVolumeHeader(Volume *vp)
 	Error error;
 	struct volHeader *hd;
 	int old;
-	
+
 	VLog(9, "Entering GetVolumeHeader()");
 	old = (vp->header != 0);
-	
+
 	if (old) {
 		hd = vp->header;
-		/* the joy of list headers Coda style: we are about to 
+		/* the joy of list headers Coda style: we are about to
 		   take hd out of the list */
 		if (volumeLRU == hd)
 			volumeLRU = hd->next;
 		CODA_ASSERT(hd->back == vp);
 	} else {
 		/* not currently in use and least recently used */
-		hd = volumeLRU->prev; 
+		hd = volumeLRU->prev;
 		if (hd->back) {
 			if (hd->diskstuff.inUse) {
 				VLog(1, "storing VolumeDiskInfo (%x) to reclaim cache slot",
@@ -1772,7 +1817,7 @@ void FreeVolumeHeader(Volume *vp)
 /***************************************************/
 /*
   AddVolumeToHashTable: Add the volume (*vp) to the hash table
-  As used, hashid is always the id of the volume.  
+  As used, hashid is always the id of the volume.
 */
 static void AddVolumeToHashTable(Volume *vp, int hashid)
 {
@@ -1831,7 +1876,7 @@ void DeleteVolumeFromHashTable(Volume *vp)
     vp->hashid = 0;
 }
 
-void VPrintCacheStats(FILE *fp) 
+void VPrintCacheStats(FILE *fp)
 {
     struct VnodeClassInfo *vcp;
     vcp = &VnodeClassInfo_Array[vLarge];
@@ -1883,7 +1928,7 @@ Error VAdjustDiskUsage(Volume *vp, int blocks)
 }
 
 int GetVolObj(VolumeId Vid, Volume **volptr,
-	      int LockLevel, int Enque, int LockerAddress)
+	      int LockLevel, int Enque, unsigned LockerAddress)
 {
     int errorCode = 0;
 
@@ -1916,7 +1961,7 @@ int GetVolObj(VolumeId Vid, Volume **volptr,
 	    }
 	    V_VolLock(*volptr).IPAddress = LockerAddress;
 	    ObtainWriteLock(&(V_VolLock(*volptr).VolumeLock));
-	    CODA_ASSERT((long)V_VolLock(*volptr).IPAddress == LockerAddress);
+	    CODA_ASSERT(V_VolLock(*volptr).IPAddress == LockerAddress);
 	    if (Enque) {
 		    lqent *lqep = new lqent(Vid);
 		    LockQueueMan->add(lqep);
@@ -1925,15 +1970,15 @@ int GetVolObj(VolumeId Vid, Volume **volptr,
     default:
 	    CODA_ASSERT(0);
     }
-    
- FreeLocks:
+
+FreeLocks:
     /* make sure the volume hash isn't resized */
     if (errorCode && *volptr) {
 	VPutVolume(*volptr);
 	*volptr = NULL;
     }
     SLog(9, "GetVolObj: returns %d", errorCode);
-    
+
     return(errorCode);
 }
 
@@ -1953,8 +1998,8 @@ void PutVolObj(Volume **volptr, int LockLevel, int Dequeue)
       case VOL_EXCL_LOCK:
 	if (Dequeue) {
 	    lqent *lqep = LockQueueMan->findanddeq(V_id(*volptr));
-	    if (!lqep) 
-		SLog(0, "PutVolObj: Couldn't find entry %x on lock queue", 
+	    if (!lqep)
+		SLog(0, "PutVolObj: Couldn't find entry %x on lock queue",
 			V_id(*volptr));
 	    else {
 		LockQueueMan->remove(lqep);
