@@ -277,7 +277,7 @@ struct Volume {
 				   that stayed around while a volume
 				   was offline */
     short	nUsers;		/* Number of users of this volume header */
-    byte	specialStatus;	/* An error code to return on VGetVolume: the
+    int32_t	specialStatus;	/* An error code to return on VGetVolume: the
 				   volume is unavailable for the reason quoted,
 				   currently VBUSY or VMOVED */
     int32_t	updateTime;	/* Time that this volume was put on the updated
@@ -371,20 +371,19 @@ extern int VInit;		/* Set to 1 when the volume package is
 				   initialized */
 extern int HInit;		/* Set to 1 when the volid hash table
 				   is initialized */
-extern char *VSalvageMessage;   /* Common message used when the volume
-				   goes off line */
+extern const char *VSalvageMessage; /* Common message used when the volume goes
+				       off line */
 extern int VolDebugLevel;	/* Controls level of debugging information */
-extern char *VSalvageMessage;	/* Canonical message when a volume is forced
-				   offline */
 extern int AllowResolution;	/* global flag to turn on dir. resolution */
 extern void VInitVolumePackage(int nLargeVnodes, int nSmallVnodes, int DoSalvage);
 extern int VInitVolUtil(ProgramType pt);
-extern void VInitServerList(char *host);
+extern void VInitServerList(const char *host);
 extern int VConnectFS();
 extern void VDisconnectFS();
 extern void VUCloneVolume(Error *, Volume *, Volume *);
 extern void VListVolumes(char **buf, unsigned int *buflen);
 extern void VGetVolumeInfo(Error *ec, char *key, VolumeInfo *info);
+extern const char *VGetVolumeLocation(VolumeId vid);
 extern Volume * VGetVolume(Error *ec, VolumeId volumeId);
 extern void VPutVolume(Volume *vp);
 extern Volume * VAttachVolume(Error *ec, VolumeId volumeId, int mode);
@@ -397,7 +396,7 @@ extern void VFreeBitMapEntry(Error *ec, struct vnodeIndex *index, int bitNumber)
 extern int VolumeNumber(char *name);
 extern char * VolumeExternalName(VolumeId volumeId);
 extern Volume * VAttachVolumeById(Error *ec, char *partition, VolumeId volid, int mode);
-extern void VOffline(Volume *vp, char *message);
+extern void VOffline(Volume *vp, const char *message);
 extern void VForceOffline(Volume *vp);
 extern void VPurgeVolume(Volume *vp);
 extern void VShutdown();
@@ -447,7 +446,7 @@ Error VAdjustDiskUsage(Volume *vp, int blocks);
 Error VCheckDiskUsage(Volume *vp, int blocks);
 void PutVolObj(Volume **volptr, int LockLevel, int Dequeue);
 int GetVolObj(VolumeId Vid, Volume **volptr, 
-	      int LockLevel, int Enque, int LockerAddress);
+	      int LockLevel, int Enque, unsigned LockerAddress);
 
 
 #endif /* VOLUME_INCLUDED */
