@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -47,7 +47,6 @@ extern "C" {
 #include <unistd.h>
 #include <stdlib.h>
 #include <netinet/in.h>
-#include <netdb.h>
 #include <errno.h>
 #include "coda_string.h"
 
@@ -79,6 +78,7 @@ extern "C" {
 #include <codaconf.h>
 #include <vice_file.h>
 #include <getsecret.h>
+#include <coda_getservbyname.h>
 
 
 static RPC2_EncryptionKey vkey;    /* Encryption key for bind authentication */
@@ -87,27 +87,28 @@ static int Debug = 0;		   /* Global debugging flag */
 static int Naptime = 30;	   /* Sleep period for PollLWP */
 
 struct hostinfo {
-    bit32      	address;  /* Assume host IP addresses are 32 bits */
+    bit32	address;  /* Assume host IP addresses are 32 bits */
     RPC2_Handle rpcid;	  /* should be -1 if connection is dead. */
     char	name[36];
-}  Hosts[N_SERVERIDS];
+} Hosts[N_SERVERIDS];
 
 #define BADCONNECTION	(RPC2_Handle) -1
 
 bit32 HostAddress[N_SERVERIDS];		/* Need these for macros in vrdb.c */
 char *ThisHost;			/* This machine's hostname */
-int ThisServerId = -1;	       	/* this server id, as found in ../db/servers */
+int ThisServerId = -1;		/* this server id, as found in ../db/servers */
 
 /* Rock info for communicating with the DumpLWP. */
 #define ROCKTAG 12345
 struct rockInfo {
-    char dumpfile[MAXPATHLEN];
-    VolumeId volid;	      /* Volume being dumped. */
-    unsigned long numbytes; /* Number of bytes already written to the file. */
+    int dumpfd;		   /* Open filedescriptor for WriteDump */
+    VolumeId volid;	   /* Volume being dumped. */
+    unsigned int numbytes; /* Number of bytes already written to the file.
+			      (has to wrap around the same way as 'offset') */
 } Rock;
 
-struct hgram DataRate;			/* Statistics on rate of data transfer */
-struct hgram DataTransferred;   	/* and size of dumpfiles. */
+struct hgram DataRate;		/* Statistics on rate of data transfer */
+struct hgram DataTransferred;	/* and size of dumpfiles. */
 
 /* Per replica info. */
 #define LOCKED 0x1
@@ -151,7 +152,7 @@ typedef struct volinfo {
     repinfo_t *replicas;
 } volinfo_t;
 
-char *vicedir = NULL;
+const char *vicedir = NULL;
 
 
 /* Procedure definitions. */
@@ -287,7 +288,7 @@ int getReplica(repinfo_t *rep) {
     VolumeId volId = rep->repvolId;
     rep->serverNum = 0;
     
-    sprintf(volIdstr, "%08x", volId);
+    sprintf(volIdstr, "%u", volId);
     vldbp = VLDBLookup(volIdstr);
     if (vldbp == NULL) {
 	LogMsg(0, 0, stdout, "Volume replica %x doesn't exist!\n", volId);
@@ -436,8 +437,8 @@ static int lockReplicas(volinfo_t *vol)
 {
     long rc = 0;
     repinfo_t *reps = vol->replicas;
-    
-    for (char i = 0; i < vol->nReplicas; i++) {
+
+    for (int i = 0; i < vol->nReplicas; i++) {
 	if (Hosts[reps[i].serverNum].rpcid != BADCONNECTION) {
 	    rc = VolLock(Hosts[reps[i].serverNum].rpcid, reps[i].repvolId, &reps[i].vvv);
 	    if (rc != RPC2_SUCCESS) {
@@ -446,17 +447,17 @@ static int lockReplicas(volinfo_t *vol)
 		       vol->volId, reps[i].repvolId, RPC2_ErrorMsg((int)rc));
 
 		if (rc < RPC2_FLIMIT) /* Connection has been nixed by RPC2 */
-		    Hosts[reps[i].serverNum].rpcid = BADCONNECTION; 
+		    Hosts[reps[i].serverNum].rpcid = BADCONNECTION;
 
 		/* If a call fails, we should clear the flags, since LOCKING
 		 * is the first stage. However, we can continue with the backup
 		 * since optimistically the quorum for backup is 1.
 		 */
-		reps[i].flags = 0;     
+		reps[i].flags = 0;
 	    } else
 		/* Set this flag to force retry of all other operations */
-		reps[i].flags = LOCKED;	
-	}	
+		reps[i].flags = LOCKED;
+	}
     }
     return 0;
 }
@@ -470,8 +471,8 @@ static void unlockReplicas(volinfo_t *vol)
 {
     long rc;
     repinfo_t *reps = vol->replicas;
-    
-    for (char i = 0; i < vol->nReplicas; i++) {
+
+    for (int i = 0; i < vol->nReplicas; i++) {
 	/* Try to unlock all replicas, whether we think they're locked or not. */
 	if (Hosts[reps[i].serverNum].rpcid != BADCONNECTION) {
 	    rc= VolUnlock(Hosts[reps[i].serverNum].rpcid, reps[i].repvolId);
@@ -480,12 +481,11 @@ static void unlockReplicas(volinfo_t *vol)
 		       vol->volId, reps[i].repvolId, RPC2_ErrorMsg((int)rc));
 		LogMsg(0, 0, stdout, "VolUnlock failed, connection %d",
 		       Hosts[reps[i].serverNum].rpcid);
-		       
+
 		if (rc < RPC2_FLIMIT)	/* Connection has been nixed by RPC2 */
 		    Hosts[reps[i].serverNum].rpcid = BADCONNECTION;
 	    } else
 		reps[i].flags &= ~LOCKED;
-	    
 	}
     }
 }
@@ -501,14 +501,13 @@ static int backup(volinfo_t *vol) {
     repinfo_t *reps = vol->replicas;
     long rc;
     int count = 0;
-    
+
     LogMsg(0, 0, stdout, "%x: cloning ", volId);
 
-    if (lockReplicas(vol) != 0) 		/* Lock all the replicas */
+    if (lockReplicas(vol) != 0)		/* Lock all the replicas */
 	return -1;
 
-    for (char i = 0; i < vol->nReplicas; i++) {
-
+    for (int i = 0; i < vol->nReplicas; i++) {
 	/* Only clone a replica if it was locked and its server is up. */
 	if ((Hosts[reps[i].serverNum].rpcid != BADCONNECTION) && ISLOCKED(reps[i].flags)) {
 	    rc = VolMakeBackups(Hosts[reps[i].serverNum].rpcid,
@@ -519,7 +518,7 @@ static int backup(volinfo_t *vol) {
 		       reps[i].repvolId, RPC2_ErrorMsg((int)rc));
 
 		if (rc < RPC2_FLIMIT) /* Connection has been nixed by RPC2 */
-		    Hosts[reps[i].serverNum].rpcid = BADCONNECTION; 
+		    Hosts[reps[i].serverNum].rpcid = BADCONNECTION;
 
 		/* I think we can safely continue if some replica(s) fails.
 		 * The worst that happens is that some extra clones are done.
@@ -530,13 +529,12 @@ static int backup(volinfo_t *vol) {
 		count++;
 		reps[i].flags |= CLONED;
 	    }
-	} 
+	}
      }
 
     /* Don't need to unlock anymore, S_VolBackup does it as a side-effect.
      *  unlockReplicas(vol);
      */
-    
     vol->nCloned = count;			/* Store # of successful clones */
     CODA_ASSERT(count <= vol->nReplicas);
     return 0;
@@ -586,49 +584,50 @@ int dumpVolume(volinfo_t *vol)
     repinfo_t *reps = vol->replicas;
     VolumeId volId = vol->volId;
     long rc;
-    RPC2_Unsigned dumplevel = (vol->flags & DUMPLVL_MASK) >> DUMPLVL_SHFT;
+    RPC2_Integer dumplevel = (vol->flags & DUMPLVL_MASK) >> DUMPLVL_SHFT;
     int ndumped = 0;
-    
+
     for (int i = 0; i < vol->nReplicas; i++) {
 	if ((Hosts[reps[i].serverNum].rpcid == BADCONNECTION) ||
 	    !ISCLONED(reps[i].flags))
-	    continue;	
-	
+	    continue;
+
 	if (ISDUMPED(reps[i].flags)) {
 	    ndumped++; /* Count it, but don't need to redo it. */
 	    continue;
 	}
-	
+
 	CODA_ASSERT(reps[i].backupId > 0);
-	
+
 	/* get the name of the dumpfile. */
 	struct DiskPartition *part = NULL;
 	char buf[MAXPATHLEN];
 
 	part = findBestPartition();
-	if (vol->flags & REPLICATED) 
+	if (vol->flags & REPLICATED)
 		sprintf(buf, "%s/%s-%08x.%08x", part->name,
 			Hosts[reps[i].serverNum].name, volId, reps[i].repvolId);
-	    else 
+	else
 		sprintf(buf, "%s/%s-%08x", part->name, Hosts[reps[i].serverNum].name, volId);
-	
 
 	/* Remove the file if it already exists. Since we made the
 	 * dump dir it can only exist if we are retrying the
 	 * replicated dump even though it succeeded for this replica
 	 * last time around. Don't care if it fails.  */
 	unlink(buf);
-	
+
 	/* Setup the write thread to handle this operation. */
-	CODA_ASSERT(strlen(buf) < sizeof(Rock.dumpfile));
-	strcpy(Rock.dumpfile, buf);
+	Rock.dumpfd = open(buf, O_CREAT | O_WRONLY | O_TRUNC, 0644);
 	Rock.volid = reps[i].backupId;
 	Rock.numbytes = 0;
-	
+
 	VLog(0, "Dumping %x.%x to %s ...", volId, reps[i].repvolId, buf);
 
 	rc = VolNewDump(Hosts[reps[i].serverNum].rpcid, reps[i].backupId,
 			&dumplevel);
+
+	close(Rock.dumpfd);
+
 	if (rc != RPC2_SUCCESS) {
 	    LogMsg(0,0,stdout, "VolDump (%x) failed on %x with %s\n",
 		   Hosts[reps[i].serverNum].rpcid, /* For debugging. */
@@ -639,26 +638,26 @@ int dumpVolume(volinfo_t *vol)
 	    continue;
 	}
 
-	/* Incremental can be forced to be full. */	
+	/* Incremental can be forced to be full. */
 	if (dumplevel == 0) {
 		vol->flags &= ~(INCREMENTAL | DUMPLVL_MASK);
 	}
-	
+
 	/* Setup a pointer from the dump subtree to the actual dumpfile
 	 * if a different partition was used for storage.
 	 */
-	
+
 	char link[66];
 	if (vol->flags & REPLICATED)
 		sprintf(link,"%s/%08x.%08x",Hosts[reps[i].serverNum].name,
 			volId, reps[i].repvolId);
 	else
 		sprintf(link, "%s/%08x", Hosts[reps[i].serverNum].name, volId);
-	    
+
 	/* Remove the link if it exists. See comment by previous
 	   unlink. */
 	unlink(link);
-	    
+
 	if (symlink(buf, link) == -1) {
 		if (errno == EEXIST) {	/* Retrying dump. */
 			if (unlink(link) != -1)
@@ -666,7 +665,7 @@ int dumpVolume(volinfo_t *vol)
 					break;
 		}
 		perror("symlink");
-		unlink(buf);	/* Delete the dump file. */		
+		unlink(buf);	/* Delete the dump file. */
 		return -1;
 	}
 
@@ -816,14 +815,14 @@ int main(int argc, char **argv) {
 
     /* Start up thread to handle WriteDump requests. */
     PROCESS dumpPid;
-    memset((char *)&Rock, 0, sizeof(struct rockInfo));
+    memset(&Rock, 0, sizeof(struct rockInfo));
     LWP_CreateProcess(VolDumpLWP, 5 * 1024, LWP_NORMAL_PRIORITY,
-		      &Rock, "VolDumpLWP", &dumpPid);
+		      (void *)&Rock, "VolDumpLWP", &dumpPid);
 
     /* Start up thread to periodically poll down servers */
     PROCESS pollPid;
     LWP_CreateProcess(PollLWP, 8 * 1024, LWP_NORMAL_PRIORITY - 1,
-		      (void *)Naptime, "PollLWP", &pollPid);
+		      (void *)&Naptime, "PollLWP", &pollPid);
 
     /* First try to backup (clone, dump, and mark) all volumes. Do all
      * volumes for a phase before starting the next phase to localize
@@ -1052,7 +1051,7 @@ static void VUInitServerList() {
     /* Find the server id */
     char hostname[100];
     char line[200];
-    char *serverList = SERVERLISTPATH;
+    const char *serverList = SERVERLISTPATH;
     FILE *file;
 
     LogMsg(9, Debug, stdout, "Entering VUInitServerList");
@@ -1141,18 +1140,22 @@ static void V_BindToServer(char *fileserver, RPC2_Handle *RPCid)
  /* Binds to File Server on volume utility port on behalf of uName.
     Sets RPCid to the value of the connection id.    */
 
+    static struct secret_state state = { 0, };
     RPC2_HostIdent hident;
     RPC2_PortIdent pident;
     RPC2_SubsysIdent sident;
     RPC2_BindParms bparms;
     RPC2_Handle rpcid;
     RPC2_EncryptionKey secret;
-    long     rc;
+    long rc;
+    struct servent *s = coda_getservbyname("codasrv", "udp");
 
     hident.Tag = RPC2_HOSTBYNAME;
     strcpy(hident.Value.Name, fileserver);
-    pident.Tag = RPC2_PORTBYNAME;
-    strcpy(pident.Value.Name, "codasrv");
+
+    pident.Tag = RPC2_PORTBYINETNUMBER;
+    pident.Value.InetPortNumber = s->s_port;
+
     sident.Tag = RPC2_SUBSYSBYID;
     sident.Value.SubsysId = UTIL_SUBSYSID;
 
@@ -1161,7 +1164,7 @@ static void V_BindToServer(char *fileserver, RPC2_Handle *RPCid)
     bparms.EncryptionType = RPC2_XOR;
     bparms.SideEffectType = SMARTFTP;
 
-    GetSecret(vice_sharedfile(VolTKFile), secret);
+    GetSecret(vice_sharedfile(VolTKFile), secret, &state);
     bparms.SharedSecret = &secret;
 
     LogMsg(10, Debug, stdout, "V_BindToServer: binding to host %s\n", fileserver);
@@ -1190,7 +1193,7 @@ static void V_BindToServer(char *fileserver, RPC2_Handle *RPCid)
  */
 static void PollLWP(void *arg)
 {
-    int naptime = (int)arg;
+    int naptime = *(int *)arg;
     struct timeval time;
     struct DiskPartition *part;
     struct dllist_head *tmp;
@@ -1291,7 +1294,7 @@ long S_WriteDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Unsigned *nbytes,
     struct rockInfo *rockinfo;
     SE_Descriptor sed;
     char *rock;
-    
+
     CODA_ASSERT(LWP_GetRock(ROCKTAG, &rock) == LWP_SUCCESS);
     rockinfo = (struct rockInfo *)rock;
 
@@ -1309,13 +1312,11 @@ long S_WriteDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Unsigned *nbytes,
     sed.Tag = SMARTFTP;
     sed.Value.SmartFTPD.TransmissionDirection = CLIENTTOSERVER;
     sed.Value.SmartFTPD.ByteQuota = -1;
-    sed.Value.SmartFTPD.SeekOffset = offset;
+    sed.Value.SmartFTPD.SeekOffset = -1; /* setting this to 'offset' wreaks
+					    havoc with dumps > 4GB */
     sed.Value.SmartFTPD.hashmark = 0;
-    sed.Value.SmartFTPD.Tag = FILEBYNAME;
-    sed.Value.SmartFTPD.FileInfo.ByName.ProtectionBits = 0755;
-    CODA_ASSERT(strlen(rockinfo->dumpfile) <
-	   sizeof(sed.Value.SmartFTPD.FileInfo.ByName.LocalFileName));
-    strcpy(sed.Value.SmartFTPD.FileInfo.ByName.LocalFileName, rockinfo->dumpfile);
+    sed.Value.SmartFTPD.Tag = FILEBYFD;
+    sed.Value.SmartFTPD.FileInfo.ByFD.fd = rockinfo->dumpfd;
 
     struct timeval before, after;
     gettimeofday(&before, 0);
