@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2004 Carnegie Mellon University
+          Copyright (c) 2004-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -93,6 +93,7 @@ public:
   /* the following are only used by directories */
   unsigned int isdir;
   unsigned int children;
+  unsigned int owner;
   unsigned int dir_mtime;
 
   DumpObject(VnodeId, Unique_t);
@@ -147,6 +148,9 @@ int DebugLevel = 0; /* set by "-d xxx" command line switch */
 #define DEBUG_HEAVY  (DebugLevel >= 10)
 #define DEBUG_LIGHT  (DebugLevel > 0 && DebugLevel < 10)
 
+int Verbose = 0; /* set by "-v" command line switch */
+size_t Bytes = 0;
+
 /* Hash table of all objects found in dump */
 ohashtab *DumpTable;
 
@@ -175,13 +179,13 @@ void DoGlobalSetup();
 int ProcessDirectory();
 void DiscoverPathnames();
 void VerifyEverythingNamed();
-void CreateEmptyDirectories();
+void CreateDirectories(int empty);
 int ProcessFileOrSymlink();
 void ProcessHardLinks();
 
 /* helper routines */
 DumpObject *GetDumpObj(VnodeId, Unique_t);
-int LowBits(DumpObject *);
+intptr_t LowBits(void *arg);
 void FreeDirectory(PDirInode);
 int AddNameEntry(struct DirEntry *, void *);
 void CreateHardLinkRecd(DumpObject *, int idx);
@@ -265,9 +269,11 @@ int main(int argc, char **argv)
     }
   }
 
-  /* use LVNlist to add empty directories to the tarfile */
-  if (!ThisHead.Incremental)
-    CreateEmptyDirectories();
+  /* use LVNlist to add directories to the tarfile */
+  /* We only add empty directories if this is a full dump, for incremental
+   * dumps we add all directories as incremental tar dumps also contain all
+   * directories. */
+  CreateDirectories(!ThisHead.Incremental);
 
   /* Get Small Vnodes (i.e. plain files) */
 
@@ -294,6 +300,10 @@ int main(int argc, char **argv)
   delete DStream;
   TarObj.WriteZeroTrailer();
   fclose(TarFile);
+
+  if (Verbose) {
+      fprintf(stderr, "Total bytes written: %lu\n", (unsigned long)Bytes);
+  }
   exit(0);
 }
 
@@ -340,13 +350,19 @@ void ParseArgs(int argc, char **argv)
       continue;
     }
 
+    /* [ -v ] */
+    if (!strcmp(argv[i], "-v")) {
+      Verbose = 1;
+      continue;
+    }
+
     goto BogusArgs; /* unknown flag */
 
   }
   return; /* done with all args */
 
  BogusArgs:
-    printf("Usage: codadump2tar [-f <dumpfilename>] [-o <tarfilename>] [-d <debuglevel>] [-hp <hashpower>] [-rn <rootname>]\n");
+    printf("Usage: codadump2tar [-v] [-f <dumpfilename>] [-o <tarfilename>] [-d <debuglevel>] [-hp <hashpower>] [-rn <rootname>]\n");
     exit(-1);
 
 }
@@ -356,7 +372,7 @@ void DoGlobalSetup()
   /* separate routine so main() doesn't get too long */
 
   /* create dump object table */
-  DumpTable = new ohashtab((2 << (HashPower-1)), (int (*)(void *)) LowBits);
+  DumpTable = new ohashtab((2 << (HashPower-1)), LowBits);
   if (!DumpTable) {
     fprintf(stderr, "Can't create DumpTable\n");
     exit(-1);
@@ -413,6 +429,7 @@ int ProcessDirectory()
 
   /* fill in missing pieces */
   CurrentDir->isdir = 1;
+  CurrentDir->owner = (unsigned int) vdo->owner;
   CurrentDir->dir_mtime = (unsigned int) vdo->unixModifyTime;
 
   /* add pointer to this object in list of large vnodes */
@@ -439,7 +456,7 @@ int ProcessDirectory()
   return 0;
 }
 
-void CreateEmptyDirectories()
+void CreateDirectories(int empty)
 {
   /* output tar commands to create skeleton of tree */
 
@@ -448,10 +465,10 @@ void CreateEmptyDirectories()
 
   /* Loop through directories, creating tar record for each */
 
-  for (lvn = 0; lvn < LVNfillcount; lvn++) {
+  for (lvn = 1; lvn < LVNfillcount; lvn++) {
     thisd = LVNlist[lvn]; /* next dump object */
     CODA_ASSERT(thisd->isdir);  /* sheer paranoia */
-    if (thisd->children) continue;
+    if (empty && thisd->children) continue;
 
     if (DEBUG_HEAVY) {
 	char path[CODA_MAXPATHLEN];
@@ -465,13 +482,12 @@ void CreateEmptyDirectories()
        failures when trying to create subdirectories via "tar xvf" of output
        tarball in normal Unix systems.
     */
-       
     TarObj.Reset();
     TarObj.tr_type = DIRTYPE;
     TarObj.tr_mode = 0755;
-    TarObj.tr_uid = 0;
+    TarObj.tr_uid = thisd->owner;
     TarObj.tr_size = 0;
-    TarObj.tr_mtime = (unsigned int) thisd->dir_mtime;
+    TarObj.tr_mtime = thisd->dir_mtime;
     TarObj.GetNameParts(thisd);
 
     /* fill and output this tar record */
@@ -554,6 +570,8 @@ int ProcessFileOrSymlink()
     exit(-1);
   }
 
+  Bytes += smallv.length;
+
   /* We are done!  DStream should be at next small vnode (or EOF) */
   return 0;
 }
@@ -729,9 +747,9 @@ TarRecd::TarRecd()
 
 void TarRecd::Reset()
 {
-  struct posix_header *tblk = &tarblock.header; 
+  struct posix_header *tblk = &tarblock.header;
 
-  /* Initialize header block to clean state 
+  /* Initialize header block to clean state
      Fields are Ascii, with numbers expressed in octal.
      No trailing null except for name fields explicitly stated below.
      Zero-fill initialization guarantees trailing NULL if needed.
@@ -740,7 +758,7 @@ void TarRecd::Reset()
   memset(tarblock.buffer, 0, BLOCKSIZE); /* zerofill */
 
   /* set fields that never change for us */
-  memset(tblk->gid, '0', 8);
+  memset(tblk->gid, '0', 7);
   memcpy(tblk->magic, TMAGIC, TMAGLEN-1); /* trailing NULL */
   memcpy(tblk->version, TVERSION, TVERSLEN);
 
@@ -763,34 +781,34 @@ void TarRecd::GetNameParts(DumpObject *obj, int idx)
 void TarRecd::Format()
 {
   struct posix_header *tblk = &tarblock.header; 
-  char temp[100];  /* temporary to avoid NULLs in TarRecd from sprintf() */
   unsigned int sum; /* for checksum computation */
   int i;
 
-  memcpy(&tblk->typeflag, &tr_type, 1);
-  sprintf(temp,     "%08o",  tr_mode);
-  memcpy(tblk->mode, temp, 8);
-  sprintf(temp,      "%08o",  tr_uid);
-  memcpy(tblk->uid, temp, 8);
-  sprintf(temp,     "%012o", tr_size);  
-  memcpy(tblk->size, temp, 12);
-  sprintf(temp,    "%012o", tr_mtime);
-  memcpy(tblk->mtime, temp, 12);
+  tblk->typeflag = tr_type;
+  snprintf(tblk->mode,   8, "%07o", tr_mode);
+  snprintf(tblk->uid,    8, "%07o", tr_uid);
+  snprintf(tblk->size,  12, "%011o", tr_size);
+  snprintf(tblk->mtime, 12, "%011o", tr_mtime);
 
   /* copy name info; leave NULL in last byte from Reset() */
-  memcpy(tblk->name, tr_name, 99); 
-  memcpy(tblk->linkname, tr_linkname, 99); 
+  memcpy(tblk->name, tr_name, 99);
+  memcpy(tblk->linkname, tr_linkname, 99);
   memcpy(tblk->prefix, tr_prefix, 154);
 
   /* Compute checksum */
   memset(tblk->chksum, ' ', 8);       /* 8 blanks */
 
-  sum = 0; 
-  for (i = 0; i < BLOCKSIZE; i++) {
+  sum = 0;
+  for (i = 0; i < BLOCKSIZE; i++)
     sum += (unsigned int) tarblock.buffer[i];
+
+  snprintf(tblk->chksum, 7, "%06o", sum);
+
+  if (Verbose) {
+    if (*tr_prefix)
+	 fprintf(stderr, "%s/%s\n", tr_prefix, tr_name);
+    else fprintf(stderr, "%s\n", tr_name);
   }
-  sprintf(temp, "%08o", sum);
-  memcpy(tblk->chksum, temp, 8);
 }
 
 void TarRecd::Output()
@@ -802,6 +820,8 @@ void TarRecd::Output()
     perror("tar file output error");
     exit(-1);
     }
+
+  Bytes += BLOCKSIZE;
 }
 
 void TarRecd::WriteZeroTrailer()
@@ -862,15 +882,16 @@ void FreeDirectory(PDirInode pdiri)
 }
 
 
-int LowBits(DumpObject *obj)
+intptr_t LowBits(void *arg)
 {
+  DumpObject *obj = (DumpObject *)arg;
   /* hash function: use low order bits of object's vnode number */
   int bucket, shift;
 
   shift = 32-HashPower;
   bucket = (obj->oid.vnode) << shift;
   bucket = bucket >> shift;
-  return(bucket);  
+  return bucket;
 }
 
 int AddNameEntry(struct DirEntry *de, void *hook)
