@@ -3,7 +3,7 @@
                            Coda File System
                               Release 6
 
-          Copyright (c) 1987-2003 Carnegie Mellon University
+          Copyright (c) 1987-2008 Carnegie Mellon University
                   Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
@@ -35,7 +35,6 @@ extern "C" {
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <netinet/in.h>
-#include <netdb.h>
 #include <errno.h>
 #include <ctype.h>
 
@@ -51,7 +50,6 @@ extern "C" {
 #include <rpc2/sftp.h>
 #include <util.h>
 #include <partition.h>
-#include <ports.h>
 #include <vice.h>
 #include <callback.h>
 #include <volutil.h>
@@ -70,16 +68,16 @@ extern "C" {
 #include <codaconf.h>
 #include <vice_file.h>
 #include <getsecret.h>
+#include <coda_getservbyname.h>
 
-static char *vicedir = NULL;
-static int   nservers = 0;
+static const char *vicedir = NULL;
+static int nservers = 0;
 
 /* hack to make argc and argv visible to subroutines */
 static char **this_argp;
 static int these_args;
 
 static char s_hostname[100];
-static int timeout = 0;
 static RPC2_Handle rpcid;
 static long rc;
 
@@ -127,9 +125,10 @@ static void pokexmem(void);
 
 #define ROCKTAG 12345
 struct rockInfo {
-    int fd;		    /* Open filedescriptor for ReadDump. */
+    int fd;		    /* Open filedescriptor for ReadDump/WriteDump. */
     VolumeId volid;	    /* Volume being dumped. */
-    unsigned long numbytes; /* Number of bytes already written to the file. */
+    unsigned int numbytes;  /* Number of bytes already written to the file.
+			       (has to wrap around the same way as 'offset') */
 };
 
 static void V_InitRPC(int timeout);
@@ -152,13 +151,13 @@ void ReadConfigFile(void)
 int main(int argc, char **argv)
 {
     char *realm = NULL;
+    int timeout = 30;	/* Default rpc2 timeout is 30 seconds. */
 
-    /* Set the default timeout and server host */
-    timeout = 30;	/* Default rpc2 timeout is 30 seconds. */
+    /* Set the default server host */
     gethostname(s_hostname, sizeof(s_hostname) -1);
 
     ReadConfigFile();
-	
+
     while (argc > 2 && *argv[1] == '-') { /* All options require an argument. */
 	if (strcmp(argv[1], "-h") == 0) { /* User specified other host. */
 	    struct hostent *hp;
@@ -184,7 +183,7 @@ int main(int argc, char **argv)
     }
 
     if (argc < 2)
-    	goto bad_options;
+	goto bad_options;
 
     CODA_ASSERT(*s_hostname != '\0');
     V_InitRPC(timeout);
@@ -260,7 +259,7 @@ int main(int argc, char **argv)
     else if (strcmp(argv[1], "truncatervmlog") == 0)
 	truncatervmlog();
     else if (strcmp(argv[1], "getmaxvol") == 0)
-        getmaxvol();
+	getmaxvol();
     else if (strcmp(argv[1], "setmaxvol") == 0)
 	setmaxvol();
     else if (strcmp(argv[1], "peek") == 0)
@@ -276,7 +275,7 @@ int main(int argc, char **argv)
     else if (strcmp(argv[1], "pokex") == 0)
 	pokexmem();
     else
-    	goto bad_options;
+	goto bad_options;
 
     return 0;
 
@@ -544,14 +543,14 @@ static void clone(void)
 static void dump(void)
 {
     long rc = 0;
-    RPC2_Unsigned Incremental = 0;
+    RPC2_Integer Incremental = 0;
     int err = 0;
     FILE *outf;
-    
+
     while ((these_args > 2) && *this_argp[2] == '-') {
 	if (strcmp(this_argp[2], "-i") == 0) {
 	    if (these_args > 4) {
-	    	Incremental = (RPC2_Unsigned)atoi(this_argp[3]);
+		Incremental = atoi(this_argp[3]);
 		these_args--; this_argp++;
 	    } else
 		Incremental = 1;
@@ -580,18 +579,18 @@ static void dump(void)
     rock->fd = fileno(outf);
     rock->volid = volid;
     rock->numbytes = 0;
-    
+
     PROCESS dumpPid;
     LWP_CreateProcess(VolDumpLWP, 16 * 1024, LWP_NORMAL_PRIORITY,
-		      rock, "VolDumpLWP", &dumpPid);
-    
+		      (void *)rock, "VolDumpLWP", &dumpPid);
+
     rc = VolNewDump(rpcid, volid, &Incremental);
     if (rc != RPC2_SUCCESS) {
 	fprintf(stderr, "\nVolDump failed with %s\n", RPC2_ErrorMsg((int)rc));
 	exit(-1);
     }
 
-    fprintf(stderr, "\n%sVolDump completed, %lu bytes dumped\n",
+    fprintf(stderr, "\n%sVolDump completed, %u bytes dumped\n",
 	    Incremental ? "Incremental " : "", rock->numbytes);
     exit(0);
 }
@@ -665,7 +664,6 @@ static void VolDumpLWP(void *arg)
 	}
 	else fprintf(stderr, "VolDumpLWP: Get_Request failed with %s\n",RPC2_ErrorMsg((int)rc));
     }
-
 }
 
 long S_WriteDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Unsigned *nbytes, VolumeId volid, SE_Descriptor *BD)
@@ -674,7 +672,7 @@ long S_WriteDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Unsigned *nbytes,
     struct rockInfo *rockinfo;
     SE_Descriptor sed;
     char *rock;
-    
+
     CODA_ASSERT(LWP_GetRock(ROCKTAG, &rock) == LWP_SUCCESS);
     rockinfo = (struct rockInfo *)rock;
 
@@ -685,23 +683,24 @@ long S_WriteDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Unsigned *nbytes,
     }
 
     if (rockinfo->numbytes != offset) {
-	fprintf(stderr, "Offset %d != rockInfo->numbytes %ld\n",
+	fprintf(stderr, "Offset %d != rockInfo->numbytes %d\n",
 		offset, rockinfo->numbytes);
     }
-    
+
     /* fetch the file with volume data */
     memset(&sed, 0, sizeof(SE_Descriptor));
     sed.Tag = SMARTFTP;
     sed.Value.SmartFTPD.TransmissionDirection = CLIENTTOSERVER;
     sed.Value.SmartFTPD.ByteQuota = -1;
-    sed.Value.SmartFTPD.SeekOffset = offset;
+    sed.Value.SmartFTPD.SeekOffset = -1; /* setting this to 'offset' wreaks
+					    havoc with dumps > 4GB */
     sed.Value.SmartFTPD.hashmark = 0;
     sed.Value.SmartFTPD.Tag = FILEBYFD;
     sed.Value.SmartFTPD.FileInfo.ByFD.fd = rockinfo->fd;
 
     if ((rc = RPC2_InitSideEffect(rpcid, &sed)) <= RPC2_ELIMIT){
 	fprintf(stderr, "WriteDump: Error %s in InitSideEffect\n", RPC2_ErrorMsg((int)rc));
-    } else if ((rc = RPC2_CheckSideEffect(rpcid, &sed, SE_AWAITLOCALSTATUS)) 
+    } else if ((rc = RPC2_CheckSideEffect(rpcid, &sed, SE_AWAITLOCALSTATUS))
 	       <= RPC2_ELIMIT) {
 	fprintf(stderr, "WriteDump: Error %s in CheckSideEffect\n", RPC2_ErrorMsg((int)rc));
     }
@@ -783,7 +782,7 @@ static void restorefromback(void)
     
     PROCESS restorePid;
     LWP_CreateProcess(VolDumpLWP, 16 * 1024, LWP_NORMAL_PRIORITY,
-		      rock, "VolDumpLWP", &restorePid);
+		      (void *)rock, "VolDumpLWP", &restorePid);
     if (rc != LWP_SUCCESS) {
 	fprintf(stderr, "VolDump can't create child %s\n", RPC2_ErrorMsg((int)rc));
 	exit(-1);
@@ -804,7 +803,6 @@ long S_ReadDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Integer *nbytes, V
     long rc = 0;
     struct rockInfo *rockinfo;
     SE_Descriptor sed;
-    char *buf;
     char *rock;
     
     CODA_ASSERT(LWP_GetRock(ROCKTAG, &rock) == LWP_SUCCESS);
@@ -819,41 +817,17 @@ long S_ReadDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Integer *nbytes, V
 	exit(-1);
     }
 
-    /* Set up a buffer and read in the data from the dump file. */
-    buf = (char *)malloc((unsigned int)*nbytes);
-    if (!buf) {
-	perror("ReadDump: Can't malloc buffer!");
-	exit(-1);
-    }
-
     CODA_ASSERT(rockinfo->fd != 0); /* Better have been opened by restore() */
 
-    if (lseek(rockinfo->fd, offset, L_SET) == -1) {
-	perror("ReadDump: lseek");
-	*nbytes = 0;
-	free(buf);
-	return 0;
-    }
-
-    *nbytes = read(rockinfo->fd, buf, (int)*nbytes);
-    if (*nbytes == -1) {
-	perror("ReadDump: read");
-	*nbytes = 0;
-	free(buf);
-	return 0;
-    }
-    
     /* fetch the file with volume data */
     memset(&sed, 0, sizeof(SE_Descriptor));
     sed.Tag = SMARTFTP;
     sed.Value.SmartFTPD.TransmissionDirection = SERVERTOCLIENT;
-    sed.Value.SmartFTPD.ByteQuota = -1;
-    sed.Value.SmartFTPD.SeekOffset = 0;
+    sed.Value.SmartFTPD.ByteQuota = *nbytes;
+    sed.Value.SmartFTPD.SeekOffset = -1;
     sed.Value.SmartFTPD.hashmark = 0;
-    sed.Value.SmartFTPD.Tag = FILEINVM;
-    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqBody = (RPC2_ByteSeq)buf;
-    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.MaxSeqLen = 
-    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqLen = *nbytes;
+    sed.Value.SmartFTPD.Tag = FILEBYFD;
+    sed.Value.SmartFTPD.FileInfo.ByFD.fd = rockinfo->fd;
 
     if ((rc = RPC2_InitSideEffect(rpcid, &sed)) <= RPC2_ELIMIT){
 	fprintf(stderr, "ReadDump: Error %s in InitSideEffect\n", RPC2_ErrorMsg((int)rc));
@@ -868,7 +842,6 @@ long S_ReadDump(RPC2_Handle rpcid, RPC2_Unsigned offset, RPC2_Integer *nbytes, V
     fprintf(stderr, ".");
 #endif
     rockinfo->numbytes += sed.Value.SmartFTPD.BytesTransferred;
-    free(buf);
     return rc;
 }
 
@@ -1751,20 +1724,21 @@ static void setmaxvol(void)
 
 static void peekpokeerr(void)
 {
-    static	char	*msgs[]={
+    static const char *msgs[] = {
 	/*0*/ "Path to server file not known",
 	/*1*/ "Cannot read symbols from the server file",
 	/*2*/ "Symbol not found in the server file",
 	/*3*/ "Address not in server virtual space",
 	/*4*/ "Memory protection error",
-	/*5*/ "Unaligned integer/pointer reference"};
+	/*5*/ "Unaligned integer/pointer reference"
+    };
     fprintf(stderr, "Couldn't %s at %s: %s\n", this_argp[1], this_argp[2],
 	    (int)rc >= -10040L && rc < (int)(sizeof(msgs)/sizeof(*msgs)-10040L) ?
 	    msgs[10040 + (int) rc] : RPC2_ErrorMsg((int) rc));
     exit (-1);
 }
 
-static void usageerr(char *args)
+static void usageerr(const char *args)
 {
     fprintf(stderr, "Usage: %s %s %s\n", this_argp[0], this_argp[1], args);
     exit(-1);
@@ -1964,7 +1938,7 @@ static void V_InitRPC(int timeout)
     memset(&options, 0, sizeof(options));
     options.Flags = RPC2_OPTION_IPV6;
 
-    rcode = RPC2_Init(RPC2_VERSION, &options, NULL, 3, &tout);
+    rcode = RPC2_Init(RPC2_VERSION, &options, NULL, 5, &tout);
     if (rcode != RPC2_SUCCESS) {
 	fprintf(stderr, "RPC2_Init failed with %s\n", RPC2_ErrorMsg((int)rcode));
 	exit(-1);
@@ -1974,6 +1948,7 @@ static void V_InitRPC(int timeout)
 
 static int V_BindToServer(char *fileserver, char *realm, RPC2_Handle *RPCid)
 {
+    static struct secret_state state;
  /* Binds to File Server on volume utility port on behalf of uName.
     Sets RPCid to the value of the connection id.    */
 
@@ -1985,17 +1960,14 @@ static int V_BindToServer(char *fileserver, char *realm, RPC2_Handle *RPCid)
     EncryptedSecretToken stok;
     RPC2_CountedBS clientident;
     long rcode;
+    struct servent *s = coda_getservbyname("codasrv", "udp");
 
     hident.Tag = RPC2_HOSTBYNAME;
     strcpy(hident.Value.Name, fileserver);
-#ifdef __CYGWIN32__
-	/* XXX -JJK */
-	pident.Tag = RPC2_PORTBYINETNUMBER;
-	pident.Value.InetPortNumber = htons(PORT_codasrv);
-#else
-    pident.Tag = RPC2_PORTBYNAME;
-    strcpy(pident.Value.Name, "codasrv");
-#endif
+
+    pident.Tag = RPC2_PORTBYINETNUMBER;
+    pident.Value.InetPortNumber = s->s_port;
+
     sident.Tag = RPC2_SUBSYSBYID;
     sident.Value.SubsysId = UTIL_SUBSYSID;
 
@@ -2006,7 +1978,7 @@ static int V_BindToServer(char *fileserver, char *realm, RPC2_Handle *RPCid)
     bparms.EncryptionType = RPC2_XOR;
     bparms.SideEffectType = SMARTFTP;
 
-    if (GetSecret(vice_sharedfile(VolTKFile), secret) == 0)
+    if (GetSecret(vice_sharedfile(VolTKFile), secret, &state) == 0)
     {
 	bparms.AuthenticationType = AUTH_METHOD_VICEKEY;
 	bparms.SharedSecret = &secret;
