@@ -31,7 +31,6 @@ extern "C" {
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <netdb.h>
 #include <netinet/in.h>
 #include <sys/time.h>
 #include <time.h>
@@ -46,7 +45,6 @@ extern "C" {
 #include <curses.h>
 #endif
 
-#include <ports.h>
 #include <lwp/lwp.h>
 #include <lwp/timer.h>
 #include <rpc2/rpc2.h>
@@ -61,6 +59,9 @@ extern "C" {
 }
 #endif
 
+#include <coda_getservbyname.h>
+#include <coda_getaddrinfo.h>
+
 static time_t MonBirthTime; /* when this monitor was born */
 
 typedef enum {DEAD, NEWBORN, ALIVE} SrvState;
@@ -182,7 +183,8 @@ int main(int argc, char *argv[])
     LWP_CreateProcess(kbdlwp, 0x4000, LWP_NORMAL_PRIORITY, NULL, "KBD", &lwpid);
     for (i = 0; i < SrvCount; i++)
     {
-	LWP_CreateProcess(srvlwp, 0x8000, LWP_NORMAL_PRIORITY, (char *)i, (char *)srv[i].srvname, &srv[i].pid);
+	LWP_CreateProcess(srvlwp, 0x8000, LWP_NORMAL_PRIORITY, (void *)&i,
+			  (char *)srv[i].srvname, &srv[i].pid);
     }
     
     LWP_WaitProcess(&Dummy); /* wait for Godot */
@@ -191,24 +193,24 @@ int main(int argc, char *argv[])
 
 static void srvlwp(void *arg)
 {
-    int slot = (int)arg;
+    int slot = *(int*)arg;
     struct server *moi;
     RPC2_HostIdent hi;
     RPC2_PortIdent pi;
     RPC2_SubsysIdent si;
     RPC2_Handle cid;
     int rc;
-    
+    struct servent *s = coda_getservbyname("codasrv", "udp");
+
     moi = &srv[slot];
 
     hi.Tag = RPC2_HOSTBYNAME;
     strcpy(hi.Value.Name, moi->srvname);
     pi.Tag = RPC2_PORTBYINETNUMBER;
-    pi.Value.InetPortNumber = htons(PORT_codasrv);
+    pi.Value.InetPortNumber = s->s_port;
     si.Tag = RPC2_SUBSYSBYID;
     si.Value.SubsysId= SUBSYS_SRV;
-    
-    
+
     moi->state = DEAD;
     /* Get this out quick so we don't have to wait for a
        RPC2_NOBINDING to tell the user there is NO BINDING */
@@ -257,7 +259,7 @@ static void srvlwp(void *arg)
 
 	IOMGR_Select(32, 0, 0, 0, &ProbeInterval);  /* sleep */
 	}
-    }
+}
 
 static void kbdlwp(void *arg)
     /* LWP to listen for keyboard activity */
@@ -335,7 +337,7 @@ static void GetArgs(int argc, char *argv[])
 		exit(-1);
 		}
 	    srv[SrvCount].srvname = argv[next];
-	    c = index(argv[next], ':');
+	    c = strchr(argv[next], ':');
 	    if (c) {
 	        *c = 0; /* so servername is null terminated */
 	    }
@@ -678,9 +680,9 @@ static int ValidServer(char *s)
     hints.ai_socktype = SOCK_DGRAM;
     hints.ai_protocol = IPPROTO_UDP;
 
-    ret = RPC2_getaddrinfo(s, "codasrv", &hints, &res);
+    ret = coda_getaddrinfo(s, "codasrv", &hints, &res);
     RPC2_freeaddrinfo(res);
-    
+
     return (ret == 0);
 }
 
