@@ -33,29 +33,23 @@ extern "C" {
 #endif
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include "coda_string.h"
 #include <unistd.h>
 #include <stdlib.h>
-#include <codaconf.h>
 
 #ifdef __cplusplus
 }
 #endif
 
-/* FreeBSD 2.2.5 defines this in rpc/types.h, all others in netinet/in.h */
-#ifndef INADDR_LOOPBACK
-#define INADDR_LOOPBACK 0x7f000001
-#endif
+#include <codaconf.h>
+#include <coda_getaddrinfo.h>
 
-const char MarinerService[] = "venus";
 #define MARINERBUFSIZE 1000
 
-
-int Bind(const char *, const char *);
+int Bind(const char *);
 void CheckMariner(FILE *);
 void CheckTheMariner(char *);
 
@@ -80,7 +74,7 @@ int main(int argc, char *argv[])
     while (running)
     {
 	/* Bind to Venus and ask it to send us Fetch/Store messages. */
-	venusSocket = Bind(MarinerService, host);
+	venusSocket = Bind(host);
 	if (venusSocket < 0) {
 	    fprintf(stderr, "codacon: bind to %s failed, venus not running?\n",
 		    host ? host : "localhost");
@@ -88,7 +82,7 @@ int main(int argc, char *argv[])
 	    continue;
 	}
 	FILE *fp = fdopen(venusSocket, "r");
-	char *p = "set:fetch\n";
+	const char *p = "set:fetch\n";
 	ssize_t len = strlen(p);
 	if (write(venusSocket, p, len) != len) {
 	    fprintf(stderr, "codacon: set:fetch command failed (%d)\n", errno);
@@ -102,62 +96,54 @@ int main(int argc, char *argv[])
 }
 
 
-int Bind(const char *service, const char *host)
+int Bind(const char *host)
 {
-    int s;
-    
+    int s = -1;
+
 #ifdef HAVE_SYS_UN_H
     if (!use_tcp) {
-        struct sockaddr_un s_un;
-        char *MarinerSocketPath;
-        
-        codaconf_init("venus.conf");
-        MarinerSocketPath = codaconf_lookup("marinersocket",
-					    "/usr/coda/spool/mariner"); 
-        memset(&s_un, 0, sizeof(s_un));
-        s_un.sun_family = AF_UNIX;
-        strcpy(s_un.sun_path, MarinerSocketPath);
+	struct sockaddr_un s_un;
+	const char *MarinerSocketPath;
 
-        if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) < 0){
-            return(-1);
-        }
-        if (connect(s, (sockaddr *)&s_un, sizeof(s_un)) < 0) {
-            close(s);
-            return(-1);
-        }
+	codaconf_init("venus.conf");
+	MarinerSocketPath = codaconf_lookup("marinersocket",
+					    "/usr/coda/spool/mariner");
+	memset(&s_un, 0, sizeof(s_un));
+	s_un.sun_family = AF_UNIX;
+	strcpy(s_un.sun_path, MarinerSocketPath);
+
+	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) < 0){
+	    return(-1);
+	}
+	if (connect(s, (sockaddr *)&s_un, sizeof(s_un)) < 0) {
+	    close(s);
+	    return(-1);
+	}
     } else
 #endif /* !HAVE_SYS_UN_H */
     {
-        struct sockaddr_in sin;
-        struct servent *sp;
-        struct hostent *hp;
+	struct RPC2_addrinfo hints, *p, *ai = NULL;
+	int rc;
 
-        memset(&sin, 0, sizeof(sin));
+	memset(&hints, 0, sizeof(struct RPC2_addrinfo));
+	hints.ai_family   = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
 
-        if (host) {
-            hp = gethostbyname(host);
-            if (hp == NULL) return(-1);
-            memcpy((char *)&sin.sin_addr, hp->h_addr, hp->h_length);
-            sin.sin_family = hp->h_addrtype;
-        } else {
-            sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-            sin.sin_family = AF_INET;
-        }
+	rc = coda_getaddrinfo(NULL, "venus", &hints, &ai);
+	if (rc) return -1;
 
-        sp = getservbyname(service, "tcp");
-        if (sp == NULL) return(-1);
-        sin.sin_port = sp->s_port;
+	for (p = ai; p != NULL; p = p->ai_next) {
+	    s = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
+	    if (s < 0) continue;
 
-        if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0){
-            return(-1);
-        }
-        if (connect(s, (sockaddr *)&sin, sizeof(sin)) < 0) {
-            close(s);
-            return(-1);
-        }
+	    if (connect(s, p->ai_addr, p->ai_addrlen) == 0) break;
+	    
+	    close(s);
+	    s = -1;
+	}
+	RPC2_freeaddrinfo(ai);
     }
-
-    return(s);
+    return s;
 }
 
 
@@ -195,7 +181,7 @@ void CheckMariner(FILE *fp) {
 
 void CheckTheMariner(char *buf)
 {
-    char *string, *prefix = "";
+    char *string, *prefix = NULL;
 
     /* search for a :: separator between the prefix and the actual message */
     string = strstr(buf, "::");
@@ -203,18 +189,15 @@ void CheckTheMariner(char *buf)
 	*string = '\0';
 	string += 2;
 	prefix = buf;
+
+	/* Some prefixes we do not want to display if the remaining string
+	 * contains ' done ' */
+	if ((strcmp(prefix, "fetch") == 0 || strcmp(prefix, "store") == 0 ||
+	     strcmp(prefix, "mond") == 0) && strstr(string, " done ") != NULL)
+	    return;
     } else
 	string = buf;
 
-    /* Some prefixes we do not want to display if they contain ' done ' */
-    if (strcmp(prefix, "fetch") == 0 ||
-	strcmp(prefix, "store") == 0 ||
-	strcmp(prefix, "mond") == 0)
-    {
-	if (strstr(string, " done "))
-	    return;
-    }
-
     printf("%s", string);
     fflush(stdout);
 }
