@@ -82,7 +82,7 @@ vuid_t ruid;
 vuid_t euid;
 int Debug = 0;
 int Verbose = 0;
-char *mountpoint = NULL;
+const char *mountpoint = NULL;
 
 
 #define	DEBUG(stmt) { if (Debug) { stmt ; fflush(stdout); } }
@@ -98,7 +98,7 @@ static int canonicalize(char *path, VolumeId *vid, char *realm, char *name,
 			char (*srealm)[MAXHOSTNAMELEN+1],
 			char (*spath)[MAXPATHLEN]);
 static char *vol_getwd(VolumeId *vid, char *realm, char *head, char *tail);
-static int  GetVid(VolumeId *, char *, char *);
+static int  GetVid(VolumeId *, char *, const char *);
 static void DoClears(olist&);
 static void DoAdds(olist&);
 static void DoDeletes(olist&);
@@ -110,7 +110,7 @@ static void DoDisables(olist&);
 static void MetaExpand(olist&, char *, int, int);
 static void ExpandNode(char *, VolumeId, char *, char *, olist&, int, int);
 static void RenameOutFile(char *, char *);
-static void error(int, char * ...);
+static void error(int, const char * ...);
 static void usage();
 static void parse_error(char *);
 
@@ -192,7 +192,7 @@ class verify_entry : public olink {
     hdb_verify_msg msg;
     char tname[MAXPATHLEN];
 
-    verify_entry(char *vname, vuid_t vuid, int verbosity) {
+    verify_entry(const char *vname, vuid_t vuid, int verbosity) {
 	tmpnam(msg.outfile);
 	strcpy(tname, vname);
 	msg.luid = vuid;
@@ -534,7 +534,6 @@ next_cmd:
 		verbosity = 0;
 		strcpy(vname, "/dev/tty");
 		vuid = ALL_UIDS;
-		
 
 		/* Verify command. */
 		if (!STREQ(token, "v") && !STREQ(token, "verify")) {
@@ -922,7 +921,7 @@ static char *vol_getwd(VolumeId *vp, char *realm, char *head, char *tail)
 }
 
 
-static int GetVid(VolumeId *vid, char *realm, char *name)
+static int GetVid(VolumeId *vid, char *realm, const char *name)
 {
 /**/
     DEBUG(printf("Entering GetVid (%s)\n", name););
@@ -940,7 +939,7 @@ static int GetVid(VolumeId *vid, char *realm, char *name)
     vi.out = (char *)&gf_msg;
     vi.out_size = sizeof(struct getfid_msg);
     err = pioctl(name, _VICEIOCTL(_VIOC_GETFID), &vi, 1);
-    
+
     *vid = 0;
     if (!err) {
 	*vid = gf_msg.fid.Volume;
@@ -963,7 +962,7 @@ static void DoClears(olist& Clear) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_CLEAR), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_CLEAR), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Clear(%d, %d): %s",
 		  c->msg.cuid, ruid, strerror(errno));
 	}
@@ -981,7 +980,7 @@ static void DoAdds(olist& Add) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_ADD), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_ADD), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Add(%x@%s, %s, %d, %d, %d): %s",
 		  a->msg.vid, a->msg.realm, a->msg.name,
 		  a->msg.priority, a->msg.attributes, ruid, strerror(errno));
@@ -1000,7 +999,7 @@ static void DoDeletes(olist& Delete) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_DELETE), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_DELETE), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Delete(%x@%s, %s, %d): %s",
 		  d->msg.vid, d->msg.realm, d->msg.name, ruid,
 		  strerror(errno));
@@ -1019,7 +1018,7 @@ static void DoLists(olist& List) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_LIST), &vi, 0) == 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_LIST), &vi, 0) == 0) {
 	    RenameOutFile(l->msg.outfile, l->tname);
 	}
 	else {
@@ -1041,7 +1040,7 @@ static void DoWalks(olist& Walk) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_WALK), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_WALK), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Walk(%d): %s",
 		  ruid, strerror(errno));
 	}
@@ -1062,7 +1061,7 @@ static void DoVerifies(olist& Verify) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_VERIFY), &vi, 0) == 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_VERIFY), &vi, 0) == 0) {
 	    RenameOutFile(v->msg.outfile, v->tname);
 	}
 	else {
@@ -1085,7 +1084,7 @@ static void DoEnables(olist& Enable) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_ENABLE), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_ENABLE), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Enable(%d): %s",
 		  ruid, strerror(errno));
 	}
@@ -1104,7 +1103,7 @@ static void DoDisables(olist& Disable) {
 	vi.out = 0;
 	vi.out_size = 0;
 
-	if (pioctl(mountpoint, _VICEIOCTL(_VIOC_HDB_DISABLE), &vi, 0) != 0) {
+	if (pioctl(NULL, _VICEIOCTL(_VIOC_HDB_DISABLE), &vi, 0) != 0) {
 	    error(!FATAL, "pioctl:Disable(%d): %s",
 		  ruid, strerror(errno));
 	}
@@ -1137,8 +1136,8 @@ static void MetaExpand(olist& Add, char *FullName, int priority, int attributes)
 	error(FATAL, "MetaExpand: chdir(%s/..) failed (%s)", FullName,
 	      strerror(errno));
     char mtpt[MAXPATHLEN];
-    char *cp;
-    if ((cp = rindex(VRPath, '/')) == NULL) cp = mountpoint;
+    const char *cp;
+    if ((cp = strrchr(VRPath, '/')) == NULL) cp = mountpoint;
     strcpy(mtpt, cp + 1);
     ExpandNode(mtpt, vid, realm, NodeName, Add, priority, attributes);
 
@@ -1168,7 +1167,7 @@ static void ExpandNode(char *mtpt, VolumeId vid, char *realm, char *name,
 
 	/* Determine the next component and attempt to cd into it. */
 	{
-	    char *cp = rindex(name, '/');
+	    char *cp = strrchr(name, '/');
 	    if (cp) cp++;
 	    else cp = mtpt;
 	    DEBUG(printf("ExpandNode: chdir(%s)\n", cp););
@@ -1250,8 +1249,8 @@ static void RenameOutFile(char *from, char *to) {
 	    error(!FATAL, "RenameOutFile: dup2() failed(%s)", strerror(errno));
 	    exit(errno);
 	}
-	char *argv[2]; argv[0] = "cat"; argv[1] = 0;
-	if (execvp("cat", argv) < 0) {
+	char *argv[2]; argv[0] = (char *)"cat"; argv[1] = NULL;
+	if (execvp(argv[0], argv) < 0) {
 	    error(!FATAL, "RenameOutFile: execvp(\"cat\") failed(%s)", strerror(errno));
 	    exit(errno);
 	}
@@ -1264,11 +1263,11 @@ static void RenameOutFile(char *from, char *to) {
 
 	if (!Debug)
 	    unlink(from);
-    }    
+    }
 }
 
 
-static void error(int fatal, char *fmt ...) {
+static void error(int fatal, const char *fmt ...) {
     va_list ap;
 
     char msg[240];
