@@ -31,10 +31,10 @@ extern int codaconf_quiet;
 /* conf_init reads (or merges) the name=value tuples from the conffile. If a
  * name is seen multiple times, only the last value is remembered. Empty lines
  * and lines starting with '#' are ignored. */
-int codaconf_init_one(char *conffile);
+int codaconf_init_one(const char *conffile);
 
 /* conf_lookup returns the value associated with name, or NULL on error. */
-char *codaconf_lookup(char *name, char *defaultvalue);
+const char *codaconf_lookup(const char *name, const char *defaultvalue);
 
 /* conf_free releases all resources allocated for the configuration data */
 void codaconf_free(void);
